// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.apache.http.impl.conn;

import java.io.Closeable;
import java.io.IOException;
import java.util.Date;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.HttpClientConnection;
import org.apache.http.HttpHost;
import org.apache.http.annotation.GuardedBy;
import org.apache.http.annotation.ThreadSafe;
import org.apache.http.config.ConnectionConfig;
import org.apache.http.config.Lookup;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.config.SocketConfig;
import org.apache.http.conn.ConnectionRequest;
import org.apache.http.conn.DnsResolver;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.conn.HttpConnectionFactory;
import org.apache.http.conn.ManagedHttpClientConnection;
import org.apache.http.conn.SchemePortResolver;
import org.apache.http.conn.routing.HttpRoute;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.Args;
import org.apache.http.util.Asserts;
import org.apache.http.util.LangUtils;

@ThreadSafe public class BasicHttpClientConnectionManager implements Closeable, HttpClientConnectionManager {
    @GuardedBy(value="this") private ManagedHttpClientConnection conn;
    @GuardedBy(value="this") private ConnectionConfig connConfig;
    private final HttpConnectionFactory connFactory;
    private final HttpClientConnectionOperator connectionOperator;
    @GuardedBy(value="this") private long expiry;
    private final AtomicBoolean isShutdown;
    @GuardedBy(value="this") private boolean leased;
    private final Log log;
    @GuardedBy(value="this") private HttpRoute route;
    @GuardedBy(value="this") private SocketConfig socketConfig;
    @GuardedBy(value="this") private Object state;
    @GuardedBy(value="this") private long updated;

    public BasicHttpClientConnectionManager() {
        this(BasicHttpClientConnectionManager.getDefaultRegistry(), null, null, null);
    }

    public BasicHttpClientConnectionManager(Lookup arg3, HttpConnectionFactory arg4, SchemePortResolver arg5, DnsResolver arg6) {
        ManagedHttpClientConnectionFactory v4;
        super();
        this.log = LogFactory.getLog(this.getClass());
        this.connectionOperator = new HttpClientConnectionOperator(arg3, arg5, arg6);
        if(arg4 == null) {
            v4 = ManagedHttpClientConnectionFactory.INSTANCE;
        }

        this.connFactory = ((HttpConnectionFactory)v4);
        this.expiry = 9223372036854775807L;
        this.socketConfig = SocketConfig.DEFAULT;
        this.connConfig = ConnectionConfig.DEFAULT;
        this.isShutdown = new AtomicBoolean(false);
    }

    public BasicHttpClientConnectionManager(Lookup arg2) {
        this(arg2, null, null, null);
    }

    public BasicHttpClientConnectionManager(Lookup arg2, HttpConnectionFactory arg3) {
        this(arg2, arg3, null, null);
    }

    private void checkExpiry() {
        if(this.conn != null && System.currentTimeMillis() >= this.expiry) {
            if(this.log.isDebugEnabled()) {
                this.log.debug("Connection expired @ " + new Date(this.expiry));
            }

            this.closeConnection();
        }
    }

    public void close() {
        this.shutdown();
    }

    private void closeConnection() {
        if(this.conn != null) {
            this.log.debug("Closing connection");
            try {
                this.conn.close();
            }
            catch(IOException v0) {
                if(!this.log.isDebugEnabled()) {
                    goto label_7;
                }

                this.log.debug("I/O exception closing connection", ((Throwable)v0));
            }

        label_7:
            this.conn = null;
        }
    }

    public void closeExpiredConnections() {
        __monitor_enter(this);
        try {
            if(!this.isShutdown.get() && !this.leased) {
                this.checkExpiry();
            }
        }
        catch(Throwable v0) {
            __monitor_exit(this);
            throw v0;
        }

        __monitor_exit(this);
    }

    public void closeIdleConnections(long arg6, TimeUnit arg8) {
        long v2;
        long v0 = 0;
        __monitor_enter(this);
        try {
            Args.notNull(arg8, "Time unit");
            if(!this.isShutdown.get() && !this.leased) {
                v2 = arg8.toMillis(arg6);
                if(v2 >= v0) {
                    goto label_22;
                }

                goto label_13;
            }

            goto label_7;
        }
        catch(Throwable v0_1) {
            goto label_20;
        }

    label_22:
        v0 = v2;
        try {
        label_13:
            if(this.updated > System.currentTimeMillis() - v0) {
                goto label_7;
            }

            this.closeConnection();
        }
        catch(Throwable v0_1) {
        label_20:
            __monitor_exit(this);
            throw v0_1;
        }

    label_7:
        __monitor_exit(this);
    }

    public void connect(HttpClientConnection arg8, HttpRoute arg9, int arg10, HttpContext arg11) {
        Args.notNull(arg8, "Connection");
        Args.notNull(arg9, "HTTP route");
        boolean v0 = arg8 == this.conn ? true : false;
        Asserts.check(v0, "Connection not obtained from this manager");
        HttpHost v2 = arg9.getProxyHost() != null ? arg9.getProxyHost() : arg9.getTargetHost();
        this.connectionOperator.connect(this.conn, v2, arg9.getLocalSocketAddress(), arg10, this.socketConfig, arg11);
    }

    protected void finalize() {
        try {
            this.shutdown();
            super.finalize();
            return;
        }
        catch(Throwable v0) {
            super.finalize();
            throw v0;
        }
    }

    HttpClientConnection getConnection(HttpRoute arg6, Object arg7) {
        ManagedHttpClientConnection v0_2;
        boolean v2;
        boolean v0 = true;
        __monitor_enter(this);
        try {
            if(!this.isShutdown.get()) {
                v2 = true;
            }
            else {
                goto label_45;
            }

            goto label_7;
        }
        catch(Throwable v0_1) {
            goto label_50;
        }

    label_45:
        v2 = false;
        try {
        label_7:
            Asserts.check(v2, "Connection manager has been shut down");
            if(this.log.isDebugEnabled()) {
                this.log.debug("Get connection for route " + arg6);
            }

            if(this.leased) {
                goto label_47;
            }

            goto label_22;
        }
        catch(Throwable v0_1) {
            goto label_50;
        }

    label_47:
        v0 = false;
        try {
        label_22:
            Asserts.check(v0, "Connection is still allocated");
            if(!LangUtils.equals(this.route, arg6) || !LangUtils.equals(this.state, arg7)) {
                this.closeConnection();
            }

            this.route = arg6;
            this.state = arg7;
            this.checkExpiry();
            if(this.conn == null) {
                this.conn = this.connFactory.create(arg6, this.connConfig);
            }

            this.leased = true;
            v0_2 = this.conn;
        }
        catch(Throwable v0_1) {
        label_50:
            __monitor_exit(this);
            throw v0_1;
        }

        __monitor_exit(this);
        return ((HttpClientConnection)v0_2);
    }

    public ConnectionConfig getConnectionConfig() {
        ConnectionConfig v0_1;
        __monitor_enter(this);
        try {
            v0_1 = this.connConfig;
        }
        catch(Throwable v0) {
            __monitor_exit(this);
            throw v0;
        }

        __monitor_exit(this);
        return v0_1;
    }

    private static Registry getDefaultRegistry() {
        return RegistryBuilder.create().register("http", PlainConnectionSocketFactory.getSocketFactory()).register("https", SSLConnectionSocketFactory.getSocketFactory()).build();
    }

    HttpRoute getRoute() {
        return this.route;
    }

    public SocketConfig getSocketConfig() {
        SocketConfig v0_1;
        __monitor_enter(this);
        try {
            v0_1 = this.socketConfig;
        }
        catch(Throwable v0) {
            __monitor_exit(this);
            throw v0;
        }

        __monitor_exit(this);
        return v0_1;
    }

    Object getState() {
        return this.state;
    }

    public void releaseConnection(HttpClientConnection arg7, Object arg8, long arg9, TimeUnit arg11) {
        long v4 = 0;
        boolean v0 = false;
        __monitor_enter(this);
        try {
            Args.notNull(arg7, "Connection");
            if(arg7 == this.conn) {
                v0 = true;
            }

            Asserts.check(v0, "Connection not obtained from this manager");
            if(this.log.isDebugEnabled()) {
                this.log.debug("Releasing connection " + arg7);
            }

            if(!this.isShutdown.get()) {
                goto label_26;
            }

            goto label_24;
        }
        catch(Throwable v0_1) {
            goto label_43;
        }

        try {
        label_26:
            this.updated = System.currentTimeMillis();
            if(!this.conn.isOpen()) {
                this.conn = null;
                this.route = null;
                this.conn = null;
                this.expiry = 9223372036854775807L;
            }
            else {
                goto label_45;
            }
        }
        catch(Throwable v0_1) {
            goto label_75;
        }

        goto label_40;
        try {
        label_45:
            this.state = arg8;
            if(this.log.isDebugEnabled()) {
                String v0_2 = arg9 > v4 ? "for " + arg9 + " " + arg11 : "indefinitely";
                this.log.debug("Connection can be kept alive " + v0_2);
            }

            if(arg9 > v4) {
                this.expiry = this.updated + arg11.toMillis(arg9);
                goto label_40;
            }

            this.expiry = 9223372036854775807L;
            goto label_40;
        }
        catch(Throwable v0_1) {
            try {
            label_75:
                this.leased = false;
                throw v0_1;
            label_40:
                this.leased = false;
            }
            catch(Throwable v0_1) {
            label_43:
                __monitor_exit(this);
                throw v0_1;
            }
        }

    label_24:
        __monitor_exit(this);
    }

    public final ConnectionRequest requestConnection(HttpRoute arg2, Object arg3) {
        Args.notNull(arg2, "Route");
        return new BasicHttpClientConnectionManager$1(this, arg2, arg3);
    }

    public void routeComplete(HttpClientConnection arg1, HttpRoute arg2, HttpContext arg3) {
    }

    public void setConnectionConfig(ConnectionConfig arg2) {
        __monitor_enter(this);
        if(arg2 == null) {
            try {
                arg2 = ConnectionConfig.DEFAULT;
            label_2:
                this.connConfig = arg2;
                goto label_3;
            }
            catch(Throwable v0) {
                __monitor_exit(this);
                throw v0;
            }
        }

        goto label_2;
    label_3:
        __monitor_exit(this);
    }

    public void setSocketConfig(SocketConfig arg2) {
        __monitor_enter(this);
        if(arg2 == null) {
            try {
                arg2 = SocketConfig.DEFAULT;
            label_2:
                this.socketConfig = arg2;
                goto label_3;
            }
            catch(Throwable v0) {
                __monitor_exit(this);
                throw v0;
            }
        }

        goto label_2;
    label_3:
        __monitor_exit(this);
    }

    public void shutdown() {
        __monitor_enter(this);
        try {
            if(this.isShutdown.compareAndSet(false, true)) {
                this.shutdownConnection();
            }
        }
        catch(Throwable v0) {
            __monitor_exit(this);
            throw v0;
        }

        __monitor_exit(this);
    }

    private void shutdownConnection() {
        if(this.conn != null) {
            this.log.debug("Shutting down connection");
            try {
                this.conn.shutdown();
            }
            catch(IOException v0) {
                if(!this.log.isDebugEnabled()) {
                    goto label_7;
                }

                this.log.debug("I/O exception shutting down connection", ((Throwable)v0));
            }

        label_7:
            this.conn = null;
        }
    }

    public void upgrade(HttpClientConnection arg4, HttpRoute arg5, HttpContext arg6) {
        Args.notNull(arg4, "Connection");
        Args.notNull(arg5, "HTTP route");
        boolean v0 = arg4 == this.conn ? true : false;
        Asserts.check(v0, "Connection not obtained from this manager");
        this.connectionOperator.upgrade(this.conn, arg5.getTargetHost(), arg6);
    }
}

