// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.asn1.x509;

import java.io.IOException;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1GeneralizedTime;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1Primitive;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1UTCTime;
import org.bouncycastle.asn1.DEROctetString;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.DERTaggedObject;
import org.bouncycastle.asn1.x500.X500Name;

public class V2TBSCertListGenerator {
    private ASN1EncodableVector crlentries;
    private Extensions extensions;
    private X500Name issuer;
    private Time nextUpdate;
    private static final ASN1Sequence[] reasons;
    private AlgorithmIdentifier signature;
    private Time thisUpdate;
    private ASN1Integer version;

    static {
        V2TBSCertListGenerator.reasons = new ASN1Sequence[11];
        V2TBSCertListGenerator.reasons[0] = V2TBSCertListGenerator.createReasonExtension(0);
        V2TBSCertListGenerator.reasons[1] = V2TBSCertListGenerator.createReasonExtension(1);
        V2TBSCertListGenerator.reasons[2] = V2TBSCertListGenerator.createReasonExtension(2);
        V2TBSCertListGenerator.reasons[3] = V2TBSCertListGenerator.createReasonExtension(3);
        V2TBSCertListGenerator.reasons[4] = V2TBSCertListGenerator.createReasonExtension(4);
        V2TBSCertListGenerator.reasons[5] = V2TBSCertListGenerator.createReasonExtension(5);
        V2TBSCertListGenerator.reasons[6] = V2TBSCertListGenerator.createReasonExtension(6);
        V2TBSCertListGenerator.reasons[7] = V2TBSCertListGenerator.createReasonExtension(7);
        V2TBSCertListGenerator.reasons[8] = V2TBSCertListGenerator.createReasonExtension(8);
        V2TBSCertListGenerator.reasons[9] = V2TBSCertListGenerator.createReasonExtension(9);
        V2TBSCertListGenerator.reasons[10] = V2TBSCertListGenerator.createReasonExtension(10);
    }

    public V2TBSCertListGenerator() {
        super();
        this.version = new ASN1Integer(1);
        this.nextUpdate = null;
        this.extensions = null;
        this.crlentries = new ASN1EncodableVector();
    }

    public void addCRLEntry(ASN1Sequence arg2) {
        this.crlentries.add(((ASN1Encodable)arg2));
    }

    public void addCRLEntry(ASN1Integer arg2, ASN1UTCTime arg3, int arg4) {
        this.addCRLEntry(arg2, new Time(((ASN1Primitive)arg3)), arg4);
    }

    public void addCRLEntry(ASN1Integer arg2, Time arg3, int arg4) {
        this.addCRLEntry(arg2, arg3, arg4, null);
    }

    public void addCRLEntry(ASN1Integer arg4, Time arg5, int arg6, ASN1GeneralizedTime arg7) {
        ASN1EncodableVector v0;
        if(arg6 != 0) {
            v0 = new ASN1EncodableVector();
            if(arg6 >= V2TBSCertListGenerator.reasons.length) {
                v0.add(V2TBSCertListGenerator.createReasonExtension(arg6));
            }
            else if(arg6 < 0) {
                throw new IllegalArgumentException("invalid reason value: " + arg6);
            }
            else {
                v0.add(V2TBSCertListGenerator.reasons[arg6]);
            }

            if(arg7 != null) {
                v0.add(V2TBSCertListGenerator.createInvalidityDateExtension(arg7));
            }

            this.internalAddCRLEntry(arg4, arg5, new DERSequence(v0));
        }
        else {
            if(arg7 != null) {
                v0 = new ASN1EncodableVector();
                v0.add(V2TBSCertListGenerator.createInvalidityDateExtension(arg7));
                this.internalAddCRLEntry(arg4, arg5, new DERSequence(v0));
                return;
            }

            this.addCRLEntry(arg4, arg5, null);
        }
    }

    public void addCRLEntry(ASN1Integer arg3, Time arg4, Extensions arg5) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        v0.add(((ASN1Encodable)arg3));
        v0.add(((ASN1Encodable)arg4));
        if(arg5 != null) {
            v0.add(((ASN1Encodable)arg5));
        }

        this.addCRLEntry(new DERSequence(v0));
    }

    private static ASN1Sequence createInvalidityDateExtension(ASN1GeneralizedTime arg4) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        try {
            v0.add(Extension.invalidityDate);
            v0.add(new DEROctetString(arg4.getEncoded()));
        }
        catch(IOException v0_1) {
            throw new IllegalArgumentException("error encoding reason: " + v0_1);
        }

        return new DERSequence(v0);
    }

    private static ASN1Sequence createReasonExtension(int arg4) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        CRLReason v1 = CRLReason.lookup(arg4);
        try {
            v0.add(Extension.reasonCode);
            v0.add(new DEROctetString(v1.getEncoded()));
        }
        catch(IOException v0_1) {
            throw new IllegalArgumentException("error encoding reason: " + v0_1);
        }

        return new DERSequence(v0);
    }

    public TBSCertList generateTBSCertList() {
        if(this.signature != null && this.issuer != null && this.thisUpdate != null) {
            ASN1EncodableVector v0 = new ASN1EncodableVector();
            v0.add(this.version);
            v0.add(this.signature);
            v0.add(this.issuer);
            v0.add(this.thisUpdate);
            if(this.nextUpdate != null) {
                v0.add(this.nextUpdate);
            }

            if(this.crlentries.size() != 0) {
                v0.add(new DERSequence(this.crlentries));
            }

            if(this.extensions != null) {
                v0.add(new DERTaggedObject(0, this.extensions));
            }

            return new TBSCertList(new DERSequence(v0));
        }

        throw new IllegalStateException("Not all mandatory fields set in V2 TBSCertList generator.");
    }

    private void internalAddCRLEntry(ASN1Integer arg3, Time arg4, ASN1Sequence arg5) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        v0.add(((ASN1Encodable)arg3));
        v0.add(((ASN1Encodable)arg4));
        if(arg5 != null) {
            v0.add(((ASN1Encodable)arg5));
        }

        this.addCRLEntry(new DERSequence(v0));
    }

    public void setExtensions(Extensions arg1) {
        this.extensions = arg1;
    }

    public void setExtensions(X509Extensions arg2) {
        this.setExtensions(Extensions.getInstance(arg2));
    }

    public void setIssuer(X500Name arg1) {
        this.issuer = arg1;
    }

    public void setIssuer(X509Name arg2) {
        this.issuer = X500Name.getInstance(arg2.toASN1Primitive());
    }

    public void setNextUpdate(ASN1UTCTime arg2) {
        this.nextUpdate = new Time(((ASN1Primitive)arg2));
    }

    public void setNextUpdate(Time arg1) {
        this.nextUpdate = arg1;
    }

    public void setSignature(AlgorithmIdentifier arg1) {
        this.signature = arg1;
    }

    public void setThisUpdate(ASN1UTCTime arg2) {
        this.thisUpdate = new Time(((ASN1Primitive)arg2));
    }

    public void setThisUpdate(Time arg1) {
        this.thisUpdate = arg1;
    }
}

