// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.agreement.jpake;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.SHA256Digest;
import org.bouncycastle.util.Arrays;

public class JPAKEParticipant {
    public static final int STATE_INITIALIZED = 0;
    public static final int STATE_KEY_CALCULATED = 50;
    public static final int STATE_ROUND_1_CREATED = 10;
    public static final int STATE_ROUND_1_VALIDATED = 20;
    public static final int STATE_ROUND_2_CREATED = 30;
    public static final int STATE_ROUND_2_VALIDATED = 40;
    public static final int STATE_ROUND_3_CREATED = 60;
    public static final int STATE_ROUND_3_VALIDATED = 70;
    private BigInteger b;
    private final Digest digest;
    private final BigInteger g;
    private BigInteger gx1;
    private BigInteger gx2;
    private BigInteger gx3;
    private BigInteger gx4;
    private final BigInteger p;
    private final String participantId;
    private String partnerParticipantId;
    private char[] password;
    private final BigInteger q;
    private final SecureRandom random;
    private int state;
    private BigInteger x1;
    private BigInteger x2;

    public JPAKEParticipant(String arg2, char[] arg3) {
        this(arg2, arg3, JPAKEPrimeOrderGroups.NIST_3072);
    }

    public JPAKEParticipant(String arg7, char[] arg8, JPAKEPrimeOrderGroup arg9) {
        this(arg7, arg8, arg9, new SHA256Digest(), new SecureRandom());
    }

    public JPAKEParticipant(String arg3, char[] arg4, JPAKEPrimeOrderGroup arg5, Digest arg6, SecureRandom arg7) {
        super();
        JPAKEUtil.validateNotNull(arg3, "participantId");
        JPAKEUtil.validateNotNull(arg4, "password");
        JPAKEUtil.validateNotNull(arg5, "p");
        JPAKEUtil.validateNotNull(arg6, "digest");
        JPAKEUtil.validateNotNull(arg7, "random");
        if(arg4.length == 0) {
            throw new IllegalArgumentException("Password must not be empty.");
        }

        this.participantId = arg3;
        this.password = Arrays.copyOf(arg4, arg4.length);
        this.p = arg5.getP();
        this.q = arg5.getQ();
        this.g = arg5.getG();
        this.digest = arg6;
        this.random = arg7;
        this.state = 0;
    }

    public BigInteger calculateKeyingMaterial() {
        int v7 = 50;
        char[] v6 = null;
        if(this.state >= v7) {
            throw new IllegalStateException("Key already calculated for " + this.participantId);
        }

        if(this.state < 40) {
            throw new IllegalStateException("Round2 payload must be validated prior to creating key for " + this.participantId);
        }

        BigInteger v4 = JPAKEUtil.calculateS(this.password);
        Arrays.fill(this.password, '\u0000');
        this.password = v6;
        BigInteger v0 = JPAKEUtil.calculateKeyingMaterial(this.p, this.q, this.gx4, this.x2, v4, this.b);
        this.x1 = ((BigInteger)v6);
        this.x2 = ((BigInteger)v6);
        this.b = ((BigInteger)v6);
        this.state = v7;
        return v0;
    }

    public JPAKERound1Payload createRound1PayloadToSend() {
        int v9 = 10;
        if(this.state >= v9) {
            throw new IllegalStateException("Round1 payload already created for " + this.participantId);
        }

        this.x1 = JPAKEUtil.generateX1(this.q, this.random);
        this.x2 = JPAKEUtil.generateX2(this.q, this.random);
        this.gx1 = JPAKEUtil.calculateGx(this.p, this.g, this.x1);
        this.gx2 = JPAKEUtil.calculateGx(this.p, this.g, this.x2);
        BigInteger[] v8 = JPAKEUtil.calculateZeroKnowledgeProof(this.p, this.q, this.g, this.gx1, this.x1, this.participantId, this.digest, this.random);
        BigInteger[] v5 = JPAKEUtil.calculateZeroKnowledgeProof(this.p, this.q, this.g, this.gx2, this.x2, this.participantId, this.digest, this.random);
        this.state = v9;
        return new JPAKERound1Payload(this.participantId, this.gx1, this.gx2, v8, v5);
    }

    public JPAKERound2Payload createRound2PayloadToSend() {
        int v8 = 30;
        if(this.state >= v8) {
            throw new IllegalStateException("Round2 payload already created for " + this.participantId);
        }

        if(this.state < 20) {
            throw new IllegalStateException("Round1 payload must be validated prior to creating Round2 payload for " + this.participantId);
        }

        BigInteger v2 = JPAKEUtil.calculateGA(this.p, this.gx1, this.gx3, this.gx4);
        BigInteger v4 = JPAKEUtil.calculateX2s(this.q, this.x2, JPAKEUtil.calculateS(this.password));
        BigInteger v3 = JPAKEUtil.calculateA(this.p, this.q, v2, v4);
        BigInteger[] v0 = JPAKEUtil.calculateZeroKnowledgeProof(this.p, this.q, v2, v3, v4, this.participantId, this.digest, this.random);
        this.state = v8;
        return new JPAKERound2Payload(this.participantId, v3, v0);
    }

    public JPAKERound3Payload createRound3PayloadToSend(BigInteger arg10) {
        int v8 = 60;
        if(this.state >= v8) {
            throw new IllegalStateException("Round3 payload already created for " + this.participantId);
        }

        if(this.state < 50) {
            throw new IllegalStateException("Keying material must be calculated prior to creating Round3 payload for " + this.participantId);
        }

        BigInteger v0 = JPAKEUtil.calculateMacTag(this.participantId, this.partnerParticipantId, this.gx1, this.gx2, this.gx3, this.gx4, arg10, this.digest);
        this.state = v8;
        return new JPAKERound3Payload(this.participantId, v0);
    }

    public int getState() {
        return this.state;
    }

    public void validateRound1PayloadReceived(JPAKERound1Payload arg10) {
        int v8 = 20;
        if(this.state >= v8) {
            throw new IllegalStateException("Validation already attempted for round1 payload for" + this.participantId);
        }

        this.partnerParticipantId = arg10.getParticipantId();
        this.gx3 = arg10.getGx1();
        this.gx4 = arg10.getGx2();
        BigInteger[] v4 = arg10.getKnowledgeProofForX1();
        BigInteger[] v7 = arg10.getKnowledgeProofForX2();
        JPAKEUtil.validateParticipantIdsDiffer(this.participantId, arg10.getParticipantId());
        JPAKEUtil.validateGx4(this.gx4);
        JPAKEUtil.validateZeroKnowledgeProof(this.p, this.q, this.g, this.gx3, v4, arg10.getParticipantId(), this.digest);
        JPAKEUtil.validateZeroKnowledgeProof(this.p, this.q, this.g, this.gx4, v7, arg10.getParticipantId(), this.digest);
        this.state = v8;
    }

    public void validateRound2PayloadReceived(JPAKERound2Payload arg9) {
        int v7 = 40;
        if(this.state >= v7) {
            throw new IllegalStateException("Validation already attempted for round2 payload for" + this.participantId);
        }

        if(this.state < 20) {
            throw new IllegalStateException("Round1 payload must be validated prior to validating Round2 payload for " + this.participantId);
        }

        BigInteger v2 = JPAKEUtil.calculateGA(this.p, this.gx3, this.gx1, this.gx2);
        this.b = arg9.getA();
        BigInteger[] v4 = arg9.getKnowledgeProofForX2s();
        JPAKEUtil.validateParticipantIdsDiffer(this.participantId, arg9.getParticipantId());
        JPAKEUtil.validateParticipantIdsEqual(this.partnerParticipantId, arg9.getParticipantId());
        JPAKEUtil.validateGa(v2);
        JPAKEUtil.validateZeroKnowledgeProof(this.p, this.q, v2, this.b, v4, arg9.getParticipantId(), this.digest);
        this.state = v7;
    }

    public void validateRound3PayloadReceived(JPAKERound3Payload arg12, BigInteger arg13) {
        int v10 = 70;
        BigInteger v9 = null;
        if(this.state >= v10) {
            throw new IllegalStateException("Validation already attempted for round3 payload for" + this.participantId);
        }

        if(this.state < 50) {
            throw new IllegalStateException("Keying material must be calculated validated prior to validating Round3 payload for " + this.participantId);
        }

        JPAKEUtil.validateParticipantIdsDiffer(this.participantId, arg12.getParticipantId());
        JPAKEUtil.validateParticipantIdsEqual(this.partnerParticipantId, arg12.getParticipantId());
        JPAKEUtil.validateMacTag(this.participantId, this.partnerParticipantId, this.gx1, this.gx2, this.gx3, this.gx4, arg13, this.digest, arg12.getMacTag());
        this.gx1 = v9;
        this.gx2 = v9;
        this.gx3 = v9;
        this.gx4 = v9;
        this.state = v10;
    }
}

