// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.agreement.jpake;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.Mac;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.Strings;

public class JPAKEUtil {
    static final BigInteger ONE;
    static final BigInteger ZERO;

    static {
        JPAKEUtil.ZERO = BigInteger.valueOf(0);
        JPAKEUtil.ONE = BigInteger.valueOf(1);
    }

    public JPAKEUtil() {
        super();
    }

    public static BigInteger calculateA(BigInteger arg1, BigInteger arg2, BigInteger arg3, BigInteger arg4) {
        return arg3.modPow(arg4, arg1);
    }

    public static BigInteger calculateGA(BigInteger arg1, BigInteger arg2, BigInteger arg3, BigInteger arg4) {
        return arg2.multiply(arg3).multiply(arg4).mod(arg1);
    }

    public static BigInteger calculateGx(BigInteger arg1, BigInteger arg2, BigInteger arg3) {
        return arg2.modPow(arg3, arg1);
    }

    private static BigInteger calculateHashForZeroKnowledgeProof(BigInteger arg2, BigInteger arg3, BigInteger arg4, String arg5, Digest arg6) {
        arg6.reset();
        JPAKEUtil.updateDigestIncludingSize(arg6, arg2);
        JPAKEUtil.updateDigestIncludingSize(arg6, arg3);
        JPAKEUtil.updateDigestIncludingSize(arg6, arg4);
        JPAKEUtil.updateDigestIncludingSize(arg6, arg5);
        byte[] v0 = new byte[arg6.getDigestSize()];
        arg6.doFinal(v0, 0);
        return new BigInteger(v0);
    }

    public static BigInteger calculateKeyingMaterial(BigInteger arg1, BigInteger arg2, BigInteger arg3, BigInteger arg4, BigInteger arg5, BigInteger arg6) {
        return arg3.modPow(arg4.multiply(arg5).negate().mod(arg2), arg1).multiply(arg6).modPow(arg4, arg1);
    }

    private static byte[] calculateMacKey(BigInteger arg2, Digest arg3) {
        arg3.reset();
        JPAKEUtil.updateDigest(arg3, arg2);
        JPAKEUtil.updateDigest(arg3, "JPAKE_KC");
        byte[] v0 = new byte[arg3.getDigestSize()];
        arg3.doFinal(v0, 0);
        return v0;
    }

    public static BigInteger calculateMacTag(String arg5, String arg6, BigInteger arg7, BigInteger arg8, BigInteger arg9, BigInteger arg10, BigInteger arg11, Digest arg12) {
        byte[] v0 = JPAKEUtil.calculateMacKey(arg11, arg12);
        HMac v1 = new HMac(arg12);
        byte[] v2 = new byte[v1.getMacSize()];
        v1.init(new KeyParameter(v0));
        JPAKEUtil.updateMac(((Mac)v1), "KC_1_U");
        JPAKEUtil.updateMac(((Mac)v1), arg5);
        JPAKEUtil.updateMac(((Mac)v1), arg6);
        JPAKEUtil.updateMac(((Mac)v1), arg7);
        JPAKEUtil.updateMac(((Mac)v1), arg8);
        JPAKEUtil.updateMac(((Mac)v1), arg9);
        JPAKEUtil.updateMac(((Mac)v1), arg10);
        v1.doFinal(v2, 0);
        Arrays.fill(v0, 0);
        return new BigInteger(v2);
    }

    public static BigInteger calculateS(char[] arg2) {
        return new BigInteger(Strings.toUTF8ByteArray(arg2));
    }

    public static BigInteger calculateX2s(BigInteger arg1, BigInteger arg2, BigInteger arg3) {
        return arg2.multiply(arg3).mod(arg1);
    }

    public static BigInteger[] calculateZeroKnowledgeProof(BigInteger arg5, BigInteger arg6, BigInteger arg7, BigInteger arg8, BigInteger arg9, String arg10, Digest arg11, SecureRandom arg12) {
        BigInteger[] v0 = new BigInteger[2];
        BigInteger v1 = BigIntegers.createRandomInRange(JPAKEUtil.ZERO, arg6.subtract(JPAKEUtil.ONE), arg12);
        BigInteger v2 = arg7.modPow(v1, arg5);
        BigInteger v3 = JPAKEUtil.calculateHashForZeroKnowledgeProof(arg7, v2, arg8, arg10, arg11);
        v0[0] = v2;
        v0[1] = v1.subtract(arg9.multiply(v3)).mod(arg6);
        return v0;
    }

    public static BigInteger generateX1(BigInteger arg2, SecureRandom arg3) {
        return BigIntegers.createRandomInRange(JPAKEUtil.ZERO, arg2.subtract(JPAKEUtil.ONE), arg3);
    }

    public static BigInteger generateX2(BigInteger arg2, SecureRandom arg3) {
        return BigIntegers.createRandomInRange(JPAKEUtil.ONE, arg2.subtract(JPAKEUtil.ONE), arg3);
    }

    private static byte[] intToByteArray(int arg3) {
        return new byte[]{((byte)(arg3 >>> 24)), ((byte)(arg3 >>> 16)), ((byte)(arg3 >>> 8)), ((byte)arg3)};
    }

    private static void updateDigest(Digest arg3, BigInteger arg4) {
        byte[] v0 = BigIntegers.asUnsignedByteArray(arg4);
        arg3.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    private static void updateDigest(Digest arg3, String arg4) {
        byte[] v0 = Strings.toUTF8ByteArray(arg4);
        arg3.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    private static void updateDigestIncludingSize(Digest arg4, BigInteger arg5) {
        byte[] v0 = BigIntegers.asUnsignedByteArray(arg5);
        arg4.update(JPAKEUtil.intToByteArray(v0.length), 0, 4);
        arg4.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    private static void updateDigestIncludingSize(Digest arg4, String arg5) {
        byte[] v0 = Strings.toUTF8ByteArray(arg5);
        arg4.update(JPAKEUtil.intToByteArray(v0.length), 0, 4);
        arg4.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    private static void updateMac(Mac arg3, String arg4) {
        byte[] v0 = Strings.toUTF8ByteArray(arg4);
        arg3.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    private static void updateMac(Mac arg3, BigInteger arg4) {
        byte[] v0 = BigIntegers.asUnsignedByteArray(arg4);
        arg3.update(v0, 0, v0.length);
        Arrays.fill(v0, 0);
    }

    public static void validateGa(BigInteger arg2) {
        if(arg2.equals(JPAKEUtil.ONE)) {
            throw new CryptoException("ga is equal to 1.  It should not be.  The chances of this happening are on the order of 2^160 for a 160-bit q.  Try again.");
        }
    }

    public static void validateGx4(BigInteger arg2) {
        if(arg2.equals(JPAKEUtil.ONE)) {
            throw new CryptoException("g^x validation failed.  g^x should not be 1.");
        }
    }

    public static void validateMacTag(String arg9, String arg10, BigInteger arg11, BigInteger arg12, BigInteger arg13, BigInteger arg14, BigInteger arg15, Digest arg16, BigInteger arg17) {
        if(!JPAKEUtil.calculateMacTag(arg10, arg9, arg13, arg14, arg11, arg12, arg15, arg16).equals(arg17)) {
            throw new CryptoException("Partner MacTag validation failed. Therefore, the password, MAC, or digest algorithm of each participant does not match.");
        }
    }

    public static void validateNotNull(Object arg3, String arg4) {
        if(arg3 == null) {
            throw new NullPointerException(arg4 + " must not be null");
        }
    }

    public static void validateParticipantIdsDiffer(String arg3, String arg4) {
        if(arg3.equals(arg4)) {
            throw new CryptoException("Both participants are using the same participantId (" + arg3 + "). This is not allowed. " + "Each participant must use a unique participantId.");
        }
    }

    public static void validateParticipantIdsEqual(String arg3, String arg4) {
        if(!arg3.equals(arg4)) {
            throw new CryptoException("Received payload from incorrect partner (" + arg4 + "). Expected to receive payload from " + arg3 + ".");
        }
    }

    public static void validateZeroKnowledgeProof(BigInteger arg5, BigInteger arg6, BigInteger arg7, BigInteger arg8, BigInteger[] arg9, String arg10, Digest arg11) {
        BigInteger v0 = arg9[0];
        BigInteger v1 = arg9[1];
        BigInteger v2 = JPAKEUtil.calculateHashForZeroKnowledgeProof(arg7, v0, arg8, arg10, arg11);
        if(arg8.compareTo(JPAKEUtil.ZERO) == 1 && arg8.compareTo(arg5) == -1 && arg8.modPow(arg6, arg5).compareTo(JPAKEUtil.ONE) == 0 && arg7.modPow(v1, arg5).multiply(arg8.modPow(v2, arg5)).mod(arg5).compareTo(v0) == 0) {
            return;
        }

        throw new CryptoException("Zero-knowledge proof validation failed");
    }
}

