// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.encodings;

import java.security.SecureRandom;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.params.ParametersWithRandom;

public class OAEPEncoding implements AsymmetricBlockCipher {
    private byte[] defHash;
    private AsymmetricBlockCipher engine;
    private boolean forEncryption;
    private Digest mgf1Hash;
    private SecureRandom random;

    public OAEPEncoding(AsymmetricBlockCipher arg3) {
        this(arg3, new SHA1Digest(), null);
    }

    public OAEPEncoding(AsymmetricBlockCipher arg1, Digest arg2, byte[] arg3) {
        this(arg1, arg2, arg2, arg3);
    }

    public OAEPEncoding(AsymmetricBlockCipher arg2, Digest arg3) {
        this(arg2, arg3, null);
    }

    public OAEPEncoding(AsymmetricBlockCipher arg3, Digest arg4, Digest arg5, byte[] arg6) {
        super();
        this.engine = arg3;
        this.mgf1Hash = arg5;
        this.defHash = new byte[arg4.getDigestSize()];
        arg4.reset();
        if(arg6 != null) {
            arg4.update(arg6, 0, arg6.length);
        }

        arg4.doFinal(this.defHash, 0);
    }

    private void ItoOSP(int arg3, byte[] arg4) {
        arg4[0] = ((byte)(arg3 >>> 24));
        arg4[1] = ((byte)(arg3 >>> 16));
        arg4[2] = ((byte)(arg3 >>> 8));
        arg4[3] = ((byte)(arg3 >>> 0));
    }

    public byte[] decodeBlock(byte[] arg8, int arg9, int arg10) {
        byte[] v0;
        byte[] v1 = this.engine.processBlock(arg8, arg9, arg10);
        if(v1.length < this.engine.getOutputBlockSize()) {
            v0 = new byte[this.engine.getOutputBlockSize()];
            System.arraycopy(v1, 0, v0, v0.length - v1.length, v1.length);
        }
        else {
            v0 = v1;
        }

        if(v0.length < this.defHash.length * 2 + 1) {
            throw new InvalidCipherTextException("data too short");
        }

        byte[] v3 = this.maskGeneratorFunction1(v0, this.defHash.length, v0.length - this.defHash.length, this.defHash.length);
        int v1_1;
        for(v1_1 = 0; v1_1 != this.defHash.length; ++v1_1) {
            v0[v1_1] = ((byte)(v0[v1_1] ^ v3[v1_1]));
        }

        v3 = this.maskGeneratorFunction1(v0, 0, this.defHash.length, v0.length - this.defHash.length);
        for(v1_1 = this.defHash.length; v1_1 != v0.length; ++v1_1) {
            v0[v1_1] = ((byte)(v0[v1_1] ^ v3[v1_1 - this.defHash.length]));
        }

        v1_1 = 0;
        int v3_1 = 0;
        while(v1_1 != this.defHash.length) {
            if(this.defHash[v1_1] != v0[this.defHash.length + v1_1]) {
                v3_1 = 1;
            }

            ++v1_1;
        }

        if(v3_1 != 0) {
            throw new InvalidCipherTextException("data hash wrong");
        }

        for(v1_1 = this.defHash.length * 2; v1_1 != v0.length; ++v1_1) {
            if(v0[v1_1] != 0) {
                break;
            }
        }

        if(v1_1 < v0.length - 1 && v0[v1_1] == 1) {
            ++v1_1;
            v3 = new byte[v0.length - v1_1];
            System.arraycopy(v0, v1_1, v3, 0, v3.length);
            return v3;
        }

        throw new InvalidCipherTextException("data start wrong " + v1_1);
    }

    public byte[] encodeBlock(byte[] arg8, int arg9, int arg10) {
        byte[] v2 = new byte[this.getInputBlockSize() + 1 + this.defHash.length * 2];
        System.arraycopy(arg8, arg9, v2, v2.length - arg10, arg10);
        v2[v2.length - arg10 - 1] = 1;
        System.arraycopy(this.defHash, 0, v2, this.defHash.length, this.defHash.length);
        byte[] v3 = new byte[this.defHash.length];
        this.random.nextBytes(v3);
        byte[] v4 = this.maskGeneratorFunction1(v3, 0, v3.length, v2.length - this.defHash.length);
        int v0;
        for(v0 = this.defHash.length; v0 != v2.length; ++v0) {
            v2[v0] = ((byte)(v2[v0] ^ v4[v0 - this.defHash.length]));
        }

        System.arraycopy(v3, 0, v2, 0, this.defHash.length);
        v3 = this.maskGeneratorFunction1(v2, this.defHash.length, v2.length - this.defHash.length, this.defHash.length);
        for(v0 = 0; v0 != this.defHash.length; ++v0) {
            v2[v0] = ((byte)(v2[v0] ^ v3[v0]));
        }

        return this.engine.processBlock(v2, 0, v2.length);
    }

    public int getInputBlockSize() {
        int v0 = this.engine.getInputBlockSize();
        if(this.forEncryption) {
            v0 = v0 - 1 - this.defHash.length * 2;
        }

        return v0;
    }

    public int getOutputBlockSize() {
        int v0 = this.engine.getOutputBlockSize();
        if(!this.forEncryption) {
            v0 = v0 - 1 - this.defHash.length * 2;
        }

        return v0;
    }

    public AsymmetricBlockCipher getUnderlyingCipher() {
        return this.engine;
    }

    public void init(boolean arg2, CipherParameters arg3) {
        this.random = (arg3 instanceof ParametersWithRandom) ? arg3.getRandom() : new SecureRandom();
        this.engine.init(arg2, arg3);
        this.forEncryption = arg2;
    }

    private byte[] maskGeneratorFunction1(byte[] arg8, int arg9, int arg10, int arg11) {
        byte[] v2 = new byte[arg11];
        byte[] v3 = new byte[this.mgf1Hash.getDigestSize()];
        byte[] v4 = new byte[4];
        this.mgf1Hash.reset();
        int v0;
        for(v0 = 0; v0 < arg11 / v3.length; ++v0) {
            this.ItoOSP(v0, v4);
            this.mgf1Hash.update(arg8, arg9, arg10);
            this.mgf1Hash.update(v4, 0, v4.length);
            this.mgf1Hash.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, v3.length * v0, v3.length);
        }

        if(v3.length * v0 < arg11) {
            this.ItoOSP(v0, v4);
            this.mgf1Hash.update(arg8, arg9, arg10);
            this.mgf1Hash.update(v4, 0, v4.length);
            this.mgf1Hash.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, v3.length * v0, v2.length - v0 * v3.length);
        }

        return v2;
    }

    public byte[] processBlock(byte[] arg2, int arg3, int arg4) {
        byte[] v0 = this.forEncryption ? this.encodeBlock(arg2, arg3, arg4) : this.decodeBlock(arg2, arg3, arg4);
        return v0;
    }
}

