// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.encodings;

import java.security.AccessController;
import java.security.SecureRandom;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.ParametersWithRandom;

public class PKCS1Encoding implements AsymmetricBlockCipher {
    private static final int HEADER_LENGTH = 10;
    public static final String STRICT_LENGTH_ENABLED_PROPERTY = "org.bouncycastle.pkcs1.strict";
    private AsymmetricBlockCipher engine;
    private boolean forEncryption;
    private boolean forPrivateKey;
    private SecureRandom random;
    private boolean useStrictLength;

    public PKCS1Encoding(AsymmetricBlockCipher arg2) {
        super();
        this.engine = arg2;
        this.useStrictLength = this.useStrict();
    }

    private byte[] decodeBlock(byte[] arg8, int arg9, int arg10) {
        byte[] v2 = this.engine.processBlock(arg8, arg9, arg10);
        if(v2.length < this.getOutputBlockSize()) {
            throw new InvalidCipherTextException("block truncated");
        }

        int v3 = v2[0];
        if(this.forPrivateKey) {
            if(v3 != 2) {
                throw new InvalidCipherTextException("unknown block type");
            }
        }
        else if(v3 != 1) {
            throw new InvalidCipherTextException("unknown block type");
        }

        if((this.useStrictLength) && v2.length != this.engine.getOutputBlockSize()) {
            throw new InvalidCipherTextException("block incorrect size");
        }

        int v0;
        for(v0 = 1; v0 != v2.length; ++v0) {
            int v4 = v2[v0];
            if(v4 == 0) {
                break;
            }

            if(v3 == 1 && v4 != -1) {
                throw new InvalidCipherTextException("block padding incorrect");
            }
        }

        ++v0;
        if(v0 <= v2.length && v0 >= 10) {
            byte[] v1 = new byte[v2.length - v0];
            System.arraycopy(v2, v0, v1, 0, v1.length);
            return v1;
        }

        throw new InvalidCipherTextException("no data in block");
    }

    private byte[] encodeBlock(byte[] arg5, int arg6, int arg7) {
        int v0 = 1;
        if(arg7 > this.getInputBlockSize()) {
            throw new IllegalArgumentException("input data too large");
        }

        byte[] v1 = new byte[this.engine.getInputBlockSize()];
        if(this.forPrivateKey) {
            v1[0] = 1;
            while(v0 != v1.length - arg7 - 1) {
                v1[v0] = -1;
                ++v0;
            }
        }
        else {
            this.random.nextBytes(v1);
            v1[0] = 2;
            while(v0 != v1.length - arg7 - 1) {
                while(v1[v0] == 0) {
                    v1[v0] = ((byte)this.random.nextInt());
                }

                ++v0;
            }
        }

        v1[v1.length - arg7 - 1] = 0;
        System.arraycopy(arg5, arg6, v1, v1.length - arg7, arg7);
        return this.engine.processBlock(v1, 0, v1.length);
    }

    public int getInputBlockSize() {
        int v0 = this.engine.getInputBlockSize();
        if(this.forEncryption) {
            v0 += -10;
        }

        return v0;
    }

    public int getOutputBlockSize() {
        int v0 = this.engine.getOutputBlockSize();
        if(!this.forEncryption) {
            v0 += -10;
        }

        return v0;
    }

    public AsymmetricBlockCipher getUnderlyingCipher() {
        return this.engine;
    }

    public void init(boolean arg3, CipherParameters arg4) {
        CipherParameters v0;
        if((arg4 instanceof ParametersWithRandom)) {
            this.random = arg4.getRandom();
            v0 = arg4.getParameters();
        }
        else {
            this.random = new SecureRandom();
            v0 = arg4;
        }

        this.engine.init(arg3, arg4);
        this.forPrivateKey = ((AsymmetricKeyParameter)v0).isPrivate();
        this.forEncryption = arg3;
    }

    public byte[] processBlock(byte[] arg2, int arg3, int arg4) {
        byte[] v0 = this.forEncryption ? this.encodeBlock(arg2, arg3, arg4) : this.decodeBlock(arg2, arg3, arg4);
        return v0;
    }

    private boolean useStrict() {
        Object v0 = AccessController.doPrivileged(new PKCS1Encoding$1(this));
        boolean v0_1 = v0 == null || (((String)v0).equals("true")) ? true : false;
        return v0_1;
    }
}

