// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.params.KeyParameter;

public class CamelliaLightEngine implements BlockCipher {
    private static final int BLOCK_SIZE = 16;
    private static final int MASK8 = 255;
    private static final byte[] SBOX1;
    private static final int[] SIGMA;
    private boolean _keyis128;
    private boolean initialized;
    private int[] ke;
    private int[] kw;
    private int[] state;
    private int[] subkey;

    static {
        CamelliaLightEngine.SIGMA = new int[]{-1600231809, 1003262091, -1233459112, 1286239154, -957401297, -380665154, 1426019237, -237801700, 283453434, -563598051, -1336506174, -1276722691};
        CamelliaLightEngine.SBOX1 = new byte[]{112, -126, 44, -20, -77, 39, -64, -27, -28, -123, 87, 53, -22, 12, -82, 65, 35, -17, 107, -109, 69, 25, -91, 33, -19, 14, 79, 78, 29, 101, -110, -67, -122, -72, -81, -113, 124, -21, 31, -50, 62, 48, -36, 95, 94, -59, 11, 26, -90, -31, 57, -54, -43, 71, 93, 61, -39, 1, 90, -42, 81, 86, 108, 77, -117, 13, -102, 102, -5, -52, -80, 45, 116, 18, 43, 32, -16, -79, -124, -103, -33, 76, -53, -62, 52, 126, 118, 5, 109, -73, -87, 49, -47, 23, 4, -41, 20, 88, 58, 97, -34, 27, 17, 28, 50, 15, -100, 22, 83, 24, -14, 34, -2, 68, -49, -78, -61, -75, 122, -111, 36, 8, -24, -88, 96, -4, 105, 80, -86, -48, -96, 125, -95, -119, 98, -105, 84, 91, 30, -107, -32, -1, 100, -46, 16, -60, 0, 72, -93, -9, 117, -37, -118, 3, -26, -38, 9, 63, -35, -108, -121, 92, -125, 2, -51, 74, -112, 51, 115, 103, -10, -13, -99, 127, -65, -30, 82, -101, -40, 38, -56, 55, -58, 59, -127, -106, 111, 75, 19, -66, 99, 46, -23, 121, -89, -116, -97, 110, -68, -114, 41, -11, -7, -74, 47, -3, -76, 89, 120, -104, 6, 106, -25, 70, 113, -70, -44, 37, -85, 66, -120, -94, -115, -6, 114, 7, -71, 85, -8, -18, -84, 10, 54, 73, 42, 104, 60, 56, -15, -92, 64, 40, -45, 123, -69, -55, 67, -63, 21, -29, -83, -12, 119, -57, -128, -98};
    }

    public CamelliaLightEngine() {
        super();
        this.subkey = new int[96];
        this.kw = new int[8];
        this.ke = new int[12];
        this.state = new int[4];
    }

    private int bytes2int(byte[] arg4, int arg5) {
        int v0 = 0;
        int v1 = 0;
        while(v0 < 4) {
            v1 = (v1 << 8) + (arg4[v0 + arg5] & 255);
            ++v0;
        }

        return v1;
    }

    private void camelliaF2(int[] arg10, int[] arg11, int arg12) {
        int v0 = arg10[0] ^ arg11[arg12];
        v0 = (CamelliaLightEngine.SBOX1[v0 >>> 24 & 255] & 255) << 24 | (this.sbox4(v0 & 255) | this.sbox3(v0 >>> 8 & 255) << 8 | this.sbox2(v0 >>> 16 & 255) << 16);
        int v1 = arg10[1] ^ arg11[arg12 + 1];
        v1 = CamelliaLightEngine.leftRotate(this.sbox2(v1 >>> 24 & 255) << 24 | (CamelliaLightEngine.SBOX1[v1 & 255] & 255 | this.sbox4(v1 >>> 8 & 255) << 8 | this.sbox3(v1 >>> 16 & 255) << 16), 8);
        v0 ^= v1;
        v1 = CamelliaLightEngine.leftRotate(v1, 8) ^ v0;
        v0 = CamelliaLightEngine.rightRotate(v0, 8) ^ v1;
        arg10[2] ^= CamelliaLightEngine.leftRotate(v1, 16) ^ v0;
        arg10[3] ^= CamelliaLightEngine.leftRotate(v0, 8);
        v0 = arg10[2] ^ arg11[arg12 + 2];
        v0 = (CamelliaLightEngine.SBOX1[v0 >>> 24 & 255] & 255) << 24 | (this.sbox4(v0 & 255) | this.sbox3(v0 >>> 8 & 255) << 8 | this.sbox2(v0 >>> 16 & 255) << 16);
        v1 = arg10[3] ^ arg11[arg12 + 3];
        v1 = CamelliaLightEngine.leftRotate(this.sbox2(v1 >>> 24 & 255) << 24 | (CamelliaLightEngine.SBOX1[v1 & 255] & 255 | this.sbox4(v1 >>> 8 & 255) << 8 | this.sbox3(v1 >>> 16 & 255) << 16), 8);
        v0 ^= v1;
        v1 = CamelliaLightEngine.leftRotate(v1, 8) ^ v0;
        v0 = CamelliaLightEngine.rightRotate(v0, 8) ^ v1;
        arg10[0] ^= CamelliaLightEngine.leftRotate(v1, 16) ^ v0;
        arg10[1] ^= CamelliaLightEngine.leftRotate(v0, 8);
    }

    private void camelliaFLs(int[] arg8, int[] arg9, int arg10) {
        arg8[1] ^= CamelliaLightEngine.leftRotate(arg8[0] & arg9[arg10], 1);
        arg8[0] ^= arg9[arg10 + 1] | arg8[1];
        arg8[2] ^= arg9[arg10 + 3] | arg8[3];
        arg8[3] ^= CamelliaLightEngine.leftRotate(arg9[arg10 + 2] & arg8[2], 1);
    }

    private static void decroldq(int arg4, int[] arg5, int arg6, int[] arg7, int arg8) {
        arg7[arg8 + 2] = arg5[arg6] << arg4 | arg5[arg6 + 1] >>> 32 - arg4;
        arg7[arg8 + 3] = arg5[arg6 + 1] << arg4 | arg5[arg6 + 2] >>> 32 - arg4;
        arg7[arg8] = arg5[arg6 + 2] << arg4 | arg5[arg6 + 3] >>> 32 - arg4;
        arg7[arg8 + 1] = arg5[arg6 + 3] << arg4 | arg5[arg6] >>> 32 - arg4;
        arg5[arg6] = arg7[arg8 + 2];
        arg5[arg6 + 1] = arg7[arg8 + 3];
        arg5[arg6 + 2] = arg7[arg8];
        arg5[arg6 + 3] = arg7[arg8 + 1];
    }

    private static void decroldqo32(int arg4, int[] arg5, int arg6, int[] arg7, int arg8) {
        arg7[arg8 + 2] = arg5[arg6 + 1] << arg4 - 32 | arg5[arg6 + 2] >>> 64 - arg4;
        arg7[arg8 + 3] = arg5[arg6 + 2] << arg4 - 32 | arg5[arg6 + 3] >>> 64 - arg4;
        arg7[arg8] = arg5[arg6 + 3] << arg4 - 32 | arg5[arg6] >>> 64 - arg4;
        arg7[arg8 + 1] = arg5[arg6] << arg4 - 32 | arg5[arg6 + 1] >>> 64 - arg4;
        arg5[arg6] = arg7[arg8 + 2];
        arg5[arg6 + 1] = arg7[arg8 + 3];
        arg5[arg6 + 2] = arg7[arg8];
        arg5[arg6 + 3] = arg7[arg8 + 1];
    }

    public String getAlgorithmName() {
        return "Camellia";
    }

    public int getBlockSize() {
        return 16;
    }

    public void init(boolean arg3, CipherParameters arg4) {
        if(!(arg4 instanceof KeyParameter)) {
            throw new IllegalArgumentException("only simple KeyParameter expected.");
        }

        this.setKey(arg3, ((KeyParameter)arg4).getKey());
        this.initialized = true;
    }

    private void int2bytes(int arg4, byte[] arg5, int arg6) {
        int v0;
        for(v0 = 0; v0 < 4; ++v0) {
            arg5[3 - v0 + arg6] = ((byte)arg4);
            arg4 >>>= 8;
        }
    }

    private byte lRot8(byte arg4, int arg5) {
        return ((byte)(arg4 << arg5 | (arg4 & 255) >>> 8 - arg5));
    }

    private static int leftRotate(int arg2, int arg3) {
        return (arg2 << arg3) + (arg2 >>> 32 - arg3);
    }

    public int processBlock(byte[] arg3, int arg4, byte[] arg5, int arg6) {
        if(!this.initialized) {
            throw new IllegalStateException("Camellia is not initialized");
        }

        if(arg4 + 16 > arg3.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg6 + 16 > arg5.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0 = this._keyis128 ? this.processBlock128(arg3, arg4, arg5, arg6) : this.processBlock192or256(arg3, arg4, arg5, arg6);
        return v0;
    }

    private int processBlock128(byte[] arg10, int arg11, byte[] arg12, int arg13) {
        int v8 = 3;
        int v7 = 2;
        int v5 = 4;
        int v0;
        for(v0 = 0; v0 < v5; ++v0) {
            this.state[v0] = this.bytes2int(arg10, v0 * 4 + arg11);
            this.state[v0] ^= this.kw[v0];
        }

        this.camelliaF2(this.state, this.subkey, 0);
        this.camelliaF2(this.state, this.subkey, v5);
        this.camelliaF2(this.state, this.subkey, 8);
        this.camelliaFLs(this.state, this.ke, 0);
        this.camelliaF2(this.state, this.subkey, 12);
        this.camelliaF2(this.state, this.subkey, 16);
        this.camelliaF2(this.state, this.subkey, 20);
        this.camelliaFLs(this.state, this.ke, v5);
        this.camelliaF2(this.state, this.subkey, 24);
        this.camelliaF2(this.state, this.subkey, 28);
        this.camelliaF2(this.state, this.subkey, 32);
        this.state[v7] ^= this.kw[v5];
        this.state[v8] ^= this.kw[5];
        this.state[0] ^= this.kw[6];
        this.state[1] ^= this.kw[7];
        this.int2bytes(this.state[v7], arg12, arg13);
        this.int2bytes(this.state[v8], arg12, arg13 + 4);
        this.int2bytes(this.state[0], arg12, arg13 + 8);
        this.int2bytes(this.state[1], arg12, arg13 + 12);
        return 16;
    }

    private int processBlock192or256(byte[] arg10, int arg11, byte[] arg12, int arg13) {
        int v8 = 3;
        int v7 = 2;
        int v5 = 4;
        int v0;
        for(v0 = 0; v0 < v5; ++v0) {
            this.state[v0] = this.bytes2int(arg10, v0 * 4 + arg11);
            this.state[v0] ^= this.kw[v0];
        }

        this.camelliaF2(this.state, this.subkey, 0);
        this.camelliaF2(this.state, this.subkey, v5);
        this.camelliaF2(this.state, this.subkey, 8);
        this.camelliaFLs(this.state, this.ke, 0);
        this.camelliaF2(this.state, this.subkey, 12);
        this.camelliaF2(this.state, this.subkey, 16);
        this.camelliaF2(this.state, this.subkey, 20);
        this.camelliaFLs(this.state, this.ke, v5);
        this.camelliaF2(this.state, this.subkey, 24);
        this.camelliaF2(this.state, this.subkey, 28);
        this.camelliaF2(this.state, this.subkey, 32);
        this.camelliaFLs(this.state, this.ke, 8);
        this.camelliaF2(this.state, this.subkey, 36);
        this.camelliaF2(this.state, this.subkey, 40);
        this.camelliaF2(this.state, this.subkey, 44);
        this.state[v7] ^= this.kw[v5];
        this.state[v8] ^= this.kw[5];
        this.state[0] ^= this.kw[6];
        this.state[1] ^= this.kw[7];
        this.int2bytes(this.state[v7], arg12, arg13);
        this.int2bytes(this.state[v8], arg12, arg13 + 4);
        this.int2bytes(this.state[0], arg12, arg13 + 8);
        this.int2bytes(this.state[1], arg12, arg13 + 12);
        return 16;
    }

    public void reset() {
    }

    private static int rightRotate(int arg2, int arg3) {
        return (arg2 >>> arg3) + (arg2 << 32 - arg3);
    }

    private static void roldq(int arg4, int[] arg5, int arg6, int[] arg7, int arg8) {
        arg7[arg8] = arg5[arg6] << arg4 | arg5[arg6 + 1] >>> 32 - arg4;
        arg7[arg8 + 1] = arg5[arg6 + 1] << arg4 | arg5[arg6 + 2] >>> 32 - arg4;
        arg7[arg8 + 2] = arg5[arg6 + 2] << arg4 | arg5[arg6 + 3] >>> 32 - arg4;
        arg7[arg8 + 3] = arg5[arg6 + 3] << arg4 | arg5[arg6] >>> 32 - arg4;
        arg5[arg6] = arg7[arg8];
        arg5[arg6 + 1] = arg7[arg8 + 1];
        arg5[arg6 + 2] = arg7[arg8 + 2];
        arg5[arg6 + 3] = arg7[arg8 + 3];
    }

    private static void roldqo32(int arg4, int[] arg5, int arg6, int[] arg7, int arg8) {
        arg7[arg8] = arg5[arg6 + 1] << arg4 - 32 | arg5[arg6 + 2] >>> 64 - arg4;
        arg7[arg8 + 1] = arg5[arg6 + 2] << arg4 - 32 | arg5[arg6 + 3] >>> 64 - arg4;
        arg7[arg8 + 2] = arg5[arg6 + 3] << arg4 - 32 | arg5[arg6] >>> 64 - arg4;
        arg7[arg8 + 3] = arg5[arg6] << arg4 - 32 | arg5[arg6 + 1] >>> 64 - arg4;
        arg5[arg6] = arg7[arg8];
        arg5[arg6 + 1] = arg7[arg8 + 1];
        arg5[arg6 + 2] = arg7[arg8 + 2];
        arg5[arg6 + 3] = arg7[arg8 + 3];
    }

    private int sbox2(int arg3) {
        return this.lRot8(CamelliaLightEngine.SBOX1[arg3], 1) & 255;
    }

    private int sbox3(int arg3) {
        return this.lRot8(CamelliaLightEngine.SBOX1[arg3], 7) & 255;
    }

    private int sbox4(int arg4) {
        return CamelliaLightEngine.SBOX1[this.lRot8(((byte)arg4), 1) & 255] & 255;
    }

    private void setKey(boolean arg13, byte[] arg14) {
        int v11 = 3;
        int v10 = 2;
        int v8 = 4;
        int[] v2 = new int[8];
        int[] v3 = new int[v8];
        int[] v4 = new int[v8];
        int[] v5 = new int[v8];
        switch(arg14.length) {
            case 16: {
                goto label_16;
            }
            case 24: {
                goto label_43;
            }
            case 32: {
                goto label_71;
            }
        }

        throw new IllegalArgumentException("key sizes are only 16/24/32 bytes.");
    label_71:
        v2[0] = this.bytes2int(arg14, 0);
        v2[1] = this.bytes2int(arg14, v8);
        v2[v10] = this.bytes2int(arg14, 8);
        v2[v11] = this.bytes2int(arg14, 12);
        v2[v8] = this.bytes2int(arg14, 16);
        v2[5] = this.bytes2int(arg14, 20);
        v2[6] = this.bytes2int(arg14, 24);
        v2[7] = this.bytes2int(arg14, 28);
        this._keyis128 = false;
        goto label_34;
    label_16:
        this._keyis128 = true;
        v2[0] = this.bytes2int(arg14, 0);
        v2[1] = this.bytes2int(arg14, v8);
        v2[v10] = this.bytes2int(arg14, 8);
        v2[v11] = this.bytes2int(arg14, 12);
        v2[7] = 0;
        v2[6] = 0;
        v2[5] = 0;
        v2[v8] = 0;
        goto label_34;
    label_43:
        v2[0] = this.bytes2int(arg14, 0);
        v2[1] = this.bytes2int(arg14, v8);
        v2[v10] = this.bytes2int(arg14, 8);
        v2[v11] = this.bytes2int(arg14, 12);
        v2[v8] = this.bytes2int(arg14, 16);
        v2[5] = this.bytes2int(arg14, 20);
        v2[6] = v2[v8] ^ -1;
        v2[7] = v2[5] ^ -1;
        this._keyis128 = false;
    label_34:
        int v0;
        for(v0 = 0; v0 < v8; ++v0) {
            v3[v0] = v2[v0] ^ v2[v0 + 4];
        }

        this.camelliaF2(v3, CamelliaLightEngine.SIGMA, 0);
        for(v0 = 0; v0 < v8; ++v0) {
            v3[v0] ^= v2[v0];
        }

        this.camelliaF2(v3, CamelliaLightEngine.SIGMA, v8);
        if(!this._keyis128) {
            for(v0 = 0; v0 < v8; ++v0) {
                v4[v0] = v3[v0] ^ v2[v0 + 4];
            }

            this.camelliaF2(v4, CamelliaLightEngine.SIGMA, 8);
            if(arg13) {
                this.kw[0] = v2[0];
                this.kw[1] = v2[1];
                this.kw[v10] = v2[v10];
                this.kw[v11] = v2[v11];
                CamelliaLightEngine.roldqo32(45, v2, 0, this.subkey, 16);
                CamelliaLightEngine.roldq(15, v2, 0, this.ke, v8);
                CamelliaLightEngine.roldq(17, v2, 0, this.subkey, 32);
                CamelliaLightEngine.roldqo32(34, v2, 0, this.subkey, 44);
                CamelliaLightEngine.roldq(15, v2, v8, this.subkey, v8);
                CamelliaLightEngine.roldq(15, v2, v8, this.ke, 0);
                CamelliaLightEngine.roldq(30, v2, v8, this.subkey, 24);
                CamelliaLightEngine.roldqo32(34, v2, v8, this.subkey, 36);
                CamelliaLightEngine.roldq(15, v3, 0, this.subkey, 8);
                CamelliaLightEngine.roldq(30, v3, 0, this.subkey, 20);
                this.ke[8] = v3[1];
                this.ke[9] = v3[v10];
                this.ke[10] = v3[v11];
                this.ke[11] = v3[0];
                CamelliaLightEngine.roldqo32(49, v3, 0, this.subkey, 40);
                this.subkey[0] = v4[0];
                this.subkey[1] = v4[1];
                this.subkey[v10] = v4[v10];
                this.subkey[v11] = v4[v11];
                CamelliaLightEngine.roldq(30, v4, 0, this.subkey, 12);
                CamelliaLightEngine.roldq(30, v4, 0, this.subkey, 28);
                CamelliaLightEngine.roldqo32(51, v4, 0, this.kw, v8);
            }
            else {
                this.kw[v8] = v2[0];
                this.kw[5] = v2[1];
                this.kw[6] = v2[v10];
                this.kw[7] = v2[v11];
                CamelliaLightEngine.decroldqo32(45, v2, 0, this.subkey, 28);
                CamelliaLightEngine.decroldq(15, v2, 0, this.ke, v8);
                CamelliaLightEngine.decroldq(17, v2, 0, this.subkey, 12);
                CamelliaLightEngine.decroldqo32(34, v2, 0, this.subkey, 0);
                CamelliaLightEngine.decroldq(15, v2, v8, this.subkey, 40);
                CamelliaLightEngine.decroldq(15, v2, v8, this.ke, 8);
                CamelliaLightEngine.decroldq(30, v2, v8, this.subkey, 20);
                CamelliaLightEngine.decroldqo32(34, v2, v8, this.subkey, 8);
                CamelliaLightEngine.decroldq(15, v3, 0, this.subkey, 36);
                CamelliaLightEngine.decroldq(30, v3, 0, this.subkey, 24);
                this.ke[v10] = v3[1];
                this.ke[v11] = v3[v10];
                this.ke[0] = v3[v11];
                this.ke[1] = v3[0];
                CamelliaLightEngine.decroldqo32(49, v3, 0, this.subkey, v8);
                this.subkey[46] = v4[0];
                this.subkey[47] = v4[1];
                this.subkey[44] = v4[v10];
                this.subkey[45] = v4[v11];
                CamelliaLightEngine.decroldq(30, v4, 0, this.subkey, 32);
                CamelliaLightEngine.decroldq(30, v4, 0, this.subkey, 16);
                CamelliaLightEngine.roldqo32(51, v4, 0, this.kw, 0);
            }
        }
        else if(arg13) {
            this.kw[0] = v2[0];
            this.kw[1] = v2[1];
            this.kw[v10] = v2[v10];
            this.kw[v11] = v2[v11];
            CamelliaLightEngine.roldq(15, v2, 0, this.subkey, v8);
            CamelliaLightEngine.roldq(30, v2, 0, this.subkey, 12);
            CamelliaLightEngine.roldq(15, v2, 0, v5, 0);
            this.subkey[18] = v5[v10];
            this.subkey[19] = v5[v11];
            CamelliaLightEngine.roldq(17, v2, 0, this.ke, v8);
            CamelliaLightEngine.roldq(17, v2, 0, this.subkey, 24);
            CamelliaLightEngine.roldq(17, v2, 0, this.subkey, 32);
            this.subkey[0] = v3[0];
            this.subkey[1] = v3[1];
            this.subkey[v10] = v3[v10];
            this.subkey[v11] = v3[v11];
            CamelliaLightEngine.roldq(15, v3, 0, this.subkey, 8);
            CamelliaLightEngine.roldq(15, v3, 0, this.ke, 0);
            CamelliaLightEngine.roldq(15, v3, 0, v5, 0);
            this.subkey[16] = v5[0];
            this.subkey[17] = v5[1];
            CamelliaLightEngine.roldq(15, v3, 0, this.subkey, 20);
            CamelliaLightEngine.roldqo32(34, v3, 0, this.subkey, 28);
            CamelliaLightEngine.roldq(17, v3, 0, this.kw, v8);
        }
        else {
            this.kw[v8] = v2[0];
            this.kw[5] = v2[1];
            this.kw[6] = v2[v10];
            this.kw[7] = v2[v11];
            CamelliaLightEngine.decroldq(15, v2, 0, this.subkey, 28);
            CamelliaLightEngine.decroldq(30, v2, 0, this.subkey, 20);
            CamelliaLightEngine.decroldq(15, v2, 0, v5, 0);
            this.subkey[16] = v5[0];
            this.subkey[17] = v5[1];
            CamelliaLightEngine.decroldq(17, v2, 0, this.ke, 0);
            CamelliaLightEngine.decroldq(17, v2, 0, this.subkey, 8);
            CamelliaLightEngine.decroldq(17, v2, 0, this.subkey, 0);
            this.subkey[34] = v3[0];
            this.subkey[35] = v3[1];
            this.subkey[32] = v3[v10];
            this.subkey[33] = v3[v11];
            CamelliaLightEngine.decroldq(15, v3, 0, this.subkey, 24);
            CamelliaLightEngine.decroldq(15, v3, 0, this.ke, v8);
            CamelliaLightEngine.decroldq(15, v3, 0, v5, 0);
            this.subkey[18] = v5[v10];
            this.subkey[19] = v5[v11];
            CamelliaLightEngine.decroldq(15, v3, 0, this.subkey, 12);
            CamelliaLightEngine.decroldqo32(34, v3, 0, this.subkey, v8);
            CamelliaLightEngine.roldq(17, v3, 0, this.kw, 0);
        }
    }
}

