// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import java.security.SecureRandom;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.Wrapper;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.util.Arrays;

public class DESedeWrapEngine implements Wrapper {
    private static final byte[] IV2;
    byte[] digest;
    private CBCBlockCipher engine;
    private boolean forWrapping;
    private byte[] iv;
    private KeyParameter param;
    private ParametersWithIV paramPlusIV;
    Digest sha1;

    static {
        DESedeWrapEngine.IV2 = new byte[]{74, -35, -94, 44, 121, -24, 33, 5};
    }

    public DESedeWrapEngine() {
        super();
        this.sha1 = new SHA1Digest();
        this.digest = new byte[20];
    }

    private byte[] calculateCMSKeyChecksum(byte[] arg6) {
        byte[] v0 = new byte[8];
        this.sha1.update(arg6, 0, arg6.length);
        this.sha1.doFinal(this.digest, 0);
        System.arraycopy(this.digest, 0, v0, 0, 8);
        return v0;
    }

    private boolean checkCMSKeyChecksum(byte[] arg2, byte[] arg3) {
        return Arrays.constantTimeAreEqual(this.calculateCMSKeyChecksum(arg2), arg3);
    }

    public String getAlgorithmName() {
        return "DESede";
    }

    public void init(boolean arg6, CipherParameters arg7) {
        SecureRandom v1;
        CipherParameters v0;
        int v3 = 8;
        this.forWrapping = arg6;
        this.engine = new CBCBlockCipher(new DESedeEngine());
        if((arg7 instanceof ParametersWithRandom)) {
            v0 = ((ParametersWithRandom)arg7).getParameters();
            v1 = ((ParametersWithRandom)arg7).getRandom();
        }
        else {
            v1 = new SecureRandom();
            v0 = arg7;
        }

        if((v0 instanceof KeyParameter)) {
            this.param = ((KeyParameter)v0);
            if(this.forWrapping) {
                this.iv = new byte[v3];
                v1.nextBytes(this.iv);
                this.paramPlusIV = new ParametersWithIV(this.param, this.iv);
            }
        }
        else if((v0 instanceof ParametersWithIV)) {
            this.paramPlusIV = ((ParametersWithIV)v0);
            this.iv = this.paramPlusIV.getIV();
            this.param = this.paramPlusIV.getParameters();
            if(this.forWrapping) {
                if(this.iv != null && this.iv.length == v3) {
                    return;
                }

                throw new IllegalArgumentException("IV is not 8 octets");
            }
            else {
                throw new IllegalArgumentException("You should not supply an IV for unwrapping");
            }
        }
    }

    private static byte[] reverse(byte[] arg4) {
        byte[] v1 = new byte[arg4.length];
        int v0;
        for(v0 = 0; v0 < arg4.length; ++v0) {
            v1[v0] = arg4[arg4.length - (v0 + 1)];
        }

        return v1;
    }

    public byte[] unwrap(byte[] arg8, int arg9, int arg10) {
        int v6 = 8;
        if(this.forWrapping) {
            throw new IllegalStateException("Not set for unwrapping");
        }

        if(arg8 == null) {
            throw new InvalidCipherTextException("Null pointer as ciphertext");
        }

        int v2 = this.engine.getBlockSize();
        if(arg10 % v2 != 0) {
            throw new InvalidCipherTextException("Ciphertext not multiple of " + v2);
        }

        this.engine.init(false, new ParametersWithIV(this.param, DESedeWrapEngine.IV2));
        byte[] v3 = new byte[arg10];
        int v0;
        for(v0 = 0; v0 != arg10; v0 += v2) {
            this.engine.processBlock(arg8, arg9 + v0, v3, v0);
        }

        byte[] v0_1 = DESedeWrapEngine.reverse(v3);
        this.iv = new byte[v6];
        v3 = new byte[v0_1.length - 8];
        System.arraycopy(v0_1, 0, this.iv, 0, v6);
        System.arraycopy(v0_1, v6, v3, 0, v0_1.length - 8);
        this.paramPlusIV = new ParametersWithIV(this.param, this.iv);
        this.engine.init(false, this.paramPlusIV);
        byte[] v4 = new byte[v3.length];
        for(v0 = 0; v0 != v4.length; v0 += v2) {
            this.engine.processBlock(v3, v0, v4, v0);
        }

        v0_1 = new byte[v4.length - 8];
        byte[] v2_1 = new byte[v6];
        System.arraycopy(v4, 0, v0_1, 0, v4.length - 8);
        System.arraycopy(v4, v4.length - 8, v2_1, 0, v6);
        if(!this.checkCMSKeyChecksum(v0_1, v2_1)) {
            throw new InvalidCipherTextException("Checksum inside ciphertext is corrupted");
        }

        return v0_1;
    }

    public byte[] wrap(byte[] arg8, int arg9, int arg10) {
        int v1 = 0;
        if(!this.forWrapping) {
            throw new IllegalStateException("Not initialized for wrapping");
        }

        byte[] v0 = new byte[arg10];
        System.arraycopy(arg8, arg9, v0, 0, arg10);
        byte[] v2 = this.calculateCMSKeyChecksum(v0);
        byte[] v3 = new byte[v0.length + v2.length];
        System.arraycopy(v0, 0, v3, 0, v0.length);
        System.arraycopy(v2, 0, v3, v0.length, v2.length);
        int v2_1 = this.engine.getBlockSize();
        if(v3.length % v2_1 != 0) {
            throw new IllegalStateException("Not multiple of block length");
        }

        this.engine.init(true, this.paramPlusIV);
        byte[] v4 = new byte[v3.length];
        int v0_1;
        for(v0_1 = 0; v0_1 != v3.length; v0_1 += v2_1) {
            this.engine.processBlock(v3, v0_1, v4, v0_1);
        }

        v0 = new byte[this.iv.length + v4.length];
        System.arraycopy(this.iv, 0, v0, 0, this.iv.length);
        System.arraycopy(v4, 0, v0, this.iv.length, v4.length);
        v0 = DESedeWrapEngine.reverse(v0);
        this.engine.init(true, new ParametersWithIV(this.param, DESedeWrapEngine.IV2));
        while(v1 != v0.length) {
            this.engine.processBlock(v0, v1, v0, v1);
            v1 += v2_1;
        }

        return v0;
    }
}

