// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.params.ElGamalKeyParameters;
import org.bouncycastle.crypto.params.ElGamalPrivateKeyParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.util.BigIntegers;

public class ElGamalEngine implements AsymmetricBlockCipher {
    private static final BigInteger ONE;
    private static final BigInteger TWO;
    private static final BigInteger ZERO;
    private int bitSize;
    private boolean forEncryption;
    private ElGamalKeyParameters key;
    private SecureRandom random;

    static {
        ElGamalEngine.ZERO = BigInteger.valueOf(0);
        ElGamalEngine.ONE = BigInteger.valueOf(1);
        ElGamalEngine.TWO = BigInteger.valueOf(2);
    }

    public ElGamalEngine() {
        super();
    }

    public int getInputBlockSize() {
        int v0 = this.forEncryption ? (this.bitSize - 1) / 8 : (this.bitSize + 7) / 8 * 2;
        return v0;
    }

    public int getOutputBlockSize() {
        int v0 = this.forEncryption ? (this.bitSize + 7) / 8 * 2 : (this.bitSize - 1) / 8;
        return v0;
    }

    public void init(boolean arg3, CipherParameters arg4) {
        if((arg4 instanceof ParametersWithRandom)) {
            this.key = ((ParametersWithRandom)arg4).getParameters();
            this.random = ((ParametersWithRandom)arg4).getRandom();
        }
        else {
            this.key = ((ElGamalKeyParameters)arg4);
            this.random = new SecureRandom();
        }

        this.forEncryption = arg3;
        this.bitSize = this.key.getParameters().getP().bitLength();
        if(arg3) {
            if(!(this.key instanceof ElGamalPublicKeyParameters)) {
                throw new IllegalArgumentException("ElGamalPublicKeyParameters are required for encryption.");
            }
        }
        else if(!(this.key instanceof ElGamalPrivateKeyParameters)) {
            throw new IllegalArgumentException("ElGamalPrivateKeyParameters are required for decryption.");
        }
    }

    public byte[] processBlock(byte[] arg9, int arg10, int arg11) {
        byte[] v1;
        byte[] v0_1;
        if(this.key == null) {
            throw new IllegalStateException("ElGamal engine not initialised");
        }

        int v0 = this.forEncryption ? (this.bitSize + 6) / 8 : this.getInputBlockSize();
        if(arg11 > v0) {
            throw new DataLengthException("input too large for ElGamal cipher.\n");
        }

        BigInteger v2 = this.key.getParameters().getP();
        if((this.key instanceof ElGamalPrivateKeyParameters)) {
            v0_1 = new byte[arg11 / 2];
            v1 = new byte[arg11 / 2];
            System.arraycopy(arg9, arg10, v0_1, 0, v0_1.length);
            System.arraycopy(arg9, v0_1.length + arg10, v1, 0, v1.length);
            v0_1 = BigIntegers.asUnsignedByteArray(new BigInteger(1, v0_1).modPow(v2.subtract(ElGamalEngine.ONE).subtract(this.key.getX()), v2).multiply(new BigInteger(1, v1)).mod(v2));
        }
        else {
            if(arg10 != 0 || arg11 != arg9.length) {
                v0_1 = new byte[arg11];
                System.arraycopy(arg9, arg10, v0_1, 0, arg11);
                arg9 = v0_1;
            }

            BigInteger v3 = new BigInteger(1, arg9);
            if(v3.bitLength() >= v2.bitLength()) {
                throw new DataLengthException("input too large for ElGamal cipher.\n");
            }

            ElGamalKeyParameters v0_2 = this.key;
            int v4 = v2.bitLength();
            BigInteger v1_1;
            for(v1_1 = new BigInteger(v4, this.random); true; v1_1 = new BigInteger(v4, this.random)) {
                if(!v1_1.equals(ElGamalEngine.ZERO) && v1_1.compareTo(v2.subtract(ElGamalEngine.TWO)) <= 0) {
                    BigInteger v4_1 = this.key.getParameters().getG().modPow(v1_1, v2);
                    BigInteger v0_3 = v3.multiply(((ElGamalPublicKeyParameters)v0_2).getY().modPow(v1_1, v2)).mod(v2);
                    v1 = v4_1.toByteArray();
                    byte[] v2_1 = v0_3.toByteArray();
                    v0_1 = new byte[this.getOutputBlockSize()];
                    if(v1.length > v0_1.length / 2) {
                        System.arraycopy(v1, 1, v0_1, v0_1.length / 2 - (v1.length - 1), v1.length - 1);
                    }
                    else {
                        System.arraycopy(v1, 0, v0_1, v0_1.length / 2 - v1.length, v1.length);
                    }

                    if(v2_1.length > v0_1.length / 2) {
                        System.arraycopy(v2_1, 1, v0_1, v0_1.length - (v2_1.length - 1), v2_1.length - 1);
                        return v0_1;
                    }

                    System.arraycopy(v2_1, 0, v0_1, v0_1.length - v2_1.length, v2_1.length);
                    return v0_1;
                }
            }
        }

        return v0_1;
    }
}

