// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;

public class Grain128Engine implements StreamCipher {
    private static final int STATE_SIZE = 4;
    private int index;
    private boolean initialised;
    private int[] lfsr;
    private int[] nfsr;
    private byte[] out;
    private int output;
    private byte[] workingIV;
    private byte[] workingKey;

    public Grain128Engine() {
        super();
        this.index = 4;
        this.initialised = false;
    }

    public String getAlgorithmName() {
        return "Grain-128";
    }

    private byte getKeyStream() {
        if(this.index > 3) {
            this.oneRound();
            this.index = 0;
        }

        byte[] v0 = this.out;
        int v1 = this.index;
        this.index = v1 + 1;
        return v0[v1];
    }

    private int getOutput() {
        int v1 = this.nfsr[0] >>> 2 | this.nfsr[1] << 30;
        int v2 = this.nfsr[0] >>> 12 | this.nfsr[1] << 20;
        int v3 = this.nfsr[0] >>> 15 | this.nfsr[1] << 17;
        int v4 = this.nfsr[1] >>> 4 | this.nfsr[2] << 28;
        int v5 = this.nfsr[1] >>> 13 | this.nfsr[2] << 19;
        int v6 = this.nfsr[2];
        int v7 = this.nfsr[2] >>> 9 | this.nfsr[3] << 23;
        int v8 = this.nfsr[2] >>> 25 | this.nfsr[3] << 7;
        int v9 = this.nfsr[2] >>> 31 | this.nfsr[3] << 1;
        return v1 ^ (v2 & v9 & (this.lfsr[2] >>> 31 | this.lfsr[3] << 1) ^ ((this.lfsr[0] >>> 8 | this.lfsr[1] << 24) & v2 ^ (this.lfsr[0] >>> 13 | this.lfsr[1] << 19) & (this.lfsr[0] >>> 20 | this.lfsr[1] << 12) ^ v9 & (this.lfsr[1] >>> 10 | this.lfsr[2] << 22) ^ (this.lfsr[1] >>> 28 | this.lfsr[2] << 4) & (this.lfsr[2] >>> 15 | this.lfsr[3] << 17)) ^ (this.lfsr[2] >>> 29 | this.lfsr[3] << 3)) ^ v3 ^ v4 ^ v5 ^ v6 ^ v7 ^ v8;
    }

    private int getOutputLFSR() {
        return this.lfsr[0] ^ (this.lfsr[0] >>> 7 | this.lfsr[1] << 25) ^ (this.lfsr[1] >>> 6 | this.lfsr[2] << 26) ^ (this.lfsr[2] >>> 6 | this.lfsr[3] << 26) ^ (this.lfsr[2] >>> 17 | this.lfsr[3] << 15) ^ this.lfsr[3];
    }

    private int getOutputNFSR() {
        return this.nfsr[0] ^ (this.nfsr[0] >>> 26 | this.nfsr[1] << 6) ^ (this.nfsr[1] >>> 24 | this.nfsr[2] << 8) ^ (this.nfsr[2] >>> 27 | this.nfsr[3] << 5) ^ this.nfsr[3] ^ (this.nfsr[0] >>> 3 | this.nfsr[1] << 29) & (this.nfsr[2] >>> 3 | this.nfsr[3] << 29) ^ (this.nfsr[0] >>> 11 | this.nfsr[1] << 21) & (this.nfsr[0] >>> 13 | this.nfsr[1] << 19) ^ (this.nfsr[0] >>> 17 | this.nfsr[1] << 15) & (this.nfsr[0] >>> 18 | this.nfsr[1] << 14) ^ (this.nfsr[0] >>> 27 | this.nfsr[1] << 5) & (this.nfsr[1] >>> 27 | this.nfsr[2] << 5) ^ (this.nfsr[1] >>> 8 | this.nfsr[2] << 24) & (this.nfsr[1] >>> 16 | this.nfsr[2] << 16) ^ (this.nfsr[1] >>> 29 | this.nfsr[2] << 3) & (this.nfsr[2] >>> 1 | this.nfsr[3] << 31) ^ (this.nfsr[2] >>> 4 | this.nfsr[3] << 28) & (this.nfsr[2] >>> 20 | this.nfsr[3] << 12);
    }

    public void init(boolean arg6, CipherParameters arg7) {
        int v3 = 4;
        if(!(arg7 instanceof ParametersWithIV)) {
            throw new IllegalArgumentException("Grain-128 Init parameters must include an IV");
        }

        byte[] v1 = ((ParametersWithIV)arg7).getIV();
        if(v1 != null && v1.length == 12) {
            if(!(((ParametersWithIV)arg7).getParameters() instanceof KeyParameter)) {
                throw new IllegalArgumentException("Grain-128 Init parameters must include a key");
            }
            else {
                CipherParameters v0 = ((ParametersWithIV)arg7).getParameters();
                this.workingIV = new byte[((KeyParameter)v0).getKey().length];
                this.workingKey = new byte[((KeyParameter)v0).getKey().length];
                this.lfsr = new int[v3];
                this.nfsr = new int[v3];
                this.out = new byte[v3];
                System.arraycopy(v1, 0, this.workingIV, 0, v1.length);
                System.arraycopy(((KeyParameter)v0).getKey(), 0, this.workingKey, 0, ((KeyParameter)v0).getKey().length);
                this.setKey(this.workingKey, this.workingIV);
                this.initGrain();
                return;
            }
        }

        throw new IllegalArgumentException("Grain-128  requires exactly 12 bytes of IV");
    }

    private void initGrain() {
        int v0;
        for(v0 = 0; v0 < 8; ++v0) {
            this.output = this.getOutput();
            this.nfsr = this.shift(this.nfsr, this.getOutputNFSR() ^ this.lfsr[0] ^ this.output);
            this.lfsr = this.shift(this.lfsr, this.getOutputLFSR() ^ this.output);
        }

        this.initialised = true;
    }

    private void oneRound() {
        this.output = this.getOutput();
        this.out[0] = ((byte)this.output);
        this.out[1] = ((byte)(this.output >> 8));
        this.out[2] = ((byte)(this.output >> 16));
        this.out[3] = ((byte)(this.output >> 24));
        this.nfsr = this.shift(this.nfsr, this.getOutputNFSR() ^ this.lfsr[0]);
        this.lfsr = this.shift(this.lfsr, this.getOutputLFSR());
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            arg8[arg9 + v0] = ((byte)(arg5[arg6 + v0] ^ this.getKeyStream()));
        }
    }

    public void reset() {
        this.index = 4;
        this.setKey(this.workingKey, this.workingIV);
        this.initGrain();
    }

    public byte returnByte(byte arg4) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        return ((byte)(this.getKeyStream() ^ arg4));
    }

    private void setKey(byte[] arg9, byte[] arg10) {
        int v7 = 16711680;
        int v6 = 65280;
        int v0 = 0;
        arg10[12] = -1;
        arg10[13] = -1;
        arg10[14] = -1;
        arg10[15] = -1;
        this.workingKey = arg9;
        this.workingIV = arg10;
        int v1 = 0;
        while(v0 < this.nfsr.length) {
            this.nfsr[v0] = this.workingKey[v1 + 3] << 24 | this.workingKey[v1 + 2] << 16 & v7 | this.workingKey[v1 + 1] << 8 & v6 | this.workingKey[v1] & 255;
            this.lfsr[v0] = this.workingIV[v1 + 3] << 24 | this.workingIV[v1 + 2] << 16 & v7 | this.workingIV[v1 + 1] << 8 & v6 | this.workingIV[v1] & 255;
            v1 += 4;
            ++v0;
        }
    }

    private int[] shift(int[] arg6, int arg7) {
        arg6[0] = arg6[1];
        arg6[1] = arg6[2];
        arg6[2] = arg6[3];
        arg6[3] = arg7;
        return arg6;
    }
}

