// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;

public class Grainv1Engine implements StreamCipher {
    private static final int STATE_SIZE = 5;
    private int index;
    private boolean initialised;
    private int[] lfsr;
    private int[] nfsr;
    private byte[] out;
    private int output;
    private byte[] workingIV;
    private byte[] workingKey;

    public Grainv1Engine() {
        super();
        this.index = 2;
        this.initialised = false;
    }

    public String getAlgorithmName() {
        return "Grain v1";
    }

    private byte getKeyStream() {
        if(this.index > 1) {
            this.oneRound();
            this.index = 0;
        }

        byte[] v0 = this.out;
        int v1 = this.index;
        this.index = v1 + 1;
        return v0[v1];
    }

    private int getOutput() {
        int v0 = this.nfsr[0] >>> 1 | this.nfsr[1] << 15;
        int v1 = this.nfsr[0] >>> 2 | this.nfsr[1] << 14;
        int v2 = this.nfsr[0] >>> 4 | this.nfsr[1] << 12;
        int v3 = this.nfsr[0] >>> 10 | this.nfsr[1] << 6;
        int v4 = this.nfsr[1] >>> 15 | this.nfsr[2] << 1;
        int v5 = this.nfsr[2] >>> 11 | this.nfsr[3] << 5;
        int v6 = this.nfsr[3] >>> 8 | this.nfsr[4] << 8;
        int v7 = this.nfsr[3] >>> 15 | this.nfsr[4] << 1;
        int v8 = this.lfsr[0] >>> 3 | this.lfsr[1] << 13;
        int v9 = this.lfsr[1] >>> 9 | this.lfsr[2] << 7;
        int v10 = this.lfsr[2] >>> 14 | this.lfsr[3] << 2;
        return (v0 ^ (v7 & (v10 & this.lfsr[4]) ^ (v8 & v10 & v7 ^ (v9 ^ v7 ^ v8 & this.lfsr[4] ^ v10 & this.lfsr[4] ^ this.lfsr[4] & v7 ^ v8 & v9 & v10 ^ v8 & v10 & this.lfsr[4]) ^ v9 & v10 & v7)) ^ v1 ^ v2 ^ v3 ^ v4 ^ v5 ^ v6) & 65535;
    }

    private int getOutputLFSR() {
        return (this.lfsr[0] ^ (this.lfsr[0] >>> 13 | this.lfsr[1] << 3) ^ (this.lfsr[1] >>> 7 | this.lfsr[2] << 9) ^ (this.lfsr[2] >>> 6 | this.lfsr[3] << 10) ^ (this.lfsr[3] >>> 3 | this.lfsr[4] << 13) ^ (this.lfsr[3] >>> 14 | this.lfsr[4] << 2)) & 65535;
    }

    private int getOutputNFSR() {
        int v0 = this.nfsr[0];
        int v1 = this.nfsr[0] >>> 9 | this.nfsr[1] << 7;
        int v2 = this.nfsr[0] >>> 14 | this.nfsr[1] << 2;
        int v3 = this.nfsr[0] >>> 15 | this.nfsr[1] << 1;
        int v4 = this.nfsr[1] >>> 5 | this.nfsr[2] << 11;
        int v5 = this.nfsr[1] >>> 12 | this.nfsr[2] << 4;
        int v6 = this.nfsr[2] >>> 1 | this.nfsr[3] << 15;
        int v7 = this.nfsr[2] >>> 5 | this.nfsr[3] << 11;
        int v8 = this.nfsr[2] >>> 13 | this.nfsr[3] << 3;
        int v9 = this.nfsr[3] >>> 4 | this.nfsr[4] << 12;
        int v10 = this.nfsr[3] >>> 12 | this.nfsr[4] << 4;
        int v11 = this.nfsr[3] >>> 14 | this.nfsr[4] << 2;
        int v12 = this.nfsr[3] >>> 15 | this.nfsr[4] << 1;
        return (v0 ^ (v2 ^ (v11 ^ v10 ^ v9 ^ v8 ^ v7 ^ v6 ^ v5 ^ v4) ^ v1) ^ v12 & v10 ^ v7 & v6 ^ v3 & v1 ^ v10 & v9 & v8 ^ v6 & v5 & v4 ^ v12 & v8 & v5 & v1 ^ v10 & v9 & v7 & v6 ^ v12 & v10 & v4 & v3 ^ v12 & v10 & v9 & v8 & v7 ^ v1 & (v6 & v5 & v4 & v3) ^ v9 & v8 & v7 & v6 & v5 & v4) & 65535;
    }

    public void init(boolean arg6, CipherParameters arg7) {
        int v3 = 5;
        if(!(arg7 instanceof ParametersWithIV)) {
            throw new IllegalArgumentException("Grain v1 Init parameters must include an IV");
        }

        byte[] v1 = ((ParametersWithIV)arg7).getIV();
        if(v1 != null && v1.length == 8) {
            if(!(((ParametersWithIV)arg7).getParameters() instanceof KeyParameter)) {
                throw new IllegalArgumentException("Grain v1 Init parameters must include a key");
            }
            else {
                CipherParameters v0 = ((ParametersWithIV)arg7).getParameters();
                this.workingIV = new byte[((KeyParameter)v0).getKey().length];
                this.workingKey = new byte[((KeyParameter)v0).getKey().length];
                this.lfsr = new int[v3];
                this.nfsr = new int[v3];
                this.out = new byte[2];
                System.arraycopy(v1, 0, this.workingIV, 0, v1.length);
                System.arraycopy(((KeyParameter)v0).getKey(), 0, this.workingKey, 0, ((KeyParameter)v0).getKey().length);
                this.setKey(this.workingKey, this.workingIV);
                this.initGrain();
                return;
            }
        }

        throw new IllegalArgumentException("Grain v1 requires exactly 8 bytes of IV");
    }

    private void initGrain() {
        int v0;
        for(v0 = 0; v0 < 10; ++v0) {
            this.output = this.getOutput();
            this.nfsr = this.shift(this.nfsr, this.getOutputNFSR() ^ this.lfsr[0] ^ this.output);
            this.lfsr = this.shift(this.lfsr, this.getOutputLFSR() ^ this.output);
        }

        this.initialised = true;
    }

    private void oneRound() {
        this.output = this.getOutput();
        this.out[0] = ((byte)this.output);
        this.out[1] = ((byte)(this.output >> 8));
        this.nfsr = this.shift(this.nfsr, this.getOutputNFSR() ^ this.lfsr[0]);
        this.lfsr = this.shift(this.lfsr, this.getOutputLFSR());
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            arg8[arg9 + v0] = ((byte)(arg5[arg6 + v0] ^ this.getKeyStream()));
        }
    }

    public void reset() {
        this.index = 2;
        this.setKey(this.workingKey, this.workingIV);
        this.initGrain();
    }

    public byte returnByte(byte arg4) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        return ((byte)(this.getKeyStream() ^ arg4));
    }

    private void setKey(byte[] arg7, byte[] arg8) {
        int v5 = 65535;
        int v0 = 0;
        arg8[8] = -1;
        arg8[9] = -1;
        this.workingKey = arg7;
        this.workingIV = arg8;
        int v1 = 0;
        while(v0 < this.nfsr.length) {
            this.nfsr[v0] = (this.workingKey[v1 + 1] << 8 | this.workingKey[v1] & 255) & v5;
            this.lfsr[v0] = (this.workingIV[v1 + 1] << 8 | this.workingIV[v1] & 255) & v5;
            v1 += 2;
            ++v0;
        }
    }

    private int[] shift(int[] arg7, int arg8) {
        arg7[0] = arg7[1];
        arg7[1] = arg7[2];
        arg7[2] = arg7[3];
        arg7[3] = arg7[4];
        arg7[4] = arg8;
        return arg7;
    }
}

