// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;

public class HC128Engine implements StreamCipher {
    private byte[] buf;
    private int cnt;
    private int idx;
    private boolean initialised;
    private byte[] iv;
    private byte[] key;
    private int[] p;
    private int[] q;

    public HC128Engine() {
        super();
        this.p = new int[512];
        this.q = new int[512];
        this.cnt = 0;
        this.buf = new byte[4];
        this.idx = 0;
    }

    private static int dim(int arg1, int arg2) {
        return HC128Engine.mod512(arg1 - arg2);
    }

    private static int f1(int arg2) {
        return HC128Engine.rotateRight(arg2, 7) ^ HC128Engine.rotateRight(arg2, 18) ^ arg2 >>> 3;
    }

    private static int f2(int arg2) {
        return HC128Engine.rotateRight(arg2, 17) ^ HC128Engine.rotateRight(arg2, 19) ^ arg2 >>> 10;
    }

    private int g1(int arg3, int arg4, int arg5) {
        return (HC128Engine.rotateRight(arg3, 10) ^ HC128Engine.rotateRight(arg5, 23)) + HC128Engine.rotateRight(arg4, 8);
    }

    private int g2(int arg3, int arg4, int arg5) {
        return (HC128Engine.rotateLeft(arg3, 10) ^ HC128Engine.rotateLeft(arg5, 23)) + HC128Engine.rotateLeft(arg4, 8);
    }

    public String getAlgorithmName() {
        return "HC-128";
    }

    private byte getByte() {
        if(this.idx == 0) {
            int v0 = this.step();
            this.buf[0] = ((byte)(v0 & 255));
            v0 >>= 8;
            this.buf[1] = ((byte)(v0 & 255));
            v0 >>= 8;
            this.buf[2] = ((byte)(v0 & 255));
            this.buf[3] = ((byte)(v0 >> 8 & 255));
        }

        byte v0_1 = this.buf[this.idx];
        this.idx = this.idx + 1 & 3;
        return v0_1;
    }

    private int h1(int arg4) {
        return this.q[arg4 & 255] + this.q[(arg4 >> 16 & 255) + 256];
    }

    private int h2(int arg4) {
        return this.p[arg4 & 255] + this.p[(arg4 >> 16 & 255) + 256];
    }

    private void init() {
        int v4;
        int v10 = 1280;
        int v9 = 4;
        int v8 = 512;
        int v2 = 16;
        if(this.key.length != v2) {
            throw new IllegalArgumentException("The key must be 128 bits long");
        }

        this.cnt = 0;
        int[] v3 = new int[v10];
        int v0;
        for(v0 = 0; v0 < v2; ++v0) {
            v4 = v0 >> 2;
            v3[v4] |= (this.key[v0] & 255) << (v0 & 3) * 8;
        }

        System.arraycopy(v3, 0, v3, v9, v9);
        for(v0 = 0; v0 < this.iv.length; ++v0) {
            if(v0 >= v2) {
                break;
            }

            v4 = (v0 >> 2) + 8;
            v3[v4] |= (this.iv[v0] & 255) << (v0 & 3) * 8;
        }

        System.arraycopy(v3, 8, v3, 12, v9);
        for(v0 = v2; v0 < v10; ++v0) {
            v3[v0] = HC128Engine.f2(v3[v0 - 2]) + v3[v0 - 7] + HC128Engine.f1(v3[v0 - 15]) + v3[v0 - 16] + v0;
        }

        System.arraycopy(v3, 256, this.p, 0, v8);
        System.arraycopy(v3, 768, this.q, 0, v8);
        for(v0 = 0; v0 < v8; ++v0) {
            this.p[v0] = this.step();
        }

        for(v0 = 0; v0 < v8; ++v0) {
            this.q[v0] = this.step();
        }

        this.cnt = 0;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        CipherParameters v0;
        if((arg5 instanceof ParametersWithIV)) {
            this.iv = arg5.getIV();
            v0 = arg5.getParameters();
        }
        else {
            this.iv = new byte[0];
            v0 = arg5;
        }

        if((v0 instanceof KeyParameter)) {
            this.key = ((KeyParameter)v0).getKey();
            this.init();
            this.initialised = true;
            return;
        }

        throw new IllegalArgumentException("Invalid parameter passed to HC128 init - " + arg5.getClass().getName());
    }

    private static int mod1024(int arg1) {
        return arg1 & 1023;
    }

    private static int mod512(int arg1) {
        return arg1 & 511;
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            arg8[arg9 + v0] = ((byte)(arg5[arg6 + v0] ^ this.getByte()));
        }
    }

    public void reset() {
        this.idx = 0;
        this.init();
    }

    public byte returnByte(byte arg2) {
        return ((byte)(this.getByte() ^ arg2));
    }

    private static int rotateLeft(int arg2, int arg3) {
        return arg2 << arg3 | arg2 >>> -arg3;
    }

    private static int rotateRight(int arg2, int arg3) {
        return arg2 >>> arg3 | arg2 << -arg3;
    }

    private int step() {
        int v6 = 511;
        int v7 = 12;
        int v5 = 10;
        int v4 = 3;
        int v0 = HC128Engine.mod512(this.cnt);
        if(this.cnt < 512) {
            this.p[v0] += this.g1(this.p[HC128Engine.dim(v0, v4)], this.p[HC128Engine.dim(v0, v5)], this.p[HC128Engine.dim(v0, v6)]);
            v0 = this.p[v0] ^ this.h1(this.p[HC128Engine.dim(v0, v7)]);
        }
        else {
            this.q[v0] += this.g2(this.q[HC128Engine.dim(v0, v4)], this.q[HC128Engine.dim(v0, v5)], this.q[HC128Engine.dim(v0, v6)]);
            v0 = this.q[v0] ^ this.h2(this.q[HC128Engine.dim(v0, v7)]);
        }

        this.cnt = HC128Engine.mod1024(this.cnt + 1);
        return v0;
    }
}

