// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;

public class HC256Engine implements StreamCipher {
    private byte[] buf;
    private int cnt;
    private int idx;
    private boolean initialised;
    private byte[] iv;
    private byte[] key;
    private int[] p;
    private int[] q;

    public HC256Engine() {
        super();
        this.p = new int[1024];
        this.q = new int[1024];
        this.cnt = 0;
        this.buf = new byte[4];
        this.idx = 0;
    }

    public String getAlgorithmName() {
        return "HC-256";
    }

    private byte getByte() {
        if(this.idx == 0) {
            int v0 = this.step();
            this.buf[0] = ((byte)(v0 & 255));
            v0 >>= 8;
            this.buf[1] = ((byte)(v0 & 255));
            v0 >>= 8;
            this.buf[2] = ((byte)(v0 & 255));
            this.buf[3] = ((byte)(v0 >> 8 & 255));
        }

        byte v0_1 = this.buf[this.idx];
        this.idx = this.idx + 1 & 3;
        return v0_1;
    }

    private void init() {
        int v4;
        byte[] v2;
        int v10 = 2560;
        int v9 = 1024;
        int v0 = 16;
        int v8 = 32;
        if(this.key.length != v8 && this.key.length != v0) {
            throw new IllegalArgumentException("The key must be 128/256 bits long");
        }

        if(this.iv.length < v0) {
            throw new IllegalArgumentException("The IV must be at least 128 bits long");
        }

        if(this.key.length != v8) {
            v2 = new byte[v8];
            System.arraycopy(this.key, 0, v2, 0, this.key.length);
            System.arraycopy(this.key, 0, v2, v0, this.key.length);
            this.key = v2;
        }

        if(this.iv.length < v8) {
            v2 = new byte[v8];
            System.arraycopy(this.iv, 0, v2, 0, this.iv.length);
            System.arraycopy(this.iv, 0, v2, this.iv.length, v2.length - this.iv.length);
            this.iv = v2;
        }

        this.cnt = 0;
        int[] v3 = new int[v10];
        int v2_1;
        for(v2_1 = 0; v2_1 < v8; ++v2_1) {
            v4 = v2_1 >> 2;
            v3[v4] |= (this.key[v2_1] & 255) << (v2_1 & 3) * 8;
        }

        for(v2_1 = 0; v2_1 < v8; ++v2_1) {
            v4 = (v2_1 >> 2) + 8;
            v3[v4] |= (this.iv[v2_1] & 255) << (v2_1 & 3) * 8;
        }

        while(v0 < v10) {
            v3[v0] = (v3[v0 - 2] >>> 10 ^ (HC256Engine.rotateRight(v3[v0 - 2], 17) ^ HC256Engine.rotateRight(v3[v0 - 2], 19))) + v3[v0 - 7] + (v3[v0 - 15] >>> 3 ^ (HC256Engine.rotateRight(v3[v0 - 15], 7) ^ HC256Engine.rotateRight(v3[v0 - 15], 18))) + v3[v0 - 16] + v0;
            ++v0;
        }

        System.arraycopy(v3, 512, this.p, 0, v9);
        System.arraycopy(v3, 1536, this.q, 0, v9);
        for(v0 = 0; v0 < 4096; ++v0) {
            this.step();
        }

        this.cnt = 0;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        CipherParameters v0;
        if((arg5 instanceof ParametersWithIV)) {
            this.iv = arg5.getIV();
            v0 = arg5.getParameters();
        }
        else {
            this.iv = new byte[0];
            v0 = arg5;
        }

        if((v0 instanceof KeyParameter)) {
            this.key = ((KeyParameter)v0).getKey();
            this.init();
            this.initialised = true;
            return;
        }

        throw new IllegalArgumentException("Invalid parameter passed to HC256 init - " + arg5.getClass().getName());
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            arg8[arg9 + v0] = ((byte)(arg5[arg6 + v0] ^ this.getByte()));
        }
    }

    public void reset() {
        this.idx = 0;
        this.init();
    }

    public byte returnByte(byte arg2) {
        return ((byte)(this.getByte() ^ arg2));
    }

    private static int rotateRight(int arg2, int arg3) {
        return arg2 >>> arg3 | arg2 << -arg3;
    }

    private int step() {
        int v8 = 23;
        int v7 = 10;
        int v0 = this.cnt & 1023;
        if(this.cnt < 1024) {
            this.p[v0] = this.q[(this.p[v0 - 3 & 1023] ^ this.p[v0 - 1023 & 1023]) & 1023] + (this.p[v0 - 10 & 1023] + (HC256Engine.rotateRight(this.p[v0 - 3 & 1023], v7) ^ HC256Engine.rotateRight(this.p[v0 - 1023 & 1023], v8))) + this.p[v0];
            v0 = this.p[v0] ^ this.q[(this.p[v0 - 12 & 1023] >> 24 & 255) + 768] + (this.q[this.p[v0 - 12 & 1023] & 255] + this.q[(this.p[v0 - 12 & 1023] >> 8 & 255) + 256] + this.q[(this.p[v0 - 12 & 1023] >> 16 & 255) + 512]);
        }
        else {
            this.q[v0] = this.p[(this.q[v0 - 3 & 1023] ^ this.q[v0 - 1023 & 1023]) & 1023] + (this.q[v0 - 10 & 1023] + (HC256Engine.rotateRight(this.q[v0 - 3 & 1023], v7) ^ HC256Engine.rotateRight(this.q[v0 - 1023 & 1023], v8))) + this.q[v0];
            v0 = this.q[v0] ^ this.p[(this.q[v0 - 12 & 1023] >> 24 & 255) + 768] + (this.p[this.q[v0 - 12 & 1023] & 255] + this.p[(this.q[v0 - 12 & 1023] >> 8 & 255) + 256] + this.p[(this.q[v0 - 12 & 1023] >> 16 & 255) + 512]);
        }

        this.cnt = this.cnt + 1 & 2047;
        return v0;
    }
}

