// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.util.Pack;

public class ISAACEngine implements StreamCipher {
    private int a;
    private int b;
    private int c;
    private int[] engineState;
    private int index;
    private boolean initialised;
    private byte[] keyStream;
    private int[] results;
    private final int sizeL;
    private final int stateArraySize;
    private byte[] workingKey;

    public ISAACEngine() {
        super();
        this.sizeL = 8;
        this.stateArraySize = 256;
        this.engineState = null;
        this.results = null;
        this.a = 0;
        this.b = 0;
        this.c = 0;
        this.index = 0;
        this.keyStream = new byte[1024];
        this.workingKey = null;
        this.initialised = false;
    }

    public String getAlgorithmName() {
        return "ISAAC";
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to ISAAC init - " + arg5.getClass().getName());
        }

        this.setKey(((KeyParameter)arg5).getKey());
    }

    private void isaac() {
        int v0 = this.b;
        int v1 = this.c + 1;
        this.c = v1;
        this.b = v0 + v1;
        for(v0 = 0; v0 < 256; ++v0) {
            v1 = this.engineState[v0];
            switch(v0 & 3) {
                case 0: {
                    this.a ^= this.a << 13;
                    break;
                }
                case 1: {
                    this.a ^= this.a >>> 6;
                    break;
                }
                case 2: {
                    this.a ^= this.a << 2;
                    break;
                }
                case 3: {
                    this.a ^= this.a >>> 16;
                    break;
                }
            }

            this.a += this.engineState[v0 + 128 & 255];
            int[] v2 = this.engineState;
            int v3 = this.engineState[v1 >>> 2 & 255] + this.a + this.b;
            v2[v0] = v3;
            v2 = this.results;
            v1 += this.engineState[v3 >>> 10 & 255];
            this.b = v1;
            v2[v0] = v1;
        }
    }

    private void mix(int[] arg9) {
        arg9[0] ^= arg9[1] << 11;
        arg9[3] += arg9[0];
        arg9[1] += arg9[2];
        arg9[1] ^= arg9[2] >>> 2;
        arg9[4] += arg9[1];
        arg9[2] += arg9[3];
        arg9[2] ^= arg9[3] << 8;
        arg9[5] += arg9[2];
        arg9[3] += arg9[4];
        arg9[3] ^= arg9[4] >>> 16;
        arg9[6] += arg9[3];
        arg9[4] += arg9[5];
        arg9[4] ^= arg9[5] << 10;
        arg9[7] += arg9[4];
        arg9[5] += arg9[6];
        arg9[5] ^= arg9[6] >>> 4;
        arg9[0] += arg9[5];
        arg9[6] += arg9[7];
        arg9[6] ^= arg9[7] << 8;
        arg9[1] += arg9[6];
        arg9[7] += arg9[0];
        arg9[7] ^= arg9[0] >>> 9;
        arg9[2] += arg9[7];
        arg9[0] += arg9[1];
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            if(this.index == 0) {
                this.isaac();
                this.keyStream = Pack.intToBigEndian(this.results);
            }

            arg8[v0 + arg9] = ((byte)(this.keyStream[this.index] ^ arg5[v0 + arg6]));
            this.index = this.index + 1 & 1023;
        }
    }

    public void reset() {
        this.setKey(this.workingKey);
    }

    public byte returnByte(byte arg3) {
        if(this.index == 0) {
            this.isaac();
            this.keyStream = Pack.intToBigEndian(this.results);
        }

        byte v0 = ((byte)(this.keyStream[this.index] ^ arg3));
        this.index = this.index + 1 & 1023;
        return v0;
    }

    private void setKey(byte[] arg12) {
        int v9 = 256;
        int v8 = 8;
        this.workingKey = arg12;
        if(this.engineState == null) {
            this.engineState = new int[v9];
        }

        if(this.results == null) {
            this.results = new int[v9];
        }

        int v0;
        for(v0 = 0; v0 < v9; ++v0) {
            int[] v2 = this.engineState;
            this.results[v0] = 0;
            v2[v0] = 0;
        }

        this.c = 0;
        this.b = 0;
        this.a = 0;
        this.index = 0;
        byte[] v2_1 = new byte[arg12.length + (arg12.length & 3)];
        System.arraycopy(arg12, 0, v2_1, 0, arg12.length);
        for(v0 = 0; v0 < v2_1.length; v0 += 4) {
            this.results[v0 >>> 2] = Pack.littleEndianToInt(v2_1, v0);
        }

        int[] v5 = new int[v8];
        for(v0 = 0; v0 < v8; ++v0) {
            v5[v0] = -1640531527;
        }

        for(v0 = 0; v0 < 4; ++v0) {
            this.mix(v5);
        }

        int v4;
        for(v4 = 0; v4 < 2; ++v4) {
            int v3;
            for(v3 = 0; v3 < v9; v3 += 8) {
                int v2_2;
                for(v2_2 = 0; v2_2 < v8; ++v2_2) {
                    int v6 = v5[v2_2];
                    v0 = v4 < 1 ? this.results[v3 + v2_2] : this.engineState[v3 + v2_2];
                    v5[v2_2] = v0 + v6;
                }

                this.mix(v5);
                for(v0 = 0; v0 < v8; ++v0) {
                    this.engineState[v3 + v0] = v5[v0];
                }
            }
        }

        this.isaac();
        this.initialised = true;
    }
}

