// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import java.math.BigInteger;
import java.util.Vector;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.params.NaccacheSternKeyParameters;
import org.bouncycastle.crypto.params.NaccacheSternPrivateKeyParameters;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.util.Arrays;

public class NaccacheSternEngine implements AsymmetricBlockCipher {
    private static BigInteger ONE;
    private static BigInteger ZERO;
    private boolean debug;
    private boolean forEncryption;
    private NaccacheSternKeyParameters key;
    private Vector[] lookup;

    static {
        NaccacheSternEngine.ZERO = BigInteger.valueOf(0);
        NaccacheSternEngine.ONE = BigInteger.valueOf(1);
    }

    public NaccacheSternEngine() {
        super();
        this.lookup = null;
        this.debug = false;
    }

    public byte[] addCryptedBlocks(byte[] arg8, byte[] arg9) {
        if(this.forEncryption) {
            if(arg8.length <= this.getOutputBlockSize() && arg9.length <= this.getOutputBlockSize()) {
                goto label_24;
            }

            throw new InvalidCipherTextException("BlockLength too large for simple addition.\n");
        }
        else {
            if(arg8.length <= this.getInputBlockSize() && arg9.length <= this.getInputBlockSize()) {
                goto label_24;
            }

            goto label_20;
        }

    label_24:
        BigInteger v0 = new BigInteger(1, arg8);
        BigInteger v1 = new BigInteger(1, arg9);
        BigInteger v2 = v0.multiply(v1).mod(this.key.getModulus());
        if(this.debug) {
            System.out.println("c(m1) as BigInteger:....... " + v0);
            System.out.println("c(m2) as BigInteger:....... " + v1);
            System.out.println("c(m1)*c(m2)%n = c(m1+m2)%n: " + v2);
        }

        byte[] v0_1 = this.key.getModulus().toByteArray();
        Arrays.fill(v0_1, 0);
        System.arraycopy(v2.toByteArray(), 0, v0_1, v0_1.length - v2.toByteArray().length, v2.toByteArray().length);
        return v0_1;
    label_20:
        throw new InvalidCipherTextException("BlockLength too large for simple addition.\n");
    }

    private static BigInteger chineseRemainder(Vector arg5, Vector arg6) {
        int v2 = 0;
        BigInteger v4 = NaccacheSternEngine.ZERO;
        int v1 = 0;
        BigInteger v3 = NaccacheSternEngine.ONE;
        while(v1 < arg6.size()) {
            v3 = v3.multiply(arg6.elementAt(v1));
            ++v1;
        }

        BigInteger v1_1;
        for(v1_1 = v4; v2 < arg6.size(); v1_1 = v0_1) {
            Object v0 = arg6.elementAt(v2);
            v4 = v3.divide(((BigInteger)v0));
            BigInteger v0_1 = v1_1.add(v4.multiply(v4.modInverse(((BigInteger)v0))).multiply(arg5.elementAt(v2)));
            ++v2;
        }

        return v1_1.mod(v3);
    }

    public byte[] encrypt(BigInteger arg6) {
        byte[] v0 = this.key.getModulus().toByteArray();
        Arrays.fill(v0, 0);
        byte[] v1 = this.key.getG().modPow(arg6, this.key.getModulus()).toByteArray();
        System.arraycopy(v1, 0, v0, v0.length - v1.length, v1.length);
        if(this.debug) {
            System.out.println("Encrypted value is:  " + new BigInteger(v0));
        }

        return v0;
    }

    public int getInputBlockSize() {
        int v0 = this.forEncryption ? (this.key.getLowerSigmaBound() + 7) / 8 - 1 : this.key.getModulus().toByteArray().length;
        return v0;
    }

    public int getOutputBlockSize() {
        int v0 = this.forEncryption ? this.key.getModulus().toByteArray().length : (this.key.getLowerSigmaBound() + 7) / 8 - 1;
        return v0;
    }

    public void init(boolean arg12, CipherParameters arg13) {
        this.forEncryption = arg12;
        CipherParameters v0 = (arg13 instanceof ParametersWithRandom) ? ((ParametersWithRandom)arg13).getParameters() : arg13;
        this.key = ((NaccacheSternKeyParameters)v0);
        if(!this.forEncryption) {
            if(this.debug) {
                System.out.println("Constructing lookup Array");
            }

            NaccacheSternKeyParameters v0_1 = this.key;
            Vector v5 = ((NaccacheSternPrivateKeyParameters)v0_1).getSmallPrimes();
            this.lookup = new Vector[v5.size()];
            int v2;
            for(v2 = 0; v2 < v5.size(); ++v2) {
                Object v1 = v5.elementAt(v2);
                int v6 = ((BigInteger)v1).intValue();
                this.lookup[v2] = new Vector();
                this.lookup[v2].addElement(NaccacheSternEngine.ONE);
                if(this.debug) {
                    System.out.println("Constructing lookup ArrayList for " + v6);
                }

                BigInteger v4 = NaccacheSternEngine.ZERO;
                int v3;
                for(v3 = 1; v3 < v6; ++v3) {
                    v4 = v4.add(((NaccacheSternPrivateKeyParameters)v0_1).getPhi_n());
                    this.lookup[v2].addElement(((NaccacheSternPrivateKeyParameters)v0_1).getG().modPow(v4.divide(((BigInteger)v1)), ((NaccacheSternPrivateKeyParameters)v0_1).getModulus()));
                }
            }
        }
    }

    public byte[] processBlock(byte[] arg11, int arg12, int arg13) {
        int v2;
        byte[] v0;
        int v3 = 0;
        if(this.key == null) {
            throw new IllegalStateException("NaccacheStern engine not initialised");
        }

        if(arg13 > this.getInputBlockSize() + 1) {
            throw new DataLengthException("input too large for Naccache-Stern cipher.\n");
        }

        if(!this.forEncryption && arg13 < this.getInputBlockSize()) {
            throw new InvalidCipherTextException("BlockLength does not match modulus for Naccache-Stern cipher.\n");
        }

        if(arg12 != 0 || arg13 != arg11.length) {
            v0 = new byte[arg13];
            System.arraycopy(arg11, arg12, v0, 0, arg13);
            arg11 = v0;
        }

        BigInteger v4 = new BigInteger(1, arg11);
        if(this.debug) {
            System.out.println("input as BigInteger: " + v4);
        }

        if(this.forEncryption) {
            v0 = this.encrypt(v4);
        }
        else {
            Vector v5 = new Vector();
            NaccacheSternKeyParameters v0_1 = this.key;
            Vector v6 = ((NaccacheSternPrivateKeyParameters)v0_1).getSmallPrimes();
            v2 = 0;
            while(true) {
                if(v2 < v6.size()) {
                    BigInteger v7 = v4.modPow(((NaccacheSternPrivateKeyParameters)v0_1).getPhi_n().divide(v6.elementAt(v2)), ((NaccacheSternPrivateKeyParameters)v0_1).getModulus());
                    Vector v8 = this.lookup[v2];
                    if(this.lookup[v2].size() != v6.elementAt(v2).intValue()) {
                        if(this.debug) {
                            System.out.println("Prime is " + v6.elementAt(v2) + ", lookup table has size " + v8.size());
                        }

                        throw new InvalidCipherTextException("Error in lookup Array for " + v6.elementAt(v2).intValue() + ": Size mismatch. Expected ArrayList with length " + v6.elementAt(v2).intValue() + " but found ArrayList of length " + this.lookup[v2].size());
                    }
                    else {
                        int v1 = v8.indexOf(v7);
                        if(v1 != -1) {
                            goto label_156;
                        }

                        if(this.debug) {
                            System.out.println("Actual prime is " + v6.elementAt(v2));
                            System.out.println("Decrypted value is " + v7);
                            System.out.println("LookupList for " + v6.elementAt(v2) + " with size " + this.lookup[v2].size() + " is: ");
                            goto label_141;
                        label_156:
                            v5.addElement(BigInteger.valueOf(((long)v1)));
                            ++v2;
                            continue;
                        }

                        goto label_152;
                    }
                }
                else {
                    break;
                }

                return v0;
            }

            v0 = NaccacheSternEngine.chineseRemainder(v5, v6).toByteArray();
        }

        return v0;
    label_141:
        while(v3 < this.lookup[v2].size()) {
            System.out.println(this.lookup[v2].elementAt(v3));
            ++v3;
        }

    label_152:
        throw new InvalidCipherTextException("Lookup failed");
    }

    public byte[] processData(byte[] arg10) {
        byte[] v0_1;
        byte[] v3;
        if(this.debug) {
            System.out.println();
        }

        if(arg10.length > this.getInputBlockSize()) {
            int v4 = this.getInputBlockSize();
            int v0 = this.getOutputBlockSize();
            if(this.debug) {
                System.out.println("Input blocksize is:  " + v4 + " bytes");
                System.out.println("Output blocksize is: " + v0 + " bytes");
                System.out.println("Data has length:.... " + arg10.length + " bytes");
            }

            byte[] v5 = new byte[v0 * (arg10.length / v4 + 1)];
            v0 = 0;
            int v2 = 0;
            while(true) {
                if(v2 >= arg10.length) {
                    goto label_89;
                }

                if(v2 + v4 < arg10.length) {
                    v3 = this.processBlock(arg10, v2, v4);
                    v2 += v4;
                }
                else {
                    v3 = this.processBlock(arg10, v2, arg10.length - v2);
                    v2 += arg10.length - v2;
                }

                if(this.debug) {
                    System.out.println("new datapos is " + v2);
                }

                if(v3 == null) {
                    break;
                }

                System.arraycopy(v3, 0, v5, v0, v3.length);
                v0 += v3.length;
            }

            if(this.debug) {
                System.out.println("cipher returned null");
            }

            throw new InvalidCipherTextException("cipher returned null");
        label_89:
            byte[] v2_1 = new byte[v0];
            System.arraycopy(v5, 0, v2_1, 0, v0);
            if(this.debug) {
                System.out.println("returning " + v2_1.length + " bytes");
            }

            v0_1 = v2_1;
        }
        else {
            if(this.debug) {
                System.out.println("data size is less then input block size, processing directly");
            }

            v0_1 = this.processBlock(arg10, 0, arg10.length);
        }

        return v0_1;
    }

    public void setDebug(boolean arg1) {
        this.debug = arg1;
    }
}

