// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.params.KeyParameter;

public class NoekeonEngine implements BlockCipher {
    private boolean _forEncryption;
    private boolean _initialised;
    private int[] decryptKeys;
    private static final int genericSize = 16;
    private static final int[] nullVector;
    private static final int[] roundConstants;
    private int[] state;
    private int[] subKeys;

    static {
        NoekeonEngine.nullVector = new int[]{0, 0, 0, 0};
        NoekeonEngine.roundConstants = new int[]{128, 27, 54, 108, 216, 171, 77, 154, 47, 94, 188, 99, 198, 151, 53, 106, 212};
    }

    public NoekeonEngine() {
        super();
        this.state = new int[4];
        this.subKeys = new int[4];
        this.decryptKeys = new int[4];
        this._initialised = false;
    }

    private int bytesToIntBig(byte[] arg4, int arg5) {
        int v0 = arg5 + 1;
        int v2 = v0 + 1;
        return (arg4[v0] & 255) << 16 | arg4[arg5] << 24 | (arg4[v2] & 255) << 8 | arg4[v2 + 1] & 255;
    }

    private int decryptBlock(byte[] arg10, int arg11, byte[] arg12, int arg13) {
        int v1 = 16;
        int v8 = 3;
        int v7 = 2;
        this.state[0] = this.bytesToIntBig(arg10, arg11);
        this.state[1] = this.bytesToIntBig(arg10, arg11 + 4);
        this.state[v7] = this.bytesToIntBig(arg10, arg11 + 8);
        this.state[v8] = this.bytesToIntBig(arg10, arg11 + 12);
        System.arraycopy(this.subKeys, 0, this.decryptKeys, 0, this.subKeys.length);
        this.theta(this.decryptKeys, NoekeonEngine.nullVector);
        int v0;
        for(v0 = v1; v0 > 0; --v0) {
            this.theta(this.state, this.decryptKeys);
            this.state[0] ^= NoekeonEngine.roundConstants[v0];
            this.pi1(this.state);
            this.gamma(this.state);
            this.pi2(this.state);
        }

        this.theta(this.state, this.decryptKeys);
        this.state[0] ^= NoekeonEngine.roundConstants[v0];
        this.intToBytesBig(this.state[0], arg12, arg13);
        this.intToBytesBig(this.state[1], arg12, arg13 + 4);
        this.intToBytesBig(this.state[v7], arg12, arg13 + 8);
        this.intToBytesBig(this.state[v8], arg12, arg13 + 12);
        return v1;
    }

    private int encryptBlock(byte[] arg10, int arg11, byte[] arg12, int arg13) {
        int v8 = 16;
        int v7 = 3;
        int v6 = 2;
        this.state[0] = this.bytesToIntBig(arg10, arg11);
        this.state[1] = this.bytesToIntBig(arg10, arg11 + 4);
        this.state[v6] = this.bytesToIntBig(arg10, arg11 + 8);
        this.state[v7] = this.bytesToIntBig(arg10, arg11 + 12);
        int v0;
        for(v0 = 0; v0 < v8; ++v0) {
            this.state[0] ^= NoekeonEngine.roundConstants[v0];
            this.theta(this.state, this.subKeys);
            this.pi1(this.state);
            this.gamma(this.state);
            this.pi2(this.state);
        }

        this.state[0] ^= NoekeonEngine.roundConstants[v0];
        this.theta(this.state, this.subKeys);
        this.intToBytesBig(this.state[0], arg12, arg13);
        this.intToBytesBig(this.state[1], arg12, arg13 + 4);
        this.intToBytesBig(this.state[v6], arg12, arg13 + 8);
        this.intToBytesBig(this.state[v7], arg12, arg13 + 12);
        return v8;
    }

    private void gamma(int[] arg8) {
        arg8[1] ^= (arg8[3] ^ -1) & (arg8[2] ^ -1);
        arg8[0] ^= arg8[2] & arg8[1];
        int v0 = arg8[3];
        arg8[3] = arg8[0];
        arg8[0] = v0;
        arg8[2] ^= arg8[0] ^ arg8[1] ^ arg8[3];
        arg8[1] ^= (arg8[3] ^ -1) & (arg8[2] ^ -1);
        arg8[0] ^= arg8[2] & arg8[1];
    }

    public String getAlgorithmName() {
        return "Noekeon";
    }

    public int getBlockSize() {
        return 16;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to Noekeon init - " + arg5.getClass().getName());
        }

        this._forEncryption = arg4;
        this._initialised = true;
        this.setKey(((KeyParameter)arg5).getKey());
    }

    private void intToBytesBig(int arg4, byte[] arg5, int arg6) {
        int v0 = arg6 + 1;
        arg5[arg6] = ((byte)(arg4 >>> 24));
        int v1 = v0 + 1;
        arg5[v0] = ((byte)(arg4 >>> 16));
        arg5[v1] = ((byte)(arg4 >>> 8));
        arg5[v1 + 1] = ((byte)arg4);
    }

    private void pi1(int[] arg5) {
        arg5[1] = this.rotl(arg5[1], 1);
        arg5[2] = this.rotl(arg5[2], 5);
        arg5[3] = this.rotl(arg5[3], 2);
    }

    private void pi2(int[] arg6) {
        arg6[1] = this.rotl(arg6[1], 31);
        arg6[2] = this.rotl(arg6[2], 27);
        arg6[3] = this.rotl(arg6[3], 30);
    }

    public int processBlock(byte[] arg4, int arg5, byte[] arg6, int arg7) {
        if(!this._initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg5 + 16 > arg4.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg7 + 16 > arg6.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0 = this._forEncryption ? this.encryptBlock(arg4, arg5, arg6, arg7) : this.decryptBlock(arg4, arg5, arg6, arg7);
        return v0;
    }

    public void reset() {
    }

    private int rotl(int arg3, int arg4) {
        return arg3 << arg4 | arg3 >>> 32 - arg4;
    }

    private void setKey(byte[] arg4) {
        this.subKeys[0] = this.bytesToIntBig(arg4, 0);
        this.subKeys[1] = this.bytesToIntBig(arg4, 4);
        this.subKeys[2] = this.bytesToIntBig(arg4, 8);
        this.subKeys[3] = this.bytesToIntBig(arg4, 12);
    }

    private void theta(int[] arg9, int[] arg10) {
        int v7 = 8;
        int v6 = 3;
        int v5 = 2;
        int v0 = arg9[0] ^ arg9[v5];
        v0 ^= this.rotl(v0, v7) ^ this.rotl(v0, 24);
        arg9[1] ^= v0;
        arg9[v6] ^= v0;
        for(v0 = 0; v0 < 4; ++v0) {
            arg9[v0] ^= arg10[v0];
        }

        v0 = arg9[1] ^ arg9[v6];
        v0 ^= this.rotl(v0, v7) ^ this.rotl(v0, 24);
        arg9[0] ^= v0;
        arg9[v5] ^= v0;
    }
}

