// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import java.security.SecureRandom;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.Wrapper;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.util.Arrays;

public class RC2WrapEngine implements Wrapper {
    private static final byte[] IV2;
    byte[] digest;
    private CBCBlockCipher engine;
    private boolean forWrapping;
    private byte[] iv;
    private CipherParameters param;
    private ParametersWithIV paramPlusIV;
    Digest sha1;
    private SecureRandom sr;

    static {
        RC2WrapEngine.IV2 = new byte[]{74, -35, -94, 44, 121, -24, 33, 5};
    }

    public RC2WrapEngine() {
        super();
        this.sha1 = new SHA1Digest();
        this.digest = new byte[20];
    }

    private byte[] calculateCMSKeyChecksum(byte[] arg6) {
        byte[] v0 = new byte[8];
        this.sha1.update(arg6, 0, arg6.length);
        this.sha1.doFinal(this.digest, 0);
        System.arraycopy(this.digest, 0, v0, 0, 8);
        return v0;
    }

    private boolean checkCMSKeyChecksum(byte[] arg2, byte[] arg3) {
        return Arrays.constantTimeAreEqual(this.calculateCMSKeyChecksum(arg2), arg3);
    }

    public String getAlgorithmName() {
        return "RC2";
    }

    public void init(boolean arg4, CipherParameters arg5) {
        CipherParameters v0;
        int v2 = 8;
        this.forWrapping = arg4;
        this.engine = new CBCBlockCipher(new RC2Engine());
        if((arg5 instanceof ParametersWithRandom)) {
            this.sr = ((ParametersWithRandom)arg5).getRandom();
            v0 = ((ParametersWithRandom)arg5).getParameters();
        }
        else {
            this.sr = new SecureRandom();
            v0 = arg5;
        }

        if((v0 instanceof ParametersWithIV)) {
            this.paramPlusIV = ((ParametersWithIV)v0);
            this.iv = this.paramPlusIV.getIV();
            this.param = this.paramPlusIV.getParameters();
            if(this.forWrapping) {
                if(this.iv != null && this.iv.length == v2) {
                    return;
                }

                throw new IllegalArgumentException("IV is not 8 octets");
            }
            else {
                throw new IllegalArgumentException("You should not supply an IV for unwrapping");
            }
        }
        else {
            this.param = v0;
            if(!this.forWrapping) {
                return;
            }

            this.iv = new byte[v2];
            this.sr.nextBytes(this.iv);
            this.paramPlusIV = new ParametersWithIV(this.param, this.iv);
        }
    }

    public byte[] unwrap(byte[] arg8, int arg9, int arg10) {
        int v3;
        int v6 = 8;
        if(this.forWrapping) {
            throw new IllegalStateException("Not set for unwrapping");
        }

        if(arg8 == null) {
            throw new InvalidCipherTextException("Null pointer as ciphertext");
        }

        if(arg10 % this.engine.getBlockSize() != 0) {
            throw new InvalidCipherTextException("Ciphertext not multiple of " + this.engine.getBlockSize());
        }

        this.engine.init(false, new ParametersWithIV(this.param, RC2WrapEngine.IV2));
        byte[] v2 = new byte[arg10];
        System.arraycopy(arg8, arg9, v2, 0, arg10);
        int v0;
        for(v0 = 0; v0 < v2.length / this.engine.getBlockSize(); ++v0) {
            v3 = this.engine.getBlockSize() * v0;
            this.engine.processBlock(v2, v3, v2, v3);
        }

        byte[] v3_1 = new byte[v2.length];
        for(v0 = 0; v0 < v2.length; ++v0) {
            v3_1[v0] = v2[v2.length - (v0 + 1)];
        }

        this.iv = new byte[v6];
        byte[] v0_1 = new byte[v3_1.length - 8];
        System.arraycopy(v3_1, 0, this.iv, 0, v6);
        System.arraycopy(v3_1, v6, v0_1, 0, v3_1.length - 8);
        this.paramPlusIV = new ParametersWithIV(this.param, this.iv);
        this.engine.init(false, this.paramPlusIV);
        v2 = new byte[v0_1.length];
        System.arraycopy(v0_1, 0, v2, 0, v0_1.length);
        for(v0 = 0; v0 < v2.length / this.engine.getBlockSize(); ++v0) {
            v3 = this.engine.getBlockSize() * v0;
            this.engine.processBlock(v2, v3, v2, v3);
        }

        v0_1 = new byte[v2.length - 8];
        v3_1 = new byte[v6];
        System.arraycopy(v2, 0, v0_1, 0, v2.length - 8);
        System.arraycopy(v2, v2.length - 8, v3_1, 0, v6);
        if(!this.checkCMSKeyChecksum(v0_1, v3_1)) {
            throw new InvalidCipherTextException("Checksum inside ciphertext is corrupted");
        }

        if(v0_1.length - ((v0_1[0] & 255) + 1) > 7) {
            throw new InvalidCipherTextException("too many pad bytes (" + (v0_1.length - ((v0_1[0] & 255) + 1)) + ")");
        }

        v2 = new byte[v0_1[0]];
        System.arraycopy(v0_1, 1, v2, 0, v2.length);
        return v2;
    }

    public byte[] wrap(byte[] arg9, int arg10, int arg11) {
        int v1 = 0;
        if(!this.forWrapping) {
            throw new IllegalStateException("Not initialized for wrapping");
        }

        int v0 = arg11 + 1;
        if(v0 % 8 != 0) {
            v0 += 8 - v0 % 8;
        }

        byte[] v0_1 = new byte[v0];
        v0_1[0] = ((byte)arg11);
        System.arraycopy(arg9, arg10, v0_1, 1, arg11);
        byte[] v2 = new byte[v0_1.length - arg11 - 1];
        if(v2.length > 0) {
            this.sr.nextBytes(v2);
            System.arraycopy(v2, 0, v0_1, arg11 + 1, v2.length);
        }

        v2 = this.calculateCMSKeyChecksum(v0_1);
        byte[] v3 = new byte[v0_1.length + v2.length];
        System.arraycopy(v0_1, 0, v3, 0, v0_1.length);
        System.arraycopy(v2, 0, v3, v0_1.length, v2.length);
        v2 = new byte[v3.length];
        System.arraycopy(v3, 0, v2, 0, v3.length);
        int v4 = v3.length / this.engine.getBlockSize();
        if(v3.length % this.engine.getBlockSize() != 0) {
            throw new IllegalStateException("Not multiple of block length");
        }

        this.engine.init(true, this.paramPlusIV);
        for(v0 = 0; v0 < v4; ++v0) {
            int v3_1 = this.engine.getBlockSize() * v0;
            this.engine.processBlock(v2, v3_1, v2, v3_1);
        }

        v3 = new byte[this.iv.length + v2.length];
        System.arraycopy(this.iv, 0, v3, 0, this.iv.length);
        System.arraycopy(v2, 0, v3, this.iv.length, v2.length);
        v2 = new byte[v3.length];
        for(v0 = 0; v0 < v3.length; ++v0) {
            v2[v0] = v3[v3.length - (v0 + 1)];
        }

        this.engine.init(true, new ParametersWithIV(this.param, RC2WrapEngine.IV2));
        while(v1 < v4 + 1) {
            v0 = this.engine.getBlockSize() * v1;
            this.engine.processBlock(v2, v0, v2, v0);
            ++v1;
        }

        return v2;
    }
}

