// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.RC5Parameters;

public class RC532Engine implements BlockCipher {
    private static final int P32 = -1209970333;
    private static final int Q32 = -1640531527;
    private int[] _S;
    private int _noRounds;
    private boolean forEncryption;

    public RC532Engine() {
        super();
        this._noRounds = 12;
        this._S = null;
    }

    private int bytesToWord(byte[] arg3, int arg4) {
        return arg3[arg4] & 255 | (arg3[arg4 + 1] & 255) << 8 | (arg3[arg4 + 2] & 255) << 16 | (arg3[arg4 + 3] & 255) << 24;
    }

    private int decryptBlock(byte[] arg7, int arg8, byte[] arg9, int arg10) {
        int v2 = this.bytesToWord(arg7, arg8);
        int v1 = this.bytesToWord(arg7, arg8 + 4);
        int v0;
        for(v0 = this._noRounds; v0 >= 1; --v0) {
            v1 = this.rotateRight(v1 - this._S[v0 * 2 + 1], v2) ^ v2;
            v2 = this.rotateRight(v2 - this._S[v0 * 2], v1) ^ v1;
        }

        this.wordToBytes(v2 - this._S[0], arg9, arg10);
        this.wordToBytes(v1 - this._S[1], arg9, arg10 + 4);
        return 8;
    }

    private int encryptBlock(byte[] arg6, int arg7, byte[] arg8, int arg9) {
        int v0 = 1;
        int v2 = this._S[0] + this.bytesToWord(arg6, arg7);
        int v1 = this.bytesToWord(arg6, arg7 + 4) + this._S[1];
        while(v0 <= this._noRounds) {
            v2 = this.rotateLeft(v2 ^ v1, v1) + this._S[v0 * 2];
            v1 = this.rotateLeft(v1 ^ v2, v2) + this._S[v0 * 2 + 1];
            ++v0;
        }

        this.wordToBytes(v2, arg8, arg9);
        this.wordToBytes(v1, arg8, arg9 + 4);
        return 8;
    }

    public String getAlgorithmName() {
        return "RC5-32";
    }

    public int getBlockSize() {
        return 8;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if((arg5 instanceof RC5Parameters)) {
            this._noRounds = ((RC5Parameters)arg5).getRounds();
            this.setKey(((RC5Parameters)arg5).getKey());
        }
        else if((arg5 instanceof KeyParameter)) {
            this.setKey(((KeyParameter)arg5).getKey());
        }
        else {
            goto label_13;
        }

        this.forEncryption = arg4;
        return;
    label_13:
        throw new IllegalArgumentException("invalid parameter passed to RC532 init - " + arg5.getClass().getName());
    }

    public int processBlock(byte[] arg2, int arg3, byte[] arg4, int arg5) {
        int v0 = this.forEncryption ? this.encryptBlock(arg2, arg3, arg4, arg5) : this.decryptBlock(arg2, arg3, arg4, arg5);
        return v0;
    }

    public void reset() {
    }

    private int rotateLeft(int arg3, int arg4) {
        return arg3 << (arg4 & 31) | arg3 >>> 32 - (arg4 & 31);
    }

    private int rotateRight(int arg3, int arg4) {
        return arg3 >>> (arg4 & 31) | arg3 << 32 - (arg4 & 31);
    }

    private void setKey(byte[] arg10) {
        int v2;
        int v1 = 0;
        int[] v6 = new int[(arg10.length + 3) / 4];
        int v0;
        for(v0 = 0; v0 != arg10.length; ++v0) {
            v2 = v0 / 4;
            v6[v2] += (arg10[v0] & 255) << v0 % 4 * 8;
        }

        this._S = new int[(this._noRounds + 1) * 2];
        this._S[0] = -1209970333;
        for(v0 = 1; v0 < this._S.length; ++v0) {
            this._S[v0] = this._S[v0 - 1] - 1640531527;
        }

        v0 = v6.length > this._S.length ? v6.length * 3 : this._S.length * 3;
        v2 = 0;
        int v3 = 0;
        int v4 = 0;
        int v5 = 0;
        while(v1 < v0) {
            int[] v7 = this._S;
            v5 = this.rotateLeft(v5 + this._S[v3] + v4, 3);
            v7[v3] = v5;
            v4 = this.rotateLeft(v6[v2] + v5 + v4, v4 + v5);
            v6[v2] = v4;
            v3 = (v3 + 1) % this._S.length;
            v2 = (v2 + 1) % v6.length;
            ++v1;
        }
    }

    private void wordToBytes(int arg3, byte[] arg4, int arg5) {
        arg4[arg5] = ((byte)arg3);
        arg4[arg5 + 1] = ((byte)(arg3 >> 8));
        arg4[arg5 + 2] = ((byte)(arg3 >> 16));
        arg4[arg5 + 3] = ((byte)(arg3 >> 24));
    }
}

