// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.params.RC5Parameters;

public class RC564Engine implements BlockCipher {
    private static final long P64 = -5196783011329398165L;
    private static final long Q64 = -7046029254386353131L;
    private long[] _S;
    private int _noRounds;
    private static final int bytesPerWord = 8;
    private boolean forEncryption;
    private static final int wordSize = 64;

    public RC564Engine() {
        super();
        this._noRounds = 12;
        this._S = null;
    }

    private long bytesToWord(byte[] arg6, int arg7) {
        long v1 = 0;
        int v0;
        for(v0 = 7; v0 >= 0; --v0) {
            v1 = (v1 << 8) + (((long)(arg6[v0 + arg7] & 255)));
        }

        return v1;
    }

    private int decryptBlock(byte[] arg9, int arg10, byte[] arg11, int arg12) {
        long v3 = this.bytesToWord(arg9, arg10);
        long v1 = this.bytesToWord(arg9, arg10 + 8);
        int v0;
        for(v0 = this._noRounds; v0 >= 1; --v0) {
            v1 = this.rotateRight(v1 - this._S[v0 * 2 + 1], v3) ^ v3;
            v3 = this.rotateRight(v3 - this._S[v0 * 2], v1) ^ v1;
        }

        this.wordToBytes(v3 - this._S[0], arg11, arg12);
        this.wordToBytes(v1 - this._S[1], arg11, arg12 + 8);
        return 16;
    }

    private int encryptBlock(byte[] arg8, int arg9, byte[] arg10, int arg11) {
        int v0 = 1;
        long v3 = this._S[0] + this.bytesToWord(arg8, arg9);
        long v1 = this.bytesToWord(arg8, arg9 + 8) + this._S[1];
        while(v0 <= this._noRounds) {
            v3 = this.rotateLeft(v3 ^ v1, v1) + this._S[v0 * 2];
            v1 = this.rotateLeft(v1 ^ v3, v3) + this._S[v0 * 2 + 1];
            ++v0;
        }

        this.wordToBytes(v3, arg10, arg11);
        this.wordToBytes(v1, arg10, arg11 + 8);
        return 16;
    }

    public String getAlgorithmName() {
        return "RC5-64";
    }

    public int getBlockSize() {
        return 16;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof RC5Parameters)) {
            throw new IllegalArgumentException("invalid parameter passed to RC564 init - " + arg5.getClass().getName());
        }

        this.forEncryption = arg4;
        this._noRounds = ((RC5Parameters)arg5).getRounds();
        this.setKey(((RC5Parameters)arg5).getKey());
    }

    public int processBlock(byte[] arg2, int arg3, byte[] arg4, int arg5) {
        int v0 = this.forEncryption ? this.encryptBlock(arg2, arg3, arg4, arg5) : this.decryptBlock(arg2, arg3, arg4, arg5);
        return v0;
    }

    public void reset() {
    }

    private long rotateLeft(long arg7, long arg9) {
        return arg7 << (((int)(arg9 & 63))) | arg7 >>> (((int)(64 - (63 & arg9))));
    }

    private long rotateRight(long arg7, long arg9) {
        return arg7 >>> (((int)(arg9 & 63))) | arg7 << (((int)(64 - (63 & arg9))));
    }

    private void setKey(byte[] arg13) {
        long v2 = 0;
        int v1 = 0;
        long[] v8 = new long[(arg13.length + 7) / 8];
        int v0;
        for(v0 = 0; v0 != arg13.length; ++v0) {
            int v4 = v0 / 8;
            v8[v4] += (((long)(arg13[v0] & 255))) << v0 % 8 * 8;
        }

        this._S = new long[(this._noRounds + 1) * 2];
        this._S[0] = -5196783011329398165L;
        for(v0 = 1; v0 < this._S.length; ++v0) {
            this._S[v0] = this._S[v0 - 1] - 7046029254386353131L;
        }

        v0 = v8.length > this._S.length ? v8.length * 3 : this._S.length * 3;
        long v4_1 = v2;
        long v6 = v2;
        int v2_1 = 0;
        int v3 = 0;
        while(v1 < v0) {
            long[] v9 = this._S;
            v6 = this.rotateLeft(v6 + this._S[v3] + v4_1, 3);
            v9[v3] = v6;
            v4_1 = this.rotateLeft(v8[v2_1] + v6 + v4_1, v4_1 + v6);
            v8[v2_1] = v4_1;
            v3 = (v3 + 1) % this._S.length;
            v2_1 = (v2_1 + 1) % v8.length;
            ++v1;
        }
    }

    private void wordToBytes(long arg5, byte[] arg7, int arg8) {
        int v3 = 8;
        int v0;
        for(v0 = 0; v0 < v3; ++v0) {
            arg7[v0 + arg8] = ((byte)(((int)arg5)));
            arg5 >>>= v3;
        }
    }
}

