// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.params.KeyParameter;

public class RC6Engine implements BlockCipher {
    private static final int LGW = 5;
    private static final int P32 = -1209970333;
    private static final int Q32 = -1640531527;
    private int[] _S;
    private static final int _noRounds = 20;
    private static final int bytesPerWord = 4;
    private boolean forEncryption;
    private static final int wordSize = 32;

    public RC6Engine() {
        super();
        this._S = null;
    }

    private int bytesToWord(byte[] arg4, int arg5) {
        int v1 = 0;
        int v0;
        for(v0 = 3; v0 >= 0; --v0) {
            v1 = (v1 << 8) + (arg4[v0 + arg5] & 255);
        }

        return v1;
    }

    private int decryptBlock(byte[] arg11, int arg12, byte[] arg13, int arg14) {
        int v0 = this.bytesToWord(arg11, arg12);
        int v3 = this.bytesToWord(arg11, arg12 + 4);
        int v2 = this.bytesToWord(arg11, arg12 + 8);
        int v1 = this.bytesToWord(arg11, arg12 + 12);
        v2 -= this._S[43];
        int v4 = v0 - this._S[42];
        v0 = 20;
        while(v0 >= 1) {
            int v5 = this.rotateLeft((v4 * 2 + 1) * v4, 5);
            int v6 = this.rotateLeft((v2 * 2 + 1) * v2, 5);
            v3 = this.rotateRight(v3 - this._S[v0 * 2 + 1], v5) ^ v6;
            v1 = this.rotateRight(v1 - this._S[v0 * 2], v6) ^ v5;
            --v0;
            int v9 = v2;
            v2 = v3;
            v3 = v4;
            v4 = v1;
            v1 = v9;
        }

        v0 = v1 - this._S[1];
        v1 = v3 - this._S[0];
        this.wordToBytes(v4, arg13, arg14);
        this.wordToBytes(v1, arg13, arg14 + 4);
        this.wordToBytes(v2, arg13, arg14 + 8);
        this.wordToBytes(v0, arg13, arg14 + 12);
        return 16;
    }

    private int encryptBlock(byte[] arg12, int arg13, byte[] arg14, int arg15) {
        int v9 = 5;
        int v0 = 1;
        int v4 = this.bytesToWord(arg12, arg13);
        int v1 = this.bytesToWord(arg12, arg13 + 4);
        int v2 = this.bytesToWord(arg12, arg13 + 8);
        int v5 = this.bytesToWord(arg12, arg13 + 12);
        int v3 = this._S[0] + v1;
        v1 = this._S[1] + v5;
        int v10 = v2;
        v2 = v4;
        v4 = v3;
        v3 = v10;
        while(v0 <= 20) {
            v5 = this.rotateLeft((v4 * 2 + 1) * v4, v9);
            int v6 = this.rotateLeft((v1 * 2 + 1) * v1, v9);
            v2 = this.rotateLeft(v2 ^ v5, v6) + this._S[v0 * 2];
            v3 = this.rotateLeft(v3 ^ v6, v5) + this._S[v0 * 2 + 1];
            ++v0;
            v10 = v2;
            v2 = v4;
            v4 = v3;
            v3 = v1;
            v1 = v10;
        }

        v0 = this._S[42] + v2;
        v2 = this._S[43] + v3;
        this.wordToBytes(v0, arg14, arg15);
        this.wordToBytes(v4, arg14, arg15 + 4);
        this.wordToBytes(v2, arg14, arg15 + 8);
        this.wordToBytes(v1, arg14, arg15 + 12);
        return 16;
    }

    public String getAlgorithmName() {
        return "RC6";
    }

    public int getBlockSize() {
        return 16;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to RC6 init - " + arg5.getClass().getName());
        }

        this.forEncryption = arg4;
        this.setKey(((KeyParameter)arg5).getKey());
    }

    public int processBlock(byte[] arg4, int arg5, byte[] arg6, int arg7) {
        int v0 = this.getBlockSize();
        if(this._S == null) {
            throw new IllegalStateException("RC6 engine not initialised");
        }

        if(arg5 + v0 > arg4.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(v0 + arg7 > arg6.length) {
            throw new OutputLengthException("output buffer too short");
        }

        return this.forEncryption ? this.encryptBlock(arg4, arg5, arg6, arg7) : this.decryptBlock(arg4, arg5, arg6, arg7);
    }

    public void reset() {
    }

    private int rotateLeft(int arg3, int arg4) {
        return arg3 << arg4 | arg3 >>> -arg4;
    }

    private int rotateRight(int arg3, int arg4) {
        return arg3 >>> arg4 | arg3 << -arg4;
    }

    private void setKey(byte[] arg10) {
        int v1 = 0;
        int[] v6 = new int[(arg10.length + 3) / 4];
        int v0;
        for(v0 = arg10.length - 1; v0 >= 0; --v0) {
            v6[v0 / 4] = (v6[v0 / 4] << 8) + (arg10[v0] & 255);
        }

        this._S = new int[44];
        this._S[0] = -1209970333;
        for(v0 = 1; v0 < this._S.length; ++v0) {
            this._S[v0] = this._S[v0 - 1] - 1640531527;
        }

        v0 = v6.length > this._S.length ? v6.length * 3 : this._S.length * 3;
        int v2 = 0;
        int v3 = 0;
        int v4 = 0;
        int v5 = 0;
        while(v1 < v0) {
            int[] v7 = this._S;
            v5 = this.rotateLeft(v5 + this._S[v3] + v4, 3);
            v7[v3] = v5;
            v4 = this.rotateLeft(v6[v2] + v5 + v4, v4 + v5);
            v6[v2] = v4;
            v3 = (v3 + 1) % this._S.length;
            v2 = (v2 + 1) % v6.length;
            ++v1;
        }
    }

    private void wordToBytes(int arg4, byte[] arg5, int arg6) {
        int v0;
        for(v0 = 0; v0 < 4; ++v0) {
            arg5[v0 + arg6] = ((byte)arg4);
            arg4 >>>= 8;
        }
    }
}

