// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import java.security.SecureRandom;
import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.Wrapper;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.crypto.params.ParametersWithRandom;

public class RFC3211WrapEngine implements Wrapper {
    private CBCBlockCipher engine;
    private boolean forWrapping;
    private ParametersWithIV param;
    private SecureRandom rand;

    public RFC3211WrapEngine(BlockCipher arg2) {
        super();
        this.engine = new CBCBlockCipher(arg2);
    }

    public String getAlgorithmName() {
        return this.engine.getUnderlyingCipher().getAlgorithmName() + "/RFC3211Wrap";
    }

    public void init(boolean arg2, CipherParameters arg3) {
        this.forWrapping = arg2;
        if((arg3 instanceof ParametersWithRandom)) {
            this.rand = ((ParametersWithRandom)arg3).getRandom();
            this.param = ((ParametersWithRandom)arg3).getParameters();
        }
        else {
            if(arg2) {
                this.rand = new SecureRandom();
            }

            this.param = ((ParametersWithIV)arg3);
        }
    }

    public byte[] unwrap(byte[] arg8, int arg9, int arg10) {
        int v2 = 0;
        if(this.forWrapping) {
            throw new IllegalStateException("not set for unwrapping");
        }

        int v1 = this.engine.getBlockSize();
        if(arg10 < v1 * 2) {
            throw new InvalidCipherTextException("input too short");
        }

        byte[] v3 = new byte[arg10];
        byte[] v4 = new byte[v1];
        System.arraycopy(arg8, arg9, v3, 0, arg10);
        System.arraycopy(arg8, arg9, v4, 0, v4.length);
        this.engine.init(false, new ParametersWithIV(this.param.getParameters(), v4));
        int v0;
        for(v0 = v1; v0 < v3.length; v0 += v1) {
            this.engine.processBlock(v3, v0, v3, v0);
        }

        System.arraycopy(v3, v3.length - v4.length, v4, 0, v4.length);
        this.engine.init(false, new ParametersWithIV(this.param.getParameters(), v4));
        this.engine.processBlock(v3, 0, v3, 0);
        this.engine.init(false, this.param);
        for(v0 = 0; v0 < v3.length; v0 += v1) {
            this.engine.processBlock(v3, v0, v3, v0);
        }

        if((v3[0] & 255) > v3.length - 4) {
            throw new InvalidCipherTextException("wrapped key corrupted");
        }

        byte[] v1_1 = new byte[v3[0] & 255];
        System.arraycopy(v3, 4, v1_1, 0, v3[0]);
        v0 = 0;
        while(v2 != 3) {
            v0 |= (((byte)(v3[v2 + 1] ^ -1))) ^ v1_1[v2];
            ++v2;
        }

        if(v0 != 0) {
            throw new InvalidCipherTextException("wrapped key fails checksum");
        }

        return v1_1;
    }

    public byte[] wrap(byte[] arg6, int arg7, int arg8) {
        byte[] v0;
        int v2 = 0;
        if(!this.forWrapping) {
            throw new IllegalStateException("not set for wrapping");
        }

        this.engine.init(true, this.param);
        int v3 = this.engine.getBlockSize();
        if(arg8 + 4 < v3 * 2) {
            v0 = new byte[v3 * 2];
        }
        else {
            int v0_1 = (arg8 + 4) % v3 == 0 ? arg8 + 4 : ((arg8 + 4) / v3 + 1) * v3;
            v0 = new byte[v0_1];
        }

        v0[0] = ((byte)arg8);
        v0[1] = ((byte)(arg6[arg7] ^ -1));
        v0[2] = ((byte)(arg6[arg7 + 1] ^ -1));
        v0[3] = ((byte)(arg6[arg7 + 2] ^ -1));
        System.arraycopy(arg6, arg7, v0, 4, arg8);
        int v1;
        for(v1 = arg8 + 4; v1 < v0.length; ++v1) {
            v0[v1] = ((byte)this.rand.nextInt());
        }

        for(v1 = 0; v1 < v0.length; v1 += v3) {
            this.engine.processBlock(v0, v1, v0, v1);
        }

        while(v2 < v0.length) {
            this.engine.processBlock(v0, v2, v0, v2);
            v2 += v3;
        }

        return v0;
    }
}

