// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.MaxBytesExceededException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.StreamCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.crypto.util.Pack;
import org.bouncycastle.util.Strings;

public class Salsa20Engine implements StreamCipher {
    private static final int STATE_SIZE = 16;
    private int cW0;
    private int cW1;
    private int cW2;
    private int[] engineState;
    private int index;
    private boolean initialised;
    private byte[] keyStream;
    private static final byte[] sigma;
    private static final byte[] tau;
    private byte[] workingIV;
    private byte[] workingKey;
    private int[] x;

    static {
        Salsa20Engine.sigma = Strings.toByteArray("expand 32-byte k");
        Salsa20Engine.tau = Strings.toByteArray("expand 16-byte k");
    }

    public Salsa20Engine() {
        super();
        this.index = 0;
        this.engineState = new int[16];
        this.x = new int[16];
        this.keyStream = new byte[64];
        this.workingKey = null;
        this.workingIV = null;
        this.initialised = false;
    }

    private void generateKeyStream(byte[] arg4) {
        Salsa20Engine.salsaCore(20, this.engineState, this.x);
        Pack.intToLittleEndian(this.x, arg4, 0);
    }

    public String getAlgorithmName() {
        return "Salsa20";
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof ParametersWithIV)) {
            throw new IllegalArgumentException("Salsa20 Init parameters must include an IV");
        }

        byte[] v1 = ((ParametersWithIV)arg5).getIV();
        if(v1 != null && v1.length == 8) {
            if(!(((ParametersWithIV)arg5).getParameters() instanceof KeyParameter)) {
                throw new IllegalArgumentException("Salsa20 Init parameters must include a key");
            }
            else {
                this.workingKey = ((ParametersWithIV)arg5).getParameters().getKey();
                this.workingIV = v1;
                this.setKey(this.workingKey, this.workingIV);
                return;
            }
        }

        throw new IllegalArgumentException("Salsa20 requires exactly 8 bytes of IV");
    }

    private boolean limitExceeded() {
        boolean v0 = false;
        int v1 = this.cW0 + 1;
        this.cW0 = v1;
        if(v1 == 0) {
            v1 = this.cW1 + 1;
            this.cW1 = v1;
            if(v1 == 0) {
                v1 = this.cW2 + 1;
                this.cW2 = v1;
                if((v1 & 32) != 0) {
                    v0 = true;
                }
            }
        }

        return v0;
    }

    private boolean limitExceeded(int arg3) {
        boolean v0 = false;
        this.cW0 += arg3;
        if(this.cW0 < arg3 && this.cW0 >= 0) {
            int v1 = this.cW1 + 1;
            this.cW1 = v1;
            if(v1 == 0) {
                v1 = this.cW2 + 1;
                this.cW2 = v1;
                if((v1 & 32) != 0) {
                    v0 = true;
                }
            }
        }

        return v0;
    }

    public void processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        if(!this.initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg6 + arg7 > arg5.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg9 + arg7 > arg8.length) {
            throw new OutputLengthException("output buffer too short");
        }

        if(this.limitExceeded(arg7)) {
            throw new MaxBytesExceededException("2^70 byte limit per IV would be exceeded; Change IV");
        }

        int v0;
        for(v0 = 0; v0 < arg7; ++v0) {
            if(this.index == 0) {
                this.generateKeyStream(this.keyStream);
                int[] v1 = this.engineState;
                int v3 = v1[8] + 1;
                v1[8] = v3;
                if(v3 == 0) {
                    ++this.engineState[9];
                }
            }

            arg8[v0 + arg9] = ((byte)(this.keyStream[this.index] ^ arg5[v0 + arg6]));
            this.index = this.index + 1 & 63;
        }
    }

    public void reset() {
        this.setKey(this.workingKey, this.workingIV);
    }

    private void resetCounter() {
        this.cW0 = 0;
        this.cW1 = 0;
        this.cW2 = 0;
    }

    public byte returnByte(byte arg4) {
        if(this.limitExceeded()) {
            throw new MaxBytesExceededException("2^70 byte limit per IV; Change IV");
        }

        if(this.index == 0) {
            this.generateKeyStream(this.keyStream);
            int[] v0 = this.engineState;
            int v2 = v0[8] + 1;
            v0[8] = v2;
            if(v2 == 0) {
                ++this.engineState[9];
            }
        }

        byte v0_1 = ((byte)(this.keyStream[this.index] ^ arg4));
        this.index = this.index + 1 & 63;
        return v0_1;
    }

    private static int rotl(int arg2, int arg3) {
        return arg2 << arg3 | arg2 >>> -arg3;
    }

    public static void salsaCore(int arg9, int[] arg10, int[] arg11) {
        int v8 = 18;
        int v0 = 0;
        int v7 = 13;
        int v6 = 9;
        int v5 = 7;
        System.arraycopy(arg10, 0, arg11, 0, arg10.length);
        while(arg9 > 0) {
            arg11[4] ^= Salsa20Engine.rotl(arg11[0] + arg11[12], v5);
            arg11[8] ^= Salsa20Engine.rotl(arg11[4] + arg11[0], v6);
            arg11[12] ^= Salsa20Engine.rotl(arg11[8] + arg11[4], v7);
            arg11[0] ^= Salsa20Engine.rotl(arg11[12] + arg11[8], v8);
            arg11[v6] ^= Salsa20Engine.rotl(arg11[5] + arg11[1], v5);
            arg11[v7] ^= Salsa20Engine.rotl(arg11[v6] + arg11[5], v6);
            arg11[1] ^= Salsa20Engine.rotl(arg11[v7] + arg11[v6], v7);
            arg11[5] ^= Salsa20Engine.rotl(arg11[1] + arg11[v7], v8);
            arg11[14] ^= Salsa20Engine.rotl(arg11[10] + arg11[6], v5);
            arg11[2] ^= Salsa20Engine.rotl(arg11[14] + arg11[10], v6);
            arg11[6] ^= Salsa20Engine.rotl(arg11[2] + arg11[14], v7);
            arg11[10] ^= Salsa20Engine.rotl(arg11[6] + arg11[2], v8);
            arg11[3] ^= Salsa20Engine.rotl(arg11[15] + arg11[11], v5);
            arg11[v5] ^= Salsa20Engine.rotl(arg11[3] + arg11[15], v6);
            arg11[11] ^= Salsa20Engine.rotl(arg11[v5] + arg11[3], v7);
            arg11[15] ^= Salsa20Engine.rotl(arg11[11] + arg11[v5], v8);
            arg11[1] ^= Salsa20Engine.rotl(arg11[0] + arg11[3], v5);
            arg11[2] ^= Salsa20Engine.rotl(arg11[1] + arg11[0], v6);
            arg11[3] ^= Salsa20Engine.rotl(arg11[2] + arg11[1], v7);
            arg11[0] ^= Salsa20Engine.rotl(arg11[3] + arg11[2], v8);
            arg11[6] ^= Salsa20Engine.rotl(arg11[5] + arg11[4], v5);
            arg11[v5] ^= Salsa20Engine.rotl(arg11[6] + arg11[5], v6);
            arg11[4] ^= Salsa20Engine.rotl(arg11[v5] + arg11[6], v7);
            arg11[5] ^= Salsa20Engine.rotl(arg11[4] + arg11[v5], v8);
            arg11[11] ^= Salsa20Engine.rotl(arg11[10] + arg11[v6], v5);
            arg11[8] ^= Salsa20Engine.rotl(arg11[11] + arg11[10], v6);
            arg11[v6] ^= Salsa20Engine.rotl(arg11[8] + arg11[11], v7);
            arg11[10] ^= Salsa20Engine.rotl(arg11[v6] + arg11[8], v8);
            arg11[12] ^= Salsa20Engine.rotl(arg11[15] + arg11[14], v5);
            arg11[v7] ^= Salsa20Engine.rotl(arg11[12] + arg11[15], v6);
            arg11[14] ^= Salsa20Engine.rotl(arg11[v7] + arg11[12], v7);
            arg11[15] ^= Salsa20Engine.rotl(arg11[14] + arg11[v7], v8);
            arg9 += -2;
        }

        while(v0 < 16) {
            arg11[v0] += arg10[v0];
            ++v0;
        }
    }

    private void setKey(byte[] arg12, byte[] arg13) {
        int v1;
        byte[] v0;
        int v9 = 12;
        int v8 = 8;
        int v7 = 4;
        this.workingKey = arg12;
        this.workingIV = arg13;
        this.index = 0;
        this.resetCounter();
        this.engineState[1] = Pack.littleEndianToInt(this.workingKey, 0);
        this.engineState[2] = Pack.littleEndianToInt(this.workingKey, v7);
        this.engineState[3] = Pack.littleEndianToInt(this.workingKey, v8);
        this.engineState[v7] = Pack.littleEndianToInt(this.workingKey, v9);
        if(this.workingKey.length == 32) {
            v0 = Salsa20Engine.sigma;
            v1 = 16;
        }
        else {
            v0 = Salsa20Engine.tau;
            v1 = 0;
        }

        this.engineState[11] = Pack.littleEndianToInt(this.workingKey, v1);
        this.engineState[v9] = Pack.littleEndianToInt(this.workingKey, v1 + 4);
        this.engineState[13] = Pack.littleEndianToInt(this.workingKey, v1 + 8);
        this.engineState[14] = Pack.littleEndianToInt(this.workingKey, v1 + 12);
        this.engineState[0] = Pack.littleEndianToInt(v0, 0);
        this.engineState[5] = Pack.littleEndianToInt(v0, v7);
        this.engineState[10] = Pack.littleEndianToInt(v0, v8);
        this.engineState[15] = Pack.littleEndianToInt(v0, v9);
        this.engineState[6] = Pack.littleEndianToInt(this.workingIV, 0);
        this.engineState[7] = Pack.littleEndianToInt(this.workingIV, v7);
        int[] v0_1 = this.engineState;
        this.engineState[9] = 0;
        v0_1[v8] = 0;
        this.initialised = true;
    }
}

