// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.engines;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.OutputLengthException;
import org.bouncycastle.crypto.params.KeyParameter;

public class XTEAEngine implements BlockCipher {
    private int[] _S;
    private boolean _forEncryption;
    private boolean _initialised;
    private int[] _sum0;
    private int[] _sum1;
    private static final int block_size = 8;
    private static final int delta = -1640531527;
    private static final int rounds = 32;

    public XTEAEngine() {
        super();
        this._S = new int[4];
        this._sum0 = new int[32];
        this._sum1 = new int[32];
        this._initialised = false;
    }

    private int bytesToInt(byte[] arg4, int arg5) {
        int v0 = arg5 + 1;
        int v2 = v0 + 1;
        return (arg4[v0] & 255) << 16 | arg4[arg5] << 24 | (arg4[v2] & 255) << 8 | arg4[v2 + 1] & 255;
    }

    private int decryptBlock(byte[] arg6, int arg7, byte[] arg8, int arg9) {
        int v2 = this.bytesToInt(arg6, arg7);
        int v1 = this.bytesToInt(arg6, arg7 + 4);
        int v0;
        for(v0 = 31; v0 >= 0; --v0) {
            v1 -= (v2 << 4 ^ v2 >>> 5) + v2 ^ this._sum1[v0];
            v2 -= (v1 << 4 ^ v1 >>> 5) + v1 ^ this._sum0[v0];
        }

        this.unpackInt(v2, arg8, arg9);
        this.unpackInt(v1, arg8, arg9 + 4);
        return 8;
    }

    private int encryptBlock(byte[] arg6, int arg7, byte[] arg8, int arg9) {
        int v2 = this.bytesToInt(arg6, arg7);
        int v1 = this.bytesToInt(arg6, arg7 + 4);
        int v0;
        for(v0 = 0; v0 < 32; ++v0) {
            v2 += (v1 << 4 ^ v1 >>> 5) + v1 ^ this._sum0[v0];
            v1 += (v2 << 4 ^ v2 >>> 5) + v2 ^ this._sum1[v0];
        }

        this.unpackInt(v2, arg8, arg9);
        this.unpackInt(v1, arg8, arg9 + 4);
        return 8;
    }

    public String getAlgorithmName() {
        return "XTEA";
    }

    public int getBlockSize() {
        return 8;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        if(!(arg5 instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to TEA init - " + arg5.getClass().getName());
        }

        this._forEncryption = arg4;
        this._initialised = true;
        this.setKey(((KeyParameter)arg5).getKey());
    }

    public int processBlock(byte[] arg4, int arg5, byte[] arg6, int arg7) {
        if(!this._initialised) {
            throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
        }

        if(arg5 + 8 > arg4.length) {
            throw new DataLengthException("input buffer too short");
        }

        if(arg7 + 8 > arg6.length) {
            throw new OutputLengthException("output buffer too short");
        }

        int v0 = this._forEncryption ? this.encryptBlock(arg4, arg5, arg6, arg7) : this.decryptBlock(arg4, arg5, arg6, arg7);
        return v0;
    }

    public void reset() {
    }

    private void setKey(byte[] arg6) {
        int v0 = 0;
        int v1 = 0;
        int v2 = 0;
        while(v2 < 4) {
            this._S[v2] = this.bytesToInt(arg6, v1);
            ++v2;
            v1 += 4;
        }

        for(v1 = 0; v1 < 32; ++v1) {
            this._sum0[v1] = this._S[v0 & 3] + v0;
            v0 -= 1640531527;
            this._sum1[v1] = this._S[v0 >>> 11 & 3] + v0;
        }
    }

    private void unpackInt(int arg4, byte[] arg5, int arg6) {
        int v0 = arg6 + 1;
        arg5[arg6] = ((byte)(arg4 >>> 24));
        int v1 = v0 + 1;
        arg5[v0] = ((byte)(arg4 >>> 16));
        arg5[v1] = ((byte)(arg4 >>> 8));
        arg5[v1 + 1] = ((byte)arg4);
    }
}

