// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.examples;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.agreement.jpake.JPAKEParticipant;
import org.bouncycastle.crypto.agreement.jpake.JPAKEPrimeOrderGroup;
import org.bouncycastle.crypto.agreement.jpake.JPAKEPrimeOrderGroups;
import org.bouncycastle.crypto.agreement.jpake.JPAKERound1Payload;
import org.bouncycastle.crypto.agreement.jpake.JPAKERound2Payload;
import org.bouncycastle.crypto.agreement.jpake.JPAKERound3Payload;
import org.bouncycastle.crypto.digests.SHA256Digest;

public class JPAKEExample {
    public JPAKEExample() {
        super();
    }

    private static BigInteger deriveSessionKey(BigInteger arg5) {
        SHA256Digest v0 = new SHA256Digest();
        byte[] v1 = arg5.toByteArray();
        byte[] v2 = new byte[v0.getDigestSize()];
        v0.update(v1, 0, v1.length);
        v0.doFinal(v2, 0);
        return new BigInteger(v2);
    }

    public static void main(String[] arg15) {
        JPAKEPrimeOrderGroup v3 = JPAKEPrimeOrderGroups.NIST_3072;
        BigInteger v0 = v3.getP();
        BigInteger v1 = v3.getQ();
        BigInteger v2 = v3.getG();
        System.out.println("********* Initialization **********");
        System.out.println("Public parameters for the cyclic group:");
        System.out.println("p (" + v0.bitLength() + " bits): " + v0.toString(16));
        System.out.println("q (" + v1.bitLength() + " bits): " + v1.toString(16));
        System.out.println("g (" + v0.bitLength() + " bits): " + v2.toString(16));
        System.out.println("p mod q = " + v0.mod(v1).toString(16));
        System.out.println("g^{q} mod p = " + v2.modPow(v1, v0).toString(16));
        System.out.println("");
        System.out.println("(Secret passwords used by Alice and Bob: \"" + "password" + "\" and \"" + "password" + "\")\n");
        SHA256Digest v4 = new SHA256Digest();
        SecureRandom v5 = new SecureRandom();
        JPAKEParticipant v0_1 = new JPAKEParticipant("alice", "password".toCharArray(), v3, ((Digest)v4), v5);
        JPAKEParticipant v6 = new JPAKEParticipant("bob", "password".toCharArray(), v3, v4, v5);
        JPAKERound1Payload v1_1 = v0_1.createRound1PayloadToSend();
        JPAKERound1Payload v2_1 = v6.createRound1PayloadToSend();
        System.out.println("************ Round 1 **************");
        System.out.println("Alice sends to Bob: ");
        System.out.println("g^{x1}=" + v1_1.getGx1().toString(16));
        System.out.println("g^{x2}=" + v1_1.getGx2().toString(16));
        System.out.println("KP{x1}={" + v1_1.getKnowledgeProofForX1()[0].toString(16) + "};{" + v1_1.getKnowledgeProofForX1()[1].toString(16) + "}");
        System.out.println("KP{x2}={" + v1_1.getKnowledgeProofForX2()[0].toString(16) + "};{" + v1_1.getKnowledgeProofForX2()[1].toString(16) + "}");
        System.out.println("");
        System.out.println("Bob sends to Alice: ");
        System.out.println("g^{x3}=" + v2_1.getGx1().toString(16));
        System.out.println("g^{x4}=" + v2_1.getGx2().toString(16));
        System.out.println("KP{x3}={" + v2_1.getKnowledgeProofForX1()[0].toString(16) + "};{" + v2_1.getKnowledgeProofForX1()[1].toString(16) + "}");
        System.out.println("KP{x4}={" + v2_1.getKnowledgeProofForX2()[0].toString(16) + "};{" + v2_1.getKnowledgeProofForX2()[1].toString(16) + "}");
        System.out.println("");
        v0_1.validateRound1PayloadReceived(v2_1);
        System.out.println("Alice checks g^{x4}!=1: OK");
        System.out.println("Alice checks KP{x3}: OK");
        System.out.println("Alice checks KP{x4}: OK");
        System.out.println("");
        v6.validateRound1PayloadReceived(v1_1);
        System.out.println("Bob checks g^{x2}!=1: OK");
        System.out.println("Bob checks KP{x1},: OK");
        System.out.println("Bob checks KP{x2},: OK");
        System.out.println("");
        JPAKERound2Payload v1_2 = v0_1.createRound2PayloadToSend();
        JPAKERound2Payload v2_2 = v6.createRound2PayloadToSend();
        System.out.println("************ Round 2 **************");
        System.out.println("Alice sends to Bob: ");
        System.out.println("A=" + v1_2.getA().toString(16));
        System.out.println("KP{x2*s}={" + v1_2.getKnowledgeProofForX2s()[0].toString(16) + "},{" + v1_2.getKnowledgeProofForX2s()[1].toString(16) + "}");
        System.out.println("");
        System.out.println("Bob sends to Alice");
        System.out.println("B=" + v2_2.getA().toString(16));
        System.out.println("KP{x4*s}={" + v2_2.getKnowledgeProofForX2s()[0].toString(16) + "},{" + v2_2.getKnowledgeProofForX2s()[1].toString(16) + "}");
        System.out.println("");
        v0_1.validateRound2PayloadReceived(v2_2);
        System.out.println("Alice checks KP{x4*s}: OK\n");
        v6.validateRound2PayloadReceived(v1_2);
        System.out.println("Bob checks KP{x2*s}: OK\n");
        v1 = v0_1.calculateKeyingMaterial();
        v2 = v6.calculateKeyingMaterial();
        System.out.println("********* After round 2 ***********");
        System.out.println("Alice computes key material \t K=" + v1.toString(16));
        System.out.println("Bob computes key material \t K=" + v2.toString(16));
        System.out.println();
        JPAKEExample.deriveSessionKey(v1);
        JPAKEExample.deriveSessionKey(v2);
        JPAKERound3Payload v3_1 = v0_1.createRound3PayloadToSend(v1);
        JPAKERound3Payload v4_1 = v6.createRound3PayloadToSend(v2);
        System.out.println("************ Round 3 **************");
        System.out.println("Alice sends to Bob: ");
        System.out.println("MacTag=" + v3_1.getMacTag().toString(16));
        System.out.println("");
        System.out.println("Bob sends to Alice: ");
        System.out.println("MacTag=" + v4_1.getMacTag().toString(16));
        System.out.println("");
        v0_1.validateRound3PayloadReceived(v4_1, v1);
        System.out.println("Alice checks MacTag: OK\n");
        v6.validateRound3PayloadReceived(v3_1, v2);
        System.out.println("Bob checks MacTag: OK\n");
        System.out.println();
        System.out.println("MacTags validated, therefore the keying material matches.");
    }
}

