// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.generators;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.params.DSAParameterGenerationParameters;
import org.bouncycastle.crypto.params.DSAParameters;
import org.bouncycastle.crypto.params.DSAValidationParameters;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;

public class DSAParametersGenerator {
    private int L;
    private int N;
    private static final BigInteger ONE;
    private static final BigInteger TWO;
    private static final BigInteger ZERO;
    private int certainty;
    private Digest digest;
    private SecureRandom random;
    private int usageIndex;
    private boolean use186_3;

    static {
        DSAParametersGenerator.ZERO = BigInteger.valueOf(0);
        DSAParametersGenerator.ONE = BigInteger.valueOf(1);
        DSAParametersGenerator.TWO = BigInteger.valueOf(2);
    }

    public DSAParametersGenerator() {
        this(new SHA1Digest());
    }

    public DSAParametersGenerator(Digest arg1) {
        super();
        this.digest = arg1;
    }

    private static BigInteger calculateGenerator_FIPS186_2(BigInteger arg5, BigInteger arg6, SecureRandom arg7) {
        BigInteger v2;
        BigInteger v0 = arg5.subtract(DSAParametersGenerator.ONE).divide(arg6);
        BigInteger v1 = arg5.subtract(DSAParametersGenerator.TWO);
        do {
            v2 = BigIntegers.createRandomInRange(DSAParametersGenerator.TWO, v1, arg7).modPow(v0, arg5);
        }
        while(v2.bitLength() <= 1);

        return v2;
    }

    private static BigInteger calculateGenerator_FIPS186_3_Unverifiable(BigInteger arg1, BigInteger arg2, SecureRandom arg3) {
        return DSAParametersGenerator.calculateGenerator_FIPS186_2(arg1, arg2, arg3);
    }

    private static BigInteger calculateGenerator_FIPS186_3_Verifiable(Digest arg7, BigInteger arg8, BigInteger arg9, byte[] arg10, int arg11) {
        BigInteger v0_1;
        BigInteger v3 = arg8.subtract(DSAParametersGenerator.ONE).divide(arg9);
        byte[] v0 = Hex.decode("6767656E");
        byte[] v4 = new byte[arg10.length + v0.length + 3];
        System.arraycopy(arg10, 0, v4, 0, arg10.length);
        System.arraycopy(v0, 0, v4, arg10.length, v0.length);
        v4[v4.length - 3] = ((byte)arg11);
        byte[] v5 = new byte[arg7.getDigestSize()];
        int v1 = 1;
        while(true) {
            if(v1 < 65536) {
                DSAParametersGenerator.inc(v4);
                DSAParametersGenerator.hash(arg7, v4, v5);
                v0_1 = new BigInteger(1, v5).modPow(v3, arg8);
                if(v0_1.compareTo(DSAParametersGenerator.TWO) < 0) {
                    ++v1;
                    continue;
                }
            }
            else {
                return null;
            }

            return v0_1;
        }

        return null;
    }

    public DSAParameters generateParameters() {
        DSAParameters v0 = this.use186_3 ? this.generateParameters_FIPS186_3() : this.generateParameters_FIPS186_2();
        return v0;
    }

    private DSAParameters generateParameters_FIPS186_2() {
        BigInteger v9;
        byte[] v3 = new byte[20];
        byte[] v4 = new byte[20];
        byte[] v5 = new byte[20];
        byte[] v6 = new byte[20];
        int v7 = (this.L - 1) / 160;
        byte[] v8 = new byte[this.L / 8];
        if(!(this.digest instanceof SHA1Digest)) {
            throw new IllegalStateException("can only use SHA-1 for generating FIPS 186-2 parameters");
        }

        do {
        label_34:
            this.random.nextBytes(v3);
            DSAParametersGenerator.hash(this.digest, v3, v4);
            System.arraycopy(v3, 0, v5, 0, v3.length);
            DSAParametersGenerator.inc(v5);
            DSAParametersGenerator.hash(this.digest, v5, v5);
            int v0;
            for(v0 = 0; v0 != v6.length; ++v0) {
                v6[v0] = ((byte)(v4[v0] ^ v5[v0]));
            }

            v6[0] = ((byte)(v6[0] | -128));
            v6[19] = ((byte)(v6[19] | 1));
            v9 = new BigInteger(1, v6);
            if(v9.isProbablePrime(this.certainty)) {
                break;
            }
        }
        while(true);

        byte[] v10 = Arrays.clone(v3);
        DSAParametersGenerator.inc(v10);
        int v2;
        for(v2 = 0; true; ++v2) {
            if(v2 >= 4096) {
                goto label_34;
            }

            for(v0 = 0; v0 < v7; ++v0) {
                DSAParametersGenerator.inc(v10);
                DSAParametersGenerator.hash(this.digest, v10, v4);
                System.arraycopy(v4, 0, v8, v8.length - (v0 + 1) * v4.length, v4.length);
            }

            DSAParametersGenerator.inc(v10);
            DSAParametersGenerator.hash(this.digest, v10, v4);
            System.arraycopy(v4, v4.length - (v8.length - v4.length * v7), v8, 0, v8.length - v4.length * v7);
            v8[0] = ((byte)(v8[0] | -128));
            BigInteger v0_1 = new BigInteger(1, v8);
            v0_1 = v0_1.subtract(v0_1.mod(v9.shiftLeft(1)).subtract(DSAParametersGenerator.ONE));
            if(v0_1.bitLength() == this.L && (v0_1.isProbablePrime(this.certainty))) {
                return new DSAParameters(v0_1, v9, DSAParametersGenerator.calculateGenerator_FIPS186_2(v0_1, v9, this.random), new DSAValidationParameters(v3, v2));
            }
        }
    }

    private DSAParameters generateParameters_FIPS186_3() {
        DSAParameters v0_1;
        BigInteger v11;
        BigInteger v0;
        Digest v5 = this.digest;
        int v6 = v5.getDigestSize() * 8;
        byte[] v7 = new byte[this.N / 8];
        int v8 = (this.L - 1) / v6;
        int v9 = (this.L - 1) % v6;
        byte[] v10 = new byte[v5.getDigestSize()];
        do {
        label_14:
            this.random.nextBytes(v7);
            DSAParametersGenerator.hash(v5, v7, v10);
            v0 = new BigInteger(1, v10).mod(DSAParametersGenerator.ONE.shiftLeft(this.N - 1));
            v11 = DSAParametersGenerator.ONE.shiftLeft(this.N - 1).add(v0).add(DSAParametersGenerator.ONE).subtract(v0.mod(DSAParametersGenerator.TWO));
        }
        while(!v11.isProbablePrime(this.certainty));

        byte[] v12 = Arrays.clone(v7);
        int v13 = this.L * 4;
        int v4;
        for(v4 = 0; true; ++v4) {
            if(v4 >= v13) {
                goto label_14;
            }

            BigInteger v3 = DSAParametersGenerator.ZERO;
            int v2 = 0;
            int v1;
            for(v1 = 0; v2 <= v8; v1 += v6) {
                DSAParametersGenerator.inc(v12);
                DSAParametersGenerator.hash(v5, v12, v10);
                v0 = new BigInteger(1, v10);
                if(v2 == v8) {
                    v0 = v0.mod(DSAParametersGenerator.ONE.shiftLeft(v9));
                }

                v3 = v3.add(v0.shiftLeft(v1));
                ++v2;
            }

            v0 = v3.add(DSAParametersGenerator.ONE.shiftLeft(this.L - 1));
            BigInteger v1_1 = v0.subtract(v0.mod(v11.shiftLeft(1)).subtract(DSAParametersGenerator.ONE));
            if(v1_1.bitLength() == this.L && (v1_1.isProbablePrime(this.certainty))) {
                if(this.usageIndex >= 0) {
                    BigInteger v2_1 = DSAParametersGenerator.calculateGenerator_FIPS186_3_Verifiable(v5, v1_1, v11, v7, this.usageIndex);
                    if(v2_1 != null) {
                        v0_1 = new DSAParameters(v1_1, v11, v2_1, new DSAValidationParameters(v7, v4, this.usageIndex));
                    }
                    else {
                        goto label_97;
                    }
                }
                else {
                label_97:
                    v0_1 = new DSAParameters(v1_1, v11, DSAParametersGenerator.calculateGenerator_FIPS186_3_Unverifiable(v1_1, v11, this.random), new DSAValidationParameters(v7, v4));
                }

                return v0_1;
            }
        }
    }

    private static int getDefaultN(int arg1) {
        int v0 = arg1 > 1024 ? 256 : 160;
        return v0;
    }

    private static void hash(Digest arg2, byte[] arg3, byte[] arg4) {
        arg2.update(arg3, 0, arg3.length);
        arg2.doFinal(arg4, 0);
    }

    private static void inc(byte[] arg2) {
        int v0;
        for(v0 = arg2.length - 1; v0 >= 0; --v0) {
            byte v1 = ((byte)(arg2[v0] + 1 & 255));
            arg2[v0] = v1;
            if(v1 != 0) {
                return;
            }
        }
    }

    public void init(int arg2, int arg3, SecureRandom arg4) {
        this.use186_3 = false;
        this.L = arg2;
        this.N = DSAParametersGenerator.getDefaultN(arg2);
        this.certainty = arg3;
        this.random = arg4;
    }

    public void init(DSAParameterGenerationParameters arg5) {
        int v3 = 3072;
        int v1 = 1024;
        int v2 = 256;
        this.use186_3 = true;
        this.L = arg5.getL();
        this.N = arg5.getN();
        this.certainty = arg5.getCertainty();
        this.random = arg5.getRandom();
        this.usageIndex = arg5.getUsageIndex();
        if(this.L >= v1 && this.L <= v3 && this.L % 1024 == 0) {
            if(this.L == v1 && this.N != 160) {
                throw new IllegalArgumentException("N must be 160 for L = 1024");
            }

            if(this.L == 2048 && this.N != 224 && this.N != v2) {
                throw new IllegalArgumentException("N must be 224 or 256 for L = 2048");
            }

            if(this.L == v3 && this.N != v2) {
                throw new IllegalArgumentException("N must be 256 for L = 3072");
            }

            if(this.digest.getDigestSize() * 8 < this.N) {
                throw new IllegalStateException("Digest output size too small for value of N");
            }

            return;
        }

        throw new IllegalArgumentException("L values must be between 1024 and 3072 and a multiple of 1024");
    }
}

