// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.generators;

import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.DerivationFunction;
import org.bouncycastle.crypto.DerivationParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.params.HKDFParameters;
import org.bouncycastle.crypto.params.KeyParameter;

public class HKDFBytesGenerator implements DerivationFunction {
    private byte[] currentT;
    private int generatedBytes;
    private HMac hMacHash;
    private int hashLen;
    private byte[] info;

    public HKDFBytesGenerator(Digest arg2) {
        super();
        this.hMacHash = new HMac(arg2);
        this.hashLen = arg2.getDigestSize();
    }

    private void expandNext() {
        int v0 = this.generatedBytes / this.hashLen + 1;
        if(v0 >= 256) {
            throw new DataLengthException("HKDF cannot generate more than 255 blocks of HashLen size");
        }

        if(this.generatedBytes != 0) {
            this.hMacHash.update(this.currentT, 0, this.hashLen);
        }

        this.hMacHash.update(this.info, 0, this.info.length);
        this.hMacHash.update(((byte)v0));
        this.hMacHash.doFinal(this.currentT, 0);
    }

    private KeyParameter extract(byte[] arg5, byte[] arg6) {
        this.hMacHash.init(new KeyParameter(arg6));
        if(arg5 == null) {
            this.hMacHash.init(new KeyParameter(new byte[this.hashLen]));
        }
        else {
            this.hMacHash.init(new KeyParameter(arg5));
        }

        this.hMacHash.update(arg6, 0, arg6.length);
        byte[] v0 = new byte[this.hashLen];
        this.hMacHash.doFinal(v0, 0);
        return new KeyParameter(v0);
    }

    public int generateBytes(byte[] arg6, int arg7, int arg8) {
        if(this.generatedBytes + arg8 > this.hashLen * 255) {
            throw new DataLengthException("HKDF may only be used for 255 * HashLen bytes of output");
        }

        if(this.generatedBytes % this.hashLen == 0) {
            this.expandNext();
        }

        int v0 = this.generatedBytes % this.hashLen;
        int v1 = Math.min(this.hashLen - this.generatedBytes % this.hashLen, arg8);
        System.arraycopy(this.currentT, v0, arg6, arg7, v1);
        this.generatedBytes += v1;
        v0 = arg8 - v1;
        v1 += arg7;
        while(v0 > 0) {
            this.expandNext();
            int v2 = Math.min(this.hashLen, v0);
            System.arraycopy(this.currentT, 0, arg6, v1, v2);
            this.generatedBytes += v2;
            v0 -= v2;
            v1 += v2;
        }

        return arg8;
    }

    public Digest getDigest() {
        return this.hMacHash.getUnderlyingDigest();
    }

    public void init(DerivationParameters arg4) {
        if(!(arg4 instanceof HKDFParameters)) {
            throw new IllegalArgumentException("HKDF parameters required for HKDFBytesGenerator");
        }

        if(((HKDFParameters)arg4).skipExtract()) {
            this.hMacHash.init(new KeyParameter(((HKDFParameters)arg4).getIKM()));
        }
        else {
            this.hMacHash.init(this.extract(((HKDFParameters)arg4).getSalt(), ((HKDFParameters)arg4).getIKM()));
        }

        this.info = ((HKDFParameters)arg4).getInfo();
        this.generatedBytes = 0;
        this.currentT = new byte[this.hashLen];
    }
}

