// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.macs;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Mac;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.paddings.ISO7816d4Padding;

public class CMac implements Mac {
    private static final byte CONSTANT_128 = -121;
    private static final byte CONSTANT_64 = 27;
    private byte[] L;
    private byte[] Lu;
    private byte[] Lu2;
    private byte[] ZEROES;
    private byte[] buf;
    private int bufOff;
    private BlockCipher cipher;
    private byte[] mac;
    private int macSize;

    public CMac(BlockCipher arg2) {
        this(arg2, arg2.getBlockSize() * 8);
    }

    public CMac(BlockCipher arg4, int arg5) {
        super();
        if(arg5 % 8 != 0) {
            throw new IllegalArgumentException("MAC size must be multiple of 8");
        }

        if(arg5 > arg4.getBlockSize() * 8) {
            throw new IllegalArgumentException("MAC size must be less or equal to " + arg4.getBlockSize() * 8);
        }

        if(arg4.getBlockSize() != 8 && arg4.getBlockSize() != 16) {
            throw new IllegalArgumentException("Block size must be either 64 or 128 bits");
        }

        this.cipher = new CBCBlockCipher(arg4);
        this.macSize = arg5 / 8;
        this.mac = new byte[arg4.getBlockSize()];
        this.buf = new byte[arg4.getBlockSize()];
        this.ZEROES = new byte[arg4.getBlockSize()];
        this.bufOff = 0;
    }

    public int doFinal(byte[] arg7, int arg8) {
        byte[] v0;
        if(this.bufOff == this.cipher.getBlockSize()) {
            v0 = this.Lu;
        }
        else {
            new ISO7816d4Padding().addPadding(this.buf, this.bufOff);
            v0 = this.Lu2;
        }

        int v1;
        for(v1 = 0; v1 < this.mac.length; ++v1) {
            this.buf[v1] = ((byte)(this.buf[v1] ^ v0[v1]));
        }

        this.cipher.processBlock(this.buf, 0, this.mac, 0);
        System.arraycopy(this.mac, 0, arg7, arg8, this.macSize);
        this.reset();
        return this.macSize;
    }

    private static byte[] doubleLu(byte[] arg5) {
        int v0 = 0;
        int v1 = (arg5[0] & 255) >> 7;
        byte[] v2 = new byte[arg5.length];
        while(v0 < arg5.length - 1) {
            v2[v0] = ((byte)((arg5[v0] << 1) + ((arg5[v0 + 1] & 255) >> 7)));
            ++v0;
        }

        v2[arg5.length - 1] = ((byte)(arg5[arg5.length - 1] << 1));
        if(v1 == 1) {
            v1 = arg5.length - 1;
            int v3 = v2[v1];
            v0 = arg5.length == 16 ? -121 : 27;
            v2[v1] = ((byte)(v0 ^ v3));
        }

        return v2;
    }

    public String getAlgorithmName() {
        return this.cipher.getAlgorithmName();
    }

    public int getMacSize() {
        return this.macSize;
    }

    public void init(CipherParameters arg5) {
        if(arg5 != null) {
            this.cipher.init(true, arg5);
            this.L = new byte[this.ZEROES.length];
            this.cipher.processBlock(this.ZEROES, 0, this.L, 0);
            this.Lu = CMac.doubleLu(this.L);
            this.Lu2 = CMac.doubleLu(this.Lu);
        }

        this.reset();
    }

    public void reset() {
        int v0;
        for(v0 = 0; v0 < this.buf.length; ++v0) {
            this.buf[v0] = 0;
        }

        this.bufOff = 0;
        this.cipher.reset();
    }

    public void update(byte arg5) {
        if(this.bufOff == this.buf.length) {
            this.cipher.processBlock(this.buf, 0, this.mac, 0);
            this.bufOff = 0;
        }

        byte[] v0 = this.buf;
        int v1 = this.bufOff;
        this.bufOff = v1 + 1;
        v0[v1] = arg5;
    }

    public void update(byte[] arg7, int arg8, int arg9) {
        if(arg9 < 0) {
            throw new IllegalArgumentException("Can\'t have a negative input length!");
        }

        int v0 = this.cipher.getBlockSize();
        int v1 = v0 - this.bufOff;
        if(arg9 > v1) {
            System.arraycopy(arg7, arg8, this.buf, this.bufOff, v1);
            this.cipher.processBlock(this.buf, 0, this.mac, 0);
            this.bufOff = 0;
            arg9 -= v1;
            arg8 += v1;
            while(arg9 > v0) {
                this.cipher.processBlock(arg7, arg8, this.mac, 0);
                arg9 -= v0;
                arg8 += v0;
            }
        }

        System.arraycopy(arg7, arg8, this.buf, this.bufOff, arg9);
        this.bufOff += arg9;
    }
}

