// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.macs;

import java.util.Hashtable;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.ExtendedDigest;
import org.bouncycastle.crypto.Mac;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.util.Integers;
import org.bouncycastle.util.Memoable;

public class HMac implements Mac {
    private static final byte IPAD = 54;
    private static final byte OPAD = 92;
    private int blockLength;
    private static Hashtable blockLengths;
    private Digest digest;
    private int digestSize;
    private byte[] inputPad;
    private Memoable ipadState;
    private Memoable opadState;
    private byte[] outputBuf;

    static {
        HMac.blockLengths = new Hashtable();
        HMac.blockLengths.put("GOST3411", Integers.valueOf(32));
        HMac.blockLengths.put("MD2", Integers.valueOf(16));
        HMac.blockLengths.put("MD4", Integers.valueOf(64));
        HMac.blockLengths.put("MD5", Integers.valueOf(64));
        HMac.blockLengths.put("RIPEMD128", Integers.valueOf(64));
        HMac.blockLengths.put("RIPEMD160", Integers.valueOf(64));
        HMac.blockLengths.put("SHA-1", Integers.valueOf(64));
        HMac.blockLengths.put("SHA-224", Integers.valueOf(64));
        HMac.blockLengths.put("SHA-256", Integers.valueOf(64));
        HMac.blockLengths.put("SHA-384", Integers.valueOf(128));
        HMac.blockLengths.put("SHA-512", Integers.valueOf(128));
        HMac.blockLengths.put("Tiger", Integers.valueOf(64));
        HMac.blockLengths.put("Whirlpool", Integers.valueOf(64));
    }

    public HMac(Digest arg2) {
        this(arg2, HMac.getByteLength(arg2));
    }

    private HMac(Digest arg3, int arg4) {
        super();
        this.digest = arg3;
        this.digestSize = arg3.getDigestSize();
        this.blockLength = arg4;
        this.inputPad = new byte[this.blockLength];
        this.outputBuf = new byte[this.blockLength + this.digestSize];
    }

    public int doFinal(byte[] arg6, int arg7) {
        this.digest.doFinal(this.outputBuf, this.blockLength);
        if(this.opadState != null) {
            this.digest.reset(this.opadState);
            this.digest.update(this.outputBuf, this.blockLength, this.digest.getDigestSize());
        }
        else {
            this.digest.update(this.outputBuf, 0, this.outputBuf.length);
        }

        int v1 = this.digest.doFinal(arg6, arg7);
        int v0;
        for(v0 = this.blockLength; v0 < this.outputBuf.length; ++v0) {
            this.outputBuf[v0] = 0;
        }

        if(this.ipadState != null) {
            this.digest.reset(this.ipadState);
        }
        else {
            this.digest.update(this.inputPad, 0, this.inputPad.length);
        }

        return v1;
    }

    public String getAlgorithmName() {
        return this.digest.getAlgorithmName() + "/HMAC";
    }

    private static int getByteLength(Digest arg3) {
        int v0;
        if((arg3 instanceof ExtendedDigest)) {
            v0 = ((ExtendedDigest)arg3).getByteLength();
        }
        else {
            Object v0_1 = HMac.blockLengths.get(arg3.getAlgorithmName());
            if(v0_1 == null) {
                throw new IllegalArgumentException("unknown digest passed: " + arg3.getAlgorithmName());
            }
            else {
                v0 = ((Integer)v0_1).intValue();
            }
        }

        return v0;
    }

    public int getMacSize() {
        return this.digestSize;
    }

    public Digest getUnderlyingDigest() {
        return this.digest;
    }

    public void init(CipherParameters arg5) {
        this.digest.reset();
        byte[] v1 = ((KeyParameter)arg5).getKey();
        int v0 = v1.length;
        if(v0 > this.blockLength) {
            this.digest.update(v1, 0, v0);
            this.digest.doFinal(this.inputPad, 0);
            v0 = this.digestSize;
        }
        else {
            System.arraycopy(v1, 0, this.inputPad, 0, v0);
        }

        while(v0 < this.inputPad.length) {
            this.inputPad[v0] = 0;
            ++v0;
        }

        System.arraycopy(this.inputPad, 0, this.outputBuf, 0, this.blockLength);
        HMac.xorPad(this.inputPad, this.blockLength, 54);
        HMac.xorPad(this.outputBuf, this.blockLength, 92);
        if((this.digest instanceof Memoable)) {
            this.opadState = this.digest.copy();
            this.opadState.update(this.outputBuf, 0, this.blockLength);
        }

        this.digest.update(this.inputPad, 0, this.inputPad.length);
        if((this.digest instanceof Memoable)) {
            this.ipadState = this.digest.copy();
        }
    }

    public void reset() {
        this.digest.reset();
        this.digest.update(this.inputPad, 0, this.inputPad.length);
    }

    public void update(byte[] arg2, int arg3, int arg4) {
        this.digest.update(arg2, arg3, arg4);
    }

    public void update(byte arg2) {
        this.digest.update(arg2);
    }

    private static void xorPad(byte[] arg2, int arg3, byte arg4) {
        int v0;
        for(v0 = 0; v0 < arg3; ++v0) {
            arg2[v0] = ((byte)(arg2[v0] ^ arg4));
        }
    }
}

