// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.modes;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.Mac;
import org.bouncycastle.crypto.macs.CMac;
import org.bouncycastle.crypto.params.AEADParameters;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.util.Arrays;

public class EAXBlockCipher implements AEADBlockCipher {
    private byte[] associatedTextMac;
    private int blockSize;
    private byte[] bufBlock;
    private int bufOff;
    private static final byte cTAG = 2;
    private SICBlockCipher cipher;
    private boolean cipherInitialized;
    private boolean forEncryption;
    private static final byte hTAG = 1;
    private byte[] initialAssociatedText;
    private Mac mac;
    private byte[] macBlock;
    private int macSize;
    private static final byte nTAG;
    private byte[] nonceMac;

    public EAXBlockCipher(BlockCipher arg2) {
        super();
        this.blockSize = arg2.getBlockSize();
        this.mac = new CMac(arg2);
        this.macBlock = new byte[this.blockSize];
        this.bufBlock = new byte[this.blockSize * 2];
        this.associatedTextMac = new byte[this.mac.getMacSize()];
        this.nonceMac = new byte[this.mac.getMacSize()];
        this.cipher = new SICBlockCipher(arg2);
    }

    private void calculateMac() {
        int v0 = 0;
        byte[] v1 = new byte[this.blockSize];
        this.mac.doFinal(v1, 0);
        while(v0 < this.macBlock.length) {
            this.macBlock[v0] = ((byte)(this.nonceMac[v0] ^ this.associatedTextMac[v0] ^ v1[v0]));
            ++v0;
        }
    }

    public int doFinal(byte[] arg8, int arg9) {
        this.initCipher();
        int v0 = this.bufOff;
        byte[] v1 = new byte[this.bufBlock.length];
        this.bufOff = 0;
        if(this.forEncryption) {
            this.cipher.processBlock(this.bufBlock, 0, v1, 0);
            this.cipher.processBlock(this.bufBlock, this.blockSize, v1, this.blockSize);
            System.arraycopy(v1, 0, arg8, arg9, v0);
            this.mac.update(v1, 0, v0);
            this.calculateMac();
            System.arraycopy(this.macBlock, 0, arg8, arg9 + v0, this.macSize);
            this.reset(false);
            v0 += this.macSize;
        }
        else {
            if(v0 > this.macSize) {
                this.mac.update(this.bufBlock, 0, v0 - this.macSize);
                this.cipher.processBlock(this.bufBlock, 0, v1, 0);
                this.cipher.processBlock(this.bufBlock, this.blockSize, v1, this.blockSize);
                System.arraycopy(v1, 0, arg8, arg9, v0 - this.macSize);
            }

            this.calculateMac();
            if(!this.verifyMac(this.bufBlock, v0 - this.macSize)) {
                throw new InvalidCipherTextException("mac check in EAX failed");
            }

            this.reset(false);
            v0 -= this.macSize;
        }

        return v0;
    }

    public String getAlgorithmName() {
        return this.cipher.getUnderlyingCipher().getAlgorithmName() + "/EAX";
    }

    public int getBlockSize() {
        return this.cipher.getBlockSize();
    }

    public byte[] getMac() {
        byte[] v0 = new byte[this.macSize];
        System.arraycopy(this.macBlock, 0, v0, 0, this.macSize);
        return v0;
    }

    public int getOutputSize(int arg3) {
        int v0 = this.bufOff + arg3;
        if(this.forEncryption) {
            v0 += this.macSize;
        }
        else if(v0 < this.macSize) {
            v0 = 0;
        }
        else {
            v0 -= this.macSize;
        }

        return v0;
    }

    public BlockCipher getUnderlyingCipher() {
        return this.cipher.getUnderlyingCipher();
    }

    public int getUpdateOutputSize(int arg3) {
        int v0 = this.bufOff + arg3;
        if(this.forEncryption) {
        label_10:
            v0 -= v0 % this.blockSize;
        }
        else if(v0 < this.macSize) {
            v0 = 0;
        }
        else {
            v0 -= this.macSize;
            goto label_10;
        }

        return v0;
    }

    public void init(boolean arg8, CipherParameters arg9) {
        KeyParameter v0;
        byte[] v1;
        CipherParameters v6 = null;
        this.forEncryption = arg8;
        if((arg9 instanceof AEADParameters)) {
            v1 = ((AEADParameters)arg9).getNonce();
            this.initialAssociatedText = ((AEADParameters)arg9).getAssociatedText();
            this.macSize = ((AEADParameters)arg9).getMacSize() / 8;
            v0 = ((AEADParameters)arg9).getKey();
        }
        else if((arg9 instanceof ParametersWithIV)) {
            v1 = ((ParametersWithIV)arg9).getIV();
            this.initialAssociatedText = ((byte[])v6);
            this.macSize = this.mac.getMacSize() / 2;
            CipherParameters v0_1 = ((ParametersWithIV)arg9).getParameters();
        }
        else {
            goto label_57;
        }

        byte[] v2 = new byte[this.blockSize];
        this.mac.init(((CipherParameters)v0));
        v2[this.blockSize - 1] = 0;
        this.mac.update(v2, 0, this.blockSize);
        this.mac.update(v1, 0, v1.length);
        this.mac.doFinal(this.nonceMac, 0);
        v2[this.blockSize - 1] = 1;
        this.mac.update(v2, 0, this.blockSize);
        if(this.initialAssociatedText != null) {
            this.processAADBytes(this.initialAssociatedText, 0, this.initialAssociatedText.length);
        }

        this.cipher.init(true, new ParametersWithIV(v6, this.nonceMac));
        return;
    label_57:
        throw new IllegalArgumentException("invalid parameters passed to EAX");
    }

    private void initCipher() {
        if(!this.cipherInitialized) {
            this.cipherInitialized = true;
            this.mac.doFinal(this.associatedTextMac, 0);
            byte[] v0 = new byte[this.blockSize];
            v0[this.blockSize - 1] = 2;
            this.mac.update(v0, 0, this.blockSize);
        }
    }

    private int process(byte arg7, byte[] arg8, int arg9) {
        int v0_1;
        byte[] v0 = this.bufBlock;
        int v2 = this.bufOff;
        this.bufOff = v2 + 1;
        v0[v2] = arg7;
        if(this.bufOff == this.bufBlock.length) {
            if(this.forEncryption) {
                v0_1 = this.cipher.processBlock(this.bufBlock, 0, arg8, arg9);
                this.mac.update(arg8, arg9, this.blockSize);
            }
            else {
                this.mac.update(this.bufBlock, 0, this.blockSize);
                v0_1 = this.cipher.processBlock(this.bufBlock, 0, arg8, arg9);
            }

            this.bufOff = this.blockSize;
            System.arraycopy(this.bufBlock, this.blockSize, this.bufBlock, 0, this.blockSize);
        }
        else {
            v0_1 = 0;
        }

        return v0_1;
    }

    public void processAADByte(byte arg3) {
        if(this.cipherInitialized) {
            throw new IllegalStateException("AAD data cannot be added after encryption/decription processing has begun.");
        }

        this.mac.update(arg3);
    }

    public void processAADBytes(byte[] arg3, int arg4, int arg5) {
        if(this.cipherInitialized) {
            throw new IllegalStateException("AAD data cannot be added after encryption/decription processing has begun.");
        }

        this.mac.update(arg3, arg4, arg5);
    }

    public int processByte(byte arg2, byte[] arg3, int arg4) {
        this.initCipher();
        return this.process(arg2, arg3, arg4);
    }

    public int processBytes(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        int v0 = 0;
        this.initCipher();
        int v1 = 0;
        while(v0 != arg7) {
            v1 += this.process(arg5[arg6 + v0], arg8, arg9 + v1);
            ++v0;
        }

        return v1;
    }

    private void reset(boolean arg5) {
        this.cipher.reset();
        this.mac.reset();
        this.bufOff = 0;
        Arrays.fill(this.bufBlock, 0);
        if(arg5) {
            Arrays.fill(this.macBlock, 0);
        }

        byte[] v0 = new byte[this.blockSize];
        v0[this.blockSize - 1] = 1;
        this.mac.update(v0, 0, this.blockSize);
        this.cipherInitialized = false;
        if(this.initialAssociatedText != null) {
            this.processAADBytes(this.initialAssociatedText, 0, this.initialAssociatedText.length);
        }
    }

    public void reset() {
        this.reset(true);
    }

    private boolean verifyMac(byte[] arg6, int arg7) {
        boolean v1 = false;
        int v0 = 0;
        int v2 = 0;
        while(v0 < this.macSize) {
            v2 |= this.macBlock[v0] ^ arg6[arg7 + v0];
            ++v0;
        }

        if(v2 == 0) {
            v1 = true;
        }

        return v1;
    }
}

