// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.modes;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.modes.gcm.GCMExponentiator;
import org.bouncycastle.crypto.modes.gcm.GCMMultiplier;
import org.bouncycastle.crypto.modes.gcm.Tables1kGCMExponentiator;
import org.bouncycastle.crypto.modes.gcm.Tables8kGCMMultiplier;
import org.bouncycastle.crypto.params.AEADParameters;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.crypto.util.Pack;
import org.bouncycastle.util.Arrays;

public class GCMBlockCipher implements AEADBlockCipher {
    private static final int BLOCK_SIZE = 16;
    private byte[] H;
    private byte[] J0;
    private byte[] S;
    private byte[] S_at;
    private byte[] S_atPre;
    private byte[] atBlock;
    private int atBlockPos;
    private long atLength;
    private long atLengthPre;
    private byte[] bufBlock;
    private int bufOff;
    private BlockCipher cipher;
    private byte[] counter;
    private GCMExponentiator exp;
    private boolean forEncryption;
    private byte[] initialAssociatedText;
    private byte[] macBlock;
    private int macSize;
    private GCMMultiplier multiplier;
    private byte[] nonce;
    private long totalLength;

    public GCMBlockCipher(BlockCipher arg2) {
        this(arg2, null);
    }

    public GCMBlockCipher(BlockCipher arg3, GCMMultiplier arg4) {
        Tables8kGCMMultiplier v4;
        super();
        if(arg3.getBlockSize() != 16) {
            throw new IllegalArgumentException("cipher required with a block size of 16.");
        }

        if(arg4 == null) {
            v4 = new Tables8kGCMMultiplier();
        }

        this.cipher = arg3;
        this.multiplier = ((GCMMultiplier)v4);
    }

    public int doFinal(byte[] arg13, int arg14) {
        long v10 = 0;
        long v8 = 8;
        int v7 = 16;
        if(this.totalLength == v10) {
            this.initCipher();
        }

        int v3 = this.bufOff;
        if(!this.forEncryption) {
            if(v3 < this.macSize) {
                throw new InvalidCipherTextException("data too short");
            }
            else {
                v3 -= this.macSize;
            }
        }

        if(v3 > 0) {
            this.gCTRPartial(this.bufBlock, 0, v3, arg13, arg14);
        }

        this.atLength += ((long)this.atBlockPos);
        if(this.atLength > this.atLengthPre) {
            if(this.atBlockPos > 0) {
                this.gHASHPartial(this.S_at, this.atBlock, 0, this.atBlockPos);
            }

            if(this.atLengthPre > v10) {
                GCMBlockCipher.xor(this.S_at, this.S_atPre);
            }

            long v0 = this.totalLength * v8 + 127 >>> 7;
            byte[] v4 = new byte[v7];
            if(this.exp == null) {
                this.exp = new Tables1kGCMExponentiator();
                this.exp.init(this.H);
            }

            this.exp.exponentiateX(v0, v4);
            GCMBlockCipher.multiply(this.S_at, v4);
            GCMBlockCipher.xor(this.S, this.S_at);
        }

        byte[] v0_1 = new byte[v7];
        Pack.longToBigEndian(this.atLength * v8, v0_1, 0);
        Pack.longToBigEndian(this.totalLength * v8, v0_1, 8);
        this.gHASHBlock(this.S, v0_1);
        v0_1 = new byte[v7];
        this.cipher.processBlock(this.J0, 0, v0_1, 0);
        GCMBlockCipher.xor(v0_1, this.S);
        this.macBlock = new byte[this.macSize];
        System.arraycopy(v0_1, 0, this.macBlock, 0, this.macSize);
        if(this.forEncryption) {
            System.arraycopy(this.macBlock, 0, arg13, this.bufOff + arg14, this.macSize);
            v3 += this.macSize;
        }
        else {
            v0_1 = new byte[this.macSize];
            System.arraycopy(this.bufBlock, v3, v0_1, 0, this.macSize);
            if(!Arrays.constantTimeAreEqual(this.macBlock, v0_1)) {
                throw new InvalidCipherTextException("mac check in GCM failed");
            }
        }

        this.reset(false);
        return v3;
    }

    private void gCTRBlock(byte[] arg5, byte[] arg6, int arg7) {
        byte[] v0 = this.getNextCounterBlock();
        GCMBlockCipher.xor(v0, arg5);
        System.arraycopy(v0, 0, arg6, arg7, 16);
        byte[] v1 = this.S;
        if(this.forEncryption) {
            arg5 = v0;
        }

        this.gHASHBlock(v1, arg5);
        this.totalLength += 16;
    }

    private void gCTRPartial(byte[] arg5, int arg6, int arg7, byte[] arg8, int arg9) {
        byte[] v0 = this.getNextCounterBlock();
        GCMBlockCipher.xor(v0, arg5, arg6, arg7);
        System.arraycopy(v0, 0, arg8, arg9, arg7);
        byte[] v1 = this.S;
        if(this.forEncryption) {
            arg5 = v0;
        }

        this.gHASHPartial(v1, arg5, 0, arg7);
        this.totalLength += ((long)arg7);
    }

    private void gHASH(byte[] arg4, byte[] arg5, int arg6) {
        int v0;
        for(v0 = 0; v0 < arg6; v0 += 16) {
            this.gHASHPartial(arg4, arg5, v0, Math.min(arg6 - v0, 16));
        }
    }

    private void gHASHBlock(byte[] arg2, byte[] arg3) {
        GCMBlockCipher.xor(arg2, arg3);
        this.multiplier.multiplyH(arg2);
    }

    private void gHASHPartial(byte[] arg2, byte[] arg3, int arg4, int arg5) {
        GCMBlockCipher.xor(arg2, arg3, arg4, arg5);
        this.multiplier.multiplyH(arg2);
    }

    public String getAlgorithmName() {
        return this.cipher.getAlgorithmName() + "/GCM";
    }

    public byte[] getMac() {
        return Arrays.clone(this.macBlock);
    }

    private byte[] getNextCounterBlock() {
        int v0;
        for(v0 = 15; v0 >= 12; --v0) {
            byte v1 = ((byte)(this.counter[v0] + 1 & 255));
            this.counter[v0] = v1;
            if(v1 != 0) {
                break;
            }
        }

        byte[] v0_1 = new byte[16];
        this.cipher.processBlock(this.counter, 0, v0_1, 0);
        return v0_1;
    }

    public int getOutputSize(int arg3) {
        int v0 = this.bufOff + arg3;
        if(this.forEncryption) {
            v0 += this.macSize;
        }
        else if(v0 < this.macSize) {
            v0 = 0;
        }
        else {
            v0 -= this.macSize;
        }

        return v0;
    }

    public BlockCipher getUnderlyingCipher() {
        return this.cipher;
    }

    public int getUpdateOutputSize(int arg3) {
        int v0 = this.bufOff + arg3;
        if(this.forEncryption) {
        label_10:
            v0 -= v0 % 16;
        }
        else if(v0 < this.macSize) {
            v0 = 0;
        }
        else {
            v0 -= this.macSize;
            goto label_10;
        }

        return v0;
    }

    public void init(boolean arg10, CipherParameters arg11) {
        KeyParameter v2;
        int v0;
        long v7 = 0;
        byte[] v5 = null;
        int v1 = 16;
        this.forEncryption = arg10;
        this.macBlock = v5;
        if((arg11 instanceof AEADParameters)) {
            this.nonce = ((AEADParameters)arg11).getNonce();
            this.initialAssociatedText = ((AEADParameters)arg11).getAssociatedText();
            v0 = ((AEADParameters)arg11).getMacSize();
            if(v0 >= 96 && v0 <= 128 && v0 % 8 == 0) {
                this.macSize = v0 / 8;
                v2 = ((AEADParameters)arg11).getKey();
                goto label_33;
            }

            throw new IllegalArgumentException("Invalid value for MAC size: " + v0);
        }
        else {
            if(!(arg11 instanceof ParametersWithIV)) {
                goto label_55;
            }

            this.nonce = ((ParametersWithIV)arg11).getIV();
            this.initialAssociatedText = v5;
            this.macSize = v1;
            CipherParameters v2_1 = ((ParametersWithIV)arg11).getParameters();
        }

    label_33:
        v0 = arg10 ? v1 : this.macSize + 16;
        this.bufBlock = new byte[v0];
        if(this.nonce != null && this.nonce.length >= 1) {
            if(v2 != null) {
                this.cipher.init(true, ((CipherParameters)v2));
                this.H = new byte[v1];
                this.cipher.processBlock(this.H, 0, this.H, 0);
                this.multiplier.init(this.H);
                this.exp = ((GCMExponentiator)v5);
            }

            this.J0 = new byte[v1];
            if(this.nonce.length == 12) {
                System.arraycopy(this.nonce, 0, this.J0, 0, this.nonce.length);
                this.J0[15] = 1;
            }
            else {
                this.gHASH(this.J0, this.nonce, this.nonce.length);
                byte[] v0_1 = new byte[v1];
                Pack.longToBigEndian((((long)this.nonce.length)) * 8, v0_1, 8);
                this.gHASHBlock(this.J0, v0_1);
            }

            this.S = new byte[v1];
            this.S_at = new byte[v1];
            this.S_atPre = new byte[v1];
            this.atBlock = new byte[v1];
            this.atBlockPos = 0;
            this.atLength = v7;
            this.atLengthPre = v7;
            this.counter = Arrays.clone(this.J0);
            this.bufOff = 0;
            this.totalLength = v7;
            if(this.initialAssociatedText != null) {
                this.processAADBytes(this.initialAssociatedText, 0, this.initialAssociatedText.length);
            }

            return;
        }

        throw new IllegalArgumentException("IV must be at least 1 byte");
    label_55:
        throw new IllegalArgumentException("invalid parameters passed to GCM");
    }

    private void initCipher() {
        long v6 = 0;
        int v5 = 16;
        if(this.atLength > v6) {
            System.arraycopy(this.S_at, 0, this.S_atPre, 0, v5);
            this.atLengthPre = this.atLength;
        }

        if(this.atBlockPos > 0) {
            this.gHASHPartial(this.S_atPre, this.atBlock, 0, this.atBlockPos);
            this.atLengthPre += ((long)this.atBlockPos);
        }

        if(this.atLengthPre > v6) {
            System.arraycopy(this.S_atPre, 0, this.S, 0, v5);
        }
    }

    private static void multiply(byte[] arg9, byte[] arg10) {
        int v8 = 16;
        byte[] v5 = Arrays.clone(arg9);
        byte[] v6 = new byte[v8];
        int v4;
        for(v4 = 0; v4 < v8; ++v4) {
            int v7 = arg10[v4];
            int v3;
            for(v3 = 7; v3 >= 0; --v3) {
                if((1 << v3 & v7) != 0) {
                    GCMBlockCipher.xor(v6, v5);
                }

                int v0 = (v5[15] & 1) != 0 ? 1 : 0;
                GCMBlockCipher.shiftRight(v5);
                if(v0 != 0) {
                    v5[0] = ((byte)(v5[0] ^ -31));
                }
            }
        }

        System.arraycopy(v6, 0, arg9, 0, v8);
    }

    private void outputBlock(byte[] arg6, int arg7) {
        if(this.totalLength == 0) {
            this.initCipher();
        }

        this.gCTRBlock(this.bufBlock, arg6, arg7);
        if(this.forEncryption) {
            this.bufOff = 0;
        }
        else {
            System.arraycopy(this.bufBlock, 16, this.bufBlock, 0, this.macSize);
            this.bufOff = this.macSize;
        }
    }

    public void processAADByte(byte arg5) {
        this.atBlock[this.atBlockPos] = arg5;
        int v0 = this.atBlockPos + 1;
        this.atBlockPos = v0;
        if(v0 == 16) {
            this.gHASHBlock(this.S_at, this.atBlock);
            this.atBlockPos = 0;
            this.atLength += 16;
        }
    }

    public void processAADBytes(byte[] arg7, int arg8, int arg9) {
        int v0;
        for(v0 = 0; v0 < arg9; ++v0) {
            this.atBlock[this.atBlockPos] = arg7[arg8 + v0];
            int v2 = this.atBlockPos + 1;
            this.atBlockPos = v2;
            if(v2 == 16) {
                this.gHASHBlock(this.S_at, this.atBlock);
                this.atBlockPos = 0;
                this.atLength += 16;
            }
        }
    }

    public int processByte(byte arg3, byte[] arg4, int arg5) {
        this.bufBlock[this.bufOff] = arg3;
        int v0 = this.bufOff + 1;
        this.bufOff = v0;
        if(v0 == this.bufBlock.length) {
            this.outputBlock(arg4, arg5);
            v0 = 16;
        }
        else {
            v0 = 0;
        }

        return v0;
    }

    public int processBytes(byte[] arg6, int arg7, int arg8, byte[] arg9, int arg10) {
        int v0 = 0;
        int v1;
        for(v1 = 0; v1 < arg8; ++v1) {
            this.bufBlock[this.bufOff] = arg6[arg7 + v1];
            int v2 = this.bufOff + 1;
            this.bufOff = v2;
            if(v2 == this.bufBlock.length) {
                this.outputBlock(arg9, arg10 + v0);
                v0 += 16;
            }
        }

        return v0;
    }

    public void reset() {
        this.reset(true);
    }

    private void reset(boolean arg6) {
        this.cipher.reset();
        this.S = new byte[16];
        this.S_at = new byte[16];
        this.S_atPre = new byte[16];
        this.atBlock = new byte[16];
        this.atBlockPos = 0;
        this.atLength = 0;
        this.atLengthPre = 0;
        this.counter = Arrays.clone(this.J0);
        this.bufOff = 0;
        this.totalLength = 0;
        if(this.bufBlock != null) {
            Arrays.fill(this.bufBlock, 0);
        }

        if(arg6) {
            this.macBlock = null;
        }

        if(this.initialAssociatedText != null) {
            this.processAADBytes(this.initialAssociatedText, 0, this.initialAssociatedText.length);
        }
    }

    private static void shiftRight(byte[] arg4) {
        int v0 = 0;
        int v1 = 0;
        while(true) {
            int v2 = arg4[v1] & 255;
            arg4[v1] = ((byte)(v0 | v2 >>> 1));
            ++v1;
            if(v1 == 16) {
                return;
            }

            v0 = (v2 & 1) << 7;
        }
    }

    private static void xor(byte[] arg3, byte[] arg4) {
        int v0;
        for(v0 = 15; v0 >= 0; --v0) {
            arg3[v0] = ((byte)(arg3[v0] ^ arg4[v0]));
        }
    }

    private static void xor(byte[] arg3, byte[] arg4, int arg5, int arg6) {
        while(true) {
            int v0 = arg6 - 1;
            if(arg6 <= 0) {
                return;
            }

            arg3[v0] = ((byte)(arg3[v0] ^ arg4[arg5 + v0]));
            arg6 = v0;
        }
    }
}

