// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.prng.drbg;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.prng.EntropySource;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.encoders.Hex;

public class CTRSP800DRBG implements SP80090DRBG {
    private static final int AES_MAX_BITS_REQUEST = 262144;
    private static final long AES_RESEED_MAX = 140737488355328L;
    private static final byte[] K_BITS = null;
    private static final int TDEA_MAX_BITS_REQUEST = 4096;
    private static final long TDEA_RESEED_MAX = 2147483648L;
    private byte[] _Key;
    private byte[] _V;
    private BlockCipher _engine;
    private EntropySource _entropySource;
    private boolean _isTDEA;
    private int _keySizeInBits;
    private long _reseedCounter;
    private int _seedLength;

    static {
        CTRSP800DRBG.K_BITS = Hex.decode("000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F");
    }

    public CTRSP800DRBG(BlockCipher arg3, int arg4, int arg5, EntropySource arg6, byte[] arg7, byte[] arg8) {
        super();
        this._reseedCounter = 0;
        this._isTDEA = false;
        this._entropySource = arg6;
        this._engine = arg3;
        this._keySizeInBits = arg4;
        this._seedLength = arg3.getBlockSize() * 8 + arg4;
        this._isTDEA = this.isTDEA(arg3);
        if(arg5 > 256) {
            throw new IllegalArgumentException("Requested security strength is not supported by the derivation function");
        }

        if(this.getMaxSecurityStrength(arg3, arg4) < arg5) {
            throw new IllegalArgumentException("Requested security strength is not supported by block cipher and key size");
        }

        if(arg6.entropySize() < arg5) {
            throw new IllegalArgumentException("Not enough entropy for security strength required");
        }

        this.CTR_DRBG_Instantiate_algorithm(arg6.getEntropy(), arg8, arg7);
    }

    private void BCC(byte[] arg10, byte[] arg11, byte[] arg12, byte[] arg13) {
        int v2 = this._engine.getBlockSize();
        byte[] v3 = new byte[v2];
        int v4 = arg13.length / v2;
        byte[] v5 = new byte[v2];
        this._engine.init(true, new KeyParameter(this.expandKey(arg11)));
        this._engine.processBlock(arg12, 0, v3, 0);
        int v0;
        for(v0 = 0; v0 < v4; ++v0) {
            this.XOR(v5, v3, arg13, v0 * v2);
            this._engine.processBlock(v5, 0, v3, 0);
        }

        System.arraycopy(v3, 0, arg10, 0, arg10.length);
    }

    private byte[] Block_Cipher_df(byte[] arg12, int arg13) {
        int v2;
        int v3 = this._engine.getBlockSize();
        int v0 = arg12.length;
        byte[] v4 = new byte[(v0 + 9 + v3 - 1) / v3 * v3];
        this.copyIntToByteArray(v4, v0, 0);
        this.copyIntToByteArray(v4, arg13 / 8, 4);
        System.arraycopy(arg12, 0, v4, 8, v0);
        v4[v0 + 8] = -128;
        byte[] v5 = new byte[this._keySizeInBits / 8 + v3];
        byte[] v6 = new byte[v3];
        byte[] v7 = new byte[v3];
        byte[] v8 = new byte[this._keySizeInBits / 8];
        System.arraycopy(CTRSP800DRBG.K_BITS, 0, v8, 0, v8.length);
        for(v0 = 0; v0 * v3 * 8 < this._keySizeInBits + v3 * 8; ++v0) {
            this.copyIntToByteArray(v7, v0, 0);
            this.BCC(v6, v8, v7, v4);
            v2 = v5.length - v0 * v3 > v3 ? v3 : v5.length - v0 * v3;
            System.arraycopy(v6, 0, v5, v0 * v3, v2);
        }

        v4 = new byte[v3];
        System.arraycopy(v5, 0, v8, 0, v8.length);
        System.arraycopy(v5, v8.length, v4, 0, v4.length);
        v5 = new byte[arg13 / 2];
        this._engine.init(true, new KeyParameter(this.expandKey(v8)));
        for(v0 = 0; v0 * v3 < v5.length; ++v0) {
            this._engine.processBlock(v4, 0, v4, 0);
            v2 = v5.length - v0 * v3 > v3 ? v3 : v5.length - v0 * v3;
            System.arraycopy(v4, 0, v5, v0 * v3, v2);
        }

        return v5;
    }

    private void CTR_DRBG_Instantiate_algorithm(byte[] arg4, byte[] arg5, byte[] arg6) {
        byte[] v0 = this.Block_Cipher_df(Arrays.concatenate(arg4, arg5, arg6), this._seedLength);
        int v1 = this._engine.getBlockSize();
        this._Key = new byte[(this._keySizeInBits + 7) / 8];
        this._V = new byte[v1];
        this.CTR_DRBG_Update(v0, this._Key, this._V);
        this._reseedCounter = 1;
    }

    private void CTR_DRBG_Reseed_algorithm(EntropySource arg4, byte[] arg5) {
        this.CTR_DRBG_Update(this.Block_Cipher_df(Arrays.concatenate(arg4.getEntropy(), arg5), this._seedLength), this._Key, this._V);
        this._reseedCounter = 1;
    }

    private void CTR_DRBG_Update(byte[] arg9, byte[] arg10, byte[] arg11) {
        byte[] v4 = new byte[arg9.length];
        byte[] v5 = new byte[this._engine.getBlockSize()];
        int v3 = this._engine.getBlockSize();
        this._engine.init(true, new KeyParameter(this.expandKey(arg10)));
        int v0;
        for(v0 = 0; v0 * v3 < arg9.length; ++v0) {
            this.addOneTo(arg11);
            this._engine.processBlock(arg11, 0, v5, 0);
            int v2 = v4.length - v0 * v3 > v3 ? v3 : v4.length - v0 * v3;
            System.arraycopy(v5, 0, v4, v0 * v3, v2);
        }

        this.XOR(v4, arg9, v4, 0);
        System.arraycopy(v4, 0, arg10, 0, arg10.length);
        System.arraycopy(v4, arg10.length, arg11, 0, arg11.length);
    }

    private void XOR(byte[] arg4, byte[] arg5, byte[] arg6, int arg7) {
        int v0;
        for(v0 = 0; v0 < arg4.length; ++v0) {
            arg4[v0] = ((byte)(arg5[v0] ^ arg6[v0 + arg7]));
        }
    }

    private void addOneTo(byte[] arg6) {
        int v0 = 1;
        int v2 = 1;
        while(v0 <= arg6.length) {
            int v3 = (arg6[arg6.length - v0] & 255) + v2;
            v2 = v3 > 255 ? 1 : 0;
            arg6[arg6.length - v0] = ((byte)v3);
            ++v0;
        }
    }

    private void copyIntToByteArray(byte[] arg3, int arg4, int arg5) {
        arg3[arg5] = ((byte)(arg4 >> 24));
        arg3[arg5 + 1] = ((byte)(arg4 >> 16));
        arg3[arg5 + 2] = ((byte)(arg4 >> 8));
        arg3[arg5 + 3] = ((byte)arg4);
    }

    byte[] expandKey(byte[] arg4) {
        if(this._isTDEA) {
            byte[] v0 = new byte[24];
            this.padKey(arg4, 0, v0, 0);
            this.padKey(arg4, 7, v0, 8);
            this.padKey(arg4, 14, v0, 16);
            arg4 = v0;
        }

        return arg4;
    }

    public int generate(byte[] arg8, byte[] arg9, boolean arg10) {
        byte[] v0_1;
        int v0 = -1;
        if(this._isTDEA) {
            if(this._reseedCounter <= 2147483648L) {
                if(Utils.isTooLarge(arg8, 512)) {
                    throw new IllegalArgumentException("Number of bits per request limited to 4096");
                }
                else {
                    goto label_25;
                }
            }
        }
        else if(this._reseedCounter <= 140737488355328L) {
            if(Utils.isTooLarge(arg8, 32768)) {
                throw new IllegalArgumentException("Number of bits per request limited to 262144");
            }
            else {
            label_25:
                if(arg10) {
                    this.CTR_DRBG_Reseed_algorithm(this._entropySource, arg9);
                    arg9 = null;
                }

                if(0 != 0) {
                    v0_1 = this.Block_Cipher_df(arg9, this._seedLength);
                    this.CTR_DRBG_Update(v0_1, this._Key, this._V);
                }
                else {
                    v0_1 = new byte[this._seedLength];
                }

                byte[] v4 = new byte[this._V.length];
                this._engine.init(true, new KeyParameter(this.expandKey(this._Key)));
                int v1;
                for(v1 = 0; v1 < arg8.length / v4.length; ++v1) {
                    this.addOneTo(this._V);
                    this._engine.processBlock(this._V, 0, v4, 0);
                    int v3 = arg8.length - v4.length * v1 > v4.length ? v4.length : arg8.length - this._V.length * v1;
                    System.arraycopy(v4, 0, arg8, v4.length * v1, v3);
                }

                this.CTR_DRBG_Update(v0_1, this._Key, this._V);
                ++this._reseedCounter;
                v0 = arg8.length * 8;
            }
        }

        return v0;
    }

    private int getMaxSecurityStrength(BlockCipher arg3, int arg4) {
        if((this.isTDEA(arg3)) && arg4 == 168) {
            arg4 = 112;
        }
        else if(!arg3.getAlgorithmName().equals("AES")) {
            arg4 = -1;
        }

        return arg4;
    }

    private boolean isTDEA(BlockCipher arg3) {
        boolean v0 = (arg3.getAlgorithmName().equals("DESede")) || (arg3.getAlgorithmName().equals("TDEA")) ? true : false;
        return v0;
    }

    private void padKey(byte[] arg6, int arg7, byte[] arg8, int arg9) {
        arg8[arg9] = ((byte)(arg6[arg7] & 254));
        arg8[arg9 + 1] = ((byte)(arg6[arg7] << 7 | (arg6[arg7 + 1] & 252) >>> 1));
        arg8[arg9 + 2] = ((byte)(arg6[arg7 + 1] << 6 | (arg6[arg7 + 2] & 248) >>> 2));
        arg8[arg9 + 3] = ((byte)(arg6[arg7 + 2] << 5 | (arg6[arg7 + 3] & 240) >>> 3));
        arg8[arg9 + 4] = ((byte)(arg6[arg7 + 3] << 4 | (arg6[arg7 + 4] & 224) >>> 4));
        arg8[arg9 + 5] = ((byte)(arg6[arg7 + 4] << 3 | (arg6[arg7 + 5] & 192) >>> 5));
        arg8[arg9 + 6] = ((byte)(arg6[arg7 + 5] << 2 | (arg6[arg7 + 6] & 128) >>> 6));
        arg8[arg9 + 7] = ((byte)(arg6[arg7 + 6] << 1));
        int v0;
        for(v0 = arg9; v0 <= arg9 + 7; ++v0) {
            arg8[v0] = ((byte)((arg8[v0] >> 7 ^ (arg8[v0] >> 1 ^ arg8[v0] >> 2 ^ arg8[v0] >> 3 ^ arg8[v0] >> 4 ^ arg8[v0] >> 5 ^ arg8[v0] >> 6) ^ 1) & 1 | arg8[v0] & 254));
        }
    }

    public void reseed(byte[] arg2) {
        this.CTR_DRBG_Reseed_algorithm(this._entropySource, arg2);
    }
}

