// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.prng.drbg;

import java.math.BigInteger;
import org.bouncycastle.asn1.nist.NISTNamedCurves;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.prng.EntropySource;
import org.bouncycastle.math.ec.ECCurve$Fp;
import org.bouncycastle.math.ec.ECFieldElement$Fp;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.math.ec.ECPoint$Fp;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.BigIntegers;

public class DualECSP800DRBG implements SP80090DRBG {
    private static final int MAX_ADDITIONAL_INPUT = 4096;
    private static final int MAX_ENTROPY_LENGTH = 4096;
    private static final int MAX_PERSONALIZATION_STRING = 4096;
    private static final long RESEED_MAX = 2147483648L;
    private ECPoint _P;
    private ECPoint _Q;
    private ECCurve$Fp _curve;
    private Digest _digest;
    private EntropySource _entropySource;
    private int _outlen;
    private long _reseedCounter;
    private byte[] _s;
    private int _sLength;
    private int _securityStrength;
    private int _seedlen;
    private static final BigInteger p256_Px;
    private static final BigInteger p256_Py;
    private static final BigInteger p256_Qx;
    private static final BigInteger p256_Qy;
    private static final BigInteger p384_Px;
    private static final BigInteger p384_Py;
    private static final BigInteger p384_Qx;
    private static final BigInteger p384_Qy;
    private static final BigInteger p521_Px;
    private static final BigInteger p521_Py;
    private static final BigInteger p521_Qx;
    private static final BigInteger p521_Qy;

    static {
        DualECSP800DRBG.p256_Px = new BigInteger("6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296", 16);
        DualECSP800DRBG.p256_Py = new BigInteger("4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5", 16);
        DualECSP800DRBG.p256_Qx = new BigInteger("c97445f45cdef9f0d3e05e1e585fc297235b82b5be8ff3efca67c59852018192", 16);
        DualECSP800DRBG.p256_Qy = new BigInteger("b28ef557ba31dfcbdd21ac46e2a91e3c304f44cb87058ada2cb815151e610046", 16);
        DualECSP800DRBG.p384_Px = new BigInteger("aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a385502f25dbf55296c3a545e3872760ab7", 16);
        DualECSP800DRBG.p384_Py = new BigInteger("3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c00a60b1ce1d7e819d7a431d7c90ea0e5f", 16);
        DualECSP800DRBG.p384_Qx = new BigInteger("8e722de3125bddb05580164bfe20b8b432216a62926c57502ceede31c47816edd1e89769124179d0b695106428815065", 16);
        DualECSP800DRBG.p384_Qy = new BigInteger("023b1660dd701d0839fd45eec36f9ee7b32e13b315dc02610aa1b636e346df671f790f84c5e09b05674dbb7e45c803dd", 16);
        DualECSP800DRBG.p521_Px = new BigInteger("c6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31c2e5bd66", 16);
        DualECSP800DRBG.p521_Py = new BigInteger("11839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be94769fd16650", 16);
        DualECSP800DRBG.p521_Qx = new BigInteger("1b9fa3e518d683c6b65763694ac8efbaec6fab44f2276171a42726507dd08add4c3b3f4c1ebc5b1222ddba077f722943b24c3edfa0f85fe24d0c8c01591f0be6f63", 16);
        DualECSP800DRBG.p521_Qy = new BigInteger("1f3bdba585295d9a1110d1df1f9430ef8442c5018976ff3437ef91b81dc0b8132c8d5c39c32d0e004a3092b7d327c0e7a4d26d2c7b69b58f9066652911e457779de", 16);
    }

    public DualECSP800DRBG(Digest arg8, int arg9, EntropySource arg10, byte[] arg11, byte[] arg12) {
        int v5 = 4096;
        int v4 = 192;
        int v3 = 128;
        int v2 = 256;
        super();
        this._digest = arg8;
        this._entropySource = arg10;
        this._securityStrength = arg9;
        if(Utils.isTooLarge(arg11, 512)) {
            throw new IllegalArgumentException("Personalization string too large");
        }

        if(arg10.entropySize() >= arg9 && arg10.entropySize() <= v5) {
            byte[] v1 = Arrays.concatenate(arg10.getEntropy(), arg12, arg11);
            if(arg9 <= v3) {
                if(Utils.getMaxSecurityStrength(arg8) < v3) {
                    throw new IllegalArgumentException("Requested security strength is not supported by digest");
                }
                else {
                    this._seedlen = v2;
                    this._outlen = 30;
                    this._curve = NISTNamedCurves.getByName("P-256").getCurve();
                    this._P = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p256_Px), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p256_Py));
                    this._Q = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p256_Qx), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p256_Qy));
                }
            }
            else if(arg9 <= v4) {
                if(Utils.getMaxSecurityStrength(arg8) < v4) {
                    throw new IllegalArgumentException("Requested security strength is not supported by digest");
                }
                else {
                    this._seedlen = 384;
                    this._outlen = 46;
                    this._curve = NISTNamedCurves.getByName("P-384").getCurve();
                    this._P = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p384_Px), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p384_Py));
                    this._Q = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p384_Qx), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p384_Qy));
                }
            }
            else if(arg9 > v2) {
                goto label_175;
            }
            else if(Utils.getMaxSecurityStrength(arg8) < v2) {
                throw new IllegalArgumentException("Requested security strength is not supported by digest");
            }
            else {
                this._seedlen = 521;
                this._outlen = 63;
                this._curve = NISTNamedCurves.getByName("P-521").getCurve();
                this._P = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p521_Px), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p521_Py));
                this._Q = new ECPoint$Fp(this._curve, new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p521_Qx), new ECFieldElement$Fp(this._curve.getQ(), DualECSP800DRBG.p521_Qy));
            }

            this._s = Utils.hash_df(this._digest, v1, this._seedlen);
            this._sLength = this._s.length;
            this._reseedCounter = 0;
            return;
        label_175:
            throw new IllegalArgumentException("security strength cannot be greater than 256 bits");
        }

        throw new IllegalArgumentException("EntropySource must provide between " + arg9 + " and " + v5 + " bits");
    }

    public int generate(byte[] arg12, byte[] arg13, boolean arg14) {
        byte[] v0_1;
        int v0;
        byte[] v4 = null;
        int v1 = arg12.length * 8;
        int v5 = arg12.length / this._outlen;
        if(Utils.isTooLarge(arg13, 512)) {
            throw new IllegalArgumentException("Additional input too large");
        }

        if(this._reseedCounter + (((long)v5)) > 2147483648L) {
            v0 = -1;
        }
        else {
            if(arg14) {
                this.reseed(arg13);
                v0_1 = v4;
            }
            else {
                v0_1 = arg13;
            }

            if(v0_1 != null) {
                v0_1 = Utils.hash_df(this._digest, v0_1, this._seedlen);
            }

            byte[] v3 = v0_1;
            v0 = 0;
            while(v0 < v5) {
                this._s = this._P.multiply(new BigInteger(1, this.xor(this._s, v3))).getX().toBigInteger().toByteArray();
                v3 = this._Q.multiply(new BigInteger(1, this._s)).getX().toBigInteger().toByteArray();
                if(v3.length > this._outlen) {
                    System.arraycopy(v3, v3.length - this._outlen, arg12, this._outlen * v0, this._outlen);
                }
                else {
                    System.arraycopy(v3, 0, arg12, this._outlen * v0 + (this._outlen - v3.length), v3.length);
                }

                ++this._reseedCounter;
                ++v0;
                v3 = v4;
            }

            if(this._outlen * v5 < arg12.length) {
                this._s = this._P.multiply(new BigInteger(1, this.xor(this._s, v3))).getX().toBigInteger().toByteArray();
                System.arraycopy(this._Q.multiply(new BigInteger(1, this._s)).getX().toBigInteger().toByteArray(), 0, arg12, this._outlen * v5, arg12.length - v5 * this._outlen);
            }

            this._s = BigIntegers.asUnsignedByteArray(this._sLength, this._P.multiply(new BigInteger(1, this._s)).getX().toBigInteger());
            v0 = v1;
        }

        return v0;
    }

    private byte[] pad8(byte[] arg7, int arg8) {
        if(arg8 % 8 != 0) {
            int v3 = 8 - arg8 % 8;
            int v1 = 0;
            int v0 = arg7.length - 1;
            while(v0 >= 0) {
                int v2 = arg7[v0] & 255;
                arg7[v0] = ((byte)(v1 >> 8 - v3 | v2 << v3));
                --v0;
                v1 = v2;
            }
        }

        return arg7;
    }

    public void reseed(byte[] arg4) {
        if(Utils.isTooLarge(arg4, 512)) {
            throw new IllegalArgumentException("Additional input string too large");
        }

        this._s = Utils.hash_df(this._digest, Arrays.concatenate(this.pad8(this._s, this._seedlen), this._entropySource.getEntropy(), arg4), this._seedlen);
        this._reseedCounter = 0;
    }

    private byte[] xor(byte[] arg5, byte[] arg6) {
        if(arg6 != null) {
            byte[] v1 = new byte[arg5.length];
            int v0;
            for(v0 = 0; v0 != v1.length; ++v0) {
                v1[v0] = ((byte)(arg5[v0] ^ arg6[v0]));
            }

            arg5 = v1;
        }

        return arg5;
    }
}

