// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.prng.drbg;

import org.bouncycastle.crypto.Mac;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.prng.EntropySource;
import org.bouncycastle.util.Arrays;

public class HMacSP800DRBG implements SP80090DRBG {
    private static final int MAX_BITS_REQUEST = 262144;
    private static final long RESEED_MAX = 140737488355328L;
    private byte[] _K;
    private byte[] _V;
    private EntropySource _entropySource;
    private Mac _hMac;
    private long _reseedCounter;

    public HMacSP800DRBG(Mac arg4, int arg5, EntropySource arg6, byte[] arg7, byte[] arg8) {
        super();
        if(arg5 > Utils.getMaxSecurityStrength(arg4)) {
            throw new IllegalArgumentException("Requested security strength is not supported by the derivation function");
        }

        if(arg6.entropySize() < arg5) {
            throw new IllegalArgumentException("Not enough entropy for security strength required");
        }

        this._entropySource = arg6;
        this._hMac = arg4;
        byte[] v0 = Arrays.concatenate(arg6.getEntropy(), arg8, arg7);
        this._K = new byte[arg4.getMacSize()];
        this._V = new byte[this._K.length];
        Arrays.fill(this._V, 1);
        this.hmac_DRBG_Update(v0);
        this._reseedCounter = 1;
    }

    public int generate(byte[] arg9, byte[] arg10, boolean arg11) {
        int v0 = arg9.length * 8;
        if(v0 > 262144) {
            throw new IllegalArgumentException("Number of bits per request limited to 262144");
        }

        if(this._reseedCounter > 140737488355328L) {
            v0 = -1;
        }
        else {
            if(arg11) {
                this.reseed(arg10);
                arg10 = null;
            }

            if(0 != 0) {
                this.hmac_DRBG_Update(arg10);
            }

            byte[] v3 = new byte[arg9.length];
            int v4 = arg9.length / this._V.length;
            this._hMac.init(new KeyParameter(this._K));
            int v1;
            for(v1 = 0; v1 < v4; ++v1) {
                this._hMac.update(this._V, 0, this._V.length);
                this._hMac.doFinal(this._V, 0);
                System.arraycopy(this._V, 0, v3, this._V.length * v1, this._V.length);
            }

            if(this._V.length * v4 < v3.length) {
                this._hMac.update(this._V, 0, this._V.length);
                this._hMac.doFinal(this._V, 0);
                System.arraycopy(this._V, 0, v3, this._V.length * v4, v3.length - v4 * this._V.length);
            }

            this.hmac_DRBG_Update(arg10);
            ++this._reseedCounter;
            System.arraycopy(v3, 0, arg9, 0, arg9.length);
        }

        return v0;
    }

    private void hmac_DRBG_Update(byte[] arg2) {
        this.hmac_DRBG_Update_Func(arg2, 0);
        if(arg2 != null) {
            this.hmac_DRBG_Update_Func(arg2, 1);
        }
    }

    private void hmac_DRBG_Update_Func(byte[] arg5, byte arg6) {
        this._hMac.init(new KeyParameter(this._K));
        this._hMac.update(this._V, 0, this._V.length);
        this._hMac.update(arg6);
        if(arg5 != null) {
            this._hMac.update(arg5, 0, arg5.length);
        }

        this._hMac.doFinal(this._K, 0);
        this._hMac.init(new KeyParameter(this._K));
        this._hMac.update(this._V, 0, this._V.length);
        this._hMac.doFinal(this._V, 0);
    }

    public void reseed(byte[] arg3) {
        this.hmac_DRBG_Update(Arrays.concatenate(this._entropySource.getEntropy(), arg3));
        this._reseedCounter = 1;
    }
}

