// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.prng.drbg;

import java.util.Hashtable;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.prng.EntropySource;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;

public class HashSP800DRBG implements SP80090DRBG {
    private static final int MAX_BITS_REQUEST = 262144;
    private static final byte[] ONE = null;
    private static final long RESEED_MAX = 140737488355328L;
    private byte[] _C;
    private byte[] _V;
    private Digest _digest;
    private EntropySource _entropySource;
    private long _reseedCounter;
    private int _securityStrength;
    private int _seedLength;
    private static final Hashtable seedlens;

    static {
        HashSP800DRBG.ONE = new byte[]{1};
        HashSP800DRBG.seedlens = new Hashtable();
        HashSP800DRBG.seedlens.put("SHA-1", Integers.valueOf(440));
        HashSP800DRBG.seedlens.put("SHA-224", Integers.valueOf(440));
        HashSP800DRBG.seedlens.put("SHA-256", Integers.valueOf(440));
        HashSP800DRBG.seedlens.put("SHA-512/256", Integers.valueOf(440));
        HashSP800DRBG.seedlens.put("SHA-512/224", Integers.valueOf(440));
        HashSP800DRBG.seedlens.put("SHA-384", Integers.valueOf(888));
        HashSP800DRBG.seedlens.put("SHA-512", Integers.valueOf(888));
    }

    public HashSP800DRBG(Digest arg6, int arg7, EntropySource arg8, byte[] arg9, byte[] arg10) {
        super();
        if(arg7 > Utils.getMaxSecurityStrength(arg6)) {
            throw new IllegalArgumentException("Requested security strength is not supported by the derivation function");
        }

        if(arg8.entropySize() < arg7) {
            throw new IllegalArgumentException("Not enough entropy for security strength required");
        }

        this._digest = arg6;
        this._entropySource = arg8;
        this._securityStrength = arg7;
        this._seedLength = HashSP800DRBG.seedlens.get(arg6.getAlgorithmName()).intValue();
        this._V = Utils.hash_df(this._digest, Arrays.concatenate(arg8.getEntropy(), arg10, arg9), this._seedLength);
        byte[] v0 = new byte[this._V.length + 1];
        System.arraycopy(this._V, 0, v0, 1, this._V.length);
        this._C = Utils.hash_df(this._digest, v0, this._seedLength);
        this._reseedCounter = 1;
    }

    private void addTo(byte[] arg8, byte[] arg9) {
        int v4;
        int v6 = 255;
        int v0 = 1;
        int v2 = 0;
        while(v0 <= arg9.length) {
            v4 = (arg8[arg8.length - v0] & 255) + (arg9[arg9.length - v0] & 255) + v2;
            v2 = v4 > v6 ? 1 : 0;
            arg8[arg8.length - v0] = ((byte)v4);
            ++v0;
        }

        for(v0 = arg9.length + 1; v0 <= arg8.length; ++v0) {
            v4 = (arg8[arg8.length - v0] & 255) + v2;
            v2 = v4 > v6 ? 1 : 0;
            arg8[arg8.length - v0] = ((byte)v4);
        }
    }

    public int generate(byte[] arg11, byte[] arg12, boolean arg13) {
        byte[] v1;
        byte v9 = 3;
        byte v8 = 2;
        int v0 = arg11.length * 8;
        if(v0 > 262144) {
            throw new IllegalArgumentException("Number of bits per request limited to 262144");
        }

        if(this._reseedCounter > 140737488355328L) {
            v0 = -1;
        }
        else {
            if(arg13) {
                this.reseed(arg12);
                arg12 = null;
            }

            if(0 != 0) {
                v1 = new byte[this._V.length + 1 + arg12.length];
                v1[0] = v8;
                System.arraycopy(this._V, 0, v1, 1, this._V.length);
                System.arraycopy(arg12, 0, v1, this._V.length + 1, arg12.length);
                this.addTo(this._V, this.hash(v1));
            }

            v1 = this.hashgen(this._V, v0);
            byte[] v2 = new byte[this._V.length + 1];
            System.arraycopy(this._V, 0, v2, 1, this._V.length);
            v2[0] = v9;
            this.addTo(this._V, this.hash(v2));
            this.addTo(this._V, this._C);
            v2 = new byte[4];
            v2[0] = ((byte)(((int)(this._reseedCounter >> 24))));
            v2[1] = ((byte)(((int)(this._reseedCounter >> 16))));
            v2[v8] = ((byte)(((int)(this._reseedCounter >> 8))));
            v2[v9] = ((byte)(((int)this._reseedCounter)));
            this.addTo(this._V, v2);
            ++this._reseedCounter;
            System.arraycopy(v1, 0, arg11, 0, arg11.length);
        }

        return v0;
    }

    private byte[] hash(byte[] arg4) {
        this._digest.update(arg4, 0, arg4.length);
        byte[] v0 = new byte[this._digest.getDigestSize()];
        this._digest.doFinal(v0, 0);
        return v0;
    }

    private byte[] hashgen(byte[] arg9, int arg10) {
        int v3 = arg10 / 8 / this._digest.getDigestSize();
        byte[] v4 = new byte[arg9.length];
        System.arraycopy(arg9, 0, v4, 0, arg9.length);
        byte[] v5 = new byte[arg10 / 8];
        int v1;
        for(v1 = 0; v1 <= v3; ++v1) {
            byte[] v6 = this.hash(v4);
            int v0 = v5.length - v6.length * v1 > v6.length ? v6.length : v5.length - v6.length * v1;
            System.arraycopy(v6, 0, v5, v6.length * v1, v0);
            this.addTo(v4, HashSP800DRBG.ONE);
        }

        return v5;
    }

    public void reseed(byte[] arg6) {
        this._V = Utils.hash_df(this._digest, Arrays.concatenate(HashSP800DRBG.ONE, this._V, this._entropySource.getEntropy(), arg6), this._seedLength);
        byte[] v0 = new byte[this._V.length + 1];
        v0[0] = 0;
        System.arraycopy(this._V, 0, v0, 1, this._V.length);
        this._C = Utils.hash_df(this._digest, v0, this._seedLength);
        this._reseedCounter = 1;
    }
}

