// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.io.IOException;
import java.math.BigInteger;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Primitive;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERInteger;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DSA;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.Signer;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.ParametersWithRandom;

public class DSADigestSigner implements Signer {
    private final Digest digest;
    private final DSA dsaSigner;
    private boolean forSigning;

    public DSADigestSigner(DSA arg1, Digest arg2) {
        super();
        this.digest = arg2;
        this.dsaSigner = arg1;
    }

    private BigInteger[] derDecode(byte[] arg6) {
        ASN1Primitive v0 = ASN1Primitive.fromByteArray(arg6);
        return new BigInteger[]{((ASN1Sequence)v0).getObjectAt(0).getValue(), ((ASN1Sequence)v0).getObjectAt(1).getValue()};
    }

    private byte[] derEncode(BigInteger arg3, BigInteger arg4) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        v0.add(new DERInteger(arg3));
        v0.add(new DERInteger(arg4));
        return new DERSequence(v0).getEncoded("DER");
    }

    public byte[] generateSignature() {
        if(!this.forSigning) {
            throw new IllegalStateException("DSADigestSigner not initialised for signature generation.");
        }

        byte[] v0 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v0, 0);
        BigInteger[] v0_1 = this.dsaSigner.generateSignature(v0);
        try {
            return this.derEncode(v0_1[0], v0_1[1]);
        }
        catch(IOException v0_2) {
            throw new IllegalStateException("unable to encode signature");
        }
    }

    public void init(boolean arg3, CipherParameters arg4) {
        this.forSigning = arg3;
        CipherParameters v0 = (arg4 instanceof ParametersWithRandom) ? arg4.getParameters() : arg4;
        if((arg3) && !((AsymmetricKeyParameter)v0).isPrivate()) {
            throw new IllegalArgumentException("Signing Requires Private Key.");
        }

        if(!arg3 && (((AsymmetricKeyParameter)v0).isPrivate())) {
            throw new IllegalArgumentException("Verification Requires Public Key.");
        }

        this.reset();
        this.dsaSigner.init(arg3, arg4);
    }

    public void reset() {
        this.digest.reset();
    }

    public void update(byte arg2) {
        this.digest.update(arg2);
    }

    public void update(byte[] arg2, int arg3, int arg4) {
        this.digest.update(arg2, arg3, arg4);
    }

    public boolean verifySignature(byte[] arg7) {
        boolean v0;
        if(this.forSigning) {
            throw new IllegalStateException("DSADigestSigner not initialised for verification");
        }

        byte[] v1 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v1, 0);
        try {
            BigInteger[] v2 = this.derDecode(arg7);
            v0 = this.dsaSigner.verifySignature(v1, v2[0], v2[1]);
        }
        catch(IOException v1_1) {
        }

        return v0;
    }
}

