// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.Random;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DSA;
import org.bouncycastle.crypto.params.ECKeyParameters;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.math.ec.ECAlgorithms;
import org.bouncycastle.math.ec.ECCurve;
import org.bouncycastle.math.ec.ECFieldElement;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.util.Arrays;

public class DSTU4145Signer implements DSA {
    private static final BigInteger ONE;
    private ECKeyParameters key;
    private SecureRandom random;

    static {
        DSTU4145Signer.ONE = BigInteger.valueOf(1);
    }

    public DSTU4145Signer() {
        super();
    }

    private static BigInteger fieldElement2Integer(BigInteger arg3, ECFieldElement arg4) {
        BigInteger v0;
        for(v0 = arg4.toBigInteger(); v0.bitLength() >= arg3.bitLength(); v0 = v0.clearBit(v0.bitLength() - 1)) {
        }

        return v0;
    }

    private static BigInteger generateRandomInteger(BigInteger arg2, SecureRandom arg3) {
        return new BigInteger(arg2.bitLength() - 1, ((Random)arg3));
    }

    public BigInteger[] generateSignature(byte[] arg5) {
        BigInteger v0_1;
        BigInteger v3;
        ECFieldElement v0 = DSTU4145Signer.hash2FieldElement(this.key.getParameters().getCurve(), arg5);
        ECFieldElement v1 = v0.toBigInteger().signum() == 0 ? this.key.getParameters().getCurve().fromBigInteger(DSTU4145Signer.ONE) : v0;
        do {
        label_13:
            BigInteger v2 = DSTU4145Signer.generateRandomInteger(this.key.getParameters().getN(), this.random);
            v0 = this.key.getParameters().getG().multiply(v2).getX();
            if(v0.toBigInteger().signum() == 0) {
                goto label_13;
            }

            v3 = DSTU4145Signer.fieldElement2Integer(this.key.getParameters().getN(), v1.multiply(v0));
            if(v3.signum() == 0) {
                goto label_13;
            }

            v0_1 = v3.multiply(this.key.getD()).add(v2).mod(this.key.getParameters().getN());
        }
        while(v0_1.signum() == 0);

        return new BigInteger[]{v3, v0_1};
    }

    private static ECFieldElement hash2FieldElement(ECCurve arg3, byte[] arg4) {
        byte[] v1 = Arrays.clone(arg4);
        DSTU4145Signer.reverseBytes(v1);
        BigInteger v0;
        for(v0 = new BigInteger(1, v1); v0.bitLength() >= arg3.getFieldSize(); v0 = v0.clearBit(v0.bitLength() - 1)) {
        }

        return arg3.fromBigInteger(v0);
    }

    public void init(boolean arg2, CipherParameters arg3) {
        CipherParameters v0;
        if(arg2) {
            if((arg3 instanceof ParametersWithRandom)) {
                this.random = ((ParametersWithRandom)arg3).getRandom();
                v0 = ((ParametersWithRandom)arg3).getParameters();
            }
            else {
                this.random = new SecureRandom();
                v0 = arg3;
            }

            this.key = ((ECKeyParameters)v0);
        }
        else {
            this.key = ((ECKeyParameters)arg3);
        }
    }

    private static void reverseBytes(byte[] arg3) {
        int v0;
        for(v0 = 0; v0 < arg3.length / 2; ++v0) {
            byte v1 = arg3[v0];
            arg3[v0] = arg3[arg3.length - 1 - v0];
            arg3[arg3.length - 1 - v0] = v1;
        }
    }

    public boolean verifySignature(byte[] arg5, BigInteger arg6, BigInteger arg7) {
        boolean v2 = false;
        if(arg6.signum() != 0 && arg7.signum() != 0 && arg6.compareTo(this.key.getParameters().getN()) < 0 && arg7.compareTo(this.key.getParameters().getN()) < 0) {
            ECFieldElement v0 = DSTU4145Signer.hash2FieldElement(this.key.getParameters().getCurve(), arg5);
            ECFieldElement v1 = v0.toBigInteger().signum() == 0 ? this.key.getParameters().getCurve().fromBigInteger(DSTU4145Signer.ONE) : v0;
            ECPoint v0_1 = ECAlgorithms.sumOfTwoMultiplies(this.key.getParameters().getG(), arg7, this.key.getQ(), arg6);
            if(v0_1.isInfinity()) {
                return v2;
            }

            boolean v0_2 = DSTU4145Signer.fieldElement2Integer(this.key.getParameters().getN(), v1.multiply(v0_1.getX())).compareTo(arg6) == 0 ? true : false;
            v2 = v0_2;
        }

        return v2;
    }
}

