// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DSA;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.generators.ECKeyPairGenerator;
import org.bouncycastle.crypto.params.ECKeyGenerationParameters;
import org.bouncycastle.crypto.params.ECKeyParameters;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.math.ec.ECAlgorithms;
import org.bouncycastle.math.ec.ECConstants;
import org.bouncycastle.math.ec.ECPoint;

public class ECNRSigner implements DSA {
    private boolean forSigning;
    private ECKeyParameters key;
    private SecureRandom random;

    public ECNRSigner() {
        super();
    }

    public BigInteger[] generateSignature(byte[] arg9) {
        BigInteger v1_2;
        AsymmetricCipherKeyPair v4_1;
        if(!this.forSigning) {
            throw new IllegalStateException("not initialised for signing");
        }

        BigInteger v2 = this.key.getParameters().getN();
        int v1 = v2.bitLength();
        BigInteger v3 = new BigInteger(1, arg9);
        int v4 = v3.bitLength();
        ECKeyParameters v0 = this.key;
        if(v4 > v1) {
            throw new DataLengthException("input too large for ECNR key.");
        }

        do {
            ECKeyPairGenerator v1_1 = new ECKeyPairGenerator();
            v1_1.init(new ECKeyGenerationParameters(((ECPrivateKeyParameters)v0).getParameters(), this.random));
            v4_1 = v1_1.generateKeyPair();
            v1_2 = v4_1.getPublic().getQ().getX().toBigInteger().add(v3).mod(v2);
        }
        while(v1_2.equals(ECConstants.ZERO));

        return new BigInteger[]{v1_2, v4_1.getPrivate().getD().subtract(v1_2.multiply(((ECPrivateKeyParameters)v0).getD())).mod(v2)};
    }

    public void init(boolean arg2, CipherParameters arg3) {
        this.forSigning = arg2;
        if(!arg2) {
            this.key = ((ECKeyParameters)arg3);
        }
        else if((arg3 instanceof ParametersWithRandom)) {
            this.random = ((ParametersWithRandom)arg3).getRandom();
            this.key = ((ParametersWithRandom)arg3).getParameters();
        }
        else {
            this.random = new SecureRandom();
            this.key = ((ECKeyParameters)arg3);
        }
    }

    public boolean verifySignature(byte[] arg7, BigInteger arg8, BigInteger arg9) {
        boolean v0_2;
        if(this.forSigning) {
            throw new IllegalStateException("not initialised for verifying");
        }

        ECKeyParameters v0 = this.key;
        BigInteger v2 = ((ECPublicKeyParameters)v0).getParameters().getN();
        int v3 = v2.bitLength();
        BigInteger v4 = new BigInteger(1, arg7);
        if(v4.bitLength() > v3) {
            throw new DataLengthException("input too large for ECNR key.");
        }

        if(arg8.compareTo(ECConstants.ONE) < 0 || arg8.compareTo(v2) >= 0) {
            v0_2 = false;
        }
        else {
            if(arg9.compareTo(ECConstants.ZERO) >= 0 && arg9.compareTo(v2) < 0) {
                ECPoint v0_1 = ECAlgorithms.sumOfTwoMultiplies(((ECPublicKeyParameters)v0).getParameters().getG(), arg9, ((ECPublicKeyParameters)v0).getQ(), arg8);
                v0_2 = v0_1.isInfinity() ? false : arg8.subtract(v0_1.getX().toBigInteger()).mod(v2).equals(v4);
                return v0_2;
            }

            v0_2 = false;
        }

        return v0_2;
    }
}

