// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.security.SecureRandom;
import java.util.Hashtable;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.SignerWithRecovery;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.crypto.params.ParametersWithSalt;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;

public class ISO9796d2PSSSigner implements SignerWithRecovery {
    public static final int TRAILER_IMPLICIT = 188;
    public static final int TRAILER_RIPEMD128 = 13004;
    public static final int TRAILER_RIPEMD160 = 12748;
    public static final int TRAILER_SHA1 = 13260;
    public static final int TRAILER_SHA256 = 13516;
    public static final int TRAILER_SHA384 = 14028;
    public static final int TRAILER_SHA512 = 13772;
    public static final int TRAILER_WHIRLPOOL = 14284;
    private byte[] block;
    private AsymmetricBlockCipher cipher;
    private Digest digest;
    private boolean fullMessage;
    private int hLen;
    private int keyBits;
    private byte[] mBuf;
    private int messageLength;
    private byte[] preBlock;
    private int preMStart;
    private byte[] preSig;
    private int preTLength;
    private SecureRandom random;
    private byte[] recoveredMessage;
    private int saltLength;
    private byte[] standardSalt;
    private int trailer;
    private static Hashtable trailerMap;

    static {
        ISO9796d2PSSSigner.trailerMap = new Hashtable();
        ISO9796d2PSSSigner.trailerMap.put("RIPEMD128", Integers.valueOf(13004));
        ISO9796d2PSSSigner.trailerMap.put("RIPEMD160", Integers.valueOf(12748));
        ISO9796d2PSSSigner.trailerMap.put("SHA-1", Integers.valueOf(13260));
        ISO9796d2PSSSigner.trailerMap.put("SHA-256", Integers.valueOf(13516));
        ISO9796d2PSSSigner.trailerMap.put("SHA-384", Integers.valueOf(14028));
        ISO9796d2PSSSigner.trailerMap.put("SHA-512", Integers.valueOf(13772));
        ISO9796d2PSSSigner.trailerMap.put("Whirlpool", Integers.valueOf(14284));
    }

    public ISO9796d2PSSSigner(AsymmetricBlockCipher arg2, Digest arg3, int arg4) {
        this(arg2, arg3, arg4, false);
    }

    public ISO9796d2PSSSigner(AsymmetricBlockCipher arg3, Digest arg4, int arg5, boolean arg6) {
        super();
        this.cipher = arg3;
        this.digest = arg4;
        this.hLen = arg4.getDigestSize();
        this.saltLength = arg5;
        if(arg6) {
            this.trailer = 188;
        }
        else {
            Object v0 = ISO9796d2PSSSigner.trailerMap.get(arg4.getAlgorithmName());
            if(v0 != null) {
                this.trailer = ((Integer)v0).intValue();
            }
            else {
                goto label_17;
            }
        }

        return;
    label_17:
        throw new IllegalArgumentException("no valid trailer for digest");
    }

    private void ItoOSP(int arg3, byte[] arg4) {
        arg4[0] = ((byte)(arg3 >>> 24));
        arg4[1] = ((byte)(arg3 >>> 16));
        arg4[2] = ((byte)(arg3 >>> 8));
        arg4[3] = ((byte)(arg3 >>> 0));
    }

    private void LtoOSP(long arg5, byte[] arg7) {
        arg7[0] = ((byte)(((int)(arg5 >>> 56))));
        arg7[1] = ((byte)(((int)(arg5 >>> 48))));
        arg7[2] = ((byte)(((int)(arg5 >>> 40))));
        arg7[3] = ((byte)(((int)(arg5 >>> 32))));
        arg7[4] = ((byte)(((int)(arg5 >>> 24))));
        arg7[5] = ((byte)(((int)(arg5 >>> 16))));
        arg7[6] = ((byte)(((int)(arg5 >>> 8))));
        arg7[7] = ((byte)(((int)(arg5 >>> 0))));
    }

    private void clearBlock(byte[] arg4) {
        int v0;
        for(v0 = 0; v0 != arg4.length; ++v0) {
            arg4[v0] = 0;
        }
    }

    public byte[] generateSignature() {
        int v9 = 188;
        byte[] v0 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v0, 0);
        byte[] v1 = new byte[8];
        this.LtoOSP(((long)(this.messageLength * 8)), v1);
        this.digest.update(v1, 0, v1.length);
        this.digest.update(this.mBuf, 0, this.messageLength);
        this.digest.update(v0, 0, v0.length);
        if(this.standardSalt != null) {
            v0 = this.standardSalt;
        }
        else {
            v0 = new byte[this.saltLength];
            this.random.nextBytes(v0);
        }

        this.digest.update(v0, 0, v0.length);
        byte[] v4 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v4, 0);
        int v1_1 = 2;
        if(this.trailer == v9) {
            v1_1 = 1;
        }

        int v5 = this.block.length - this.messageLength - v0.length - this.hLen - v1_1 - 1;
        this.block[v5] = 1;
        System.arraycopy(this.mBuf, 0, this.block, v5 + 1, this.messageLength);
        System.arraycopy(v0, 0, this.block, v5 + 1 + this.messageLength, v0.length);
        byte[] v2 = this.maskGeneratorFunction1(v4, 0, v4.length, this.block.length - this.hLen - v1_1);
        int v0_1;
        for(v0_1 = 0; v0_1 != v2.length; ++v0_1) {
            this.block[v0_1] = ((byte)(this.block[v0_1] ^ v2[v0_1]));
        }

        System.arraycopy(v4, 0, this.block, this.block.length - this.hLen - v1_1, this.hLen);
        if(this.trailer == v9) {
            this.block[this.block.length - 1] = -68;
        }
        else {
            this.block[this.block.length - 2] = ((byte)(this.trailer >>> 8));
            this.block[this.block.length - 1] = ((byte)this.trailer);
        }

        this.block[0] = ((byte)(this.block[0] & 127));
        v0 = this.cipher.processBlock(this.block, 0, this.block.length);
        this.clearBlock(this.mBuf);
        this.clearBlock(this.block);
        this.messageLength = 0;
        return v0;
    }

    public byte[] getRecoveredMessage() {
        return this.recoveredMessage;
    }

    public boolean hasFullMessage() {
        return this.fullMessage;
    }

    public void init(boolean arg5, CipherParameters arg6) {
        int v0_1;
        CipherParameters v0;
        int v1 = this.saltLength;
        if((arg6 instanceof ParametersWithRandom)) {
            v0 = ((ParametersWithRandom)arg6).getParameters();
            if(arg5) {
                this.random = ((ParametersWithRandom)arg6).getRandom();
            }

            arg6 = v0;
            v0_1 = v1;
        }
        else {
            if((arg6 instanceof ParametersWithSalt)) {
                v0 = ((ParametersWithSalt)arg6).getParameters();
                this.standardSalt = ((ParametersWithSalt)arg6).getSalt();
                v1 = this.standardSalt.length;
                if(this.standardSalt.length != this.saltLength) {
                    throw new IllegalArgumentException("Fixed salt is of wrong length");
                }

                arg6 = v0;
                v0_1 = v1;
                goto label_9;
            }

            if(arg5) {
                this.random = new SecureRandom();
            }

            v0_1 = v1;
        }

    label_9:
        this.cipher.init(arg5, arg6);
        this.keyBits = ((RSAKeyParameters)arg6).getModulus().bitLength();
        this.block = new byte[(this.keyBits + 7) / 8];
        this.mBuf = this.trailer == 188 ? new byte[this.block.length - this.digest.getDigestSize() - v0_1 - 2] : new byte[this.block.length - this.digest.getDigestSize() - v0_1 - 3];
        this.reset();
    }

    private boolean isSameAs(byte[] arg6, byte[] arg7) {
        boolean v0 = true;
        if(this.messageLength != arg7.length) {
            v0 = false;
        }

        boolean v2 = v0;
        int v0_1;
        for(v0_1 = 0; v0_1 != arg7.length; ++v0_1) {
            if(arg6[v0_1] != arg7[v0_1]) {
                v2 = false;
            }
        }

        return v2;
    }

    private byte[] maskGeneratorFunction1(byte[] arg8, int arg9, int arg10, int arg11) {
        byte[] v2 = new byte[arg11];
        byte[] v3 = new byte[this.hLen];
        byte[] v4 = new byte[4];
        this.digest.reset();
        int v0;
        for(v0 = 0; v0 < arg11 / this.hLen; ++v0) {
            this.ItoOSP(v0, v4);
            this.digest.update(arg8, arg9, arg10);
            this.digest.update(v4, 0, v4.length);
            this.digest.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, this.hLen * v0, this.hLen);
        }

        if(this.hLen * v0 < arg11) {
            this.ItoOSP(v0, v4);
            this.digest.update(arg8, arg9, arg10);
            this.digest.update(v4, 0, v4.length);
            this.digest.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, this.hLen * v0, v2.length - v0 * this.hLen);
        }

        return v2;
    }

    public void reset() {
        byte[] v1 = null;
        this.digest.reset();
        this.messageLength = 0;
        if(this.mBuf != null) {
            this.clearBlock(this.mBuf);
        }

        if(this.recoveredMessage != null) {
            this.clearBlock(this.recoveredMessage);
            this.recoveredMessage = v1;
        }

        this.fullMessage = false;
        if(this.preSig != null) {
            this.preSig = v1;
            this.clearBlock(this.preBlock);
            this.preBlock = v1;
        }
    }

    public void update(byte arg4) {
        if(this.preSig != null || this.messageLength >= this.mBuf.length) {
            this.digest.update(arg4);
        }
        else {
            byte[] v0 = this.mBuf;
            int v1 = this.messageLength;
            this.messageLength = v1 + 1;
            v0[v1] = arg4;
        }
    }

    public void update(byte[] arg3, int arg4, int arg5) {
        if(this.preSig == null) {
            while(arg5 > 0) {
                if(this.messageLength >= this.mBuf.length) {
                    break;
                }

                this.update(arg3[arg4]);
                ++arg4;
                --arg5;
            }
        }

        if(arg5 > 0) {
            this.digest.update(arg3, arg4, arg5);
        }
    }

    public void updateWithRecoveredMessage(byte[] arg9) {
        int v3;
        int v0_1;
        boolean v2 = true;
        byte[] v1 = this.cipher.processBlock(arg9, 0, arg9.length);
        if(v1.length < (this.keyBits + 7) / 8) {
            byte[] v0 = new byte[(this.keyBits + 7) / 8];
            System.arraycopy(v1, 0, v0, v0.length - v1.length, v1.length);
            this.clearBlock(v1);
            v1 = v0;
        }

        if((v1[v1.length - 1] & 255 ^ 188) == 0) {
            v0_1 = 1;
        }
        else {
            v3 = v1[v1.length - 1] & 255 | (v1[v1.length - 2] & 255) << 8;
            Object v0_2 = ISO9796d2PSSSigner.trailerMap.get(this.digest.getAlgorithmName());
            if(v0_2 == null) {
                goto label_78;
            }
            else if(v3 != ((Integer)v0_2).intValue()) {
                throw new IllegalStateException("signer initialised with wrong digest for trailer " + v3);
            }
            else {
                v0_1 = 2;
            }
        }

        this.digest.doFinal(new byte[this.hLen], 0);
        byte[] v5 = this.maskGeneratorFunction1(v1, v1.length - this.hLen - v0_1, this.hLen, v1.length - this.hLen - v0_1);
        for(v3 = 0; v3 != v5.length; ++v3) {
            v1[v3] = ((byte)(v1[v3] ^ v5[v3]));
        }

        v1[0] = ((byte)(v1[0] & 127));
        for(v3 = 0; v3 != v1.length; ++v3) {
            if(v1[v3] == 1) {
                break;
            }
        }

        ++v3;
        if(v3 >= v1.length) {
            this.clearBlock(v1);
        }

        if(v3 <= 1) {
            v2 = false;
        }

        this.fullMessage = v2;
        this.recoveredMessage = new byte[v5.length - v3 - this.saltLength];
        System.arraycopy(v1, v3, this.recoveredMessage, 0, this.recoveredMessage.length);
        System.arraycopy(this.recoveredMessage, 0, this.mBuf, 0, this.recoveredMessage.length);
        this.preSig = arg9;
        this.preBlock = v1;
        this.preMStart = v3;
        this.preTLength = v0_1;
        return;
    label_78:
        throw new IllegalArgumentException("unrecognised hash in signature");
    }

    public boolean verifySignature(byte[] arg10) {
        byte[] v6 = null;
        boolean v1 = false;
        byte[] v0 = new byte[this.hLen];
        this.digest.doFinal(v0, 0);
        if(this.preSig == null) {
            try {
                this.updateWithRecoveredMessage(arg10);
            }
            catch(Exception v0_1) {
                return v1;
            }
        }
        else if(!Arrays.areEqual(this.preSig, arg10)) {
            throw new IllegalStateException("updateWithRecoveredMessage called on different signature");
        }

        byte[] v4 = this.preBlock;
        int v2 = this.preMStart;
        int v5 = this.preTLength;
        this.preSig = v6;
        this.preBlock = v6;
        v6 = new byte[8];
        this.LtoOSP(((long)(this.recoveredMessage.length * 8)), v6);
        this.digest.update(v6, 0, v6.length);
        if(this.recoveredMessage.length != 0) {
            this.digest.update(this.recoveredMessage, 0, this.recoveredMessage.length);
        }

        this.digest.update(v0, 0, v0.length);
        this.digest.update(v4, v2 + this.recoveredMessage.length, this.saltLength);
        v6 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v6, 0);
        v5 = v4.length - v5 - v6.length;
        int v0_2 = 0;
        v2 = 1;
        while(v0_2 != v6.length) {
            if(v6[v0_2] != v4[v5 + v0_2]) {
                v2 = 0;
            }

            ++v0_2;
        }

        this.clearBlock(v4);
        this.clearBlock(v6);
        if(v2 == 0) {
            this.fullMessage = false;
            this.clearBlock(this.recoveredMessage);
            return v1;
        }

        if(this.messageLength != 0) {
            if(!this.isSameAs(this.mBuf, this.recoveredMessage)) {
                this.clearBlock(this.mBuf);
                return v1;
            }
            else {
                this.messageLength = 0;
            }
        }

        this.clearBlock(this.mBuf);
        return true;
    }
}

