// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.util.Hashtable;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.SignerWithRecovery;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;

public class ISO9796d2Signer implements SignerWithRecovery {
    public static final int TRAILER_IMPLICIT = 188;
    public static final int TRAILER_RIPEMD128 = 13004;
    public static final int TRAILER_RIPEMD160 = 12748;
    public static final int TRAILER_SHA1 = 13260;
    public static final int TRAILER_SHA256 = 13516;
    public static final int TRAILER_SHA384 = 14028;
    public static final int TRAILER_SHA512 = 13772;
    public static final int TRAILER_WHIRLPOOL = 14284;
    private byte[] block;
    private AsymmetricBlockCipher cipher;
    private Digest digest;
    private boolean fullMessage;
    private int keyBits;
    private byte[] mBuf;
    private int messageLength;
    private byte[] preBlock;
    private byte[] preSig;
    private byte[] recoveredMessage;
    private int trailer;
    private static Hashtable trailerMap;

    static {
        ISO9796d2Signer.trailerMap = new Hashtable();
        ISO9796d2Signer.trailerMap.put("RIPEMD128", Integers.valueOf(13004));
        ISO9796d2Signer.trailerMap.put("RIPEMD160", Integers.valueOf(12748));
        ISO9796d2Signer.trailerMap.put("SHA-1", Integers.valueOf(13260));
        ISO9796d2Signer.trailerMap.put("SHA-256", Integers.valueOf(13516));
        ISO9796d2Signer.trailerMap.put("SHA-384", Integers.valueOf(14028));
        ISO9796d2Signer.trailerMap.put("SHA-512", Integers.valueOf(13772));
        ISO9796d2Signer.trailerMap.put("Whirlpool", Integers.valueOf(14284));
    }

    public ISO9796d2Signer(AsymmetricBlockCipher arg2, Digest arg3) {
        this(arg2, arg3, false);
    }

    public ISO9796d2Signer(AsymmetricBlockCipher arg3, Digest arg4, boolean arg5) {
        super();
        this.cipher = arg3;
        this.digest = arg4;
        if(arg5) {
            this.trailer = 188;
        }
        else {
            Object v0 = ISO9796d2Signer.trailerMap.get(arg4.getAlgorithmName());
            if(v0 != null) {
                this.trailer = ((Integer)v0).intValue();
            }
            else {
                goto label_14;
            }
        }

        return;
    label_14:
        throw new IllegalArgumentException("no valid trailer for digest");
    }

    private void clearBlock(byte[] arg4) {
        int v0;
        for(v0 = 0; v0 != arg4.length; ++v0) {
            arg4[v0] = 0;
        }
    }

    public byte[] generateSignature() {
        int v0;
        int v1;
        int v2 = this.digest.getDigestSize();
        if(this.trailer == 188) {
            v1 = 8;
            v0 = this.block.length - v2 - 1;
            this.digest.doFinal(this.block, v0);
            this.block[this.block.length - 1] = -68;
        }
        else {
            v1 = 16;
            v0 = this.block.length - v2 - 2;
            this.digest.doFinal(this.block, v0);
            this.block[this.block.length - 2] = ((byte)(this.trailer >>> 8));
            this.block[this.block.length - 1] = ((byte)this.trailer);
        }

        v1 = v1 + (v2 + this.messageLength) * 8 + 4 - this.keyBits;
        if(v1 > 0) {
            v2 = this.messageLength - (v1 + 7) / 8;
            v1 = 96;
            v0 -= v2;
            System.arraycopy(this.mBuf, 0, this.block, v0, v2);
            v2 = v0;
        }
        else {
            v1 = 64;
            v0 -= this.messageLength;
            System.arraycopy(this.mBuf, 0, this.block, v0, this.messageLength);
            v2 = v0;
        }

        if(v2 - 1 > 0) {
            for(v0 = v2 - 1; v0 != 0; --v0) {
                this.block[v0] = -69;
            }

            --v2;
            this.block[v2] = ((byte)(this.block[v2] ^ 1));
            this.block[0] = 11;
            this.block[0] = ((byte)(v1 | this.block[0]));
        }
        else {
            this.block[0] = 10;
            this.block[0] = ((byte)(v1 | this.block[0]));
        }

        byte[] v0_1 = this.cipher.processBlock(this.block, 0, this.block.length);
        this.clearBlock(this.mBuf);
        this.clearBlock(this.block);
        return v0_1;
    }

    public byte[] getRecoveredMessage() {
        return this.recoveredMessage;
    }

    public boolean hasFullMessage() {
        return this.fullMessage;
    }

    public void init(boolean arg3, CipherParameters arg4) {
        this.cipher.init(arg3, arg4);
        this.keyBits = ((RSAKeyParameters)arg4).getModulus().bitLength();
        this.block = new byte[(this.keyBits + 7) / 8];
        this.mBuf = this.trailer == 188 ? new byte[this.block.length - this.digest.getDigestSize() - 2] : new byte[this.block.length - this.digest.getDigestSize() - 3];
        this.reset();
    }

    private boolean isSameAs(byte[] arg6, byte[] arg7) {
        boolean v2;
        boolean v0 = true;
        if(this.messageLength > this.mBuf.length) {
            if(this.mBuf.length > arg7.length) {
                v0 = false;
            }

            v2 = v0;
            int v0_1;
            for(v0_1 = 0; v0_1 != this.mBuf.length; ++v0_1) {
                if(arg6[v0_1] != arg7[v0_1]) {
                    v2 = false;
                }
            }
        }
        else {
            if(this.messageLength != arg7.length) {
                v0 = false;
            }

            v2 = v0;
            for(v0_1 = 0; v0_1 != arg7.length; ++v0_1) {
                if(arg6[v0_1] != arg7[v0_1]) {
                    v2 = false;
                }
            }
        }

        return v2;
    }

    public void reset() {
        byte[] v1 = null;
        this.digest.reset();
        this.messageLength = 0;
        this.clearBlock(this.mBuf);
        if(this.recoveredMessage != null) {
            this.clearBlock(this.recoveredMessage);
        }

        this.recoveredMessage = v1;
        this.fullMessage = false;
        if(this.preSig != null) {
            this.preSig = v1;
            this.clearBlock(this.preBlock);
            this.preBlock = v1;
        }
    }

    private boolean returnFalse(byte[] arg2) {
        this.clearBlock(this.mBuf);
        this.clearBlock(arg2);
        return 0;
    }

    public void update(byte arg3) {
        this.digest.update(arg3);
        if(this.messageLength < this.mBuf.length) {
            this.mBuf[this.messageLength] = arg3;
        }

        ++this.messageLength;
    }

    public void update(byte[] arg3, int arg4, int arg5) {
        while(arg5 > 0) {
            if(this.messageLength >= this.mBuf.length) {
                break;
            }

            this.update(arg3[arg4]);
            ++arg4;
            --arg5;
        }

        this.digest.update(arg3, arg4, arg5);
        this.messageLength += arg5;
    }

    public void updateWithRecoveredMessage(byte[] arg7) {
        int v2;
        int v0;
        byte[] v4 = this.cipher.processBlock(arg7, 0, arg7.length);
        if((v4[0] & 192 ^ 64) != 0) {
            throw new InvalidCipherTextException("malformed signature");
        }

        if((v4[v4.length - 1] & 15 ^ 12) != 0) {
            throw new InvalidCipherTextException("malformed signature");
        }

        if((v4[v4.length - 1] & 255 ^ 188) == 0) {
            v0 = 1;
        }
        else {
            v2 = v4[v4.length - 1] & 255 | (v4[v4.length - 2] & 255) << 8;
            Object v0_1 = ISO9796d2Signer.trailerMap.get(this.digest.getAlgorithmName());
            if(v0_1 == null) {
                goto label_75;
            }
            else if(v2 != ((Integer)v0_1).intValue()) {
                throw new IllegalStateException("signer initialised with wrong digest for trailer " + v2);
            }
            else {
                v0 = 2;
            }
        }

        for(v2 = 0; v2 != v4.length; ++v2) {
            if((v4[v2] & 15 ^ 10) == 0) {
                break;
            }
        }

        ++v2;
        v0 = v4.length - v0 - this.digest.getDigestSize();
        if(v0 - v2 <= 0) {
            throw new InvalidCipherTextException("malformed block");
        }

        if((v4[0] & 32) == 0) {
            this.fullMessage = true;
            this.recoveredMessage = new byte[v0 - v2];
            System.arraycopy(v4, v2, this.recoveredMessage, 0, this.recoveredMessage.length);
        }
        else {
            this.fullMessage = false;
            this.recoveredMessage = new byte[v0 - v2];
            System.arraycopy(v4, v2, this.recoveredMessage, 0, this.recoveredMessage.length);
        }

        this.preSig = arg7;
        this.preBlock = v4;
        this.digest.update(this.recoveredMessage, 0, this.recoveredMessage.length);
        this.messageLength = this.recoveredMessage.length;
        System.arraycopy(this.recoveredMessage, 0, this.mBuf, 0, this.recoveredMessage.length);
        return;
    label_75:
        throw new IllegalArgumentException("unrecognised hash in signature");
    }

    public boolean verifySignature(byte[] arg12) {
        int v8;
        int v3;
        int v0_2;
        byte[] v1 = null;
        boolean v4 = false;
        if(this.preSig == null) {
            try {
                v1 = this.cipher.processBlock(arg12, 0, arg12.length);
            }
            catch(Exception v0) {
                return v4;
            }
        }
        else if(!Arrays.areEqual(this.preSig, arg12)) {
            throw new IllegalStateException("updateWithRecoveredMessage called on different signature");
        }
        else {
            byte[] v0_1 = this.preBlock;
            this.preSig = v1;
            this.preBlock = v1;
            v1 = v0_1;
        }

        if((v1[0] & 192 ^ 64) != 0) {
            return this.returnFalse(v1);
        }

        if((v1[v1.length - 1] & 15 ^ 12) != 0) {
            return this.returnFalse(v1);
        }

        if((v1[v1.length - 1] & 255 ^ 188) == 0) {
            v0_2 = 1;
        }
        else {
            v3 = v1[v1.length - 1] & 255 | (v1[v1.length - 2] & 255) << 8;
            Object v0_3 = ISO9796d2Signer.trailerMap.get(this.digest.getAlgorithmName());
            if(v0_3 == null) {
                goto label_88;
            }
            else if(v3 != ((Integer)v0_3).intValue()) {
                throw new IllegalStateException("signer initialised with wrong digest for trailer " + v3);
            }
            else {
                v0_2 = 2;
            }
        }

        for(v3 = 0; v3 != v1.length; ++v3) {
            if((v1[v3] & 15 ^ 10) == 0) {
                break;
            }
        }

        int v5 = v3 + 1;
        byte[] v6 = new byte[this.digest.getDigestSize()];
        int v7 = v1.length - v0_2 - v6.length;
        if(v7 - v5 <= 0) {
            return this.returnFalse(v1);
        }

        if((v1[0] & 32) == 0) {
            this.fullMessage = true;
            if(this.messageLength > v7 - v5) {
                return this.returnFalse(v1);
            }
            else {
                this.digest.reset();
                this.digest.update(v1, v5, v7 - v5);
                this.digest.doFinal(v6, 0);
                v0_2 = 0;
                v3 = 1;
                goto label_114;
            }
        }
        else {
            this.fullMessage = false;
            this.digest.doFinal(v6, 0);
            v0_2 = 0;
            v3 = 1;
            while(v0_2 != v6.length) {
                v8 = v7 + v0_2;
                v1[v8] = ((byte)(v1[v8] ^ v6[v0_2]));
                if(v1[v7 + v0_2] != 0) {
                    v3 = 0;
                }

                ++v0_2;
            }

            if(v3 == 0) {
                return this.returnFalse(v1);
            }

            this.recoveredMessage = new byte[v7 - v5];
            System.arraycopy(v1, v5, this.recoveredMessage, 0, this.recoveredMessage.length);
            goto label_138;
        label_114:
            while(v0_2 != v6.length) {
                v8 = v7 + v0_2;
                v1[v8] = ((byte)(v1[v8] ^ v6[v0_2]));
                if(v1[v7 + v0_2] != 0) {
                    v3 = 0;
                }

                ++v0_2;
            }

            if(v3 == 0) {
                return this.returnFalse(v1);
            }

            this.recoveredMessage = new byte[v7 - v5];
            System.arraycopy(v1, v5, this.recoveredMessage, 0, this.recoveredMessage.length);
        }

    label_138:
        if(this.messageLength != 0 && !this.isSameAs(this.mBuf, this.recoveredMessage)) {
            v4 = this.returnFalse(v1);
        }
        else {
            this.clearBlock(this.mBuf);
            this.clearBlock(v1);
            v4 = true;
        }

        return v4;
    label_88:
        throw new IllegalArgumentException("unrecognised hash in signature");
    }
}

