// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.security.SecureRandom;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.Signer;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.crypto.params.RSABlindingParameters;
import org.bouncycastle.crypto.params.RSAKeyParameters;

public class PSSSigner implements Signer {
    public static final byte TRAILER_IMPLICIT = -68;
    private byte[] block;
    private AsymmetricBlockCipher cipher;
    private Digest contentDigest;
    private int emBits;
    private int hLen;
    private byte[] mDash;
    private Digest mgfDigest;
    private int mgfhLen;
    private SecureRandom random;
    private int sLen;
    private byte[] salt;
    private byte trailer;

    public PSSSigner(AsymmetricBlockCipher arg2, Digest arg3, int arg4) {
        this(arg2, arg3, arg4, -68);
    }

    public PSSSigner(AsymmetricBlockCipher arg7, Digest arg8, int arg9, byte arg10) {
        this(arg7, arg8, arg8, arg9, arg10);
    }

    public PSSSigner(AsymmetricBlockCipher arg3, Digest arg4, Digest arg5, int arg6, byte arg7) {
        super();
        this.cipher = arg3;
        this.contentDigest = arg4;
        this.mgfDigest = arg5;
        this.hLen = arg4.getDigestSize();
        this.mgfhLen = arg5.getDigestSize();
        this.sLen = arg6;
        this.salt = new byte[arg6];
        this.mDash = new byte[arg6 + 8 + this.hLen];
        this.trailer = arg7;
    }

    public PSSSigner(AsymmetricBlockCipher arg7, Digest arg8, Digest arg9, int arg10) {
        this(arg7, arg8, arg9, arg10, -68);
    }

    private void ItoOSP(int arg3, byte[] arg4) {
        arg4[0] = ((byte)(arg3 >>> 24));
        arg4[1] = ((byte)(arg3 >>> 16));
        arg4[2] = ((byte)(arg3 >>> 8));
        arg4[3] = ((byte)(arg3 >>> 0));
    }

    private void clearBlock(byte[] arg4) {
        int v0;
        for(v0 = 0; v0 != arg4.length; ++v0) {
            arg4[v0] = 0;
        }
    }

    public byte[] generateSignature() {
        this.contentDigest.doFinal(this.mDash, this.mDash.length - this.hLen - this.sLen);
        if(this.sLen != 0) {
            this.random.nextBytes(this.salt);
            System.arraycopy(this.salt, 0, this.mDash, this.mDash.length - this.sLen, this.sLen);
        }

        byte[] v2 = new byte[this.hLen];
        this.contentDigest.update(this.mDash, 0, this.mDash.length);
        this.contentDigest.doFinal(v2, 0);
        this.block[this.block.length - this.sLen - 1 - this.hLen - 1] = 1;
        System.arraycopy(this.salt, 0, this.block, this.block.length - this.sLen - this.hLen - 1, this.sLen);
        byte[] v3 = this.maskGeneratorFunction1(v2, 0, v2.length, this.block.length - this.hLen - 1);
        int v0;
        for(v0 = 0; v0 != v3.length; ++v0) {
            this.block[v0] = ((byte)(this.block[v0] ^ v3[v0]));
        }

        this.block[0] = ((byte)(this.block[0] & 255 >> this.block.length * 8 - this.emBits));
        System.arraycopy(v2, 0, this.block, this.block.length - this.hLen - 1, this.hLen);
        this.block[this.block.length - 1] = this.trailer;
        byte[] v0_1 = this.cipher.processBlock(this.block, 0, this.block.length);
        this.clearBlock(this.block);
        return v0_1;
    }

    public void init(boolean arg4, CipherParameters arg5) {
        CipherParameters v0;
        if((arg5 instanceof ParametersWithRandom)) {
            v0 = ((ParametersWithRandom)arg5).getParameters();
            this.random = ((ParametersWithRandom)arg5).getRandom();
        }
        else {
            if(arg4) {
                this.random = new SecureRandom();
            }

            v0 = arg5;
        }

        this.cipher.init(arg4, v0);
        if((v0 instanceof RSABlindingParameters)) {
            RSAKeyParameters v0_1 = ((RSABlindingParameters)v0).getPublicKey();
        }

        this.emBits = ((RSAKeyParameters)v0).getModulus().bitLength() - 1;
        if(this.emBits < this.hLen * 8 + this.sLen * 8 + 9) {
            throw new IllegalArgumentException("key too small for specified hash and salt lengths");
        }

        this.block = new byte[(this.emBits + 7) / 8];
        this.reset();
    }

    private byte[] maskGeneratorFunction1(byte[] arg8, int arg9, int arg10, int arg11) {
        byte[] v2 = new byte[arg11];
        byte[] v3 = new byte[this.mgfhLen];
        byte[] v4 = new byte[4];
        this.mgfDigest.reset();
        int v0;
        for(v0 = 0; v0 < arg11 / this.mgfhLen; ++v0) {
            this.ItoOSP(v0, v4);
            this.mgfDigest.update(arg8, arg9, arg10);
            this.mgfDigest.update(v4, 0, v4.length);
            this.mgfDigest.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, this.mgfhLen * v0, this.mgfhLen);
        }

        if(this.mgfhLen * v0 < arg11) {
            this.ItoOSP(v0, v4);
            this.mgfDigest.update(arg8, arg9, arg10);
            this.mgfDigest.update(v4, 0, v4.length);
            this.mgfDigest.doFinal(v3, 0);
            System.arraycopy(v3, 0, v2, this.mgfhLen * v0, v2.length - v0 * this.mgfhLen);
        }

        return v2;
    }

    public void reset() {
        this.contentDigest.reset();
    }

    public void update(byte arg2) {
        this.contentDigest.update(arg2);
    }

    public void update(byte[] arg2, int arg3, int arg4) {
        this.contentDigest.update(arg2, arg3, arg4);
    }

    public boolean verifySignature(byte[] arg8) {
        boolean v1 = false;
        this.contentDigest.doFinal(this.mDash, this.mDash.length - this.hLen - this.sLen);
        try {
            byte[] v0_1 = this.cipher.processBlock(arg8, 0, arg8.length);
            System.arraycopy(v0_1, 0, this.block, this.block.length - v0_1.length, v0_1.length);
        }
        catch(Exception v0) {
            return v1;
        }

        if(this.block[this.block.length - 1] != this.trailer) {
            this.clearBlock(this.block);
            return v1;
        }

        byte[] v2 = this.maskGeneratorFunction1(this.block, this.block.length - this.hLen - 1, this.hLen, this.block.length - this.hLen - 1);
        int v0_2;
        for(v0_2 = 0; v0_2 != v2.length; ++v0_2) {
            this.block[v0_2] = ((byte)(this.block[v0_2] ^ v2[v0_2]));
        }

        this.block[0] = ((byte)(this.block[0] & 255 >> this.block.length * 8 - this.emBits));
        for(v0_2 = 0; v0_2 != this.block.length - this.hLen - this.sLen - 2; ++v0_2) {
            if(this.block[v0_2] != 0) {
                this.clearBlock(this.block);
                return v1;
            }
        }

        if(this.block[this.block.length - this.hLen - this.sLen - 2] != 1) {
            this.clearBlock(this.block);
            return v1;
        }

        System.arraycopy(this.block, this.block.length - this.sLen - this.hLen - 1, this.mDash, this.mDash.length - this.sLen, this.sLen);
        this.contentDigest.update(this.mDash, 0, this.mDash.length);
        this.contentDigest.doFinal(this.mDash, this.mDash.length - this.hLen);
        int v2_1 = this.block.length - this.hLen - 1;
        for(v0_2 = this.mDash.length - this.hLen; v0_2 != this.mDash.length; ++v0_2) {
            if((this.block[v2_1] ^ this.mDash[v0_2]) != 0) {
                this.clearBlock(this.mDash);
                this.clearBlock(this.block);
                return v1;
            }

            ++v2_1;
        }

        this.clearBlock(this.mDash);
        this.clearBlock(this.block);
        return true;
    }
}

