// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.signers;

import java.io.IOException;
import java.util.Hashtable;
import org.bouncycastle.asn1.DERNull;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.DigestInfo;
import org.bouncycastle.asn1.x509.X509ObjectIdentifiers;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.Signer;
import org.bouncycastle.crypto.encodings.PKCS1Encoding;
import org.bouncycastle.crypto.engines.RSABlindedEngine;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.util.Arrays;

public class RSADigestSigner implements Signer {
    private final AlgorithmIdentifier algId;
    private final Digest digest;
    private boolean forSigning;
    private static final Hashtable oidMap;
    private final AsymmetricBlockCipher rsaEngine;

    static {
        RSADigestSigner.oidMap = new Hashtable();
        RSADigestSigner.oidMap.put("RIPEMD128", TeleTrusTObjectIdentifiers.ripemd128);
        RSADigestSigner.oidMap.put("RIPEMD160", TeleTrusTObjectIdentifiers.ripemd160);
        RSADigestSigner.oidMap.put("RIPEMD256", TeleTrusTObjectIdentifiers.ripemd256);
        RSADigestSigner.oidMap.put("SHA-1", X509ObjectIdentifiers.id_SHA1);
        RSADigestSigner.oidMap.put("SHA-224", NISTObjectIdentifiers.id_sha224);
        RSADigestSigner.oidMap.put("SHA-256", NISTObjectIdentifiers.id_sha256);
        RSADigestSigner.oidMap.put("SHA-384", NISTObjectIdentifiers.id_sha384);
        RSADigestSigner.oidMap.put("SHA-512", NISTObjectIdentifiers.id_sha512);
        RSADigestSigner.oidMap.put("MD2", PKCSObjectIdentifiers.md2);
        RSADigestSigner.oidMap.put("MD4", PKCSObjectIdentifiers.md4);
        RSADigestSigner.oidMap.put("MD5", PKCSObjectIdentifiers.md5);
    }

    public RSADigestSigner(Digest arg4) {
        super();
        this.rsaEngine = new PKCS1Encoding(new RSABlindedEngine());
        this.digest = arg4;
        this.algId = new AlgorithmIdentifier(RSADigestSigner.oidMap.get(arg4.getAlgorithmName()), DERNull.INSTANCE);
    }

    private byte[] derEncode(byte[] arg3) {
        return new DigestInfo(this.algId, arg3).getEncoded("DER");
    }

    public byte[] generateSignature() {
        if(!this.forSigning) {
            throw new IllegalStateException("RSADigestSigner not initialised for signature generation.");
        }

        byte[] v0 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v0, 0);
        try {
            v0 = this.derEncode(v0);
            return this.rsaEngine.processBlock(v0, 0, v0.length);
        }
        catch(IOException v0_1) {
            throw new CryptoException("unable to encode signature: " + v0_1.getMessage(), ((Throwable)v0_1));
        }
    }

    public String getAlgorithmName() {
        return this.digest.getAlgorithmName() + "withRSA";
    }

    public void init(boolean arg3, CipherParameters arg4) {
        this.forSigning = arg3;
        CipherParameters v0 = (arg4 instanceof ParametersWithRandom) ? arg4.getParameters() : arg4;
        if((arg3) && !((AsymmetricKeyParameter)v0).isPrivate()) {
            throw new IllegalArgumentException("signing requires private key");
        }

        if(!arg3 && (((AsymmetricKeyParameter)v0).isPrivate())) {
            throw new IllegalArgumentException("verification requires public key");
        }

        this.reset();
        this.rsaEngine.init(arg3, arg4);
    }

    public void reset() {
        this.digest.reset();
    }

    public void update(byte arg2) {
        this.digest.update(arg2);
    }

    public void update(byte[] arg2, int arg3, int arg4) {
        this.digest.update(arg2, arg3, arg4);
    }

    public boolean verifySignature(byte[] arg12) {
        byte[] v6;
        byte[] v5;
        boolean v1 = false;
        if(this.forSigning) {
            throw new IllegalStateException("RSADigestSigner not initialised for verification");
        }

        byte[] v4 = new byte[this.digest.getDigestSize()];
        this.digest.doFinal(v4, 0);
        try {
            v5 = this.rsaEngine.processBlock(arg12, 0, arg12.length);
            v6 = this.derEncode(v4);
        }
        catch(Exception v0) {
            return v1;
        }

        if(v5.length == v6.length) {
            return Arrays.constantTimeAreEqual(v5, v6);
        }

        if(v5.length == v6.length - 2) {
            int v7 = v5.length - v4.length - 2;
            int v8 = v6.length - v4.length - 2;
            v6[1] = ((byte)(v6[1] - 2));
            v6[3] = ((byte)(v6[3] - 2));
            int v0_1 = 0;
            int v2 = 0;
            while(v0_1 < v4.length) {
                v2 |= v5[v7 + v0_1] ^ v6[v8 + v0_1];
                ++v0_1;
            }

            for(v0_1 = 0; v0_1 < v7; ++v0_1) {
                v2 |= v5[v0_1] ^ v6[v0_1];
            }

            if(v2 != 0) {
                return v1;
            }

            v1 = true;
        }

        return v1;
    }
}

