// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.IOException;

class DTLSRecordLayer implements DatagramTransport {
    private static final int MAX_FRAGMENT_LENGTH = 16384;
    private static final int RECORD_HEADER_LENGTH = 13;
    private static final long RETRANSMIT_TIMEOUT = 240000;
    private static final long TCP_MSL = 120000;
    private volatile boolean closed;
    private final TlsContext context;
    private DTLSEpoch currentEpoch;
    private volatile ProtocolVersion discoveredPeerVersion;
    private volatile boolean failed;
    private volatile boolean inHandshake;
    private final TlsPeer peer;
    private DTLSEpoch pendingEpoch;
    private DTLSEpoch readEpoch;
    private final ByteQueue recordQueue;
    private DTLSHandshakeRetransmit retransmit;
    private DTLSEpoch retransmitEpoch;
    private long retransmitExpiry;
    private final DatagramTransport transport;
    private DTLSEpoch writeEpoch;

    DTLSRecordLayer(DatagramTransport arg5, TlsContext arg6, TlsPeer arg7, short arg8) {
        super();
        this.recordQueue = new ByteQueue();
        this.closed = false;
        this.failed = false;
        this.discoveredPeerVersion = null;
        this.retransmit = null;
        this.retransmitEpoch = null;
        this.retransmitExpiry = 0;
        this.transport = arg5;
        this.context = arg6;
        this.peer = arg7;
        this.inHandshake = true;
        this.currentEpoch = new DTLSEpoch(0, new TlsNullCipher(arg6));
        this.pendingEpoch = null;
        this.readEpoch = this.currentEpoch;
        this.writeEpoch = this.currentEpoch;
    }

    public void close() {
        if(!this.closed) {
            if(this.inHandshake) {
                this.warn(90, "User canceled handshake");
            }

            this.closeTransport();
        }
    }

    private void closeTransport() {
        if(!this.closed) {
            try {
                if(!this.failed) {
                    this.warn(0, null);
                }

                this.transport.close();
            }
            catch(Exception v0) {
            }

            this.closed = true;
        }
    }

    void fail(short arg4) {
        if(!this.closed) {
            short v0 = 2;
            String v1 = null;
            Exception v2 = null;
            try {
                this.raiseAlert(v0, arg4, v1, v2);
            }
            catch(Exception v0_1) {
            }

            this.failed = true;
            this.closeTransport();
        }
    }

    ProtocolVersion getDiscoveredPeerVersion() {
        return this.discoveredPeerVersion;
    }

    private static long getMacSequenceNumber(int arg3, long arg4) {
        return (((long)arg3)) << 48 | arg4;
    }

    public int getReceiveLimit() {
        return Math.min(16384, this.readEpoch.getCipher().getPlaintextLimit(this.transport.getReceiveLimit() - 13));
    }

    public int getSendLimit() {
        return Math.min(16384, this.writeEpoch.getCipher().getPlaintextLimit(this.transport.getSendLimit() - 13));
    }

    void handshakeSuccessful(DTLSHandshakeRetransmit arg5) {
        if(this.readEpoch != this.currentEpoch && this.writeEpoch != this.currentEpoch) {
            if(arg5 != null) {
                this.retransmit = arg5;
                this.retransmitEpoch = this.currentEpoch;
                this.retransmitExpiry = System.currentTimeMillis() + 240000;
            }

            this.inHandshake = false;
            this.currentEpoch = this.pendingEpoch;
            this.pendingEpoch = null;
            return;
        }

        throw new IllegalStateException();
    }

    void initPendingEpoch(TlsCipher arg3) {
        if(this.pendingEpoch != null) {
            throw new IllegalStateException();
        }

        this.pendingEpoch = new DTLSEpoch(this.writeEpoch.getEpoch() + 1, arg3);
    }

    private void raiseAlert(short arg6, short arg7, String arg8, Exception arg9) {
        this.peer.notifyAlertRaised(arg6, arg7, arg8, arg9);
        this.sendRecord(21, new byte[]{((byte)arg6), ((byte)arg7)}, 0, 2);
    }

    public int receive(byte[] arg14, int arg15, int arg16, int arg17) {
        DTLSEpoch v8;
        byte[] v5 = null;
        while(true) {
        label_1:
            int v1 = Math.min(arg16, this.getReceiveLimit()) + 13;
            if(v5 == null || v5.length < v1) {
                v5 = new byte[v1];
            }

            try {
                if(this.retransmit != null && System.currentTimeMillis() > this.retransmitExpiry) {
                    this.retransmit = null;
                    this.retransmitEpoch = null;
                }

                int v7 = this.receiveRecord(v5, 0, v1, arg17);
                if(v7 < 0) {
                    v1 = v7;
                }
                else {
                    if(v7 < 13) {
                        continue;
                    }

                    if(v7 != TlsUtils.readUint16(v5, 11) + 13) {
                        continue;
                    }

                    short v4 = TlsUtils.readUint8(v5, 0);
                    switch(v4) {
                        case 20: 
                        case 21: 
                        case 22: 
                        case 23: {
                            int v9 = TlsUtils.readUint16(v5, 3);
                            DTLSEpoch v1_2 = null;
                            if(v9 == this.readEpoch.getEpoch()) {
                                v8 = this.readEpoch;
                            }
                            else {
                                if(v4 == 22 && this.retransmitEpoch != null && v9 == this.retransmitEpoch.getEpoch()) {
                                    v8 = this.retransmitEpoch;
                                    goto label_42;
                                }

                                v8 = v1_2;
                            }

                            try {
                            label_42:
                                if(v8 == null) {
                                    goto label_1;
                                }

                                long v10 = TlsUtils.readUint48(v5, 5);
                                if(v8.getReplayWindow().shouldDiscard(v10)) {
                                    goto label_1;
                                }

                                ProtocolVersion v12 = TlsUtils.readVersion(v5, 1);
                                if(this.discoveredPeerVersion != null && !this.discoveredPeerVersion.equals(v12)) {
                                    goto label_1;
                                }

                                byte[] v1_3 = v8.getCipher().decodeCiphertext(DTLSRecordLayer.getMacSequenceNumber(v8.getEpoch(), v10), v4, v5, 13, v7 - 13);
                                v8.getReplayWindow().reportAuthenticated(v10);
                                if(this.discoveredPeerVersion == null) {
                                    this.discoveredPeerVersion = v12;
                                }

                                switch(v4) {
                                    case 20: {
                                        goto label_115;
                                    }
                                    case 21: {
                                        goto label_90;
                                    }
                                    case 22: {
                                        goto label_127;
                                    }
                                    case 23: {
                                        goto label_112;
                                    }
                                }

                                goto label_67;
                            label_127:
                                if(this.inHandshake) {
                                    goto label_67;
                                }

                                if(this.retransmit == null) {
                                    goto label_1;
                                }

                                this.retransmit.receivedHandshakeRecord(v9, v1_3, 0, v1_3.length);
                                goto label_1;
                            label_112:
                                if(this.inHandshake) {
                                    goto label_1;
                                }

                            label_67:
                                if(!this.inHandshake && this.retransmit != null) {
                                    this.retransmit = null;
                                    this.retransmitEpoch = null;
                                }

                                System.arraycopy(v1_3, 0, arg14, arg15, v1_3.length);
                                return v1_3.length;
                            label_115:
                                if(v1_3.length != 1) {
                                    goto label_1;
                                }

                                if(v1_3[0] != 1) {
                                    goto label_1;
                                }

                                if(this.pendingEpoch == null) {
                                    goto label_1;
                                }

                                this.readEpoch = this.pendingEpoch;
                                goto label_1;
                            label_90:
                                if(v1_3.length != 2) {
                                    goto label_1;
                                }

                                short v2 = ((short)v1_3[0]);
                                short v1_4 = ((short)v1_3[1]);
                                this.peer.notifyAlertReceived(v2, v1_4);
                                if(v2 != 2) {
                                    goto label_109;
                                }

                                this.fail(v1_4);
                                throw new TlsFatalAlert(v1_4);
                            }
                            catch(IOException v1_1) {
                                goto label_108;
                            }
                        }
                        default: {
                            goto label_1;
                        }
                    }
                }

                return v1;
            }
            catch(IOException v1_1) {
                goto label_108;
            }

        label_109:
            if(v1_4 != 0) {
                continue;
            }

            try {
                this.closeTransport();
                continue;
            }
            catch(IOException v1_1) {
            label_108:
                throw v1_1;
            }
        }
    }

    private int receiveRecord(byte[] arg6, int arg7, int arg8, int arg9) {
        int v0_1;
        int v2 = 13;
        int v4 = 2;
        if(this.recordQueue.size() > 0) {
            if(this.recordQueue.size() >= v2) {
                byte[] v0 = new byte[v4];
                this.recordQueue.read(v0, 0, v4, 11);
                v0_1 = TlsUtils.readUint16(v0, 0);
            }
            else {
                v0_1 = 0;
            }

            v0_1 = Math.min(this.recordQueue.size(), v0_1 + 13);
            this.recordQueue.read(arg6, arg7, v0_1, 0);
            this.recordQueue.removeData(v0_1);
        }
        else {
            int v1 = this.transport.receive(arg6, arg7, arg8, arg9);
            if(v1 >= v2) {
                v0_1 = TlsUtils.readUint16(arg6, arg7 + 11) + 13;
                if(v1 > v0_1) {
                    this.recordQueue.addData(arg6, arg7 + v0_1, v1 - v0_1);
                    return v0_1;
                }
            }

            v0_1 = v1;
        }

        return v0_1;
    }

    void resetWriteEpoch() {
        this.writeEpoch = this.retransmitEpoch != null ? this.retransmitEpoch : this.currentEpoch;
    }

    public void send(byte[] arg8, int arg9, int arg10) {
        short v6 = 20;
        short v0 = 23;
        if((this.inHandshake) || this.writeEpoch == this.retransmitEpoch) {
            v0 = 22;
            if(TlsUtils.readUint8(arg8, arg9) == v6) {
                DTLSEpoch v1 = null;
                if(this.inHandshake) {
                    v1 = this.pendingEpoch;
                }
                else if(this.writeEpoch == this.retransmitEpoch) {
                    v1 = this.currentEpoch;
                }

                if(v1 == null) {
                    throw new IllegalStateException();
                }

                byte[] v2 = new byte[]{1};
                this.sendRecord(v6, v2, 0, v2.length);
                this.writeEpoch = v1;
            }
        }

        this.sendRecord(v0, arg8, arg9, arg10);
    }

    private void sendRecord(short arg11, byte[] arg12, int arg13, int arg14) {
        if(arg14 < 1 && arg11 != 23) {
            throw new TlsFatalAlert(80);
        }

        int v7 = this.writeEpoch.getEpoch();
        long v8 = this.writeEpoch.allocateSequenceNumber();
        byte[] v1 = this.writeEpoch.getCipher().encodePlaintext(DTLSRecordLayer.getMacSequenceNumber(v7, v8), arg11, arg12, arg13, arg14);
        if(v1.length > 16384) {
            throw new TlsFatalAlert(80);
        }

        byte[] v2 = new byte[v1.length + 13];
        TlsUtils.writeUint8(arg11, v2, 0);
        ProtocolVersion v0 = this.discoveredPeerVersion != null ? this.discoveredPeerVersion : this.context.getClientVersion();
        TlsUtils.writeVersion(v0, v2, 1);
        TlsUtils.writeUint16(v7, v2, 3);
        TlsUtils.writeUint48(v8, v2, 5);
        TlsUtils.writeUint16(v1.length, v2, 11);
        System.arraycopy(v1, 0, v2, 13, v1.length);
        this.transport.send(v2, 0, v2.length);
    }

    void warn(short arg3, String arg4) {
        this.raiseAlert(1, arg3, arg4, null);
    }
}

