// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import org.bouncycastle.crypto.Digest;

class RecordStream {
    private static int CIPHERTEXT_LIMIT;
    private static int COMPRESSED_LIMIT;
    private static int PLAINTEXT_LIMIT;
    private ByteArrayOutputStream buffer;
    private TlsContext context;
    private TlsProtocol handler;
    private TlsHandshakeHash hash;
    private InputStream input;
    private OutputStream output;
    private TlsCipher pendingCipher;
    private TlsCompression pendingCompression;
    private TlsCipher readCipher;
    private TlsCompression readCompression;
    private long readSeqNo;
    private ProtocolVersion readVersion;
    private boolean restrictReadVersion;
    private TlsCipher writeCipher;
    private TlsCompression writeCompression;
    private long writeSeqNo;
    private ProtocolVersion writeVersion;

    static {
        RecordStream.PLAINTEXT_LIMIT = 16384;
        RecordStream.COMPRESSED_LIMIT = RecordStream.PLAINTEXT_LIMIT + 1024;
        RecordStream.CIPHERTEXT_LIMIT = RecordStream.COMPRESSED_LIMIT + 1024;
    }

    RecordStream(TlsProtocol arg5, InputStream arg6, OutputStream arg7) {
        super();
        this.pendingCompression = null;
        this.readCompression = null;
        this.writeCompression = null;
        this.pendingCipher = null;
        this.readCipher = null;
        this.writeCipher = null;
        this.readSeqNo = 0;
        this.writeSeqNo = 0;
        this.buffer = new ByteArrayOutputStream();
        this.context = null;
        this.hash = null;
        this.readVersion = null;
        this.writeVersion = null;
        this.restrictReadVersion = true;
        this.handler = arg5;
        this.input = arg6;
        this.output = arg7;
        this.readCompression = new TlsNullCompression();
        this.writeCompression = this.readCompression;
        this.readCipher = new TlsNullCipher(this.context);
        this.writeCipher = this.readCipher;
    }

    private static void checkLength(int arg1, int arg2, short arg3) {
        if(arg1 > arg2) {
            throw new TlsFatalAlert(arg3);
        }
    }

    private static void checkType(short arg1, short arg2) {
        switch(arg1) {
            case 20: 
            case 21: 
            case 22: 
            case 23: {
                return;
            }
        }

        throw new TlsFatalAlert(arg2);
    }

    protected void close() {
        IOException v0 = null;
        try {
            this.input.close();
        }
        catch(IOException v0) {
        }

        try {
            this.output.close();
        }
        catch(IOException v0) {
        }

        if(v0 != null) {
            throw v0;
        }
    }

    protected byte[] decodeAndVerify(short arg10, InputStream arg11, int arg12) {
        RecordStream.checkLength(arg12, RecordStream.CIPHERTEXT_LIMIT, 22);
        byte[] v4 = TlsUtils.readFully(arg12, arg11);
        TlsCipher v0 = this.readCipher;
        long v1 = this.readSeqNo;
        this.readSeqNo = 1 + v1;
        byte[] v0_1 = v0.decodeCiphertext(v1, arg10, v4, 0, v4.length);
        RecordStream.checkLength(v0_1.length, RecordStream.COMPRESSED_LIMIT, 22);
        OutputStream v1_1 = this.readCompression.decompress(this.buffer);
        if(v1_1 != this.buffer) {
            v1_1.write(v0_1, 0, v0_1.length);
            v1_1.flush();
            v0_1 = this.getBufferContents();
        }

        RecordStream.checkLength(v0_1.length, RecordStream.PLAINTEXT_LIMIT, 30);
        return v0_1;
    }

    private static byte[] doFinal(Digest arg2) {
        byte[] v0 = new byte[arg2.getDigestSize()];
        arg2.doFinal(v0, 0);
        return v0;
    }

    void finaliseHandshake() {
        TlsCompression v2 = null;
        if(this.readCompression == this.pendingCompression && this.writeCompression == this.pendingCompression && this.readCipher == this.pendingCipher && this.writeCipher == this.pendingCipher) {
            this.pendingCompression = v2;
            this.pendingCipher = ((TlsCipher)v2);
            return;
        }

        throw new TlsFatalAlert(40);
    }

    protected void flush() {
        this.output.flush();
    }

    private byte[] getBufferContents() {
        byte[] v0 = this.buffer.toByteArray();
        this.buffer.reset();
        return v0;
    }

    byte[] getCurrentHash(byte[] arg4) {
        TlsHandshakeHash v0 = this.hash.fork();
        if((this.context.getServerVersion().isSSL()) && arg4 != null) {
            v0.update(arg4, 0, arg4.length);
        }

        return RecordStream.doFinal(((Digest)v0));
    }

    ProtocolVersion getReadVersion() {
        return this.readVersion;
    }

    void init(TlsContext arg2) {
        this.context = arg2;
        this.hash = new DeferredHash();
        this.hash.init(arg2);
    }

    void notifyHelloComplete() {
        this.hash = this.hash.commit();
    }

    public void readRecord() {
        short v3 = 47;
        short v0 = TlsUtils.readUint8(this.input);
        RecordStream.checkType(v0, 10);
        if(this.restrictReadVersion) {
            ProtocolVersion v1 = TlsUtils.readVersion(this.input);
            if(this.readVersion == null) {
                this.readVersion = v1;
            }
            else if(!v1.equals(this.readVersion)) {
                throw new TlsFatalAlert(v3);
            }
        }
        else if((TlsUtils.readVersionRaw(this.input) & -256) != 768) {
            throw new TlsFatalAlert(v3);
        }

        byte[] v1_1 = this.decodeAndVerify(v0, this.input, TlsUtils.readUint16(this.input));
        this.handler.processRecord(v0, v1_1, 0, v1_1.length);
    }

    void receivedReadCipherSpec() {
        if(this.pendingCompression != null && this.pendingCipher != null) {
            this.readCompression = this.pendingCompression;
            this.readCipher = this.pendingCipher;
            this.readSeqNo = 0;
            return;
        }

        throw new TlsFatalAlert(40);
    }

    void sentWriteCipherSpec() {
        if(this.pendingCompression != null && this.pendingCipher != null) {
            this.writeCompression = this.pendingCompression;
            this.writeCipher = this.pendingCipher;
            this.writeSeqNo = 0;
            return;
        }

        throw new TlsFatalAlert(40);
    }

    void setPendingConnectionState(TlsCompression arg1, TlsCipher arg2) {
        this.pendingCompression = arg1;
        this.pendingCipher = arg2;
    }

    void setReadVersion(ProtocolVersion arg1) {
        this.readVersion = arg1;
    }

    void setRestrictReadVersion(boolean arg1) {
        this.restrictReadVersion = arg1;
    }

    void setWriteVersion(ProtocolVersion arg1) {
        this.writeVersion = arg1;
    }

    void updateHandshakeData(byte[] arg2, int arg3, int arg4) {
        this.hash.update(arg2, arg3, arg4);
    }

    protected void writeRecord(short arg11, byte[] arg12, int arg13, int arg14) {
        byte[] v0_2;
        long v1;
        TlsCipher v0_1;
        long v5 = 1;
        short v7 = 80;
        RecordStream.checkType(arg11, v7);
        RecordStream.checkLength(arg14, RecordStream.PLAINTEXT_LIMIT, v7);
        if(arg14 < 1 && arg11 != 23) {
            throw new TlsFatalAlert(v7);
        }

        if(arg11 == 22) {
            this.updateHandshakeData(arg12, arg13, arg14);
        }

        OutputStream v0 = this.writeCompression.compress(this.buffer);
        if(v0 == this.buffer) {
            v0_1 = this.writeCipher;
            v1 = this.writeSeqNo;
            this.writeSeqNo = v1 + v5;
            v0_2 = v0_1.encodePlaintext(v1, arg11, arg12, arg13, arg14);
        }
        else {
            v0.write(arg12, arg13, arg14);
            v0.flush();
            byte[] v4 = this.getBufferContents();
            RecordStream.checkLength(v4.length, arg14 + 1024, v7);
            v0_1 = this.writeCipher;
            v1 = this.writeSeqNo;
            this.writeSeqNo = v5 + v1;
            v0_2 = v0_1.encodePlaintext(v1, arg11, v4, 0, v4.length);
        }

        RecordStream.checkLength(v0_2.length, RecordStream.CIPHERTEXT_LIMIT, v7);
        byte[] v1_1 = new byte[v0_2.length + 5];
        TlsUtils.writeUint8(arg11, v1_1, 0);
        TlsUtils.writeVersion(this.writeVersion, v1_1, 1);
        TlsUtils.writeUint16(v0_2.length, v1_1, 3);
        System.arraycopy(v0_2, 0, v1_1, 5, v0_2.length);
        this.output.write(v1_1);
        this.output.flush();
    }
}

