// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import org.bouncycastle.crypto.modes.AEADBlockCipher;
import org.bouncycastle.crypto.params.AEADParameters;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.util.Arrays;

public class TlsAEADCipher implements TlsCipher {
    protected TlsContext context;
    protected AEADBlockCipher decryptCipher;
    protected byte[] decryptImplicitNonce;
    protected AEADBlockCipher encryptCipher;
    protected byte[] encryptImplicitNonce;
    protected int macSize;
    protected int nonce_explicit_length;

    public TlsAEADCipher(TlsContext arg10, AEADBlockCipher arg11, AEADBlockCipher arg12, int arg13, int arg14) {
        super();
        if(!ProtocolVersion.TLSv12.isEqualOrEarlierVersionOf(arg10.getServerVersion().getEquivalentTLSVersion())) {
            throw new TlsFatalAlert(80);
        }

        this.context = arg10;
        this.macSize = arg14;
        this.nonce_explicit_length = 8;
        int v2 = 4;
        int v3 = arg13 * 2 + 8;
        byte[] v4 = TlsUtils.calculateKeyBlock(arg10, v3);
        KeyParameter v0 = new KeyParameter(v4, 0, arg13);
        int v5 = arg13;
        KeyParameter v1 = new KeyParameter(v4, v5, arg13);
        v5 += arg13;
        byte[] v6 = Arrays.copyOfRange(v4, v5, v5 + v2);
        v5 += v2;
        v4 = Arrays.copyOfRange(v4, v5, v5 + v2);
        if(v5 + v2 != v3) {
            throw new TlsFatalAlert(80);
        }

        if(arg10.isServer()) {
            this.encryptCipher = arg12;
            this.decryptCipher = arg11;
            this.encryptImplicitNonce = v4;
            this.decryptImplicitNonce = v6;
        }
        else {
            this.encryptCipher = arg11;
            this.decryptCipher = arg12;
            this.encryptImplicitNonce = v6;
            this.decryptImplicitNonce = v4;
            KeyParameter v8 = v1;
            v1 = v0;
            v0 = v8;
        }

        byte[] v2_1 = new byte[v2 + this.nonce_explicit_length];
        this.encryptCipher.init(true, new AEADParameters(v1, arg14 * 8, v2_1));
        this.decryptCipher.init(false, new AEADParameters(v0, arg14 * 8, v2_1));
    }

    public byte[] decodeCiphertext(long arg14, short arg16, byte[] arg17, int arg18, int arg19) {
        if(this.getPlaintextLimit(arg19) < 0) {
            throw new TlsFatalAlert(50);
        }

        byte[] v2 = new byte[this.decryptImplicitNonce.length + this.nonce_explicit_length];
        System.arraycopy(this.decryptImplicitNonce, 0, v2, 0, this.decryptImplicitNonce.length);
        System.arraycopy(arg17, arg18, v2, this.decryptImplicitNonce.length, this.nonce_explicit_length);
        int v4 = arg18 + this.nonce_explicit_length;
        int v5 = arg19 - this.nonce_explicit_length;
        int v3 = this.decryptCipher.getOutputSize(v5);
        byte[] v6 = new byte[v3];
        this.decryptCipher.init(false, new AEADParameters(null, this.macSize * 8, v2, this.getAdditionalData(arg14, arg16, v3)));
        int v2_1 = this.decryptCipher.processBytes(arg17, v4, v5, v6, 0);
        try {
            v3 = this.decryptCipher.doFinal(v6, v2_1);
        }
        catch(Exception v2_2) {
            throw new TlsFatalAlert(20);
        }

        if(v2_1 + v3 != v6.length) {
            throw new TlsFatalAlert(80);
        }

        return v6;
    }

    public byte[] encodePlaintext(long arg10, short arg12, byte[] arg13, int arg14, int arg15) {
        byte[] v0 = new byte[this.encryptImplicitNonce.length + this.nonce_explicit_length];
        System.arraycopy(this.encryptImplicitNonce, 0, v0, 0, this.encryptImplicitNonce.length);
        TlsUtils.writeUint64(arg10, v0, this.encryptImplicitNonce.length);
        byte[] v4 = new byte[this.encryptCipher.getOutputSize(arg15) + this.nonce_explicit_length];
        System.arraycopy(v0, this.encryptImplicitNonce.length, v4, 0, this.nonce_explicit_length);
        int v5 = this.nonce_explicit_length;
        this.encryptCipher.init(true, new AEADParameters(null, this.macSize * 8, v0, this.getAdditionalData(arg10, arg12, arg15)));
        int v0_1 = this.encryptCipher.processBytes(arg13, arg14, arg15, v4, v5) + v5;
        try {
            int v1 = this.encryptCipher.doFinal(v4, v0_1);
        }
        catch(Exception v0_2) {
            throw new TlsFatalAlert(80);
        }

        if(v0_1 + v1 != v4.length) {
            throw new TlsFatalAlert(80);
        }

        return v4;
    }

    protected byte[] getAdditionalData(long arg4, short arg6, int arg7) {
        byte[] v0 = new byte[13];
        TlsUtils.writeUint64(arg4, v0, 0);
        TlsUtils.writeUint8(arg6, v0, 8);
        TlsUtils.writeVersion(this.context.getServerVersion(), v0, 9);
        TlsUtils.writeUint16(arg7, v0, 11);
        return v0;
    }

    public int getPlaintextLimit(int arg3) {
        return arg3 - this.macSize - this.nonce_explicit_length;
    }
}

