// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.security.SecureRandom;
import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.util.Arrays;

public class TlsBlockCipher implements TlsCipher {
    protected TlsContext context;
    protected BlockCipher decryptCipher;
    protected BlockCipher encryptCipher;
    protected byte[] randomData;
    protected TlsMac readMac;
    protected boolean useExplicitIV;
    protected TlsMac writeMac;

    public TlsBlockCipher(TlsContext arg14, BlockCipher arg15, BlockCipher arg16, Digest arg17, Digest arg18, int arg19) {
        ParametersWithIV v1_2;
        ParametersWithIV v2_1;
        byte[] v3;
        super();
        this.context = arg14;
        this.randomData = new byte[256];
        arg14.getSecureRandom().nextBytes(this.randomData);
        this.useExplicitIV = ProtocolVersion.TLSv11.isEqualOrEarlierVersionOf(arg14.getServerVersion().getEquivalentTLSVersion());
        int v1 = arg19 * 2 + arg17.getDigestSize() + arg18.getDigestSize();
        int v11 = !this.useExplicitIV ? v1 + (arg15.getBlockSize() + arg16.getBlockSize()) : v1;
        byte[] v4 = TlsUtils.calculateKeyBlock(arg14, v11);
        TlsMac v1_1 = new TlsMac(arg14, arg17, v4, 0, arg17.getDigestSize());
        int v9 = arg17.getDigestSize();
        TlsMac v5 = new TlsMac(arg14, arg18, v4, v9, arg18.getDigestSize());
        int v2 = arg18.getDigestSize() + v9;
        KeyParameter v7 = new KeyParameter(v4, v2, arg19);
        v2 += arg19;
        KeyParameter v8 = new KeyParameter(v4, v2, arg19);
        int v6 = v2 + arg19;
        if(this.useExplicitIV) {
            v4 = new byte[arg15.getBlockSize()];
            v3 = new byte[arg16.getBlockSize()];
            v2 = v6;
        }
        else {
            v3 = Arrays.copyOfRange(v4, v6, arg15.getBlockSize() + v6);
            v6 += arg15.getBlockSize();
            byte[] v12 = Arrays.copyOfRange(v4, v6, arg16.getBlockSize() + v6);
            v2 = arg16.getBlockSize() + v6;
            v4 = v3;
            v3 = v12;
        }

        if(v2 != v11) {
            throw new TlsFatalAlert(80);
        }

        if(arg14.isServer()) {
            this.writeMac = v5;
            this.readMac = v1_1;
            this.encryptCipher = arg16;
            this.decryptCipher = arg15;
            v2_1 = new ParametersWithIV(((CipherParameters)v8), v3);
            v1_2 = new ParametersWithIV(((CipherParameters)v7), v4);
        }
        else {
            this.writeMac = v1_1;
            this.readMac = v5;
            this.encryptCipher = arg15;
            this.decryptCipher = arg16;
            v2_1 = new ParametersWithIV(((CipherParameters)v7), v4);
            v1_2 = new ParametersWithIV(((CipherParameters)v8), v3);
        }

        this.encryptCipher.init(true, ((CipherParameters)v2_1));
        this.decryptCipher.init(false, ((CipherParameters)v1_2));
    }

    protected int checkPaddingConstantTime(byte[] arg8, int arg9, int arg10, int arg11, int arg12) {
        byte v0_1;
        int v1;
        int v0;
        int v5 = arg9 + arg10;
        int v6 = arg8[v5 - 1];
        int v4 = (v6 & 255) + 1;
        if((this.context.getServerVersion().isSSL()) && v4 > arg11 || arg12 + v4 > arg10) {
            v0 = 0;
            v1 = 0;
            v4 = 0;
        }
        else {
            v0 = v5 - v4;
            int v2 = 0;
            while(true) {
                v1 = v0 + 1;
                v0_1 = ((byte)(arg8[v0] ^ v6 | v2));
                if(v1 >= v5) {
                    break;
                }

                v2 = v0_1;
                v0 = v1;
            }

            if(v0_1 != 0) {
                v1 = v4;
                v4 = 0;
            }
            else {
                v1 = v4;
            }
        }

        byte[] v5_1 = this.randomData;
        while(v1 < 256) {
            v0_1 = ((byte)(v0_1 | v5_1[v1] ^ v6));
            ++v1;
        }

        v5_1[0] = ((byte)(v0_1 ^ v5_1[0]));
        return v4;
    }

    protected int chooseExtraPadBlocks(SecureRandom arg2, int arg3) {
        return Math.min(this.lowestBitSet(arg2.nextInt()), arg3);
    }

    public byte[] decodeCiphertext(long arg15, short arg17, byte[] arg18, int arg19, int arg20) {
        int v5;
        int v4;
        int v6 = this.decryptCipher.getBlockSize();
        int v7 = this.readMac.getSize();
        int v2 = Math.max(v6, v7 + 1);
        if(this.useExplicitIV) {
            v2 += v6;
        }

        if(arg20 < v2) {
            throw new TlsFatalAlert(50);
        }

        if(arg20 % v6 != 0) {
            throw new TlsFatalAlert(21);
        }

        if(this.useExplicitIV) {
            this.decryptCipher.init(false, new ParametersWithIV(null, arg18, arg19, v6));
            v4 = arg19 + v6;
            v5 = arg20 - v6;
        }
        else {
            v5 = arg20;
            v4 = arg19;
        }

        for(v2 = 0; v2 < v5; v2 += v6) {
            this.decryptCipher.processBlock(arg18, v4 + v2, arg18, v4 + v2);
        }

        int v3 = this.checkPaddingConstantTime(arg18, v4, v5, v6, v7);
        int v11 = v5 - v3 - v7;
        v2 = !Arrays.constantTimeAreEqual(this.readMac.calculateMacConstantTime(arg15, arg17, arg18, v4, v11, v5 - v7, this.randomData), Arrays.copyOfRange(arg18, v4 + v11, v4 + v11 + v7)) ? 1 : 0;
        if(v2 == 0 && v3 != 0) {
            return Arrays.copyOfRange(arg18, v4, v4 + v11);
        }

        throw new TlsFatalAlert(20);
    }

    public byte[] encodePlaintext(long arg16, short arg18, byte[] arg19, int arg20, int arg21) {
        int v12;
        int v13 = this.encryptCipher.getBlockSize();
        int v4 = this.writeMac.getSize();
        ProtocolVersion v5 = this.context.getServerVersion();
        int v3 = v13 - 1 - (arg21 + v4) % v13;
        int v10 = (v5.isDTLS()) || (v5.isSSL()) ? v3 : v3 + this.chooseExtraPadBlocks(this.context.getSecureRandom(), (255 - v3) / v13) * v13;
        v3 = arg21 + v4 + v10 + 1;
        int v11 = this.useExplicitIV ? v3 + v13 : v3;
        byte[] v14 = new byte[v11];
        if(this.useExplicitIV) {
            byte[] v4_1 = new byte[v13];
            this.context.getSecureRandom().nextBytes(v4_1);
            this.encryptCipher.init(true, new ParametersWithIV(null, v4_1));
            System.arraycopy(v4_1, 0, v14, 0, v13);
            v12 = v13;
        }
        else {
            v12 = 0;
        }

        byte[] v3_1 = this.writeMac.calculateMac(arg16, arg18, arg19, arg20, arg21);
        System.arraycopy(arg19, arg20, v14, v12, arg21);
        System.arraycopy(v3_1, 0, v14, v12 + arg21, v3_1.length);
        v4 = v12 + arg21 + v3_1.length;
        for(v3 = 0; v3 <= v10; ++v3) {
            v14[v3 + v4] = ((byte)v10);
        }

        while(v12 < v11) {
            this.encryptCipher.processBlock(v14, v12, v14, v12);
            v12 += v13;
        }

        return v14;
    }

    public int getPlaintextLimit(int arg4) {
        int v1 = this.encryptCipher.getBlockSize();
        int v0 = arg4 - arg4 % v1 - this.writeMac.getSize() - 1;
        if(this.useExplicitIV) {
            v0 -= v1;
        }

        return v0;
    }

    public TlsMac getReadMac() {
        return this.readMac;
    }

    public TlsMac getWriteMac() {
        return this.writeMac;
    }

    protected int lowestBitSet(int arg3) {
        int v0;
        if(arg3 == 0) {
            v0 = 32;
        }
        else {
            v0 = 0;
            while((arg3 & 1) == 0) {
                ++v0;
                arg3 >>= 1;
            }
        }

        return v0;
    }
}

