// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.SecureRandom;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.bouncycastle.crypto.prng.ThreadedSeedGenerator;
import org.bouncycastle.util.Arrays;

public class TlsClientProtocol extends TlsProtocol {
    protected TlsAuthentication authentication;
    protected CertificateRequest certificateRequest;
    protected Hashtable clientExtensions;
    protected TlsKeyExchange keyExchange;
    protected int[] offeredCipherSuites;
    protected short[] offeredCompressionMethods;
    protected int selectedCipherSuite;
    protected short selectedCompressionMethod;
    protected TlsClient tlsClient;
    protected TlsClientContextImpl tlsClientContext;

    public TlsClientProtocol(InputStream arg2, OutputStream arg3) {
        this(arg2, arg3, TlsClientProtocol.createSecureRandom());
    }

    public TlsClientProtocol(InputStream arg2, OutputStream arg3, SecureRandom arg4) {
        super(arg2, arg3, arg4);
        this.tlsClient = null;
        this.tlsClientContext = null;
        this.offeredCipherSuites = null;
        this.offeredCompressionMethods = null;
        this.clientExtensions = null;
        this.keyExchange = null;
        this.authentication = null;
        this.certificateRequest = null;
    }

    public void connect(TlsClient arg5) {
        if(arg5 == null) {
            throw new IllegalArgumentException("\'tlsClient\' cannot be null");
        }

        if(this.tlsClient != null) {
            throw new IllegalStateException("connect can only be called once");
        }

        this.tlsClient = arg5;
        this.securityParameters = new SecurityParameters();
        this.securityParameters.entity = 1;
        this.securityParameters.clientRandom = TlsClientProtocol.createRandomBlock(this.secureRandom);
        this.tlsClientContext = new TlsClientContextImpl(this.secureRandom, this.securityParameters);
        this.tlsClient.init(this.tlsClientContext);
        this.recordStream.init(this.tlsClientContext);
        this.sendClientHelloMessage();
        this.connection_state = 1;
        this.completeHandshake();
        this.tlsClient.notifyHandshakeComplete();
    }

    private static SecureRandom createSecureRandom() {
        ThreadedSeedGenerator v0 = new ThreadedSeedGenerator();
        SecureRandom v1 = new SecureRandom();
        v1.setSeed(v0.generateSeed(20, true));
        return v1;
    }

    protected AbstractTlsContext getContext() {
        return this.tlsClientContext;
    }

    protected TlsPeer getPeer() {
        return this.tlsClient;
    }

    protected void handleChangeCipherSpecMessage() {
        short v2 = 40;
        short v1 = 2;
        switch(this.connection_state) {
            case 13: {
                if(!this.expectSessionTicket) {
                    goto label_9;
                }

                this.failWithError(v1, v2);
                break;
            }
            case 14: {
                break;
            }
            default: {
                this.failWithError(v1, v2);
                return;
            }
        }

    label_9:
        this.connection_state = 15;
    }

    protected void handleHandshakeMessage(short arg8, byte[] arg9) {
        TlsCredentials v0_2;
        short v2 = 16;
        short v6 = 12;
        Vector v1 = null;
        short v5 = 10;
        short v3 = 2;
        ByteArrayInputStream v0 = new ByteArrayInputStream(arg9);
        switch(arg8) {
            case 0: {
                goto label_162;
            }
            case 2: {
                goto label_34;
            }
            case 4: {
                goto label_159;
            }
            case 11: {
                goto label_10;
            }
            case 12: {
                goto label_126;
            }
            case 13: {
                switch(this.connection_state) {
                    case 4: {
                        this.keyExchange.skipServerKeyExchange();
                        goto label_148;
                    }
                    case 5: {
                    label_148:
                        if(this.authentication == null) {
                            this.failWithError(v3, 40);
                        }

                        this.certificateRequest = CertificateRequest.parse(((InputStream)v0));
                        TlsClientProtocol.assertEmpty(v0);
                        this.keyExchange.validateCertificateRequest(this.certificateRequest);
                    label_143:
                        this.connection_state = 6;
                        return;
                    label_159:
                        switch(this.connection_state) {
                            case 13: {
                                goto label_169;
                            }
                        }

                        this.failWithError(v3, v5);
                        goto label_162;
                    label_169:
                        if(!this.expectSessionTicket) {
                            this.failWithError(v3, v5);
                        }

                        this.receiveNewSessionTicketMessage(v0);
                        this.connection_state = 14;
                        goto label_162;
                    label_126:
                        switch(this.connection_state) {
                            case 2: {
                                this.handleSupplementalData(v1);
                                goto label_133;
                            }
                            case 3: {
                            label_133:
                                this.keyExchange.skipServerCredentials();
                                this.authentication = ((TlsAuthentication)v1);
                                goto label_136;
                            }
                            case 4: {
                            label_136:
                                this.keyExchange.processServerKeyExchange(((InputStream)v0));
                                TlsClientProtocol.assertEmpty(v0);
                            label_129:
                                this.connection_state = 5;
                                return;
                            label_34:
                                switch(this.connection_state) {
                                    case 1: {
                                        goto label_38;
                                    }
                                }

                                this.failWithError(v3, v5);
                                return;
                            label_38:
                                this.receiveServerHelloMessage(v0);
                                this.connection_state = v3;
                                this.securityParameters.prfAlgorithm = TlsClientProtocol.getPRFAlgorithm(this.selectedCipherSuite);
                                this.securityParameters.compressionAlgorithm = this.selectedCompressionMethod;
                                this.securityParameters.verifyDataLength = v6;
                                this.recordStream.notifyHelloComplete();
                                return;
                            label_52:
                                switch(this.connection_state) {
                                    case 2: {
                                        goto label_56;
                                    }
                                }

                                this.failWithError(v3, v5);
                                return;
                            label_56:
                                this.handleSupplementalData(TlsClientProtocol.readSupplementalDataMessage(v0));
                                return;
                            label_27:
                                switch(this.connection_state) {
                                    case 15: {
                                        goto label_31;
                                    }
                                }

                                this.failWithError(v3, v5);
                                return;
                            label_31:
                                this.processFinishedMessage(v0);
                                this.connection_state = v2;
                                return;
                            label_10:
                                switch(this.connection_state) {
                                    case 2: {
                                        this.handleSupplementalData(v1);
                                        goto label_17;
                                    }
                                    case 3: {
                                    label_17:
                                        Certificate v1_1 = Certificate.parse(((InputStream)v0));
                                        TlsClientProtocol.assertEmpty(v0);
                                        this.keyExchange.processServerCertificate(v1_1);
                                        this.authentication = this.tlsClient.getAuthentication();
                                        this.authentication.notifyServerCertificate(v1_1);
                                    label_13:
                                        this.connection_state = 4;
                                        return;
                                    label_162:
                                        TlsClientProtocol.assertEmpty(v0);
                                        if(this.connection_state != v2) {
                                            return;
                                        }

                                        this.raiseWarning(100, "Renegotiation not supported");
                                        return;
                                    label_59:
                                        switch(this.connection_state) {
                                            case 2: {
                                                this.handleSupplementalData(v1);
                                                goto label_65;
                                            }
                                            case 3: {
                                            label_65:
                                                this.keyExchange.skipServerCredentials();
                                                this.authentication = ((TlsAuthentication)v1);
                                                goto label_68;
                                            }
                                            case 4: {
                                            label_68:
                                                this.keyExchange.skipServerKeyExchange();
                                                goto label_70;
                                            }
                                            case 5: 
                                            case 6: {
                                            label_70:
                                                TlsClientProtocol.assertEmpty(v0);
                                                this.connection_state = 7;
                                                Vector v0_1 = this.tlsClient.getClientSupplementalData();
                                                if(v0_1 != null) {
                                                    this.sendSupplementalDataMessage(v0_1);
                                                }

                                                this.connection_state = 8;
                                                if(this.certificateRequest == null) {
                                                    this.keyExchange.skipClientCredentials();
                                                    v0_2 = ((TlsCredentials)v1);
                                                }
                                                else {
                                                    v0_2 = this.authentication.getClientCredentials(this.certificateRequest);
                                                    if(v0_2 == null) {
                                                        this.keyExchange.skipClientCredentials();
                                                        this.sendCertificateMessage(Certificate.EMPTY_CHAIN);
                                                    }
                                                    else {
                                                        this.keyExchange.processClientCredentials(v0_2);
                                                        this.sendCertificateMessage(v0_2.getCertificate());
                                                    }
                                                }

                                                this.connection_state = 9;
                                                this.sendClientKeyExchangeMessage();
                                                TlsClientProtocol.establishMasterSecret(this.getContext(), this.keyExchange);
                                                this.recordStream.setPendingConnectionState(this.tlsClient.getCompression(), this.tlsClient.getCipher());
                                                this.connection_state = v5;
                                                if(v0_2 != null && ((v0_2 instanceof TlsSignerCredentials))) {
                                                    this.sendCertificateVerifyMessage(((TlsSignerCredentials)v0_2).generateCertificateSignature(this.recordStream.getCurrentHash(((byte[])v1))));
                                                    this.connection_state = 11;
                                                }

                                                this.sendChangeCipherSpecMessage();
                                                this.connection_state = v6;
                                                this.sendFinishedMessage();
                                                this.connection_state = 13;
                                                return;
                                            }
                                            default: {
                                                this.failWithError(v3, 40);
                                                return;
                                            }
                                        }
                                    }
                                    default: {
                                        this.failWithError(v3, v5);
                                        goto label_13;
                                    }
                                }
                            }
                            default: {
                                this.failWithError(v3, v5);
                                goto label_129;
                            }
                        }
                    }
                    default: {
                        this.failWithError(v3, v5);
                        goto label_143;
                    }
                }
            }
            case 14: {
                goto label_59;
            }
            case 20: {
                goto label_27;
            }
            case 23: {
                goto label_52;
            }
            default: {
                this.failWithError(v3, v5);
                break;
            }
        }
    }

    protected void handleSupplementalData(Vector arg3) {
        this.tlsClient.processServerSupplementalData(arg3);
        this.connection_state = 3;
        this.keyExchange = this.tlsClient.getKeyExchange();
        this.keyExchange.init(this.getContext());
    }

    protected void receiveNewSessionTicketMessage(ByteArrayInputStream arg3) {
        NewSessionTicket v0 = NewSessionTicket.parse(((InputStream)arg3));
        TlsProtocol.assertEmpty(arg3);
        this.tlsClient.notifyNewSessionTicket(v0);
    }

    protected void receiveServerHelloMessage(ByteArrayInputStream arg6) {
        Object v0_3;
        int v3 = 32;
        short v2 = 47;
        short v4 = 2;
        ProtocolVersion v0 = TlsUtils.readVersion(((InputStream)arg6));
        if(v0.isDTLS()) {
            this.failWithError(v4, v2);
        }

        if(!v0.equals(this.recordStream.getReadVersion())) {
            this.failWithError(v4, v2);
        }

        if(!v0.isEqualOrEarlierVersionOf(this.getContext().getClientVersion())) {
            this.failWithError(v4, v2);
        }

        this.recordStream.setWriteVersion(v0);
        this.getContext().setServerVersion(v0);
        this.tlsClient.notifyServerVersion(v0);
        this.securityParameters.serverRandom = TlsUtils.readFully(v3, ((InputStream)arg6));
        byte[] v0_1 = TlsUtils.readOpaque8(((InputStream)arg6));
        if(v0_1.length > v3) {
            this.failWithError(v4, v2);
        }

        this.tlsClient.notifySessionID(v0_1);
        this.selectedCipherSuite = TlsUtils.readUint16(((InputStream)arg6));
        if(!TlsClientProtocol.arrayContains(this.offeredCipherSuites, this.selectedCipherSuite) || this.selectedCipherSuite == 0 || this.selectedCipherSuite == 255) {
            this.failWithError(v4, v2);
        }

        this.tlsClient.notifySelectedCipherSuite(this.selectedCipherSuite);
        short v0_2 = TlsUtils.readUint8(((InputStream)arg6));
        if(!TlsClientProtocol.arrayContains(this.offeredCompressionMethods, v0_2)) {
            this.failWithError(v4, v2);
        }

        this.tlsClient.notifySelectedCompressionMethod(v0_2);
        Hashtable v1 = TlsClientProtocol.readExtensions(arg6);
        if(v1 != null) {
            Enumeration v2_1 = v1.keys();
            while(v2_1.hasMoreElements()) {
                v0_3 = v2_1.nextElement();
                if(((Integer)v0_3).equals(TlsClientProtocol.EXT_RenegotiationInfo)) {
                    continue;
                }

                if(this.clientExtensions != null && this.clientExtensions.get(v0_3) != null) {
                    continue;
                }

                this.failWithError(v4, 110);
            }

            v0_3 = v1.get(TlsClientProtocol.EXT_RenegotiationInfo);
            if(v0_3 != null) {
                this.secure_renegotiation = true;
                if(!Arrays.constantTimeAreEqual(((byte[])v0_3), TlsClientProtocol.createRenegotiationInfo(TlsUtils.EMPTY_BYTES))) {
                    this.failWithError(v4, 40);
                }
            }

            this.expectSessionTicket = v1.containsKey(TlsClientProtocol.EXT_SessionTicket);
        }

        this.tlsClient.notifySecureRenegotiation(this.secure_renegotiation);
        if(this.clientExtensions != null) {
            this.tlsClient.processServerExtensions(v1);
        }
    }

    protected void sendCertificateVerifyMessage(byte[] arg5) {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(15, ((OutputStream)v0));
        TlsUtils.writeUint24(arg5.length + 2, ((OutputStream)v0));
        TlsUtils.writeOpaque16(arg5, ((OutputStream)v0));
        byte[] v0_1 = v0.toByteArray();
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }

    protected void sendClientHelloMessage() {
        this.recordStream.setWriteVersion(this.tlsClient.getClientHelloRecordLayerVersion());
        ByteArrayOutputStream v4 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(1, ((OutputStream)v4));
        TlsUtils.writeUint24(0, ((OutputStream)v4));
        ProtocolVersion v0 = this.tlsClient.getClientVersion();
        if(v0.isDTLS()) {
            this.failWithError(2, 80);
        }

        this.getContext().setClientVersion(v0);
        TlsUtils.writeVersion(v0, ((OutputStream)v4));
        v4.write(this.securityParameters.clientRandom);
        TlsUtils.writeOpaque8(TlsUtils.EMPTY_BYTES, ((OutputStream)v4));
        this.offeredCipherSuites = this.tlsClient.getCipherSuites();
        this.clientExtensions = this.tlsClient.getClientExtensions();
        int v0_1 = this.clientExtensions == null || this.clientExtensions.get(TlsClientProtocol.EXT_RenegotiationInfo) == null ? 1 : 0;
        int v3 = this.offeredCipherSuites.length;
        if(v0_1 != 0) {
            ++v3;
        }

        TlsUtils.writeUint16(v3 * 2, ((OutputStream)v4));
        TlsUtils.writeUint16Array(this.offeredCipherSuites, ((OutputStream)v4));
        if(v0_1 != 0) {
            TlsUtils.writeUint16(255, ((OutputStream)v4));
        }

        this.offeredCompressionMethods = this.tlsClient.getCompressionMethods();
        TlsUtils.writeUint8(((short)this.offeredCompressionMethods.length), ((OutputStream)v4));
        TlsUtils.writeUint8Array(this.offeredCompressionMethods, ((OutputStream)v4));
        if(this.clientExtensions != null) {
            TlsClientProtocol.writeExtensions(((OutputStream)v4), this.clientExtensions);
        }

        byte[] v0_2 = v4.toByteArray();
        TlsUtils.writeUint24(v0_2.length - 4, v0_2, 1);
        this.safeWriteRecord(22, v0_2, 0, v0_2.length);
    }

    protected void sendClientKeyExchangeMessage() {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(16, ((OutputStream)v0));
        TlsUtils.writeUint24(0, ((OutputStream)v0));
        this.keyExchange.generateClientKeyExchange(((OutputStream)v0));
        byte[] v0_1 = v0.toByteArray();
        TlsUtils.writeUint24(v0_1.length - 4, v0_1, 1);
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }
}

