// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.Hashtable;
import org.bouncycastle.asn1.sec.SECNamedCurves;
import org.bouncycastle.asn1.x9.X9ECParameters;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.agreement.ECDHBasicAgreement;
import org.bouncycastle.crypto.generators.ECKeyPairGenerator;
import org.bouncycastle.crypto.params.ECDomainParameters;
import org.bouncycastle.crypto.params.ECKeyGenerationParameters;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.math.ec.ECCurve;
import org.bouncycastle.math.ec.ECCurve$F2m;
import org.bouncycastle.math.ec.ECCurve$Fp;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.Integers;

public class TlsECCUtils {
    public static final Integer EXT_ec_point_formats;
    public static final Integer EXT_elliptic_curves;
    private static final String[] curveNames;

    static {
        TlsECCUtils.EXT_elliptic_curves = Integers.valueOf(10);
        TlsECCUtils.EXT_ec_point_formats = Integers.valueOf(11);
        TlsECCUtils.curveNames = new String[]{"sect163k1", "sect163r1", "sect163r2", "sect193r1", "sect193r2", "sect233k1", "sect233r1", "sect239k1", "sect283k1", "sect283r1", "sect409k1", "sect409r1", "sect571k1", "sect571r1", "secp160k1", "secp160r1", "secp160r2", "secp192k1", "secp192r1", "secp224k1", "secp224r1", "secp256k1", "secp256r1", "secp384r1", "secp521r1"};
    }

    public TlsECCUtils() {
        super();
    }

    public static void addSupportedEllipticCurvesExtension(Hashtable arg2, int[] arg3) {
        arg2.put(TlsECCUtils.EXT_elliptic_curves, TlsECCUtils.createSupportedEllipticCurvesExtension(arg3));
    }

    public static void addSupportedPointFormatsExtension(Hashtable arg2, short[] arg3) {
        arg2.put(TlsECCUtils.EXT_ec_point_formats, TlsECCUtils.createSupportedPointFormatsExtension(arg3));
    }

    public static boolean areOnSameCurve(ECDomainParameters arg2, ECDomainParameters arg3) {
        boolean v0 = !arg2.getCurve().equals(arg3.getCurve()) || !arg2.getG().equals(arg3.getG()) || !arg2.getN().equals(arg3.getN()) || !arg2.getH().equals(arg3.getH()) ? false : true;
        return v0;
    }

    public static byte[] calculateECDHBasicAgreement(ECPublicKeyParameters arg2, ECPrivateKeyParameters arg3) {
        ECDHBasicAgreement v0 = new ECDHBasicAgreement();
        v0.init(((CipherParameters)arg3));
        return BigIntegers.asUnsignedByteArray(v0.getFieldSize(), v0.calculateAgreement(((CipherParameters)arg2)));
    }

    public static boolean containsECCCipherSuites(int[] arg3) {
        boolean v1 = false;
        int v0 = 0;
        while(v0 < arg3.length) {
            if(TlsECCUtils.isECCCipherSuite(arg3[v0])) {
                v1 = true;
            }
            else {
                ++v0;
                continue;
            }

            return v1;
        }

        return v1;
    }

    public static byte[] createSupportedEllipticCurvesExtension(int[] arg2) {
        if(arg2 != null && arg2.length >= 1) {
            ByteArrayOutputStream v0 = new ByteArrayOutputStream();
            TlsUtils.writeUint16(arg2.length * 2, ((OutputStream)v0));
            TlsUtils.writeUint16Array(arg2, ((OutputStream)v0));
            return v0.toByteArray();
        }

        throw new TlsFatalAlert(80);
    }

    public static byte[] createSupportedPointFormatsExtension(short[] arg3) {
        if(arg3 == null) {
            arg3 = new short[]{0};
        }
        else if(!TlsProtocol.arrayContains(arg3, 0)) {
            short[] v0 = new short[arg3.length + 1];
            System.arraycopy(arg3, 0, v0, 0, arg3.length);
            v0[arg3.length] = 0;
            arg3 = v0;
        }

        ByteArrayOutputStream v0_1 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(((short)arg3.length), ((OutputStream)v0_1));
        TlsUtils.writeUint8Array(arg3, ((OutputStream)v0_1));
        return v0_1.toByteArray();
    }

    public static BigInteger deserializeECFieldElement(int arg2, byte[] arg3) {
        if(arg3.length != (arg2 + 7) / 8) {
            throw new TlsFatalAlert(50);
        }

        return new BigInteger(1, arg3);
    }

    public static ECPoint deserializeECPoint(short[] arg1, ECCurve arg2, byte[] arg3) {
        return arg2.decodePoint(arg3);
    }

    public static ECPublicKeyParameters deserializeECPublicKey(short[] arg2, ECDomainParameters arg3, byte[] arg4) {
        try {
            return new ECPublicKeyParameters(TlsECCUtils.deserializeECPoint(arg2, arg3.getCurve(), arg4), arg3);
        }
        catch(RuntimeException v0) {
            throw new TlsFatalAlert(47);
        }
    }

    public static AsymmetricCipherKeyPair generateECKeyPair(SecureRandom arg2, ECDomainParameters arg3) {
        ECKeyPairGenerator v0 = new ECKeyPairGenerator();
        v0.init(new ECKeyGenerationParameters(arg3, arg2));
        return v0.generateKeyPair();
    }

    public static String getNameOfNamedCurve(int arg2) {
        String v0 = TlsECCUtils.isSupportedNamedCurve(arg2) ? TlsECCUtils.curveNames[arg2 - 1] : null;
        return v0;
    }

    public static ECDomainParameters getParametersForNamedCurve(int arg6) {
        ECDomainParameters v0 = null;
        String v1 = TlsECCUtils.getNameOfNamedCurve(arg6);
        if(v1 != null) {
            X9ECParameters v5 = SECNamedCurves.getByName(v1);
            if(v5 != null) {
                v0 = new ECDomainParameters(v5.getCurve(), v5.getG(), v5.getN(), v5.getH(), v5.getSeed());
            }
        }

        return v0;
    }

    public static int[] getSupportedEllipticCurvesExtension(Hashtable arg2) {
        int[] v0;
        int[] v1 = null;
        if(arg2 == null) {
            v0 = v1;
        }
        else {
            Object v0_1 = arg2.get(TlsECCUtils.EXT_elliptic_curves);
            v0 = v0_1 == null ? v1 : TlsECCUtils.readSupportedEllipticCurvesExtension(((byte[])v0_1));
        }

        return v0;
    }

    public static short[] getSupportedPointFormatsExtension(Hashtable arg2) {
        short[] v0;
        short[] v1 = null;
        if(arg2 == null) {
            v0 = v1;
        }
        else {
            Object v0_1 = arg2.get(TlsECCUtils.EXT_ec_point_formats);
            v0 = v0_1 == null ? v1 : TlsECCUtils.readSupportedPointFormatsExtension(((byte[])v0_1));
        }

        return v0;
    }

    public static boolean hasAnySupportedNamedCurves() {
        boolean v0 = TlsECCUtils.curveNames.length > 0 ? true : false;
        return v0;
    }

    public static boolean isCompressionPreferred(short[] arg3, short arg4) {
        boolean v1 = false;
        if(arg3 != null) {
            int v0 = 0;
            while(v0 < arg3.length) {
                int v2 = arg3[v0];
                if(v2 == 0) {
                    return v1;
                }

                if(v2 == arg4) {
                    v1 = true;
                }
                else {
                    ++v0;
                    continue;
                }

                return v1;
            }
        }

        return v1;
    }

    public static boolean isECCCipherSuite(int arg1) {
        boolean v0;
        switch(arg1) {
            case 49153: 
            case 49154: 
            case 49155: 
            case 49156: 
            case 49157: 
            case 49158: 
            case 49159: 
            case 49160: 
            case 49161: 
            case 49162: 
            case 49163: 
            case 49164: 
            case 49165: 
            case 49166: 
            case 49167: 
            case 49168: 
            case 49169: 
            case 49170: 
            case 49171: 
            case 49172: 
            case 49173: 
            case 49174: 
            case 49175: 
            case 49176: 
            case 49177: 
            case 49187: 
            case 49188: 
            case 49189: 
            case 49190: 
            case 49191: 
            case 49192: 
            case 49193: 
            case 49194: 
            case 49195: 
            case 49196: 
            case 49197: 
            case 49198: 
            case 49199: 
            case 49200: 
            case 49201: 
            case 49202: {
                v0 = true;
                break;
            }
            default: {
                v0 = false;
                break;
            }
        }

        return v0;
    }

    public static boolean isSupportedNamedCurve(int arg1) {
        boolean v0 = arg1 <= 0 || arg1 > TlsECCUtils.curveNames.length ? false : true;
        return v0;
    }

    public static int readECExponent(int arg3, InputStream arg4) {
        BigInteger v0 = TlsECCUtils.readECParameter(arg4);
        if(v0.bitLength() < 32) {
            int v0_1 = v0.intValue();
            if(v0_1 > 0 && v0_1 < arg3) {
                return v0_1;
            }
        }

        throw new TlsFatalAlert(47);
    }

    public static BigInteger readECFieldElement(int arg1, InputStream arg2) {
        return TlsECCUtils.deserializeECFieldElement(arg1, TlsUtils.readOpaque8(arg2));
    }

    public static BigInteger readECParameter(InputStream arg3) {
        return new BigInteger(1, TlsUtils.readOpaque8(arg3));
    }

    public static ECDomainParameters readECParameters(int[] arg8, short[] arg9, InputStream arg10) {
        short v7 = 47;
        try {
            switch(TlsUtils.readUint8(arg10)) {
                case 1: {
                    goto label_11;
                }
                case 2: {
                    goto label_25;
                }
                case 3: {
                    goto label_54;
                }
            }

            throw new TlsFatalAlert(47);
        label_54:
            int v0_1 = TlsUtils.readUint16(arg10);
            if(!NamedCurve.refersToASpecificNamedCurve(v0_1)) {
                throw new TlsFatalAlert(47);
            }

            if(!TlsProtocol.arrayContains(arg8, v0_1)) {
                throw new TlsFatalAlert(47);
            }

            ECDomainParameters v0_2 = TlsECCUtils.getParametersForNamedCurve(v0_1);
            return v0_2;
        label_11:
            BigInteger v0_3 = TlsECCUtils.readECParameter(arg10);
            ECCurve$Fp v3 = new ECCurve$Fp(v0_3, TlsECCUtils.readECFieldElement(v0_3.bitLength(), arg10), TlsECCUtils.readECFieldElement(v0_3.bitLength(), arg10));
            return new ECDomainParameters(((ECCurve)v3), TlsECCUtils.deserializeECPoint(arg9, ((ECCurve)v3), TlsUtils.readOpaque8(arg10)), TlsECCUtils.readECParameter(arg10), TlsECCUtils.readECParameter(arg10));
        label_25:
            int v1 = TlsUtils.readUint16(arg10);
            switch(TlsUtils.readUint8(arg10)) {
                case 1: {
                    goto label_32;
                }
                case 2: {
                    goto label_45;
                }
            }

            throw new TlsFatalAlert(47);
        label_45:
            ECCurve$F2m v1_1 = new ECCurve$F2m(v1, TlsECCUtils.readECExponent(v1, arg10), TlsECCUtils.readECExponent(v1, arg10), TlsECCUtils.readECExponent(v1, arg10), TlsECCUtils.readECFieldElement(v1, arg10), TlsECCUtils.readECFieldElement(v1, arg10));
            goto label_38;
        label_32:
            v1_1 = new ECCurve$F2m(v1, TlsECCUtils.readECExponent(v1, arg10), TlsECCUtils.readECFieldElement(v1, arg10), TlsECCUtils.readECFieldElement(v1, arg10));
        label_38:
            return new ECDomainParameters(((ECCurve)v1_1), TlsECCUtils.deserializeECPoint(arg9, ((ECCurve)v1_1), TlsUtils.readOpaque8(arg10)), TlsECCUtils.readECParameter(arg10), TlsECCUtils.readECParameter(arg10));
        }
        catch(RuntimeException v0) {
            throw new TlsFatalAlert(v7);
        }
    }

    public static int[] readSupportedEllipticCurvesExtension(byte[] arg3) {
        if(arg3 == null) {
            throw new IllegalArgumentException("\'extensionValue\' cannot be null");
        }

        ByteArrayInputStream v0 = new ByteArrayInputStream(arg3);
        int v1 = TlsUtils.readUint16(((InputStream)v0));
        if(v1 >= 2 && (v1 & 1) == 0) {
            int[] v1_1 = TlsUtils.readUint16Array(v1 / 2, ((InputStream)v0));
            TlsProtocol.assertEmpty(v0);
            return v1_1;
        }

        throw new TlsFatalAlert(50);
    }

    public static short[] readSupportedPointFormatsExtension(byte[] arg3) {
        if(arg3 == null) {
            throw new IllegalArgumentException("\'extensionValue\' cannot be null");
        }

        ByteArrayInputStream v0 = new ByteArrayInputStream(arg3);
        int v1 = TlsUtils.readUint8(((InputStream)v0));
        if(v1 < 1) {
            throw new TlsFatalAlert(50);
        }

        short[] v1_1 = TlsUtils.readUint8Array(v1, ((InputStream)v0));
        TlsProtocol.assertEmpty(v0);
        if(!TlsProtocol.arrayContains(v1_1, 0)) {
            throw new TlsFatalAlert(47);
        }

        return v1_1;
    }

    public static byte[] serializeECFieldElement(int arg1, BigInteger arg2) {
        return BigIntegers.asUnsignedByteArray((arg1 + 7) / 8, arg2);
    }

    public static byte[] serializeECPoint(short[] arg3, ECPoint arg4) {
        ECCurve v1 = arg4.getCurve();
        boolean v0 = false;
        if((v1 instanceof ECCurve$F2m)) {
            v0 = TlsECCUtils.isCompressionPreferred(arg3, 2);
        }
        else if((v1 instanceof ECCurve$Fp)) {
            v0 = TlsECCUtils.isCompressionPreferred(arg3, 1);
        }

        return arg4.getEncoded(v0);
    }

    public static byte[] serializeECPublicKey(short[] arg1, ECPublicKeyParameters arg2) {
        return TlsECCUtils.serializeECPoint(arg1, arg2.getQ());
    }

    public static ECPublicKeyParameters validateECPublicKey(ECPublicKeyParameters arg0) {
        return arg0;
    }

    public static void writeECExponent(int arg2, OutputStream arg3) {
        TlsECCUtils.writeECParameter(BigInteger.valueOf(((long)arg2)), arg3);
    }

    public static void writeECFieldElement(int arg1, BigInteger arg2, OutputStream arg3) {
        TlsUtils.writeOpaque8(TlsECCUtils.serializeECFieldElement(arg1, arg2), arg3);
    }

    public static void writeECParameter(BigInteger arg1, OutputStream arg2) {
        TlsUtils.writeOpaque8(BigIntegers.asUnsignedByteArray(arg1), arg2);
    }

    public static void writeExplicitECParameters(short[] arg5, ECDomainParameters arg6, OutputStream arg7) {
        short v4 = 2;
        ECCurve v1 = arg6.getCurve();
        if((v1 instanceof ECCurve$Fp)) {
            TlsUtils.writeUint8(1, arg7);
            TlsECCUtils.writeECParameter(v1.getQ(), arg7);
        }
        else if((v1 instanceof ECCurve$F2m)) {
            TlsUtils.writeUint8(v4, arg7);
            ECCurve v0 = v1;
            TlsUtils.writeUint16(((ECCurve$F2m)v0).getM(), arg7);
            if(((ECCurve$F2m)v0).isTrinomial()) {
                TlsUtils.writeUint8(1, arg7);
                TlsECCUtils.writeECExponent(((ECCurve$F2m)v0).getK1(), arg7);
            }
            else {
                TlsUtils.writeUint8(v4, arg7);
                TlsECCUtils.writeECExponent(((ECCurve$F2m)v0).getK1(), arg7);
                TlsECCUtils.writeECExponent(((ECCurve$F2m)v0).getK2(), arg7);
                TlsECCUtils.writeECExponent(((ECCurve$F2m)v0).getK3(), arg7);
            }
        }
        else {
            goto label_45;
        }

        TlsECCUtils.writeECFieldElement(v1.getFieldSize(), v1.getA().toBigInteger(), arg7);
        TlsECCUtils.writeECFieldElement(v1.getFieldSize(), v1.getB().toBigInteger(), arg7);
        TlsUtils.writeOpaque8(TlsECCUtils.serializeECPoint(arg5, arg6.getG()), arg7);
        TlsECCUtils.writeECParameter(arg6.getN(), arg7);
        TlsECCUtils.writeECParameter(arg6.getH(), arg7);
        return;
    label_45:
        throw new IllegalArgumentException("\'ecParameters\' not a known curve type");
    }

    public static void writeNamedECParameters(int arg2, OutputStream arg3) {
        if(!NamedCurve.refersToASpecificNamedCurve(arg2)) {
            throw new TlsFatalAlert(80);
        }

        TlsUtils.writeUint8(3, arg3);
        TlsUtils.writeUint16(arg2, arg3);
    }
}

