// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.SecureRandom;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;

public abstract class TlsProtocol {
    protected static final short CS_CERTIFICATE_REQUEST = 6;
    protected static final short CS_CERTIFICATE_VERIFY = 11;
    protected static final short CS_CLIENT_CERTIFICATE = 9;
    protected static final short CS_CLIENT_CHANGE_CIPHER_SPEC = 12;
    protected static final short CS_CLIENT_FINISHED = 13;
    protected static final short CS_CLIENT_HELLO = 1;
    protected static final short CS_CLIENT_KEY_EXCHANGE = 10;
    protected static final short CS_CLIENT_SUPPLEMENTAL_DATA = 8;
    protected static final short CS_SERVER_CERTIFICATE = 4;
    protected static final short CS_SERVER_CHANGE_CIPHER_SPEC = 15;
    protected static final short CS_SERVER_FINISHED = 16;
    protected static final short CS_SERVER_HELLO = 2;
    protected static final short CS_SERVER_HELLO_DONE = 7;
    protected static final short CS_SERVER_KEY_EXCHANGE = 5;
    protected static final short CS_SERVER_SESSION_TICKET = 14;
    protected static final short CS_SERVER_SUPPLEMENTAL_DATA = 3;
    protected static final short CS_START = 0;
    protected static final Integer EXT_RenegotiationInfo = null;
    protected static final Integer EXT_SessionTicket = null;
    private static final String TLS_ERROR_MESSAGE = "Internal TLS error, this could be an attack";
    private ByteQueue alertQueue;
    private volatile boolean appDataReady;
    private ByteQueue applicationDataQueue;
    private ByteQueue changeCipherSpecQueue;
    private volatile boolean closed;
    protected short connection_state;
    protected boolean expectSessionTicket;
    private byte[] expected_verify_data;
    private volatile boolean failedWithError;
    private ByteQueue handshakeQueue;
    protected RecordStream recordStream;
    protected SecureRandom secureRandom;
    protected boolean secure_renegotiation;
    protected SecurityParameters securityParameters;
    private TlsInputStream tlsInputStream;
    private TlsOutputStream tlsOutputStream;
    private volatile boolean writeExtraEmptyRecords;

    static {
        TlsProtocol.EXT_RenegotiationInfo = Integers.valueOf(65281);
        TlsProtocol.EXT_SessionTicket = Integers.valueOf(35);
    }

    public TlsProtocol(InputStream arg4, OutputStream arg5, SecureRandom arg6) {
        super();
        this.applicationDataQueue = new ByteQueue();
        this.changeCipherSpecQueue = new ByteQueue();
        this.alertQueue = new ByteQueue();
        this.handshakeQueue = new ByteQueue();
        this.tlsInputStream = null;
        this.tlsOutputStream = null;
        this.closed = false;
        this.failedWithError = false;
        this.appDataReady = false;
        this.writeExtraEmptyRecords = true;
        this.expected_verify_data = null;
        this.securityParameters = null;
        this.connection_state = 0;
        this.secure_renegotiation = false;
        this.expectSessionTicket = false;
        this.recordStream = new RecordStream(this, arg4, arg5);
        this.secureRandom = arg6;
    }

    protected static boolean arrayContains(int[] arg3, int arg4) {
        boolean v1 = false;
        int v0 = 0;
        while(v0 < arg3.length) {
            if(arg3[v0] == arg4) {
                v1 = true;
            }
            else {
                ++v0;
                continue;
            }

            return v1;
        }

        return v1;
    }

    protected static boolean arrayContains(short[] arg3, short arg4) {
        boolean v1 = false;
        int v0 = 0;
        while(v0 < arg3.length) {
            if(arg3[v0] == arg4) {
                v1 = true;
            }
            else {
                ++v0;
                continue;
            }

            return v1;
        }

        return v1;
    }

    protected static void assertEmpty(ByteArrayInputStream arg2) {
        if(arg2.available() > 0) {
            throw new TlsFatalAlert(50);
        }
    }

    public void close() {
        this.handleClose(true);
    }

    protected void completeHandshake() {
        this.expected_verify_data = null;
        while(this.connection_state != 16) {
            this.safeReadRecord();
        }

        this.recordStream.finaliseHandshake();
        this.writeExtraEmptyRecords = this.getContext().getServerVersion().isEqualOrEarlierVersionOf(ProtocolVersion.TLSv10);
        if(!this.appDataReady) {
            this.appDataReady = true;
            this.tlsInputStream = new TlsInputStream(this);
            this.tlsOutputStream = new TlsOutputStream(this);
        }
    }

    protected static byte[] createRandomBlock(SecureRandom arg2) {
        byte[] v0 = new byte[32];
        arg2.nextBytes(v0);
        TlsUtils.writeGMTUnixTime(v0, 0);
        return v0;
    }

    protected static byte[] createRenegotiationInfo(byte[] arg1) {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeOpaque8(arg1, ((OutputStream)v0));
        return v0.toByteArray();
    }

    protected byte[] createVerifyData(boolean arg5) {
        AbstractTlsContext v0 = this.getContext();
        byte[] v0_1 = arg5 ? TlsUtils.calculateVerifyData(((TlsContext)v0), "server finished", this.recordStream.getCurrentHash(TlsUtils.SSL_SERVER)) : TlsUtils.calculateVerifyData(((TlsContext)v0), "client finished", this.recordStream.getCurrentHash(TlsUtils.SSL_CLIENT));
        return v0_1;
    }

    protected static void establishMasterSecret(TlsContext arg4, TlsKeyExchange arg5) {
        byte[] v1 = arg5.generatePremasterSecret();
        try {
            arg4.getSecurityParameters().masterSecret = TlsUtils.calculateMasterSecret(arg4, v1);
            if(v1 == null) {
                return;
            }
        }
        catch(Throwable v0) {
            if(v1 != null) {
                Arrays.fill(v1, 0);
            }

            throw v0;
        }

        Arrays.fill(v1, 0);
    }

    protected void failWithError(short arg5, short arg6) {
        String v3 = null;
        int v2 = 2;
        if(!this.closed) {
            this.closed = true;
            if(arg5 == v2) {
                this.failedWithError = true;
            }

            this.raiseAlert(arg5, arg6, v3, ((Exception)v3));
            this.recordStream.close();
            if(arg5 == v2) {
                throw new IOException("Internal TLS error, this could be an attack");
            }

            return;
        }

        throw new IOException("Internal TLS error, this could be an attack");
    }

    protected void flush() {
        this.recordStream.flush();
    }

    protected abstract AbstractTlsContext getContext();

    public InputStream getInputStream() {
        return this.tlsInputStream;
    }

    public OutputStream getOutputStream() {
        return this.tlsOutputStream;
    }

    protected static int getPRFAlgorithm(int arg1) {
        int v0;
        switch(arg1) {
            case 59: 
            case 60: 
            case 61: 
            case 62: 
            case 63: 
            case 64: 
            case 103: 
            case 104: 
            case 105: 
            case 106: 
            case 107: 
            case 156: 
            case 158: 
            case 160: 
            case 162: 
            case 164: 
            case 49187: 
            case 49189: 
            case 49191: 
            case 49193: 
            case 49195: 
            case 49197: 
            case 49199: 
            case 49201: {
                v0 = 1;
                break;
            }
            case 157: 
            case 159: 
            case 161: 
            case 163: 
            case 165: 
            case 49188: 
            case 49190: 
            case 49192: 
            case 49194: 
            case 49196: 
            case 49198: 
            case 49200: 
            case 49202: {
                v0 = 2;
                break;
            }
            default: {
                v0 = 0;
                break;
            }
        }

        return v0;
    }

    protected abstract TlsPeer getPeer();

    protected abstract void handleChangeCipherSpecMessage();

    protected void handleClose(boolean arg3) {
        if(!this.closed) {
            if((arg3) && !this.appDataReady) {
                this.raiseWarning(90, "User canceled handshake");
            }

            this.failWithError(1, 0);
        }
    }

    protected abstract void handleHandshakeMessage(short arg1, byte[] arg2);

    protected void handleWarningMessage(short arg1) {
    }

    private void processAlert() {
        int v3 = 2;
        while(this.alertQueue.size() >= v3) {
            byte[] v0 = new byte[v3];
            this.alertQueue.read(v0, 0, v3, 0);
            this.alertQueue.removeData(v3);
            short v1 = ((short)v0[0]);
            short v0_1 = ((short)v0[1]);
            this.getPeer().notifyAlertReceived(v1, v0_1);
            if(v1 == v3) {
                this.failedWithError = true;
                this.closed = true;
                try {
                    this.recordStream.close();
                }
                catch(Exception v0_2) {
                }

                throw new IOException("Internal TLS error, this could be an attack");
            }

            if(v0_1 == 0) {
                this.handleClose(false);
            }

            this.handleWarningMessage(v0_1);
        }
    }

    private void processApplicationData() {
    }

    private void processChangeCipherSpec() {
        while(this.changeCipherSpecQueue.size() > 0) {
            byte[] v0 = new byte[1];
            this.changeCipherSpecQueue.read(v0, 0, 1, 0);
            this.changeCipherSpecQueue.removeData(1);
            if(v0[0] != 1) {
                this.failWithError(2, 10);
            }

            this.recordStream.receivedReadCipherSpec();
            this.handleChangeCipherSpecMessage();
        }
    }

    protected void processFinishedMessage(ByteArrayInputStream arg3) {
        byte[] v0 = TlsUtils.readFully(this.expected_verify_data.length, ((InputStream)arg3));
        TlsProtocol.assertEmpty(arg3);
        if(!Arrays.constantTimeAreEqual(this.expected_verify_data, v0)) {
            this.failWithError(2, 51);
        }
    }

    private void processHandshake() {
        int v0_2;
        int v8 = 4;
        do {
            if(this.handshakeQueue.size() >= v8) {
                byte[] v3 = new byte[v8];
                this.handshakeQueue.read(v3, 0, v8, 0);
                ByteArrayInputStream v0 = new ByteArrayInputStream(v3);
                short v4 = TlsUtils.readUint8(((InputStream)v0));
                int v5 = TlsUtils.readUint24(((InputStream)v0));
                if(this.handshakeQueue.size() >= v5 + 4) {
                    byte[] v6 = new byte[v5];
                    this.handshakeQueue.read(v6, 0, v5, v8);
                    this.handshakeQueue.removeData(v5 + 4);
                    switch(v4) {
                        case 0: {
                            goto label_28;
                        }
                        case 20: {
                            if(this.expected_verify_data == null) {
                                boolean v0_1 = !this.getContext().isServer() ? true : false;
                                this.expected_verify_data = this.createVerifyData(v0_1);
                            }
                            else {
                            }

                            break;
                        }
                    }

                    this.recordStream.updateHandshakeData(v3, 0, v8);
                    this.recordStream.updateHandshakeData(v6, 0, v5);
                label_28:
                    this.handleHandshakeMessage(v4, v6);
                    v0_2 = 1;
                }
                else {
                    goto label_43;
                }
            }
            else {
            label_43:
                v0_2 = 0;
            }
        }
        while(v0_2 != 0);
    }

    protected void processRecord(short arg3, byte[] arg4, int arg5, int arg6) {
        switch(arg3) {
            case 20: {
                this.changeCipherSpecQueue.addData(arg4, arg5, arg6);
                this.processChangeCipherSpec();
                break;
            }
            case 21: {
                this.alertQueue.addData(arg4, arg5, arg6);
                this.processAlert();
                break;
            }
            case 22: {
                this.handshakeQueue.addData(arg4, arg5, arg6);
                this.processHandshake();
                break;
            }
            case 23: {
                if(!this.appDataReady) {
                    this.failWithError(2, 10);
                }

                this.applicationDataQueue.addData(arg4, arg5, arg6);
                this.processApplicationData();
                break;
            }
        }
    }

    protected void raiseAlert(short arg6, short arg7, String arg8, Exception arg9) {
        this.getPeer().notifyAlertRaised(arg6, arg7, arg8, arg9);
        this.safeWriteRecord(21, new byte[]{((byte)arg6), ((byte)arg7)}, 0, 2);
    }

    protected void raiseWarning(short arg3, String arg4) {
        this.raiseAlert(1, arg3, arg4, null);
    }

    protected int readApplicationData(byte[] arg4, int arg5, int arg6) {
        int v0 = 0;
        if(arg6 >= 1) {
            while(true) {
                if(this.applicationDataQueue.size() != 0) {
                    break;
                }
                else if(!this.closed) {
                    this.safeReadRecord();
                    continue;
                }
                else if(this.failedWithError) {
                    throw new IOException("Internal TLS error, this could be an attack");
                }
                else {
                    v0 = -1;
                }

                return v0;
            }

            int v1 = Math.min(arg6, this.applicationDataQueue.size());
            this.applicationDataQueue.read(arg4, arg5, v1, 0);
            this.applicationDataQueue.removeData(v1);
            v0 = v1;
        }

        return v0;
    }

    protected static Hashtable readExtensions(ByteArrayInputStream arg4) {
        Hashtable v0;
        if(arg4.available() < 1) {
            v0 = null;
        }
        else {
            byte[] v0_1 = TlsUtils.readOpaque16(((InputStream)arg4));
            TlsProtocol.assertEmpty(arg4);
            ByteArrayInputStream v1 = new ByteArrayInputStream(v0_1);
            v0 = new Hashtable();
            do {
                if(v1.available() > 0) {
                    if(v0.put(Integers.valueOf(TlsUtils.readUint16(((InputStream)v1))), TlsUtils.readOpaque16(((InputStream)v1))) == null) {
                        continue;
                    }

                    break;
                }

                return v0;
            }
            while(true);

            throw new TlsFatalAlert(47);
        }

        return v0;
    }

    protected static Vector readSupplementalDataMessage(ByteArrayInputStream arg5) {
        byte[] v0 = TlsUtils.readOpaque24(((InputStream)arg5));
        TlsProtocol.assertEmpty(arg5);
        ByteArrayInputStream v1 = new ByteArrayInputStream(v0);
        Vector v0_1 = new Vector();
        while(v1.available() > 0) {
            v0_1.addElement(new SupplementalDataEntry(TlsUtils.readUint16(((InputStream)v1)), TlsUtils.readOpaque16(((InputStream)v1))));
        }

        return v0_1;
    }

    protected void safeReadRecord() {
        short v3 = 80;
        short v2 = 2;
        try {
            this.recordStream.readRecord();
            return;
        }
        catch(RuntimeException v0) {
            if(!this.closed) {
                this.failWithError(v2, v3);
            }

            throw v0;
        }
        catch(IOException v0_1) {
            if(!this.closed) {
                this.failWithError(v2, v3);
            }

            throw v0_1;
        }
        catch(TlsFatalAlert v0_2) {
            if(!this.closed) {
                this.failWithError(v2, v0_2.getAlertDescription());
            }

            throw v0_2;
        }
    }

    protected void safeWriteRecord(short arg5, byte[] arg6, int arg7, int arg8) {
        short v3 = 80;
        short v2 = 2;
        try {
            this.recordStream.writeRecord(arg5, arg6, arg7, arg8);
            return;
        }
        catch(RuntimeException v0) {
            if(!this.closed) {
                this.failWithError(v2, v3);
            }

            throw v0;
        }
        catch(IOException v0_1) {
            if(!this.closed) {
                this.failWithError(v2, v3);
            }

            throw v0_1;
        }
        catch(TlsFatalAlert v0_2) {
            if(!this.closed) {
                this.failWithError(v2, v0_2.getAlertDescription());
            }

            throw v0_2;
        }
    }

    protected void sendCertificateMessage(Certificate arg5) {
        if(arg5 == null) {
            arg5 = Certificate.EMPTY_CHAIN;
        }

        if(arg5.getLength() != 0 || (this.getContext().isServer())) {
        label_22:
            ByteArrayOutputStream v0_1 = new ByteArrayOutputStream();
            TlsUtils.writeUint8(11, ((OutputStream)v0_1));
            TlsUtils.writeUint24(0, ((OutputStream)v0_1));
            arg5.encode(((OutputStream)v0_1));
            byte[] v0_2 = v0_1.toByteArray();
            TlsUtils.writeUint24(v0_2.length - 4, v0_2, 1);
            this.safeWriteRecord(22, v0_2, 0, v0_2.length);
        }
        else {
            ProtocolVersion v0 = this.getContext().getServerVersion();
            if(v0.isSSL()) {
                this.raiseWarning(41, v0.toString() + " client didn\'t provide credentials");
            }
            else {
                goto label_22;
            }
        }
    }

    protected void sendChangeCipherSpecMessage() {
        byte[] v0 = new byte[]{1};
        this.safeWriteRecord(20, v0, 0, v0.length);
        this.recordStream.sentWriteCipherSpec();
    }

    protected void sendFinishedMessage() {
        byte[] v0 = this.createVerifyData(this.getContext().isServer());
        ByteArrayOutputStream v1 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(20, ((OutputStream)v1));
        TlsUtils.writeUint24(v0.length, ((OutputStream)v1));
        v1.write(v0);
        v0 = v1.toByteArray();
        this.safeWriteRecord(22, v0, 0, v0.length);
    }

    protected void sendSupplementalDataMessage(Vector arg5) {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(23, ((OutputStream)v0));
        TlsUtils.writeUint24(0, ((OutputStream)v0));
        TlsProtocol.writeSupplementalData(((OutputStream)v0), arg5);
        byte[] v0_1 = v0.toByteArray();
        TlsUtils.writeUint24(v0_1.length - 4, v0_1, 1);
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }

    protected void writeData(byte[] arg4, int arg5, int arg6) {
        short v2 = 23;
        if(this.closed) {
            if(this.failedWithError) {
                throw new IOException("Internal TLS error, this could be an attack");
            }
            else {
                throw new IOException("Sorry, connection has been closed, you cannot write more data");
            }
        }

        while(arg6 > 0) {
            if(this.writeExtraEmptyRecords) {
                this.safeWriteRecord(v2, TlsUtils.EMPTY_BYTES, 0, 0);
            }

            int v0 = Math.min(arg6, 16384);
            this.safeWriteRecord(v2, arg4, arg5, v0);
            arg5 += v0;
            arg6 -= v0;
        }
    }

    protected static void writeExtensions(OutputStream arg4, Hashtable arg5) {
        ByteArrayOutputStream v2 = new ByteArrayOutputStream();
        Enumeration v3 = arg5.keys();
        while(v3.hasMoreElements()) {
            Object v0 = v3.nextElement();
            Object v1 = arg5.get(v0);
            TlsUtils.writeUint16(((Integer)v0).intValue(), ((OutputStream)v2));
            TlsUtils.writeOpaque16(((byte[])v1), ((OutputStream)v2));
        }

        TlsUtils.writeOpaque16(v2.toByteArray(), arg4);
    }

    protected static void writeSupplementalData(OutputStream arg4, Vector arg5) {
        ByteArrayOutputStream v2 = new ByteArrayOutputStream();
        int v1;
        for(v1 = 0; v1 < arg5.size(); ++v1) {
            Object v0 = arg5.elementAt(v1);
            TlsUtils.writeUint16(((SupplementalDataEntry)v0).getDataType(), ((OutputStream)v2));
            TlsUtils.writeOpaque16(((SupplementalDataEntry)v0).getData(), ((OutputStream)v2));
        }

        TlsUtils.writeOpaque24(v2.toByteArray(), arg4);
    }
}

