// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.SecureRandom;
import java.util.Hashtable;
import java.util.Vector;
import org.bouncycastle.crypto.util.PublicKeyFactory;
import org.bouncycastle.util.Arrays;

public class TlsServerProtocol extends TlsProtocol {
    protected CertificateRequest certificateRequest;
    protected byte[] certificateVerifyHash;
    protected Certificate clientCertificate;
    protected short clientCertificateType;
    protected Hashtable clientExtensions;
    protected TlsKeyExchange keyExchange;
    protected int[] offeredCipherSuites;
    protected short[] offeredCompressionMethods;
    protected int selectedCipherSuite;
    protected short selectedCompressionMethod;
    protected TlsCredentials serverCredentials;
    protected Hashtable serverExtensions;
    protected TlsServer tlsServer;
    protected TlsServerContextImpl tlsServerContext;

    public TlsServerProtocol(InputStream arg3, OutputStream arg4, SecureRandom arg5) {
        super(arg3, arg4, arg5);
        this.tlsServer = null;
        this.tlsServerContext = null;
        this.keyExchange = null;
        this.serverCredentials = null;
        this.certificateRequest = null;
        this.clientCertificateType = -1;
        this.clientCertificate = null;
        this.certificateVerifyHash = null;
    }

    public void accept(TlsServer arg5) {
        if(arg5 == null) {
            throw new IllegalArgumentException("\'tlsServer\' cannot be null");
        }

        if(this.tlsServer != null) {
            throw new IllegalStateException("accept can only be called once");
        }

        this.tlsServer = arg5;
        this.securityParameters = new SecurityParameters();
        this.securityParameters.entity = 0;
        this.securityParameters.serverRandom = TlsServerProtocol.createRandomBlock(this.secureRandom);
        this.tlsServerContext = new TlsServerContextImpl(this.secureRandom, this.securityParameters);
        this.tlsServer.init(this.tlsServerContext);
        this.recordStream.init(this.tlsServerContext);
        this.recordStream.setRestrictReadVersion(false);
        this.completeHandshake();
        this.tlsServer.notifyHandshakeComplete();
    }

    protected boolean expectCertificateVerifyMessage() {
        boolean v0 = this.clientCertificateType < 0 || !TlsUtils.hasSigningCapability(this.clientCertificateType) ? false : true;
        return v0;
    }

    protected AbstractTlsContext getContext() {
        return this.tlsServerContext;
    }

    protected TlsPeer getPeer() {
        return this.tlsServer;
    }

    protected void handleChangeCipherSpecMessage() {
        short v1 = 2;
        switch(this.connection_state) {
            case 10: {
                if(this.certificateVerifyHash == null) {
                    goto label_10;
                }

                this.failWithError(v1, 10);
                break;
            }
            case 11: {
                break;
            }
            default: {
                this.failWithError(v1, 40);
                return;
            }
        }

    label_10:
        this.connection_state = 12;
    }

    protected void handleHandshakeMessage(short arg6, byte[] arg7) {
        Vector v2 = null;
        short v4 = 10;
        short v3 = 2;
        ByteArrayInputStream v0 = new ByteArrayInputStream(arg7);
        switch(arg6) {
            case 1: {
                switch(this.connection_state) {
                    case 0: {
                        goto label_12;
                    }
                }

                this.failWithError(v3, v4);
                return;
            label_12:
                this.receiveClientHelloMessage(v0);
                this.connection_state = 1;
                this.sendServerHelloMessage();
                this.connection_state = v3;
                this.securityParameters.prfAlgorithm = TlsServerProtocol.getPRFAlgorithm(this.selectedCipherSuite);
                this.securityParameters.compressionAlgorithm = this.selectedCompressionMethod;
                this.securityParameters.verifyDataLength = 12;
                this.recordStream.notifyHelloComplete();
                Vector v0_1 = this.tlsServer.getServerSupplementalData();
                if(v0_1 != null) {
                    this.sendSupplementalDataMessage(v0_1);
                }

                this.connection_state = 3;
                this.keyExchange = this.tlsServer.getKeyExchange();
                this.keyExchange.init(this.getContext());
                this.serverCredentials = this.tlsServer.getCredentials();
                if(this.serverCredentials == null) {
                    this.keyExchange.skipServerCredentials();
                }
                else {
                    this.keyExchange.processServerCredentials(this.serverCredentials);
                    this.sendCertificateMessage(this.serverCredentials.getCertificate());
                }

                this.connection_state = 4;
                byte[] v0_2 = this.keyExchange.generateServerKeyExchange();
                if(v0_2 != null) {
                    this.sendServerKeyExchangeMessage(v0_2);
                }

                this.connection_state = 5;
                if(this.serverCredentials != null) {
                    this.certificateRequest = this.tlsServer.getCertificateRequest();
                    if(this.certificateRequest != null) {
                        this.keyExchange.validateCertificateRequest(this.certificateRequest);
                        this.sendCertificateRequestMessage(this.certificateRequest);
                    }
                }

                this.connection_state = 6;
                this.sendServerHelloDoneMessage();
                this.connection_state = 7;
                break;
            }
            case 11: {
                switch(this.connection_state) {
                    case 7: {
                        this.tlsServer.processClientSupplementalData(v2);
                        goto label_97;
                    }
                    case 8: {
                    label_97:
                        if(this.certificateRequest == null) {
                            this.failWithError(v3, v4);
                        }

                        this.receiveCertificateMessage(v0);
                        this.connection_state = 9;
                        return;
                    label_134:
                        switch(this.connection_state) {
                            case 10: {
                                goto label_138;
                            }
                        }

                        this.failWithError(v3, v4);
                        return;
                    label_138:
                        if(this.certificateVerifyHash == null) {
                            this.failWithError(v3, v4);
                        }

                        this.receiveCertificateVerifyMessage(v0);
                        this.connection_state = 11;
                        return;
                    label_104:
                        switch(this.connection_state) {
                            case 7: {
                                this.tlsServer.processClientSupplementalData(v2);
                                goto label_110;
                            }
                            case 8: {
                            label_110:
                                if(this.certificateRequest == null) {
                                    this.keyExchange.skipClientCredentials();
                                    goto label_114;
                                }

                                ProtocolVersion v1 = this.getContext().getServerVersion().getEquivalentTLSVersion();
                                if(ProtocolVersion.TLSv12.isEqualOrEarlierVersionOf(v1)) {
                                    this.failWithError(v3, v4);
                                    goto label_114;
                                }

                                if(v1.isSSL()) {
                                    if(this.clientCertificate != null) {
                                        goto label_114;
                                    }

                                    this.failWithError(v3, v4);
                                    goto label_114;
                                }

                                this.notifyClientCertificate(Certificate.EMPTY_CHAIN);
                                break;
                            }
                            case 9: {
                                break;
                            }
                            default: {
                                this.failWithError(v3, v4);
                                return;
                            }
                        }

                    label_114:
                        this.receiveClientKeyExchangeMessage(v0);
                        this.connection_state = v4;
                        return;
                    }
                    default: {
                        this.failWithError(v3, v4);
                        return;
                    }
                }
            }
            case 15: {
                goto label_134;
            }
            case 16: {
                goto label_104;
            }
            case 20: {
                switch(this.connection_state) {
                    case 12: {
                        goto label_149;
                    }
                }

                this.failWithError(v3, v4);
                return;
            label_149:
                this.processFinishedMessage(v0);
                this.connection_state = 13;
                if(this.expectSessionTicket) {
                    this.sendNewSessionTicketMessage(this.tlsServer.getNewSessionTicket());
                }

                this.connection_state = 14;
                this.sendChangeCipherSpecMessage();
                this.connection_state = 15;
                this.sendFinishedMessage();
                this.connection_state = 16;
                break;
            }
            case 23: {
                switch(this.connection_state) {
                    case 7: {
                        goto label_85;
                    }
                }

                this.failWithError(v3, v4);
                return;
            label_85:
                this.tlsServer.processClientSupplementalData(TlsServerProtocol.readSupplementalDataMessage(v0));
                this.connection_state = 8;
                break;
            }
            default: {
                this.failWithError(v3, v4);
                break;
            }
        }
    }

    protected void handleWarningMessage(short arg2) {
        switch(arg2) {
            case 41: {
                if((this.getContext().getServerVersion().isSSL()) && this.certificateRequest != null) {
                    this.notifyClientCertificate(Certificate.EMPTY_CHAIN);
                }

                break;
            }
            default: {
                super.handleWarningMessage(arg2);
                break;
            }
        }
    }

    protected void notifyClientCertificate(Certificate arg3) {
        if(this.certificateRequest == null) {
            throw new IllegalStateException();
        }

        if(this.clientCertificate != null) {
            throw new TlsFatalAlert(10);
        }

        this.clientCertificate = arg3;
        if(arg3.isEmpty()) {
            this.keyExchange.skipClientCredentials();
        }
        else {
            this.clientCertificateType = TlsUtils.getClientCertificateType(arg3, this.serverCredentials.getCertificate());
            this.keyExchange.processClientCertificate(arg3);
        }

        this.tlsServer.notifyClientCertificate(arg3);
    }

    protected void receiveCertificateMessage(ByteArrayInputStream arg2) {
        Certificate v0 = Certificate.parse(((InputStream)arg2));
        TlsServerProtocol.assertEmpty(arg2);
        this.notifyClientCertificate(v0);
    }

    protected void receiveCertificateVerifyMessage(ByteArrayInputStream arg5) {
        byte[] v0 = TlsUtils.readOpaque16(((InputStream)arg5));
        TlsServerProtocol.assertEmpty(arg5);
        try {
            TlsSigner v1 = TlsUtils.createTlsSigner(this.clientCertificateType);
            v1.init(this.getContext());
            v1.verifyRawSignature(v0, PublicKeyFactory.createKey(this.clientCertificate.getCertificateAt(0).getSubjectPublicKeyInfo()), this.certificateVerifyHash);
            return;
        }
        catch(Exception v0_1) {
            throw new TlsFatalAlert(51);
        }
    }

    protected void receiveClientHelloMessage(ByteArrayInputStream arg8) {
        int v3 = 32;
        short v6 = 47;
        short v4 = 2;
        ProtocolVersion v0 = TlsUtils.readVersion(((InputStream)arg8));
        if(v0.isDTLS()) {
            this.failWithError(v4, v6);
        }

        byte[] v1 = TlsUtils.readFully(v3, ((InputStream)arg8));
        if(TlsUtils.readOpaque8(((InputStream)arg8)).length > v3) {
            this.failWithError(v4, v6);
        }

        int v2 = TlsUtils.readUint16(((InputStream)arg8));
        if(v2 < v4 || (v2 & 1) != 0) {
            this.failWithError(v4, 50);
        }

        this.offeredCipherSuites = TlsUtils.readUint16Array(v2 / 2, ((InputStream)arg8));
        v2 = TlsUtils.readUint8(((InputStream)arg8));
        if(v2 < 1) {
            this.failWithError(v4, v6);
        }

        this.offeredCompressionMethods = TlsUtils.readUint8Array(v2, ((InputStream)arg8));
        this.clientExtensions = TlsServerProtocol.readExtensions(arg8);
        this.getContext().setClientVersion(v0);
        this.tlsServer.notifyClientVersion(v0);
        this.securityParameters.clientRandom = v1;
        this.tlsServer.notifyOfferedCipherSuites(this.offeredCipherSuites);
        this.tlsServer.notifyOfferedCompressionMethods(this.offeredCompressionMethods);
        if(TlsServerProtocol.arrayContains(this.offeredCipherSuites, 255)) {
            this.secure_renegotiation = true;
        }

        if(this.clientExtensions != null) {
            Object v0_1 = this.clientExtensions.get(TlsServerProtocol.EXT_RenegotiationInfo);
            if(v0_1 != null) {
                this.secure_renegotiation = true;
                if(!Arrays.constantTimeAreEqual(((byte[])v0_1), TlsServerProtocol.createRenegotiationInfo(TlsUtils.EMPTY_BYTES))) {
                    this.failWithError(v4, 40);
                }
            }
        }

        this.tlsServer.notifySecureRenegotiation(this.secure_renegotiation);
        if(this.clientExtensions != null) {
            this.tlsServer.processClientExtensions(this.clientExtensions);
        }
    }

    protected void receiveClientKeyExchangeMessage(ByteArrayInputStream arg4) {
        this.keyExchange.processClientKeyExchange(((InputStream)arg4));
        TlsServerProtocol.assertEmpty(arg4);
        TlsServerProtocol.establishMasterSecret(this.getContext(), this.keyExchange);
        this.recordStream.setPendingConnectionState(this.tlsServer.getCompression(), this.tlsServer.getCipher());
        if(this.expectCertificateVerifyMessage()) {
            this.certificateVerifyHash = this.recordStream.getCurrentHash(null);
        }
    }

    protected void sendCertificateRequestMessage(CertificateRequest arg5) {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(13, ((OutputStream)v0));
        TlsUtils.writeUint24(0, ((OutputStream)v0));
        arg5.encode(((OutputStream)v0));
        byte[] v0_1 = v0.toByteArray();
        TlsUtils.writeUint24(v0_1.length - 4, v0_1, 1);
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }

    protected void sendNewSessionTicketMessage(NewSessionTicket arg5) {
        if(arg5 == null) {
            throw new TlsFatalAlert(80);
        }

        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(4, ((OutputStream)v0));
        TlsUtils.writeUint24(0, ((OutputStream)v0));
        arg5.encode(((OutputStream)v0));
        byte[] v0_1 = v0.toByteArray();
        TlsUtils.writeUint24(v0_1.length - 4, v0_1, 1);
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }

    protected void sendServerHelloDoneMessage() {
        byte[] v0 = new byte[4];
        TlsUtils.writeUint8(14, v0, 0);
        TlsUtils.writeUint24(0, v0, 1);
        this.safeWriteRecord(22, v0, 0, v0.length);
    }

    protected void sendServerHelloMessage() {
        short v6 = 80;
        short v5 = 2;
        ByteArrayOutputStream v3 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(v5, ((OutputStream)v3));
        TlsUtils.writeUint24(0, ((OutputStream)v3));
        ProtocolVersion v0 = this.tlsServer.getServerVersion();
        if(!v0.isEqualOrEarlierVersionOf(this.getContext().getClientVersion())) {
            this.failWithError(v5, v6);
        }

        this.recordStream.setReadVersion(v0);
        this.recordStream.setWriteVersion(v0);
        this.recordStream.setRestrictReadVersion(true);
        this.getContext().setServerVersion(v0);
        TlsUtils.writeVersion(v0, ((OutputStream)v3));
        v3.write(this.securityParameters.serverRandom);
        TlsUtils.writeOpaque8(TlsUtils.EMPTY_BYTES, ((OutputStream)v3));
        this.selectedCipherSuite = this.tlsServer.getSelectedCipherSuite();
        if(!TlsServerProtocol.arrayContains(this.offeredCipherSuites, this.selectedCipherSuite) || this.selectedCipherSuite == 0 || this.selectedCipherSuite == 255) {
            this.failWithError(v5, v6);
        }

        this.selectedCompressionMethod = this.tlsServer.getSelectedCompressionMethod();
        if(!TlsServerProtocol.arrayContains(this.offeredCompressionMethods, this.selectedCompressionMethod)) {
            this.failWithError(v5, v6);
        }

        TlsUtils.writeUint16(this.selectedCipherSuite, ((OutputStream)v3));
        TlsUtils.writeUint8(this.selectedCompressionMethod, ((OutputStream)v3));
        this.serverExtensions = this.tlsServer.getServerExtensions();
        if(this.secure_renegotiation) {
            int v0_1 = this.serverExtensions == null || !this.serverExtensions.containsKey(TlsServerProtocol.EXT_RenegotiationInfo) ? 1 : 0;
            if(v0_1 == 0) {
                goto label_77;
            }

            if(this.serverExtensions == null) {
                this.serverExtensions = new Hashtable();
            }

            this.serverExtensions.put(TlsServerProtocol.EXT_RenegotiationInfo, TlsServerProtocol.createRenegotiationInfo(TlsUtils.EMPTY_BYTES));
        }

    label_77:
        if(this.serverExtensions != null) {
            this.expectSessionTicket = this.serverExtensions.containsKey(TlsServerProtocol.EXT_SessionTicket);
            TlsServerProtocol.writeExtensions(((OutputStream)v3), this.serverExtensions);
        }

        byte[] v0_2 = v3.toByteArray();
        TlsUtils.writeUint24(v0_2.length - 4, v0_2, 1);
        this.safeWriteRecord(22, v0_2, 0, v0_2.length);
    }

    protected void sendServerKeyExchangeMessage(byte[] arg5) {
        ByteArrayOutputStream v0 = new ByteArrayOutputStream();
        TlsUtils.writeUint8(12, ((OutputStream)v0));
        TlsUtils.writeUint24(arg5.length, ((OutputStream)v0));
        v0.write(arg5);
        byte[] v0_1 = v0.toByteArray();
        this.safeWriteRecord(22, v0_1, 0, v0_1.length);
    }
}

