// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.crypto.tls;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Vector;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x509.Extensions;
import org.bouncycastle.asn1.x509.KeyUsage;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x509.X509ObjectIdentifiers;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.MD5Digest;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.digests.SHA224Digest;
import org.bouncycastle.crypto.digests.SHA256Digest;
import org.bouncycastle.crypto.digests.SHA384Digest;
import org.bouncycastle.crypto.digests.SHA512Digest;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.DSAPublicKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.crypto.util.PublicKeyFactory;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;
import org.bouncycastle.util.Strings;
import org.bouncycastle.util.io.Streams;

public class TlsUtils {
    public static byte[] EMPTY_BYTES;
    public static final Integer EXT_signature_algorithms;
    static final byte[][] SSL3_CONST;
    static final byte[] SSL_CLIENT;
    static final byte[] SSL_SERVER;

    static {
        TlsUtils.EMPTY_BYTES = new byte[0];
        TlsUtils.EXT_signature_algorithms = Integers.valueOf(13);
        TlsUtils.SSL_CLIENT = new byte[]{67, 76, 78, 84};
        TlsUtils.SSL_SERVER = new byte[]{83, 82, 86, 82};
        TlsUtils.SSL3_CONST = TlsUtils.genConst();
    }

    public TlsUtils() {
        super();
    }

    public static byte[] PRF(TlsContext arg4, byte[] arg5, String arg6, byte[] arg7, int arg8) {
        byte[] v0_1;
        ProtocolVersion v1 = arg4.getServerVersion();
        if(v1.isSSL()) {
            throw new IllegalStateException("No PRF available for SSLv3 session");
        }

        byte[] v2 = Strings.toByteArray(arg6);
        byte[] v3 = TlsUtils.concat(v2, arg7);
        int v0 = arg4.getSecurityParameters().getPrfAlgorithm();
        if(v0 != 0) {
        label_19:
            Digest v1_1 = TlsUtils.createPRFHash(v0);
            v0_1 = new byte[arg8];
            TlsUtils.hmac_hash(v1_1, arg5, v3, v0_1);
        }
        else if(!ProtocolVersion.TLSv12.isEqualOrEarlierVersionOf(v1.getEquivalentTLSVersion())) {
            v0_1 = TlsUtils.PRF_legacy(arg5, v2, v3, arg8);
        }
        else {
            v0 = 1;
            goto label_19;
        }

        return v0_1;
    }

    static byte[] PRF_legacy(byte[] arg6, byte[] arg7, byte[] arg8, int arg9) {
        int v0 = 0;
        int v1 = (arg6.length + 1) / 2;
        byte[] v2 = new byte[v1];
        byte[] v3 = new byte[v1];
        System.arraycopy(arg6, 0, v2, 0, v1);
        System.arraycopy(arg6, arg6.length - v1, v3, 0, v1);
        byte[] v1_1 = new byte[arg9];
        byte[] v4 = new byte[arg9];
        TlsUtils.hmac_hash(new MD5Digest(), v2, arg8, v1_1);
        TlsUtils.hmac_hash(new SHA1Digest(), v3, arg8, v4);
        while(v0 < arg9) {
            v1_1[v0] = ((byte)(v1_1[v0] ^ v4[v0]));
            ++v0;
        }

        return v1_1;
    }

    public static void addSignatureAlgorithmsExtension(Hashtable arg2, Vector arg3) {
        arg2.put(TlsUtils.EXT_signature_algorithms, TlsUtils.createSignatureAlgorithmsExtension(arg3));
    }

    static byte[] calculateKeyBlock(TlsContext arg3, int arg4) {
        SecurityParameters v0 = arg3.getSecurityParameters();
        byte[] v1 = v0.getMasterSecret();
        byte[] v0_1 = TlsUtils.concat(v0.getServerRandom(), v0.getClientRandom());
        return arg3.getServerVersion().isSSL() ? TlsUtils.calculateKeyBlock_SSL(v1, v0_1, arg4) : TlsUtils.PRF(arg3, v1, "key expansion", v0_1, arg4);
    }

    static byte[] calculateKeyBlock_SSL(byte[] arg10, byte[] arg11, int arg12) {
        MD5Digest v3 = new MD5Digest();
        SHA1Digest v4 = new SHA1Digest();
        int v5 = ((Digest)v3).getDigestSize();
        byte[] v6 = new byte[((Digest)v4).getDigestSize()];
        byte[] v7 = new byte[arg12 + v5];
        int v0 = 0;
        int v2;
        for(v2 = 0; v0 < arg12; ++v2) {
            ((Digest)v4).update(TlsUtils.SSL3_CONST[v2], 0, TlsUtils.SSL3_CONST[v2].length);
            ((Digest)v4).update(arg10, 0, arg10.length);
            ((Digest)v4).update(arg11, 0, arg11.length);
            ((Digest)v4).doFinal(v6, 0);
            ((Digest)v3).update(arg10, 0, arg10.length);
            ((Digest)v3).update(v6, 0, v6.length);
            ((Digest)v3).doFinal(v7, v0);
            v0 += v5;
        }

        byte[] v0_1 = new byte[arg12];
        System.arraycopy(v7, 0, v0_1, 0, arg12);
        return v0_1;
    }

    static byte[] calculateMasterSecret(TlsContext arg3, byte[] arg4) {
        SecurityParameters v0 = arg3.getSecurityParameters();
        byte[] v0_1 = TlsUtils.concat(v0.getClientRandom(), v0.getServerRandom());
        return arg3.getServerVersion().isSSL() ? TlsUtils.calculateMasterSecret_SSL(arg4, v0_1) : TlsUtils.PRF(arg3, arg4, "master secret", v0_1, 48);
    }

    static byte[] calculateMasterSecret_SSL(byte[] arg10, byte[] arg11) {
        MD5Digest v3 = new MD5Digest();
        SHA1Digest v4 = new SHA1Digest();
        int v5 = ((Digest)v3).getDigestSize();
        byte[] v6 = new byte[((Digest)v4).getDigestSize()];
        byte[] v7 = new byte[v5 * 3];
        int v0 = 0;
        int v2 = 0;
        while(v0 < 3) {
            ((Digest)v4).update(TlsUtils.SSL3_CONST[v0], 0, TlsUtils.SSL3_CONST[v0].length);
            ((Digest)v4).update(arg10, 0, arg10.length);
            ((Digest)v4).update(arg11, 0, arg11.length);
            ((Digest)v4).doFinal(v6, 0);
            ((Digest)v3).update(arg10, 0, arg10.length);
            ((Digest)v3).update(v6, 0, v6.length);
            ((Digest)v3).doFinal(v7, v2);
            v2 += v5;
            ++v0;
        }

        return v7;
    }

    static byte[] calculateVerifyData(TlsContext arg2, String arg3, byte[] arg4) {
        if(!arg2.getServerVersion().isSSL()) {
            SecurityParameters v0 = arg2.getSecurityParameters();
            arg4 = TlsUtils.PRF(arg2, v0.getMasterSecret(), arg3, arg4, v0.getVerifyDataLength());
        }

        return arg4;
    }

    public static final Digest cloneHash(int arg2, Digest arg3) {
        switch(arg2) {
            case 1: {
                goto label_5;
            }
            case 2: {
                goto label_8;
            }
            case 3: {
                goto label_11;
            }
            case 4: {
                goto label_14;
            }
            case 5: {
                goto label_17;
            }
            case 6: {
                goto label_20;
            }
        }

        throw new IllegalArgumentException("unknown HashAlgorithm");
    label_20:
        SHA512Digest v0 = new SHA512Digest(((SHA512Digest)arg3));
        goto label_7;
    label_17:
        SHA384Digest v0_1 = new SHA384Digest(((SHA384Digest)arg3));
        goto label_7;
    label_8:
        SHA1Digest v0_2 = new SHA1Digest(((SHA1Digest)arg3));
        goto label_7;
    label_5:
        MD5Digest v0_3 = new MD5Digest(((MD5Digest)arg3));
        goto label_7;
    label_11:
        SHA224Digest v0_4 = new SHA224Digest(((SHA224Digest)arg3));
        goto label_7;
    label_14:
        SHA256Digest v0_5 = new SHA256Digest(((SHA256Digest)arg3));
    label_7:
        return ((Digest)v0_2);
    }

    public static final Digest clonePRFHash(int arg1, Digest arg2) {
        CombinedHash v0_1;
        switch(arg1) {
            case 0: {
                v0_1 = new CombinedHash(((CombinedHash)arg2));
                break;
            }
            default: {
                Digest v0 = TlsUtils.cloneHash(TlsUtils.getHashAlgorithmForPRFAlgorithm(arg1), arg2);
                break;
            }
        }

        return ((Digest)v0_1);
    }

    static byte[] concat(byte[] arg4, byte[] arg5) {
        byte[] v0 = new byte[arg4.length + arg5.length];
        System.arraycopy(arg4, 0, v0, 0, arg4.length);
        System.arraycopy(arg5, 0, v0, arg4.length, arg5.length);
        return v0;
    }

    public static final Digest createHash(int arg2) {
        switch(arg2) {
            case 1: {
                goto label_5;
            }
            case 2: {
                goto label_8;
            }
            case 3: {
                goto label_11;
            }
            case 4: {
                goto label_14;
            }
            case 5: {
                goto label_17;
            }
            case 6: {
                goto label_20;
            }
        }

        throw new IllegalArgumentException("unknown HashAlgorithm");
    label_20:
        SHA512Digest v0 = new SHA512Digest();
        goto label_7;
    label_17:
        SHA384Digest v0_1 = new SHA384Digest();
        goto label_7;
    label_8:
        SHA1Digest v0_2 = new SHA1Digest();
        goto label_7;
    label_5:
        MD5Digest v0_3 = new MD5Digest();
        goto label_7;
    label_11:
        SHA224Digest v0_4 = new SHA224Digest();
        goto label_7;
    label_14:
        SHA256Digest v0_5 = new SHA256Digest();
    label_7:
        return ((Digest)v0_4);
    }

    public static final Digest createPRFHash(int arg1) {
        CombinedHash v0_1;
        switch(arg1) {
            case 0: {
                v0_1 = new CombinedHash();
                break;
            }
            default: {
                Digest v0 = TlsUtils.createHash(TlsUtils.getHashAlgorithmForPRFAlgorithm(arg1));
                break;
            }
        }

        return ((Digest)v0_1);
    }

    public static byte[] createSignatureAlgorithmsExtension(Vector arg3) {
        if(arg3 != null && arg3.size() >= 1 && arg3.size() < 32768) {
            ByteArrayOutputStream v2 = new ByteArrayOutputStream();
            TlsUtils.writeUint16(arg3.size() * 2, ((OutputStream)v2));
            int v1;
            for(v1 = 0; v1 < arg3.size(); ++v1) {
                arg3.elementAt(v1).encode(((OutputStream)v2));
            }

            return v2.toByteArray();
        }

        throw new IllegalArgumentException("\'supportedSignatureAlgorithms\' must have length from 1 to (2^15 - 1)");
    }

    public static TlsSigner createTlsSigner(short arg2) {
        switch(arg2) {
            case 1: {
                goto label_11;
            }
            case 2: {
                goto label_5;
            }
            case 64: {
                goto label_8;
            }
        }

        throw new IllegalArgumentException("\'clientCertificateType\' is not a type with signing capability");
    label_8:
        TlsECDSASigner v0 = new TlsECDSASigner();
        goto label_7;
    label_5:
        TlsDSSSigner v0_1 = new TlsDSSSigner();
        goto label_7;
    label_11:
        TlsRSASigner v0_2 = new TlsRSASigner();
    label_7:
        return ((TlsSigner)v0_1);
    }

    private static byte[][] genConst() {
        int v1 = 10;
        byte[][] v2 = new byte[v1][];
        int v0;
        for(v0 = 0; v0 < v1; ++v0) {
            byte[] v3 = new byte[v0 + 1];
            Arrays.fill(v3, ((byte)(v0 + 65)));
            v2[v0] = v3;
        }

        return v2;
    }

    static short getClientCertificateType(Certificate arg3, Certificate arg4) {
        short v0;
        if(arg3.isEmpty()) {
            v0 = -1;
        }
        else {
            org.bouncycastle.asn1.x509.Certificate v0_1 = arg3.getCertificateAt(0);
            SubjectPublicKeyInfo v1 = v0_1.getSubjectPublicKeyInfo();
            try {
                AsymmetricKeyParameter v1_1 = PublicKeyFactory.createKey(v1);
                if(v1_1.isPrivate()) {
                    throw new TlsFatalAlert(80);
                }
                else if((v1_1 instanceof RSAKeyParameters)) {
                    TlsUtils.validateKeyUsage(v0_1, 128);
                    v0 = 1;
                }
                else if((v1_1 instanceof DSAPublicKeyParameters)) {
                    TlsUtils.validateKeyUsage(v0_1, 128);
                    v0 = 2;
                }
                else if((v1_1 instanceof ECPublicKeyParameters)) {
                    TlsUtils.validateKeyUsage(v0_1, 128);
                    v0 = 64;
                }
                else {
                    goto label_15;
                }
            }
            catch(Exception v0_2) {
                goto label_15;
            }
        }

        return v0;
    label_15:
        throw new TlsFatalAlert(43);
    }

    public static Vector getDefaultDSSSignatureAlgorithms() {
        return TlsUtils.vectorOfOne(new SignatureAndHashAlgorithm(2, 2));
    }

    public static Vector getDefaultECDSASignatureAlgorithms() {
        return TlsUtils.vectorOfOne(new SignatureAndHashAlgorithm(2, 3));
    }

    public static Vector getDefaultRSASignatureAlgorithms() {
        return TlsUtils.vectorOfOne(new SignatureAndHashAlgorithm(2, 1));
    }

    public static final short getHashAlgorithmForPRFAlgorithm(int arg2) {
        switch(arg2) {
            case 0: {
                goto label_5;
            }
            case 1: {
                return 4;
            }
            case 2: {
                goto label_11;
            }
        }

        throw new IllegalArgumentException("unknown PRFAlgorithm");
    label_5:
        throw new IllegalArgumentException("legacy PRF not a valid algorithm");
    label_11:
        short v0 = 5;
        return v0;
    }

    public static ASN1ObjectIdentifier getOIDForHashAlgorithm(int arg2) {
        switch(arg2) {
            case 1: {
                goto label_5;
            }
            case 2: {
                goto label_7;
            }
            case 3: {
                goto label_9;
            }
            case 4: {
                goto label_11;
            }
            case 5: {
                goto label_13;
            }
            case 6: {
                goto label_15;
            }
        }

        throw new IllegalArgumentException("unknown HashAlgorithm");
    label_7:
        ASN1ObjectIdentifier v0 = X509ObjectIdentifiers.id_SHA1;
        return v0;
    label_5:
        return PKCSObjectIdentifiers.md5;
    label_11:
        return NISTObjectIdentifiers.id_sha256;
    label_9:
        return NISTObjectIdentifiers.id_sha224;
    label_15:
        return NISTObjectIdentifiers.id_sha512;
    label_13:
        return NISTObjectIdentifiers.id_sha384;
    }

    public static Vector getSignatureAlgorithmsExtension(Hashtable arg2) {
        Vector v0;
        Vector v1 = null;
        if(arg2 == null) {
            v0 = v1;
        }
        else {
            Object v0_1 = arg2.get(TlsUtils.EXT_signature_algorithms);
            v0 = v0_1 == null ? v1 : TlsUtils.readSignatureAlgorithmsExtension(((byte[])v0_1));
        }

        return v0;
    }

    public static boolean hasSigningCapability(short arg1) {
        boolean v0;
        switch(arg1) {
            case 1: 
            case 2: 
            case 64: {
                v0 = true;
                break;
            }
            default: {
                v0 = false;
                break;
            }
        }

        return v0;
    }

    static void hmac_hash(Digest arg11, byte[] arg12, byte[] arg13, byte[] arg14) {
        HMac v4 = new HMac(arg11);
        KeyParameter v5 = new KeyParameter(arg12);
        int v6 = arg11.getDigestSize();
        int v7 = (arg14.length + v6 - 1) / v6;
        byte[] v3 = new byte[v4.getMacSize()];
        byte[] v8 = new byte[v4.getMacSize()];
        int v0 = 0;
        byte[] v2;
        for(v2 = arg13; v0 < v7; v2 = v3) {
            v4.init(((CipherParameters)v5));
            v4.update(v2, 0, v2.length);
            v4.doFinal(v3, 0);
            v4.init(((CipherParameters)v5));
            v4.update(v3, 0, v3.length);
            v4.update(arg13, 0, arg13.length);
            v4.doFinal(v8, 0);
            System.arraycopy(v8, 0, arg14, v6 * v0, Math.min(v6, arg14.length - v6 * v0));
            ++v0;
        }
    }

    public static boolean isSignatureAlgorithmsExtensionAllowed(ProtocolVersion arg2) {
        return ProtocolVersion.TLSv12.isEqualOrEarlierVersionOf(arg2.getEquivalentTLSVersion());
    }

    public static boolean isValidUint16(int arg1) {
        boolean v0 = (65535 & arg1) == arg1 ? true : false;
        return v0;
    }

    public static boolean isValidUint24(int arg1) {
        boolean v0 = (16777215 & arg1) == arg1 ? true : false;
        return v0;
    }

    public static boolean isValidUint32(long arg2) {
        boolean v0 = (4294967295L & arg2) == arg2 ? true : false;
        return v0;
    }

    public static boolean isValidUint48(long arg2) {
        boolean v0 = (281474976710655L & arg2) == arg2 ? true : false;
        return v0;
    }

    public static boolean isValidUint64(long arg1) {
        return 1;
    }

    public static boolean isValidUint8(short arg1) {
        boolean v0 = (arg1 & 255) == arg1 ? true : false;
        return v0;
    }

    public static byte[] readFully(int arg2, InputStream arg3) {
        byte[] v0;
        if(arg2 < 1) {
            v0 = TlsUtils.EMPTY_BYTES;
        }
        else {
            v0 = new byte[arg2];
            if(arg2 != Streams.readFully(arg3, v0)) {
                throw new EOFException();
            }
        }

        return v0;
    }

    public static void readFully(byte[] arg2, InputStream arg3) {
        int v0 = arg2.length;
        if(v0 > 0 && v0 != Streams.readFully(arg3, arg2)) {
            throw new EOFException();
        }
    }

    public static byte[] readOpaque16(InputStream arg1) {
        return TlsUtils.readFully(TlsUtils.readUint16(arg1), arg1);
    }

    public static byte[] readOpaque24(InputStream arg1) {
        return TlsUtils.readFully(TlsUtils.readUint24(arg1), arg1);
    }

    public static byte[] readOpaque8(InputStream arg1) {
        return TlsUtils.readFully(TlsUtils.readUint8(arg1), arg1);
    }

    public static Vector readSignatureAlgorithmsExtension(byte[] arg5) {
        if(arg5 == null) {
            throw new IllegalArgumentException("\'extensionValue\' cannot be null");
        }

        ByteArrayInputStream v1 = new ByteArrayInputStream(arg5);
        int v0 = TlsUtils.readUint16(((InputStream)v1));
        if(v0 >= 2 && (v0 & 1) == 0) {
            int v2 = v0 / 2;
            Vector v3 = new Vector(v2);
            for(v0 = 0; v0 < v2; ++v0) {
                v3.addElement(SignatureAndHashAlgorithm.parse(((InputStream)v1)));
            }

            TlsProtocol.assertEmpty(v1);
            return v3;
        }

        throw new TlsFatalAlert(50);
    }

    public static int readUint16(InputStream arg2) {
        int v0 = arg2.read();
        int v1 = arg2.read();
        if(v1 < 0) {
            throw new EOFException();
        }

        return v0 << 8 | v1;
    }

    public static int readUint16(byte[] arg2, int arg3) {
        return (arg2[arg3] & 255) << 8 | arg2[arg3 + 1] & 255;
    }

    public static int[] readUint16Array(int arg3, InputStream arg4) {
        int[] v1 = new int[arg3];
        int v0;
        for(v0 = 0; v0 < arg3; ++v0) {
            v1[v0] = TlsUtils.readUint16(arg4);
        }

        return v1;
    }

    public static int readUint24(InputStream arg3) {
        int v0 = arg3.read();
        int v1 = arg3.read();
        int v2 = arg3.read();
        if(v2 < 0) {
            throw new EOFException();
        }

        return v0 << 16 | v1 << 8 | v2;
    }

    public static int readUint24(byte[] arg3, int arg4) {
        int v1 = arg4 + 1;
        return (arg3[arg4] & 255) << 16 | (arg3[v1] & 255) << 8 | arg3[v1 + 1] & 255;
    }

    public static long readUint32(InputStream arg7) {
        int v0 = arg7.read();
        int v1 = arg7.read();
        int v2 = arg7.read();
        int v3 = arg7.read();
        if(v3 < 0) {
            throw new EOFException();
        }

        return (((long)v1)) << 16 | (((long)v0)) << 24 | (((long)v2)) << 8 | (((long)v3));
    }

    public static long readUint48(byte[] arg6, int arg7) {
        return (((long)TlsUtils.readUint24(arg6, arg7 + 3))) & 4294967295L | ((((long)TlsUtils.readUint24(arg6, arg7))) & 4294967295L) << 24;
    }

    public static long readUint48(InputStream arg9) {
        int v0 = arg9.read();
        int v1 = arg9.read();
        int v2 = arg9.read();
        int v3 = arg9.read();
        int v4 = arg9.read();
        int v5 = arg9.read();
        if(v5 < 0) {
            throw new EOFException();
        }

        return (((long)v1)) << 32 | (((long)v0)) << 40 | (((long)v2)) << 24 | (((long)v3)) << 16 | (((long)v4)) << 8 | (((long)v5));
    }

    public static short readUint8(InputStream arg1) {
        int v0 = arg1.read();
        if(v0 < 0) {
            throw new EOFException();
        }

        return ((short)v0);
    }

    public static short readUint8(byte[] arg1, int arg2) {
        return ((short)arg1[arg2]);
    }

    public static short[] readUint8Array(int arg3, InputStream arg4) {
        short[] v1 = new short[arg3];
        int v0;
        for(v0 = 0; v0 < arg3; ++v0) {
            v1[v0] = TlsUtils.readUint8(arg4);
        }

        return v1;
    }

    public static ProtocolVersion readVersion(InputStream arg2) {
        int v0 = arg2.read();
        int v1 = arg2.read();
        if(v1 < 0) {
            throw new EOFException();
        }

        return ProtocolVersion.get(v0, v1);
    }

    public static ProtocolVersion readVersion(byte[] arg2, int arg3) {
        return ProtocolVersion.get(arg2[arg3] & 255, arg2[arg3 + 1] & 255);
    }

    public static int readVersionRaw(InputStream arg2) {
        int v0 = arg2.read();
        int v1 = arg2.read();
        if(v1 < 0) {
            throw new EOFException();
        }

        return v0 << 8 | v1;
    }

    static void validateKeyUsage(org.bouncycastle.asn1.x509.Certificate arg2, int arg3) {
        Extensions v0 = arg2.getTBSCertificate().getExtensions();
        if(v0 != null) {
            KeyUsage v0_1 = KeyUsage.fromExtensions(v0);
            if(v0_1 != null && (v0_1.getBytes()[0] & 255 & arg3) != arg3) {
                throw new TlsFatalAlert(46);
            }
        }
    }

    private static Vector vectorOfOne(Object arg2) {
        Vector v0 = new Vector(1);
        v0.addElement(arg2);
        return v0;
    }

    public static void writeGMTUnixTime(byte[] arg4, int arg5) {
        int v0 = ((int)(System.currentTimeMillis() / 1000));
        arg4[arg5] = ((byte)(v0 >> 24));
        arg4[arg5 + 1] = ((byte)(v0 >> 16));
        arg4[arg5 + 2] = ((byte)(v0 >> 8));
        arg4[arg5 + 3] = ((byte)v0);
    }

    public static void writeOpaque16(byte[] arg1, OutputStream arg2) {
        TlsUtils.writeUint16(arg1.length, arg2);
        arg2.write(arg1);
    }

    public static void writeOpaque24(byte[] arg1, OutputStream arg2) {
        TlsUtils.writeUint24(arg1.length, arg2);
        arg2.write(arg1);
    }

    public static void writeOpaque8(byte[] arg1, OutputStream arg2) {
        TlsUtils.writeUint8(((short)arg1.length), arg2);
        arg2.write(arg1);
    }

    public static void writeUint16(int arg2, byte[] arg3, int arg4) {
        arg3[arg4] = ((byte)(arg2 >> 8));
        arg3[arg4 + 1] = ((byte)arg2);
    }

    public static void writeUint16(int arg1, OutputStream arg2) {
        arg2.write(arg1 >> 8);
        arg2.write(arg1);
    }

    public static void writeUint16Array(int[] arg2, OutputStream arg3) {
        int v0;
        for(v0 = 0; v0 < arg2.length; ++v0) {
            TlsUtils.writeUint16(arg2[v0], arg3);
        }
    }

    public static void writeUint24(int arg1, OutputStream arg2) {
        arg2.write(arg1 >> 16);
        arg2.write(arg1 >> 8);
        arg2.write(arg1);
    }

    public static void writeUint24(int arg2, byte[] arg3, int arg4) {
        arg3[arg4] = ((byte)(arg2 >> 16));
        arg3[arg4 + 1] = ((byte)(arg2 >> 8));
        arg3[arg4 + 2] = ((byte)arg2);
    }

    public static void writeUint32(long arg2, OutputStream arg4) {
        arg4.write(((int)(arg2 >> 24)));
        arg4.write(((int)(arg2 >> 16)));
        arg4.write(((int)(arg2 >> 8)));
        arg4.write(((int)arg2));
    }

    public static void writeUint32(long arg3, byte[] arg5, int arg6) {
        arg5[arg6] = ((byte)(((int)(arg3 >> 24))));
        arg5[arg6 + 1] = ((byte)(((int)(arg3 >> 16))));
        arg5[arg6 + 2] = ((byte)(((int)(arg3 >> 8))));
        arg5[arg6 + 3] = ((byte)(((int)arg3)));
    }

    public static void writeUint48(long arg3, byte[] arg5, int arg6) {
        arg5[arg6] = ((byte)(((int)(arg3 >> 40))));
        arg5[arg6 + 1] = ((byte)(((int)(arg3 >> 32))));
        arg5[arg6 + 2] = ((byte)(((int)(arg3 >> 24))));
        arg5[arg6 + 3] = ((byte)(((int)(arg3 >> 16))));
        arg5[arg6 + 4] = ((byte)(((int)(arg3 >> 8))));
        arg5[arg6 + 5] = ((byte)(((int)arg3)));
    }

    public static void writeUint64(long arg3, byte[] arg5, int arg6) {
        arg5[arg6] = ((byte)(((int)(arg3 >> 56))));
        arg5[arg6 + 1] = ((byte)(((int)(arg3 >> 48))));
        arg5[arg6 + 2] = ((byte)(((int)(arg3 >> 40))));
        arg5[arg6 + 3] = ((byte)(((int)(arg3 >> 32))));
        arg5[arg6 + 4] = ((byte)(((int)(arg3 >> 24))));
        arg5[arg6 + 5] = ((byte)(((int)(arg3 >> 16))));
        arg5[arg6 + 6] = ((byte)(((int)(arg3 >> 8))));
        arg5[arg6 + 7] = ((byte)(((int)arg3)));
    }

    public static void writeUint64(long arg2, OutputStream arg4) {
        arg4.write(((int)(arg2 >> 56)));
        arg4.write(((int)(arg2 >> 48)));
        arg4.write(((int)(arg2 >> 40)));
        arg4.write(((int)(arg2 >> 32)));
        arg4.write(((int)(arg2 >> 24)));
        arg4.write(((int)(arg2 >> 16)));
        arg4.write(((int)(arg2 >> 8)));
        arg4.write(((int)arg2));
    }

    public static void writeUint8(short arg0, OutputStream arg1) {
        arg1.write(arg0);
    }

    public static void writeUint8(short arg1, byte[] arg2, int arg3) {
        arg2[arg3] = ((byte)arg1);
    }

    public static void writeUint8Array(short[] arg2, OutputStream arg3) {
        int v0;
        for(v0 = 0; v0 < arg2.length; ++v0) {
            TlsUtils.writeUint8(arg2[v0], arg3);
        }
    }

    public static void writeVersion(ProtocolVersion arg1, OutputStream arg2) {
        arg2.write(arg1.getMajorVersion());
        arg2.write(arg1.getMinorVersion());
    }

    public static void writeVersion(ProtocolVersion arg2, byte[] arg3, int arg4) {
        arg3[arg4] = ((byte)arg2.getMajorVersion());
        arg3[arg4 + 1] = ((byte)arg2.getMinorVersion());
    }
}

