// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jcajce.provider.asymmetric.dh;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHParameterSpec;
import javax.crypto.spec.DHPublicKeySpec;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.pkcs.DHParameter;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x9.DHDomainParameters;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.crypto.params.DHPublicKeyParameters;
import org.bouncycastle.jcajce.provider.asymmetric.util.KeyUtil;

public class BCDHPublicKey implements DHPublicKey {
    private transient DHParameterSpec dhSpec;
    private transient SubjectPublicKeyInfo info;
    static final long serialVersionUID = -216691575254424324L;
    private BigInteger y;

    BCDHPublicKey(BigInteger arg1, DHParameterSpec arg2) {
        super();
        this.y = arg1;
        this.dhSpec = arg2;
    }

    BCDHPublicKey(DHPublicKey arg2) {
        super();
        this.y = arg2.getY();
        this.dhSpec = arg2.getParams();
    }

    BCDHPublicKey(DHPublicKeySpec arg4) {
        super();
        this.y = arg4.getY();
        this.dhSpec = new DHParameterSpec(arg4.getP(), arg4.getG());
    }

    public BCDHPublicKey(SubjectPublicKeyInfo arg5) {
        BigInteger v0_1;
        super();
        this.info = arg5;
        try {
            v0_1 = arg5.parsePublicKey().getValue();
        }
        catch(IOException v0) {
            throw new IllegalArgumentException("invalid info structure in DH public key");
        }

        this.y = v0_1;
        ASN1Sequence v0_2 = ASN1Sequence.getInstance(arg5.getAlgorithm().getParameters());
        ASN1ObjectIdentifier v1 = arg5.getAlgorithm().getAlgorithm();
        if((v1.equals(PKCSObjectIdentifiers.dhKeyAgreement)) || (this.isPKCSParam(v0_2))) {
            DHParameter v0_4 = DHParameter.getInstance(v0_2);
            this.dhSpec = v0_4.getL() != null ? new DHParameterSpec(v0_4.getP(), v0_4.getG(), v0_4.getL().intValue()) : new DHParameterSpec(v0_4.getP(), v0_4.getG());
        }
        else if(v1.equals(X9ObjectIdentifiers.dhpublicnumber)) {
            DHDomainParameters v0_3 = DHDomainParameters.getInstance(v0_2);
            this.dhSpec = new DHParameterSpec(v0_3.getP().getValue(), v0_3.getG().getValue());
        }
        else {
            throw new IllegalArgumentException("unknown algorithm type: " + v1);
        }
    }

    BCDHPublicKey(DHPublicKeyParameters arg5) {
        super();
        this.y = arg5.getY();
        this.dhSpec = new DHParameterSpec(arg5.getParameters().getP(), arg5.getParameters().getG(), arg5.getParameters().getL());
    }

    public boolean equals(Object arg4) {
        boolean v0 = false;
        if(((arg4 instanceof DHPublicKey)) && (this.getY().equals(((DHPublicKey)arg4).getY())) && (this.getParams().getG().equals(((DHPublicKey)arg4).getParams().getG())) && (this.getParams().getP().equals(((DHPublicKey)arg4).getParams().getP())) && this.getParams().getL() == ((DHPublicKey)arg4).getParams().getL()) {
            v0 = true;
        }

        return v0;
    }

    public String getAlgorithm() {
        return "DH";
    }

    public byte[] getEncoded() {
        byte[] v0 = this.info != null ? KeyUtil.getEncodedSubjectPublicKeyInfo(this.info) : KeyUtil.getEncodedSubjectPublicKeyInfo(new AlgorithmIdentifier(PKCSObjectIdentifiers.dhKeyAgreement, new DHParameter(this.dhSpec.getP(), this.dhSpec.getG(), this.dhSpec.getL()).toASN1Primitive()), new ASN1Integer(this.y));
        return v0;
    }

    public String getFormat() {
        return "X.509";
    }

    public DHParameterSpec getParams() {
        return this.dhSpec;
    }

    public BigInteger getY() {
        return this.y;
    }

    public int hashCode() {
        return this.getY().hashCode() ^ this.getParams().getG().hashCode() ^ this.getParams().getP().hashCode() ^ this.getParams().getL();
    }

    private boolean isPKCSParam(ASN1Sequence arg6) {
        int v4 = 2;
        boolean v0 = true;
        if(arg6.size() != v4) {
            if(arg6.size() > 3) {
                v0 = false;
            }
            else if(ASN1Integer.getInstance(arg6.getObjectAt(v4)).getValue().compareTo(BigInteger.valueOf(((long)ASN1Integer.getInstance(arg6.getObjectAt(0)).getValue().bitLength()))) > 0) {
                v0 = false;
            }
        }

        return v0;
    }

    private void readObject(ObjectInputStream arg5) {
        arg5.defaultReadObject();
        this.dhSpec = new DHParameterSpec(arg5.readObject(), arg5.readObject(), arg5.readInt());
        this.info = null;
    }

    private void writeObject(ObjectOutputStream arg2) {
        arg2.defaultWriteObject();
        arg2.writeObject(this.dhSpec.getP());
        arg2.writeObject(this.dhSpec.getG());
        arg2.writeInt(this.dhSpec.getL());
    }
}

