// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jcajce.provider.asymmetric.dh;

import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Hashtable;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;
import javax.crypto.interfaces.DHPrivateKey;
import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import org.bouncycastle.crypto.params.DESParameters;
import org.bouncycastle.util.Integers;
import org.bouncycastle.util.Strings;

public class KeyAgreementSpi extends javax.crypto.KeyAgreementSpi {
    private static final Hashtable algorithms;
    private BigInteger g;
    private BigInteger p;
    private BigInteger result;
    private BigInteger x;

    static {
        KeyAgreementSpi.algorithms = new Hashtable();
        Integer v0 = Integers.valueOf(64);
        Integer v1 = Integers.valueOf(192);
        Integer v2 = Integers.valueOf(128);
        Integer v3 = Integers.valueOf(256);
        KeyAgreementSpi.algorithms.put("DES", v0);
        KeyAgreementSpi.algorithms.put("DESEDE", v1);
        KeyAgreementSpi.algorithms.put("BLOWFISH", v2);
        KeyAgreementSpi.algorithms.put("AES", v3);
    }

    public KeyAgreementSpi() {
        super();
    }

    private byte[] bigIntToBytes(BigInteger arg6) {
        byte[] v0;
        byte[] v1 = arg6.toByteArray();
        if(v1[0] == 0) {
            v0 = new byte[v1.length - 1];
            System.arraycopy(v1, 1, v0, 0, v0.length);
        }
        else {
            v0 = v1;
        }

        return v0;
    }

    protected Key engineDoPhase(Key arg5, boolean arg6) {
        if(this.x == null) {
            throw new IllegalStateException("Diffie-Hellman not initialised.");
        }

        if(!(arg5 instanceof DHPublicKey)) {
            throw new InvalidKeyException("DHKeyAgreement doPhase requires DHPublicKey");
        }

        Key v0 = arg5;
        if((((DHPublicKey)v0).getParams().getG().equals(this.g)) && (((DHPublicKey)v0).getParams().getP().equals(this.p))) {
            if(arg6) {
                this.result = ((DHPublicKey)arg5).getY().modPow(this.x, this.p);
                v0 = null;
            }
            else {
                this.result = ((DHPublicKey)arg5).getY().modPow(this.x, this.p);
                BCDHPublicKey v0_1 = new BCDHPublicKey(this.result, ((DHPublicKey)v0).getParams());
            }

            return v0;
        }

        throw new InvalidKeyException("DHPublicKey not for this KeyAgreement!");
    }

    protected int engineGenerateSecret(byte[] arg4, int arg5) {
        if(this.x == null) {
            throw new IllegalStateException("Diffie-Hellman not initialised.");
        }

        byte[] v0 = this.bigIntToBytes(this.result);
        if(arg4.length - arg5 < v0.length) {
            throw new ShortBufferException("DHKeyAgreement - buffer too short");
        }

        System.arraycopy(v0, 0, arg4, arg5, v0.length);
        return v0.length;
    }

    protected SecretKey engineGenerateSecret(String arg6) {
        SecretKeySpec v0;
        if(this.x == null) {
            throw new IllegalStateException("Diffie-Hellman not initialised.");
        }

        String v1 = Strings.toUpperCase(arg6);
        byte[] v2 = this.bigIntToBytes(this.result);
        if(KeyAgreementSpi.algorithms.containsKey(v1)) {
            byte[] v3 = new byte[KeyAgreementSpi.algorithms.get(v1).intValue() / 8];
            System.arraycopy(v2, 0, v3, 0, v3.length);
            if(v1.startsWith("DES")) {
                DESParameters.setOddParity(v3);
            }

            v0 = new SecretKeySpec(v3, arg6);
        }
        else {
            v0 = new SecretKeySpec(v2, arg6);
        }

        return ((SecretKey)v0);
    }

    protected byte[] engineGenerateSecret() {
        if(this.x == null) {
            throw new IllegalStateException("Diffie-Hellman not initialised.");
        }

        return this.bigIntToBytes(this.result);
    }

    protected void engineInit(Key arg3, SecureRandom arg4) {
        if(!(arg3 instanceof DHPrivateKey)) {
            throw new InvalidKeyException("DHKeyAgreement requires DHPrivateKey");
        }

        this.p = ((DHPrivateKey)arg3).getParams().getP();
        this.g = ((DHPrivateKey)arg3).getParams().getG();
        BigInteger v0 = ((DHPrivateKey)arg3).getX();
        this.result = v0;
        this.x = v0;
    }

    protected void engineInit(Key arg3, AlgorithmParameterSpec arg4, SecureRandom arg5) {
        if(!(arg3 instanceof DHPrivateKey)) {
            throw new InvalidKeyException("DHKeyAgreement requires DHPrivateKey for initialisation");
        }

        if(arg4 == null) {
            this.p = ((DHPrivateKey)arg3).getParams().getP();
            this.g = ((DHPrivateKey)arg3).getParams().getG();
        }
        else if(!(arg4 instanceof DHParameterSpec)) {
            throw new InvalidAlgorithmParameterException("DHKeyAgreement only accepts DHParameterSpec");
        }
        else {
            this.p = ((DHParameterSpec)arg4).getP();
            this.g = ((DHParameterSpec)arg4).getG();
        }

        BigInteger v0 = ((DHPrivateKey)arg3).getX();
        this.result = v0;
        this.x = v0;
    }
}

