// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jcajce.provider.asymmetric.ec;

import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Hashtable;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;
import javax.crypto.spec.SecretKeySpec;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x9.X9IntegerConverter;
import org.bouncycastle.crypto.BasicAgreement;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.DerivationFunction;
import org.bouncycastle.crypto.DerivationParameters;
import org.bouncycastle.crypto.agreement.ECMQVBasicAgreement;
import org.bouncycastle.crypto.agreement.kdf.DHKDFParameters;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.ECDomainParameters;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.crypto.params.MQVPrivateParameters;
import org.bouncycastle.crypto.params.MQVPublicParameters;
import org.bouncycastle.jcajce.provider.asymmetric.util.ECUtil;
import org.bouncycastle.jce.interfaces.ECPrivateKey;
import org.bouncycastle.jce.interfaces.ECPublicKey;
import org.bouncycastle.jce.interfaces.MQVPrivateKey;
import org.bouncycastle.jce.interfaces.MQVPublicKey;
import org.bouncycastle.util.Integers;

public class KeyAgreementSpi extends javax.crypto.KeyAgreementSpi {
    private BasicAgreement agreement;
    private static final Hashtable algorithms;
    private static final X9IntegerConverter converter;
    private String kaAlgorithm;
    private DerivationFunction kdf;
    private ECDomainParameters parameters;
    private BigInteger result;

    static {
        KeyAgreementSpi.converter = new X9IntegerConverter();
        KeyAgreementSpi.algorithms = new Hashtable();
        Integer v0 = Integers.valueOf(128);
        Integer v1 = Integers.valueOf(192);
        Integer v2 = Integers.valueOf(256);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes128_CBC.getId(), v0);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes192_CBC.getId(), v1);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes256_CBC.getId(), v2);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes128_wrap.getId(), v0);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes192_wrap.getId(), v1);
        KeyAgreementSpi.algorithms.put(NISTObjectIdentifiers.id_aes256_wrap.getId(), v2);
        KeyAgreementSpi.algorithms.put(PKCSObjectIdentifiers.id_alg_CMS3DESwrap.getId(), v1);
    }

    protected KeyAgreementSpi(String arg1, BasicAgreement arg2, DerivationFunction arg3) {
        super();
        this.kaAlgorithm = arg1;
        this.agreement = arg2;
        this.kdf = arg3;
    }

    private byte[] bigIntToBytes(BigInteger arg4) {
        return KeyAgreementSpi.converter.integerToBytes(arg4, KeyAgreementSpi.converter.getByteLength(this.parameters.getG().getX()));
    }

    protected Key engineDoPhase(Key arg4, boolean arg5) {
        MQVPublicParameters v0;
        if(this.parameters == null) {
            throw new IllegalStateException(this.kaAlgorithm + " not initialised.");
        }

        if(!arg5) {
            throw new IllegalStateException(this.kaAlgorithm + " can only be between two parties.");
        }

        if((this.agreement instanceof ECMQVBasicAgreement)) {
            if(!(arg4 instanceof MQVPublicKey)) {
                throw new InvalidKeyException(this.kaAlgorithm + " key agreement requires " + KeyAgreementSpi.getSimpleName(MQVPublicKey.class) + " for doPhase");
            }
            else {
                v0 = new MQVPublicParameters(ECUtil.generatePublicKeyParameter(((MQVPublicKey)arg4).getStaticKey()), ECUtil.generatePublicKeyParameter(((MQVPublicKey)arg4).getEphemeralKey()));
            }
        }
        else if(!(arg4 instanceof PublicKey)) {
            throw new InvalidKeyException(this.kaAlgorithm + " key agreement requires " + KeyAgreementSpi.getSimpleName(ECPublicKey.class) + " for doPhase");
        }
        else {
            AsymmetricKeyParameter v0_1 = ECUtil.generatePublicKeyParameter(((PublicKey)arg4));
        }

        this.result = this.agreement.calculateAgreement(((CipherParameters)v0));
        return null;
    }

    protected int engineGenerateSecret(byte[] arg5, int arg6) {
        byte[] v0 = this.engineGenerateSecret();
        if(arg5.length - arg6 < v0.length) {
            throw new ShortBufferException(this.kaAlgorithm + " key agreement: need " + v0.length + " bytes");
        }

        System.arraycopy(v0, 0, arg5, arg6, v0.length);
        return v0.length;
    }

    protected byte[] engineGenerateSecret() {
        if(this.kdf != null) {
            throw new UnsupportedOperationException("KDF can only be used when algorithm is known");
        }

        return this.bigIntToBytes(this.result);
    }

    protected SecretKey engineGenerateSecret(String arg5) {
        byte[] v0_1;
        byte[] v1 = this.bigIntToBytes(this.result);
        if(this.kdf == null) {
            v0_1 = v1;
        }
        else if(!KeyAgreementSpi.algorithms.containsKey(arg5)) {
            throw new NoSuchAlgorithmException("unknown algorithm encountered: " + arg5);
        }
        else {
            int v0 = KeyAgreementSpi.algorithms.get(arg5).intValue();
            DHKDFParameters v2 = new DHKDFParameters(new DERObjectIdentifier(arg5), v0, v1);
            v0_1 = new byte[v0 / 8];
            this.kdf.init(((DerivationParameters)v2));
            this.kdf.generateBytes(v0_1, 0, v0_1.length);
        }

        return new SecretKeySpec(v0_1, arg5);
    }

    protected void engineInit(Key arg1, SecureRandom arg2) {
        this.initFromKey(arg1);
    }

    protected void engineInit(Key arg1, AlgorithmParameterSpec arg2, SecureRandom arg3) {
        this.initFromKey(arg1);
    }

    private static String getSimpleName(Class arg2) {
        String v0 = arg2.getName();
        return v0.substring(v0.lastIndexOf(46) + 1);
    }

    private void initFromKey(Key arg5) {
        AsymmetricKeyParameter v2_1;
        AsymmetricKeyParameter v0;
        if(!(this.agreement instanceof ECMQVBasicAgreement)) {
            if(!(arg5 instanceof PrivateKey)) {
                throw new InvalidKeyException(this.kaAlgorithm + " key agreement requires " + KeyAgreementSpi.getSimpleName(ECPrivateKey.class) + " for initialisation");
            }

            v0 = ECUtil.generatePrivateKeyParameter(((PrivateKey)arg5));
            this.parameters = ((ECPrivateKeyParameters)v0).getParameters();
            this.agreement.init(((CipherParameters)v0));
        }
        else if(!(arg5 instanceof MQVPrivateKey)) {
            throw new InvalidKeyException(this.kaAlgorithm + " key agreement requires " + KeyAgreementSpi.getSimpleName(MQVPrivateKey.class) + " for initialisation");
        }
        else {
            v0 = ECUtil.generatePrivateKeyParameter(((MQVPrivateKey)arg5).getStaticPrivateKey());
            AsymmetricKeyParameter v1 = ECUtil.generatePrivateKeyParameter(((MQVPrivateKey)arg5).getEphemeralPrivateKey());
            ECPublicKeyParameters v2 = null;
            if(((MQVPrivateKey)arg5).getEphemeralPublicKey() != null) {
                v2_1 = ECUtil.generatePublicKeyParameter(((MQVPrivateKey)arg5).getEphemeralPublicKey());
            }

            MQVPrivateParameters v3 = new MQVPrivateParameters(((ECPrivateKeyParameters)v0), ((ECPrivateKeyParameters)v1), ((ECPublicKeyParameters)v2_1));
            this.parameters = ((ECPrivateKeyParameters)v0).getParameters();
            this.agreement.init(((CipherParameters)v3));
        }
    }
}

