// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jcajce.provider.asymmetric.ecgost;

import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidParameterException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.ECGenParameterSpec;
import org.bouncycastle.asn1.cryptopro.ECGOST3410NamedCurves;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.generators.ECKeyPairGenerator;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.crypto.params.ECDomainParameters;
import org.bouncycastle.crypto.params.ECKeyGenerationParameters;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.jcajce.provider.asymmetric.util.EC5Util;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.spec.ECNamedCurveGenParameterSpec;
import org.bouncycastle.jce.spec.ECNamedCurveSpec;
import org.bouncycastle.jce.spec.ECParameterSpec;
import org.bouncycastle.math.ec.ECCurve;

public class KeyPairGeneratorSpi extends KeyPairGenerator {
    String algorithm;
    Object ecParams;
    ECKeyPairGenerator engine;
    boolean initialised;
    ECKeyGenerationParameters param;
    SecureRandom random;
    int strength;

    public KeyPairGeneratorSpi() {
        super("ECGOST3410");
        this.ecParams = null;
        this.engine = new ECKeyPairGenerator();
        this.algorithm = "ECGOST3410";
        this.strength = 239;
        this.random = null;
        this.initialised = false;
    }

    public KeyPair generateKeyPair() {
        KeyPair v0_1;
        BCECGOST3410PublicKey v3;
        Object v2;
        if(!this.initialised) {
            throw new IllegalStateException("EC Key Pair Generator not initialised");
        }

        AsymmetricCipherKeyPair v1 = this.engine.generateKeyPair();
        AsymmetricKeyParameter v0 = v1.getPublic();
        AsymmetricKeyParameter v1_1 = v1.getPrivate();
        if((this.ecParams instanceof ECParameterSpec)) {
            v2 = this.ecParams;
            v3 = new BCECGOST3410PublicKey(this.algorithm, ((ECPublicKeyParameters)v0), ((ECParameterSpec)v2));
            v0_1 = new KeyPair(((PublicKey)v3), new BCECGOST3410PrivateKey(this.algorithm, ((ECPrivateKeyParameters)v1_1), v3, ((ECParameterSpec)v2)));
        }
        else if(this.ecParams == null) {
            v0_1 = new KeyPair(new BCECGOST3410PublicKey(this.algorithm, ((ECPublicKeyParameters)v0)), new BCECGOST3410PrivateKey(this.algorithm, ((ECPrivateKeyParameters)v1_1)));
        }
        else {
            v2 = this.ecParams;
            v3 = new BCECGOST3410PublicKey(this.algorithm, ((ECPublicKeyParameters)v0), ((java.security.spec.ECParameterSpec)v2));
            v0_1 = new KeyPair(((PublicKey)v3), new BCECGOST3410PrivateKey(this.algorithm, ((ECPrivateKeyParameters)v1_1), v3, ((java.security.spec.ECParameterSpec)v2)));
        }

        return v0_1;
    }

    public void initialize(int arg3, SecureRandom arg4) {
        this.strength = arg3;
        this.random = arg4;
        if(this.ecParams != null) {
            try {
                this.initialize(this.ecParams, arg4);
                return;
            }
            catch(InvalidAlgorithmParameterException v0) {
                throw new InvalidParameterException("key size not configurable.");
            }
        }

        throw new InvalidParameterException("unknown key size.");
    }

    public void initialize(AlgorithmParameterSpec arg10, SecureRandom arg11) {
        ECCurve v1;
        if((arg10 instanceof ECParameterSpec)) {
            this.ecParams = arg10;
            this.param = new ECKeyGenerationParameters(new ECDomainParameters(arg10.getCurve(), arg10.getG(), arg10.getN()), arg11);
            this.engine.init(this.param);
            this.initialised = true;
        }
        else if((arg10 instanceof java.security.spec.ECParameterSpec)) {
            this.ecParams = arg10;
            v1 = EC5Util.convertCurve(arg10.getCurve());
            this.param = new ECKeyGenerationParameters(new ECDomainParameters(v1, EC5Util.convertPoint(v1, arg10.getGenerator(), false), arg10.getOrder(), BigInteger.valueOf(((long)arg10.getCofactor()))), arg11);
            this.engine.init(this.param);
            this.initialised = true;
        }
        else {
            if(!(arg10 instanceof ECGenParameterSpec) && !(arg10 instanceof ECNamedCurveGenParameterSpec)) {
                if(arg10 == null && BouncyCastleProvider.CONFIGURATION.getEcImplicitlyCa() != null) {
                    ECParameterSpec v0 = BouncyCastleProvider.CONFIGURATION.getEcImplicitlyCa();
                    this.ecParams = arg10;
                    this.param = new ECKeyGenerationParameters(new ECDomainParameters(v0.getCurve(), v0.getG(), v0.getN()), arg11);
                    this.engine.init(this.param);
                    this.initialised = true;
                    return;
                }

                if(arg10 == null && BouncyCastleProvider.CONFIGURATION.getEcImplicitlyCa() == null) {
                    throw new InvalidAlgorithmParameterException("null parameter passed but no implicitCA set");
                }

                throw new InvalidAlgorithmParameterException("parameter object not a ECParameterSpec: " + arg10.getClass().getName());
            }

            String v1_1 = (arg10 instanceof ECGenParameterSpec) ? ((ECGenParameterSpec)arg10).getName() : ((ECNamedCurveGenParameterSpec)arg10).getName();
            ECDomainParameters v6 = ECGOST3410NamedCurves.getByName(v1_1);
            if(v6 == null) {
                throw new InvalidAlgorithmParameterException("unknown curve name: " + v1_1);
            }

            this.ecParams = new ECNamedCurveSpec(v1_1, v6.getCurve(), v6.getG(), v6.getN(), v6.getH(), v6.getSeed());
            Object v0_1 = this.ecParams;
            v1 = EC5Util.convertCurve(((java.security.spec.ECParameterSpec)v0_1).getCurve());
            this.param = new ECKeyGenerationParameters(new ECDomainParameters(v1, EC5Util.convertPoint(v1, ((java.security.spec.ECParameterSpec)v0_1).getGenerator(), false), ((java.security.spec.ECParameterSpec)v0_1).getOrder(), BigInteger.valueOf(((long)((java.security.spec.ECParameterSpec)v0_1).getCofactor()))), arg11);
            this.engine.init(this.param);
            this.initialised = true;
        }
    }
}

