// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jcajce.provider.asymmetric.rsa;

import java.security.AlgorithmParameters;
import java.security.InvalidKeyException;
import java.security.InvalidParameterException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.SignatureSpi;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.MGF1ParameterSpec;
import java.security.spec.PSSParameterSpec;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.crypto.signers.PSSSigner;
import org.bouncycastle.jcajce.provider.util.DigestFactory;

public class PSSSignatureSpi extends SignatureSpi {
    private Digest contentDigest;
    private AlgorithmParameters engineParams;
    private boolean isRaw;
    private Digest mgfDigest;
    private PSSParameterSpec originalSpec;
    private PSSParameterSpec paramSpec;
    private PSSSigner pss;
    private int saltLength;
    private AsymmetricBlockCipher signer;
    private byte trailer;

    protected PSSSignatureSpi(AsymmetricBlockCipher arg2, PSSParameterSpec arg3) {
        this(arg2, arg3, false);
    }

    protected PSSSignatureSpi(AsymmetricBlockCipher arg2, PSSParameterSpec arg3, boolean arg4) {
        super();
        this.signer = arg2;
        this.originalSpec = arg3;
        this.paramSpec = arg3 == null ? PSSParameterSpec.DEFAULT : arg3;
        this.mgfDigest = DigestFactory.getDigest(this.paramSpec.getDigestAlgorithm());
        this.saltLength = this.paramSpec.getSaltLength();
        this.trailer = this.getTrailer(this.paramSpec.getTrailerField());
        this.isRaw = arg4;
        this.setupContentDigest();
    }

    protected Object engineGetParameter(String arg3) {
        throw new UnsupportedOperationException("engineGetParameter unsupported");
    }

    protected AlgorithmParameters engineGetParameters() {
        if(this.engineParams == null && this.paramSpec != null) {
            try {
                this.engineParams = AlgorithmParameters.getInstance("PSS", "BC");
                this.engineParams.init(this.paramSpec);
            }
            catch(Exception v0) {
                throw new RuntimeException(v0.toString());
            }
        }

        return this.engineParams;
    }

    protected void engineInitSign(PrivateKey arg7) {
        if(!(arg7 instanceof RSAPrivateKey)) {
            throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
        }

        this.pss = new PSSSigner(this.signer, this.contentDigest, this.mgfDigest, this.saltLength, this.trailer);
        this.pss.init(true, RSAUtil.generatePrivateKeyParameter(((RSAPrivateKey)arg7)));
    }

    protected void engineInitSign(PrivateKey arg7, SecureRandom arg8) {
        if(!(arg7 instanceof RSAPrivateKey)) {
            throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
        }

        this.pss = new PSSSigner(this.signer, this.contentDigest, this.mgfDigest, this.saltLength, this.trailer);
        this.pss.init(true, new ParametersWithRandom(RSAUtil.generatePrivateKeyParameter(((RSAPrivateKey)arg7)), arg8));
    }

    protected void engineInitVerify(PublicKey arg7) {
        if(!(arg7 instanceof RSAPublicKey)) {
            throw new InvalidKeyException("Supplied key is not a RSAPublicKey instance");
        }

        this.pss = new PSSSigner(this.signer, this.contentDigest, this.mgfDigest, this.saltLength, this.trailer);
        this.pss.init(false, RSAUtil.generatePublicKeyParameter(((RSAPublicKey)arg7)));
    }

    protected void engineSetParameter(String arg3, Object arg4) {
        throw new UnsupportedOperationException("engineSetParameter unsupported");
    }

    protected void engineSetParameter(AlgorithmParameterSpec arg5) {
        if((arg5 instanceof PSSParameterSpec)) {
            if(this.originalSpec != null && !DigestFactory.isSameDigest(this.originalSpec.getDigestAlgorithm(), ((PSSParameterSpec)arg5).getDigestAlgorithm())) {
                throw new InvalidParameterException("parameter must be using " + this.originalSpec.getDigestAlgorithm());
            }

            if(!((PSSParameterSpec)arg5).getMGFAlgorithm().equalsIgnoreCase("MGF1") && !((PSSParameterSpec)arg5).getMGFAlgorithm().equals(PKCSObjectIdentifiers.id_mgf1.getId())) {
                throw new InvalidParameterException("unknown mask generation function specified");
            }

            if(!(((PSSParameterSpec)arg5).getMGFParameters() instanceof MGF1ParameterSpec)) {
                throw new InvalidParameterException("unkown MGF parameters");
            }

            AlgorithmParameterSpec v0 = ((PSSParameterSpec)arg5).getMGFParameters();
            if(!DigestFactory.isSameDigest(((MGF1ParameterSpec)v0).getDigestAlgorithm(), ((PSSParameterSpec)arg5).getDigestAlgorithm())) {
                throw new InvalidParameterException("digest algorithm for MGF should be the same as for PSS parameters.");
            }

            Digest v1 = DigestFactory.getDigest(((MGF1ParameterSpec)v0).getDigestAlgorithm());
            if(v1 == null) {
                throw new InvalidParameterException("no match on MGF digest algorithm: " + ((MGF1ParameterSpec)v0).getDigestAlgorithm());
            }

            this.engineParams = null;
            this.paramSpec = ((PSSParameterSpec)arg5);
            this.mgfDigest = v1;
            this.saltLength = this.paramSpec.getSaltLength();
            this.trailer = this.getTrailer(this.paramSpec.getTrailerField());
            this.setupContentDigest();
            return;
        }

        throw new InvalidParameterException("Only PSSParameterSpec supported");
    }

    protected byte[] engineSign() {
        try {
            return this.pss.generateSignature();
        }
        catch(CryptoException v0) {
            throw new SignatureException(v0.getMessage());
        }
    }

    protected void engineUpdate(byte arg2) {
        this.pss.update(arg2);
    }

    protected void engineUpdate(byte[] arg2, int arg3, int arg4) {
        this.pss.update(arg2, arg3, arg4);
    }

    protected boolean engineVerify(byte[] arg2) {
        return this.pss.verifySignature(arg2);
    }

    private byte getTrailer(int arg3) {
        if(arg3 == 1) {
            return -68;
        }

        throw new IllegalArgumentException("unknown trailer field");
    }

    private void setupContentDigest() {
        this.contentDigest = this.isRaw ? new PSSSignatureSpi$NullPssDigest(this, this.mgfDigest) : this.mgfDigest;
    }
}

