// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.jce.provider;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map$Entry;
import java.util.Set;
import org.bouncycastle.asn1.ASN1OctetString;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralSubtree;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Integers;
import org.bouncycastle.util.Strings;

public class PKIXNameConstraintValidator {
    private Set excludedSubtreesDN;
    private Set excludedSubtreesDNS;
    private Set excludedSubtreesEmail;
    private Set excludedSubtreesIP;
    private Set excludedSubtreesURI;
    private Set permittedSubtreesDN;
    private Set permittedSubtreesDNS;
    private Set permittedSubtreesEmail;
    private Set permittedSubtreesIP;
    private Set permittedSubtreesURI;

    public PKIXNameConstraintValidator() {
        super();
        this.excludedSubtreesDN = new HashSet();
        this.excludedSubtreesDNS = new HashSet();
        this.excludedSubtreesEmail = new HashSet();
        this.excludedSubtreesURI = new HashSet();
        this.excludedSubtreesIP = new HashSet();
    }

    public void addExcludedSubtree(GeneralSubtree arg3) {
        GeneralName v0 = arg3.getBase();
        switch(v0.getTagNo()) {
            case 1: {
                this.excludedSubtreesEmail = this.unionEmail(this.excludedSubtreesEmail, this.extractNameAsString(v0));
                break;
            }
            case 2: {
                this.excludedSubtreesDNS = this.unionDNS(this.excludedSubtreesDNS, this.extractNameAsString(v0));
                break;
            }
            case 4: {
                this.excludedSubtreesDN = this.unionDN(this.excludedSubtreesDN, v0.getName().toASN1Primitive());
                break;
            }
            case 6: {
                this.excludedSubtreesURI = this.unionURI(this.excludedSubtreesURI, this.extractNameAsString(v0));
                break;
            }
            case 7: {
                this.excludedSubtreesIP = this.unionIP(this.excludedSubtreesIP, ASN1OctetString.getInstance(v0.getName()).getOctets());
                break;
            }
        }
    }

    public void checkExcluded(GeneralName arg3) {
        switch(arg3.getTagNo()) {
            case 1: {
                this.checkExcludedEmail(this.excludedSubtreesEmail, this.extractNameAsString(arg3));
                break;
            }
            case 2: {
                this.checkExcludedDNS(this.excludedSubtreesDNS, DERIA5String.getInstance(arg3.getName()).getString());
                break;
            }
            case 4: {
                this.checkExcludedDN(ASN1Sequence.getInstance(arg3.getName().toASN1Primitive()));
                break;
            }
            case 6: {
                this.checkExcludedURI(this.excludedSubtreesURI, DERIA5String.getInstance(arg3.getName()).getString());
                break;
            }
            case 7: {
                this.checkExcludedIP(this.excludedSubtreesIP, ASN1OctetString.getInstance(arg3.getName()).getOctets());
                break;
            }
        }
    }

    private void checkExcludedDN(Set arg3, ASN1Sequence arg4) {
        if(!arg3.isEmpty()) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!PKIXNameConstraintValidator.withinDNSubtree(arg4, v1.next())) {
                        continue;
                    }

                    break;
                }

                return;
            }
            while(true);

            throw new PKIXNameConstraintValidatorException("Subject distinguished name is from an excluded subtree");
        }
    }

    public void checkExcludedDN(ASN1Sequence arg2) {
        this.checkExcludedDN(this.excludedSubtreesDN, arg2);
    }

    private void checkExcludedDNS(Set arg4, String arg5) {
        if(!arg4.isEmpty()) {
            Iterator v1 = arg4.iterator();
            do {
                if(v1.hasNext()) {
                    Object v0 = v1.next();
                    if(!this.withinDomain(arg5, ((String)v0)) && !arg5.equalsIgnoreCase(((String)v0))) {
                        continue;
                    }

                    break;
                }

                return;
            }
            while(true);

            throw new PKIXNameConstraintValidatorException("DNS is from an excluded subtree.");
        }
    }

    private void checkExcludedEmail(Set arg3, String arg4) {
        if(!arg3.isEmpty()) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.emailIsConstrained(arg4, v1.next())) {
                        continue;
                    }

                    break;
                }

                return;
            }
            while(true);

            throw new PKIXNameConstraintValidatorException("Email address is from an excluded subtree.");
        }
    }

    private void checkExcludedIP(Set arg3, byte[] arg4) {
        if(!arg3.isEmpty()) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.isIPConstrained(arg4, v1.next())) {
                        continue;
                    }

                    break;
                }

                return;
            }
            while(true);

            throw new PKIXNameConstraintValidatorException("IP is from an excluded subtree.");
        }
    }

    private void checkExcludedURI(Set arg3, String arg4) {
        if(!arg3.isEmpty()) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.isUriConstrained(arg4, v1.next())) {
                        continue;
                    }

                    break;
                }

                return;
            }
            while(true);

            throw new PKIXNameConstraintValidatorException("URI is from an excluded subtree.");
        }
    }

    public void checkPermitted(GeneralName arg3) {
        switch(arg3.getTagNo()) {
            case 1: {
                this.checkPermittedEmail(this.permittedSubtreesEmail, this.extractNameAsString(arg3));
                break;
            }
            case 2: {
                this.checkPermittedDNS(this.permittedSubtreesDNS, DERIA5String.getInstance(arg3.getName()).getString());
                break;
            }
            case 4: {
                this.checkPermittedDN(ASN1Sequence.getInstance(arg3.getName().toASN1Primitive()));
                break;
            }
            case 6: {
                this.checkPermittedURI(this.permittedSubtreesURI, DERIA5String.getInstance(arg3.getName()).getString());
                break;
            }
            case 7: {
                this.checkPermittedIP(this.permittedSubtreesIP, ASN1OctetString.getInstance(arg3.getName()).getOctets());
                break;
            }
        }
    }

    private void checkPermittedDN(Set arg3, ASN1Sequence arg4) {
        if(arg3 != null && (!arg3.isEmpty() || arg4.size() != 0)) {
            Iterator v1 = arg3.iterator();
            do {
                if(!v1.hasNext()) {
                    goto label_13;
                }
            }
            while(!PKIXNameConstraintValidator.withinDNSubtree(arg4, v1.next()));

            return;
        label_13:
            throw new PKIXNameConstraintValidatorException("Subject distinguished name is not from a permitted subtree");
        }
    }

    public void checkPermittedDN(ASN1Sequence arg2) {
        this.checkPermittedDN(this.permittedSubtreesDN, arg2);
    }

    private void checkPermittedDNS(Set arg4, String arg5) {
        if(arg4 != null) {
            Iterator v1 = arg4.iterator();
            do {
                if(v1.hasNext()) {
                    Object v0 = v1.next();
                    if(!this.withinDomain(arg5, ((String)v0))) {
                        if(!arg5.equalsIgnoreCase(((String)v0))) {
                            continue;
                        }

                        return;
                    }
                }
                else {
                    goto label_11;
                }

                return;
            }
            while(true);

            return;
        label_11:
            if(arg5.length() == 0 && arg4.size() == 0) {
                return;
            }

            throw new PKIXNameConstraintValidatorException("DNS is not from a permitted subtree.");
        }
    }

    private void checkPermittedEmail(Set arg3, String arg4) {
        if(arg3 != null) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.emailIsConstrained(arg4, v1.next())) {
                        continue;
                    }

                    return;
                }
                else {
                    goto label_9;
                }
            }
            while(true);

            return;
        label_9:
            if(arg4.length() == 0 && arg3.size() == 0) {
                return;
            }

            throw new PKIXNameConstraintValidatorException("Subject email address is not from a permitted subtree.");
        }
    }

    private void checkPermittedIP(Set arg3, byte[] arg4) {
        if(arg3 != null) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.isIPConstrained(arg4, v1.next())) {
                        continue;
                    }

                    return;
                }
                else {
                    goto label_9;
                }
            }
            while(true);

            return;
        label_9:
            if(arg4.length == 0 && arg3.size() == 0) {
                return;
            }

            throw new PKIXNameConstraintValidatorException("IP is not from a permitted subtree.");
        }
    }

    private void checkPermittedURI(Set arg3, String arg4) {
        if(arg3 != null) {
            Iterator v1 = arg3.iterator();
            do {
                if(v1.hasNext()) {
                    if(!this.isUriConstrained(arg4, v1.next())) {
                        continue;
                    }

                    return;
                }
                else {
                    goto label_9;
                }
            }
            while(true);

            return;
        label_9:
            if(arg4.length() == 0 && arg3.size() == 0) {
                return;
            }

            throw new PKIXNameConstraintValidatorException("URI is not from a permitted subtree.");
        }
    }

    private boolean collectionsAreEqual(Collection arg7, Collection arg8) {
        boolean v0 = true;
        if(arg7 != arg8) {
            if(arg7 != null && arg8 != null) {
                if(arg7.size() != arg8.size()) {
                    v0 = false;
                }
                else {
                    Iterator v3 = arg7.iterator();
                    do {
                        if(v3.hasNext()) {
                            Object v2 = v3.next();
                            Iterator v4 = arg8.iterator();
                            do {
                                if(v4.hasNext()) {
                                    if(!this.equals(v2, v4.next())) {
                                        continue;
                                    }

                                    break;
                                }
                                else {
                                    goto label_27;
                                }
                            }
                            while(true);

                            int v2_1 = 1;
                            goto label_24;
                        label_27:
                            v2_1 = 0;
                        label_24:
                            if(v2_1 != 0) {
                                continue;
                            }

                            break;
                        }
                        else {
                            return v0;
                        }
                    }
                    while(true);

                    v0 = false;
                }

                return v0;
            }

            v0 = false;
        }

        return v0;
    }

    private static int compareTo(byte[] arg1, byte[] arg2) {
        int v0;
        if(Arrays.areEqual(arg1, arg2)) {
            v0 = 0;
        }
        else if(Arrays.areEqual(PKIXNameConstraintValidator.max(arg1, arg2), arg1)) {
            v0 = 1;
        }
        else {
            v0 = -1;
        }

        return v0;
    }

    private boolean emailIsConstrained(String arg6, String arg7) {
        boolean v0 = true;
        String v2 = arg6.substring(arg6.indexOf(64) + 1);
        if(arg7.indexOf(64) != -1) {
            if(!arg6.equalsIgnoreCase(arg7)) {
                goto label_17;
            }
        }
        else if(arg7.charAt(0) != 46) {
            if(v2.equalsIgnoreCase(arg7)) {
                return v0;
            }

            goto label_17;
        }
        else if(this.withinDomain(v2, arg7)) {
        }
        else {
        label_17:
            v0 = false;
        }

        return v0;
    }

    private boolean equals(Object arg2, Object arg3) {
        boolean v0;
        if(arg2 == arg3) {
            v0 = true;
        }
        else {
            if(arg2 != null && arg3 != null) {
                if(((arg2 instanceof byte[])) && ((arg3 instanceof byte[]))) {
                    return Arrays.areEqual(((byte[])arg2), ((byte[])arg3));
                }

                return arg2.equals(arg3);
            }

            v0 = false;
        }

        return v0;
    }

    public boolean equals(Object arg4) {
        boolean v0 = false;
        if(((arg4 instanceof PKIXNameConstraintValidator)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).excludedSubtreesDN, this.excludedSubtreesDN)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).excludedSubtreesDNS, this.excludedSubtreesDNS)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).excludedSubtreesEmail, this.excludedSubtreesEmail)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).excludedSubtreesIP, this.excludedSubtreesIP)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).excludedSubtreesURI, this.excludedSubtreesURI)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).permittedSubtreesDN, this.permittedSubtreesDN)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).permittedSubtreesDNS, this.permittedSubtreesDNS)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).permittedSubtreesEmail, this.permittedSubtreesEmail)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).permittedSubtreesIP, this.permittedSubtreesIP)) && (this.collectionsAreEqual(((PKIXNameConstraintValidator)arg4).permittedSubtreesURI, this.permittedSubtreesURI))) {
            v0 = true;
        }

        return v0;
    }

    private static String extractHostFromURL(String arg6) {
        int v5 = 47;
        int v3 = -1;
        int v2 = 58;
        String v0 = arg6.substring(arg6.indexOf(v2) + 1);
        if(v0.indexOf("//") != v3) {
            v0 = v0.substring(v0.indexOf("//") + 2);
        }

        if(v0.lastIndexOf(v2) != v3) {
            v0 = v0.substring(0, v0.lastIndexOf(v2));
        }

        v0 = v0.substring(v0.indexOf(v2) + 1);
        v0 = v0.substring(v0.indexOf(64) + 1);
        if(v0.indexOf(v5) != v3) {
            v0 = v0.substring(0, v0.indexOf(v5));
        }

        return v0;
    }

    private byte[][] extractIPsAndSubnetMasks(byte[] arg7, byte[] arg8) {
        int v0 = arg7.length / 2;
        byte[] v1 = new byte[v0];
        byte[] v2 = new byte[v0];
        System.arraycopy(arg7, 0, v1, 0, v0);
        System.arraycopy(arg7, v0, v2, 0, v0);
        byte[] v3 = new byte[v0];
        byte[] v4 = new byte[v0];
        System.arraycopy(arg8, 0, v3, 0, v0);
        System.arraycopy(arg8, v0, v4, 0, v0);
        return new byte[][]{v1, v2, v3, v4};
    }

    private String extractNameAsString(GeneralName arg2) {
        return DERIA5String.getInstance(arg2.getName()).getString();
    }

    public int hashCode() {
        return this.hashCollection(this.excludedSubtreesDN) + this.hashCollection(this.excludedSubtreesDNS) + this.hashCollection(this.excludedSubtreesEmail) + this.hashCollection(this.excludedSubtreesIP) + this.hashCollection(this.excludedSubtreesURI) + this.hashCollection(this.permittedSubtreesDN) + this.hashCollection(this.permittedSubtreesDNS) + this.hashCollection(this.permittedSubtreesEmail) + this.hashCollection(this.permittedSubtreesIP) + this.hashCollection(this.permittedSubtreesURI);
    }

    private int hashCollection(Collection arg5) {
        int v0 = 0;
        if(arg5 != null) {
            Iterator v2 = arg5.iterator();
            int v1;
            for(v1 = 0; v2.hasNext(); v1 = v0) {
                Object v0_1 = v2.next();
                v0 = (v0_1 instanceof byte[]) ? Arrays.hashCode(((byte[])v0_1)) + v1 : v0_1.hashCode() + v1;
            }

            v0 = v1;
        }

        return v0;
    }

    private Set intersectDN(Set arg7, Set arg8) {
        ASN1Sequence v3;
        HashSet v1 = new HashSet();
        Iterator v2 = arg8.iterator();
        while(true) {
        label_3:
            if(!v2.hasNext()) {
                goto label_26;
            }

            v3 = ASN1Sequence.getInstance(v2.next().getBase().getName().toASN1Primitive());
            if(arg7 != null) {
                break;
            }

            if(v3 == null) {
                continue;
            }

            ((Set)v1).add(v3);
        }

        Iterator v4 = arg7.iterator();
        goto label_15;
    label_26:
        return ((Set)v1);
        while(true) {
        label_15:
            if(!v4.hasNext()) {
                goto label_3;
            }

            Object v0 = v4.next();
            if(PKIXNameConstraintValidator.withinDNSubtree(v3, ((ASN1Sequence)v0))) {
                ((Set)v1).add(v3);
                continue;
            }

            if(!PKIXNameConstraintValidator.withinDNSubtree(((ASN1Sequence)v0), v3)) {
                continue;
            }

            ((Set)v1).add(v0);
        }
    }

    private Set intersectDNS(Set arg7, Set arg8) {
        String v3;
        HashSet v1 = new HashSet();
        Iterator v2 = arg8.iterator();
        while(true) {
        label_3:
            if(!v2.hasNext()) {
                goto label_24;
            }

            v3 = this.extractNameAsString(v2.next().getBase());
            if(arg7 != null) {
                break;
            }

            if(v3 == null) {
                continue;
            }

            ((Set)v1).add(v3);
        }

        Iterator v4 = arg7.iterator();
        goto label_13;
    label_24:
        return ((Set)v1);
        while(true) {
        label_13:
            if(!v4.hasNext()) {
                goto label_3;
            }

            Object v0 = v4.next();
            if(this.withinDomain(((String)v0), v3)) {
                ((Set)v1).add(v0);
                continue;
            }

            if(!this.withinDomain(v3, ((String)v0))) {
                continue;
            }

            ((Set)v1).add(v3);
        }
    }

    private Set intersectEmail(Set arg6, Set arg7) {
        String v3;
        HashSet v1 = new HashSet();
        Iterator v2 = arg7.iterator();
        while(true) {
        label_3:
            if(!v2.hasNext()) {
                goto label_18;
            }

            v3 = this.extractNameAsString(v2.next().getBase());
            if(arg6 != null) {
                break;
            }

            if(v3 == null) {
                continue;
            }

            ((Set)v1).add(v3);
        }

        Iterator v4 = arg6.iterator();
        goto label_13;
    label_18:
        return ((Set)v1);
        while(true) {
        label_13:
            if(!v4.hasNext()) {
                goto label_3;
            }

            this.intersectEmail(v3, v4.next(), ((Set)v1));
        }
    }

    private void intersectEmail(String arg4, String arg5, Set arg6) {
        int v2 = -1;
        int v1 = 64;
        if(arg4.indexOf(v1) != v2) {
            String v0 = arg4.substring(arg4.indexOf(v1) + 1);
            if(arg5.indexOf(v1) != v2) {
                if(arg4.equalsIgnoreCase(arg5)) {
                    arg6.add(arg4);
                }
            }
            else if(arg5.startsWith(".")) {
                if(this.withinDomain(v0, arg5)) {
                    arg6.add(arg4);
                }
            }
            else if(v0.equalsIgnoreCase(arg5)) {
                arg6.add(arg4);
            }
        }
        else if(!arg4.startsWith(".")) {
            if(arg5.indexOf(v1) != v2) {
                if(!arg5.substring(arg5.indexOf(v1) + 1).equalsIgnoreCase(arg4)) {
                    return;
                }

                arg6.add(arg5);
                return;
            }

            if(arg5.startsWith(".")) {
                if(!this.withinDomain(arg4, arg5)) {
                    return;
                }

                arg6.add(arg4);
                return;
            }

            if(!arg4.equalsIgnoreCase(arg5)) {
                return;
            }

            arg6.add(arg4);
        }
        else if(arg5.indexOf(v1) != v2) {
            if(this.withinDomain(arg5.substring(arg4.indexOf(v1) + 1), arg4)) {
                arg6.add(arg5);
            }
        }
        else if(arg5.startsWith(".")) {
            if(!this.withinDomain(arg4, arg5) && !arg4.equalsIgnoreCase(arg5)) {
                if(this.withinDomain(arg5, arg4)) {
                    arg6.add(arg5);
                }
                else {
                }

                return;
            }

            arg6.add(arg4);
        }
        else {
            if(!this.withinDomain(arg5, arg4)) {
                return;
            }

            arg6.add(arg5);
        }
    }

    public void intersectEmptyPermittedSubtree(int arg2) {
        switch(arg2) {
            case 1: {
                this.permittedSubtreesEmail = new HashSet();
                break;
            }
            case 2: {
                this.permittedSubtreesDNS = new HashSet();
                break;
            }
            case 4: {
                this.permittedSubtreesDN = new HashSet();
                break;
            }
            case 6: {
                this.permittedSubtreesURI = new HashSet();
                break;
            }
            case 7: {
                this.permittedSubtreesIP = new HashSet();
                break;
            }
        }
    }

    private Set intersectIP(Set arg6, Set arg7) {
        byte[] v3;
        HashSet v1 = new HashSet();
        Iterator v2 = arg7.iterator();
        while(true) {
        label_3:
            if(!v2.hasNext()) {
                goto label_21;
            }

            v3 = ASN1OctetString.getInstance(v2.next().getBase().getName()).getOctets();
            if(arg6 != null) {
                break;
            }

            if(v3 == null) {
                continue;
            }

            ((Set)v1).add(v3);
        }

        Iterator v4 = arg6.iterator();
        goto label_15;
    label_21:
        return ((Set)v1);
        while(true) {
        label_15:
            if(!v4.hasNext()) {
                goto label_3;
            }

            ((Set)v1).addAll(this.intersectIPRange(v4.next(), v3));
        }
    }

    private Set intersectIPRange(byte[] arg10, byte[] arg11) {
        Set v0;
        int v4 = 3;
        int v8 = 2;
        if(arg10.length != arg11.length) {
            v0 = Collections.EMPTY_SET;
        }
        else {
            byte[][] v0_1 = this.extractIPsAndSubnetMasks(arg10, arg11);
            byte[] v1 = v0_1[0];
            byte[] v2 = v0_1[1];
            byte[] v3 = v0_1[v8];
            byte[] v0_2 = v0_1[v4];
            byte[][] v1_1 = this.minMaxIPs(v1, v2, v3, v0_2);
            v0 = PKIXNameConstraintValidator.compareTo(PKIXNameConstraintValidator.max(v1_1[0], v1_1[v8]), PKIXNameConstraintValidator.min(v1_1[1], v1_1[v4])) == 1 ? Collections.EMPTY_SET : Collections.singleton(this.ipWithSubnetMask(PKIXNameConstraintValidator.or(v1_1[0], v1_1[v8]), PKIXNameConstraintValidator.or(v2, v0_2)));
        }

        return v0;
    }

    public void intersectPermittedSubtree(GeneralSubtree arg3) {
        this.intersectPermittedSubtree(new GeneralSubtree[]{arg3});
    }

    public void intersectPermittedSubtree(GeneralSubtree[] arg6) {
        HashMap v2 = new HashMap();
        int v1;
        for(v1 = 0; v1 != arg6.length; ++v1) {
            GeneralSubtree v3 = arg6[v1];
            Integer v0 = Integers.valueOf(v3.getBase().getTagNo());
            if(((Map)v2).get(v0) == null) {
                ((Map)v2).put(v0, new HashSet());
            }

            ((Map)v2).get(v0).add(v3);
        }

        Iterator v2_1 = ((Map)v2).entrySet().iterator();
        while(v2_1.hasNext()) {
            Object v0_1 = v2_1.next();
            switch(((Map$Entry)v0_1).getKey().intValue()) {
                case 1: {
                    goto label_29;
                }
                case 2: {
                    goto label_34;
                }
                case 4: {
                    goto label_39;
                }
                case 6: {
                    goto label_44;
                }
                case 7: {
                    goto label_49;
                }
            }

            continue;
        label_39:
            this.permittedSubtreesDN = this.intersectDN(this.permittedSubtreesDN, ((Map$Entry)v0_1).getValue());
            continue;
        label_49:
            this.permittedSubtreesIP = this.intersectIP(this.permittedSubtreesIP, ((Map$Entry)v0_1).getValue());
            continue;
        label_34:
            this.permittedSubtreesDNS = this.intersectDNS(this.permittedSubtreesDNS, ((Map$Entry)v0_1).getValue());
            continue;
        label_44:
            this.permittedSubtreesURI = this.intersectURI(this.permittedSubtreesURI, ((Map$Entry)v0_1).getValue());
            continue;
        label_29:
            this.permittedSubtreesEmail = this.intersectEmail(this.permittedSubtreesEmail, ((Map$Entry)v0_1).getValue());
        }
    }

    private Set intersectURI(Set arg6, Set arg7) {
        String v3;
        HashSet v1 = new HashSet();
        Iterator v2 = arg7.iterator();
        while(true) {
        label_3:
            if(!v2.hasNext()) {
                goto label_18;
            }

            v3 = this.extractNameAsString(v2.next().getBase());
            if(arg6 != null) {
                break;
            }

            if(v3 == null) {
                continue;
            }

            ((Set)v1).add(v3);
        }

        Iterator v4 = arg6.iterator();
        goto label_13;
    label_18:
        return ((Set)v1);
        while(true) {
        label_13:
            if(!v4.hasNext()) {
                goto label_3;
            }

            this.intersectURI(v4.next(), v3, ((Set)v1));
        }
    }

    private void intersectURI(String arg4, String arg5, Set arg6) {
        int v2 = -1;
        int v1 = 64;
        if(arg4.indexOf(v1) != v2) {
            String v0 = arg4.substring(arg4.indexOf(v1) + 1);
            if(arg5.indexOf(v1) != v2) {
                if(arg4.equalsIgnoreCase(arg5)) {
                    arg6.add(arg4);
                }
            }
            else if(arg5.startsWith(".")) {
                if(this.withinDomain(v0, arg5)) {
                    arg6.add(arg4);
                }
            }
            else if(v0.equalsIgnoreCase(arg5)) {
                arg6.add(arg4);
            }
        }
        else if(!arg4.startsWith(".")) {
            if(arg5.indexOf(v1) != v2) {
                if(!arg5.substring(arg5.indexOf(v1) + 1).equalsIgnoreCase(arg4)) {
                    return;
                }

                arg6.add(arg5);
                return;
            }

            if(arg5.startsWith(".")) {
                if(!this.withinDomain(arg4, arg5)) {
                    return;
                }

                arg6.add(arg4);
                return;
            }

            if(!arg4.equalsIgnoreCase(arg5)) {
                return;
            }

            arg6.add(arg4);
        }
        else if(arg5.indexOf(v1) != v2) {
            if(this.withinDomain(arg5.substring(arg4.indexOf(v1) + 1), arg4)) {
                arg6.add(arg5);
            }
        }
        else if(arg5.startsWith(".")) {
            if(!this.withinDomain(arg4, arg5) && !arg4.equalsIgnoreCase(arg5)) {
                if(this.withinDomain(arg5, arg4)) {
                    arg6.add(arg5);
                }
                else {
                }

                return;
            }

            arg6.add(arg4);
        }
        else {
            if(!this.withinDomain(arg5, arg4)) {
                return;
            }

            arg6.add(arg5);
        }
    }

    private byte[] ipWithSubnetMask(byte[] arg4, byte[] arg5) {
        int v0 = arg4.length;
        byte[] v1 = new byte[v0 * 2];
        System.arraycopy(arg4, 0, v1, 0, v0);
        System.arraycopy(arg5, 0, v1, v0, v0);
        return v1;
    }

    private boolean isIPConstrained(byte[] arg8, byte[] arg9) {
        boolean v0 = false;
        int v1 = arg8.length;
        if(v1 == arg9.length / 2) {
            byte[] v2 = new byte[v1];
            System.arraycopy(arg9, v1, v2, 0, v1);
            byte[] v3 = new byte[v1];
            byte[] v4 = new byte[v1];
            while((((int)v0)) < v1) {
                v3[((int)v0)] = ((byte)(arg9[((int)v0)] & v2[((int)v0)]));
                v4[((int)v0)] = ((byte)(arg8[((int)v0)] & v2[((int)v0)]));
                int v0_1 = (((int)v0)) + 1;
            }

            v0 = Arrays.areEqual(v3, v4);
        }

        return v0;
    }

    private boolean isUriConstrained(String arg4, String arg5) {
        boolean v0 = true;
        String v1 = PKIXNameConstraintValidator.extractHostFromURL(arg4);
        if(!arg5.startsWith(".")) {
            if(!v1.equalsIgnoreCase(arg5)) {
                goto label_10;
            }
        }
        else if(!this.withinDomain(v1, arg5)) {
        label_10:
            v0 = false;
        }

        return v0;
    }

    private static byte[] max(byte[] arg4, byte[] arg5) {
        int v3 = 65535;
        int v0 = 0;
        while(true) {
            if(v0 >= arg4.length) {
                return arg5;
            }
            else if((arg4[v0] & v3) <= (arg5[v0] & v3)) {
                ++v0;
                continue;
            }

            return arg4;
        }

        return arg5;
    }

    private static byte[] min(byte[] arg4, byte[] arg5) {
        int v3 = 65535;
        int v0 = 0;
        while(true) {
            if(v0 >= arg4.length) {
                return arg5;
            }
            else if((arg4[v0] & v3) >= (arg5[v0] & v3)) {
                ++v0;
                continue;
            }

            return arg4;
        }

        return arg5;
    }

    private byte[][] minMaxIPs(byte[] arg10, byte[] arg11, byte[] arg12, byte[] arg13) {
        int v2 = arg10.length;
        byte[] v3 = new byte[v2];
        byte[] v4 = new byte[v2];
        byte[] v5 = new byte[v2];
        byte[] v6 = new byte[v2];
        int v0;
        for(v0 = 0; v0 < v2; ++v0) {
            v3[v0] = ((byte)(arg10[v0] & arg11[v0]));
            v4[v0] = ((byte)(arg10[v0] & arg11[v0] | arg11[v0] ^ -1));
            v5[v0] = ((byte)(arg12[v0] & arg13[v0]));
            v6[v0] = ((byte)(arg12[v0] & arg13[v0] | arg13[v0] ^ -1));
        }

        return new byte[][]{v3, v4, v5, v6};
    }

    private static byte[] or(byte[] arg4, byte[] arg5) {
        byte[] v1 = new byte[arg4.length];
        int v0;
        for(v0 = 0; v0 < arg4.length; ++v0) {
            v1[v0] = ((byte)(arg4[v0] | arg5[v0]));
        }

        return v1;
    }

    private String stringifyIP(byte[] arg5) {
        String v2 = "";
        int v0;
        for(v0 = 0; v0 < arg5.length / 2; ++v0) {
            v2 = v2 + Integer.toString(arg5[v0] & 255) + ".";
        }

        v2 = v2.substring(0, v2.length() - 1) + "/";
        for(v0 = arg5.length / 2; v0 < arg5.length; ++v0) {
            v2 = v2 + Integer.toString(arg5[v0] & 255) + ".";
        }

        return v2.substring(0, v2.length() - 1);
    }

    private String stringifyIPCollection(Set arg4) {
        String v0 = "" + "[";
        Iterator v1 = arg4.iterator();
        while(v1.hasNext()) {
            v0 = v0 + this.stringifyIP(v1.next()) + ",";
        }

        if(v0.length() > 1) {
            v0 = v0.substring(0, v0.length() - 1);
        }

        return v0 + "]";
    }

    public String toString() {
        String v0 = "" + "permitted:\n";
        if(this.permittedSubtreesDN != null) {
            v0 = v0 + "DN:\n" + this.permittedSubtreesDN.toString() + "\n";
        }

        if(this.permittedSubtreesDNS != null) {
            v0 = v0 + "DNS:\n" + this.permittedSubtreesDNS.toString() + "\n";
        }

        if(this.permittedSubtreesEmail != null) {
            v0 = v0 + "Email:\n" + this.permittedSubtreesEmail.toString() + "\n";
        }

        if(this.permittedSubtreesURI != null) {
            v0 = v0 + "URI:\n" + this.permittedSubtreesURI.toString() + "\n";
        }

        if(this.permittedSubtreesIP != null) {
            v0 = v0 + "IP:\n" + this.stringifyIPCollection(this.permittedSubtreesIP) + "\n";
        }

        v0 += "excluded:\n";
        if(!this.excludedSubtreesDN.isEmpty()) {
            v0 = v0 + "DN:\n" + this.excludedSubtreesDN.toString() + "\n";
        }

        if(!this.excludedSubtreesDNS.isEmpty()) {
            v0 = v0 + "DNS:\n" + this.excludedSubtreesDNS.toString() + "\n";
        }

        if(!this.excludedSubtreesEmail.isEmpty()) {
            v0 = v0 + "Email:\n" + this.excludedSubtreesEmail.toString() + "\n";
        }

        if(!this.excludedSubtreesURI.isEmpty()) {
            v0 = v0 + "URI:\n" + this.excludedSubtreesURI.toString() + "\n";
        }

        if(!this.excludedSubtreesIP.isEmpty()) {
            v0 = v0 + "IP:\n" + this.stringifyIPCollection(this.excludedSubtreesIP) + "\n";
        }

        return v0;
    }

    private Set unionDN(Set arg5, ASN1Sequence arg6) {
        HashSet v5;
        if(!arg5.isEmpty()) {
            HashSet v1 = new HashSet();
            Iterator v2 = arg5.iterator();
            while(v2.hasNext()) {
                Object v0 = v2.next();
                if(PKIXNameConstraintValidator.withinDNSubtree(arg6, ((ASN1Sequence)v0))) {
                    ((Set)v1).add(v0);
                }
                else if(PKIXNameConstraintValidator.withinDNSubtree(((ASN1Sequence)v0), arg6)) {
                    ((Set)v1).add(arg6);
                }
                else {
                    ((Set)v1).add(v0);
                    ((Set)v1).add(arg6);
                }
            }

            v5 = v1;
        }
        else if(arg6 != null) {
            arg5.add(arg6);
        }

        return ((Set)v5);
    }

    protected Set unionDNS(Set arg5, String arg6) {
        HashSet v5;
        if(!arg5.isEmpty()) {
            HashSet v1 = new HashSet();
            Iterator v2 = arg5.iterator();
            while(v2.hasNext()) {
                Object v0 = v2.next();
                if(this.withinDomain(((String)v0), arg6)) {
                    ((Set)v1).add(arg6);
                }
                else if(this.withinDomain(arg6, ((String)v0))) {
                    ((Set)v1).add(v0);
                }
                else {
                    ((Set)v1).add(v0);
                    ((Set)v1).add(arg6);
                }
            }

            v5 = v1;
        }
        else if(arg6 != null) {
            arg5.add(arg6);
        }

        return ((Set)v5);
    }

    private Set unionEmail(Set arg4, String arg5) {
        HashSet v4;
        if(!arg4.isEmpty()) {
            HashSet v1 = new HashSet();
            Iterator v2 = arg4.iterator();
            while(v2.hasNext()) {
                this.unionEmail(v2.next(), arg5, ((Set)v1));
            }

            v4 = v1;
        }
        else if(arg5 != null) {
            arg4.add(arg5);
        }

        return ((Set)v4);
    }

    private void unionEmail(String arg4, String arg5, Set arg6) {
        int v2 = -1;
        int v1 = 64;
        if(arg4.indexOf(v1) != v2) {
            String v0 = arg4.substring(arg4.indexOf(v1) + 1);
            if(arg5.indexOf(v1) != v2) {
                if(arg4.equalsIgnoreCase(arg5)) {
                    arg6.add(arg4);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }
            }
            else if(arg5.startsWith(".")) {
                if(this.withinDomain(v0, arg5)) {
                    arg6.add(arg5);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }
            }
            else if(v0.equalsIgnoreCase(arg5)) {
                arg6.add(arg5);
            }
            else {
                arg6.add(arg4);
                arg6.add(arg5);
            }
        }
        else if(!arg4.startsWith(".")) {
            if(arg5.indexOf(v1) != v2) {
                if(arg5.substring(arg4.indexOf(v1) + 1).equalsIgnoreCase(arg4)) {
                    arg6.add(arg4);
                    return;
                }

                arg6.add(arg4);
                arg6.add(arg5);
                return;
            }

            if(arg5.startsWith(".")) {
                if(this.withinDomain(arg4, arg5)) {
                    arg6.add(arg5);
                    return;
                }

                arg6.add(arg4);
                arg6.add(arg5);
                return;
            }

            if(arg4.equalsIgnoreCase(arg5)) {
                arg6.add(arg4);
                return;
            }

            arg6.add(arg4);
            arg6.add(arg5);
        }
        else if(arg5.indexOf(v1) != v2) {
            if(this.withinDomain(arg5.substring(arg4.indexOf(v1) + 1), arg4)) {
                arg6.add(arg4);
            }
            else {
                arg6.add(arg4);
                arg6.add(arg5);
            }
        }
        else if(arg5.startsWith(".")) {
            if(!this.withinDomain(arg4, arg5) && !arg4.equalsIgnoreCase(arg5)) {
                if(this.withinDomain(arg5, arg4)) {
                    arg6.add(arg4);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }

                return;
            }

            arg6.add(arg5);
        }
        else {
            if(this.withinDomain(arg5, arg4)) {
                arg6.add(arg4);
                return;
            }

            arg6.add(arg4);
            arg6.add(arg5);
        }
    }

    private Set unionIP(Set arg4, byte[] arg5) {
        HashSet v4;
        if(!arg4.isEmpty()) {
            HashSet v1 = new HashSet();
            Iterator v2 = arg4.iterator();
            while(v2.hasNext()) {
                ((Set)v1).addAll(this.unionIPRange(v2.next(), arg5));
            }

            v4 = v1;
        }
        else if(arg5 != null) {
            arg4.add(arg5);
        }

        return ((Set)v4);
    }

    private Set unionIPRange(byte[] arg3, byte[] arg4) {
        HashSet v0 = new HashSet();
        if(Arrays.areEqual(arg3, arg4)) {
            ((Set)v0).add(arg3);
        }
        else {
            ((Set)v0).add(arg3);
            ((Set)v0).add(arg4);
        }

        return ((Set)v0);
    }

    private Set unionURI(Set arg4, String arg5) {
        HashSet v4;
        if(!arg4.isEmpty()) {
            HashSet v1 = new HashSet();
            Iterator v2 = arg4.iterator();
            while(v2.hasNext()) {
                this.unionURI(v2.next(), arg5, ((Set)v1));
            }

            v4 = v1;
        }
        else if(arg5 != null) {
            arg4.add(arg5);
        }

        return ((Set)v4);
    }

    private void unionURI(String arg4, String arg5, Set arg6) {
        int v2 = -1;
        int v1 = 64;
        if(arg4.indexOf(v1) != v2) {
            String v0 = arg4.substring(arg4.indexOf(v1) + 1);
            if(arg5.indexOf(v1) != v2) {
                if(arg4.equalsIgnoreCase(arg5)) {
                    arg6.add(arg4);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }
            }
            else if(arg5.startsWith(".")) {
                if(this.withinDomain(v0, arg5)) {
                    arg6.add(arg5);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }
            }
            else if(v0.equalsIgnoreCase(arg5)) {
                arg6.add(arg5);
            }
            else {
                arg6.add(arg4);
                arg6.add(arg5);
            }
        }
        else if(!arg4.startsWith(".")) {
            if(arg5.indexOf(v1) != v2) {
                if(arg5.substring(arg4.indexOf(v1) + 1).equalsIgnoreCase(arg4)) {
                    arg6.add(arg4);
                    return;
                }

                arg6.add(arg4);
                arg6.add(arg5);
                return;
            }

            if(arg5.startsWith(".")) {
                if(this.withinDomain(arg4, arg5)) {
                    arg6.add(arg5);
                    return;
                }

                arg6.add(arg4);
                arg6.add(arg5);
                return;
            }

            if(arg4.equalsIgnoreCase(arg5)) {
                arg6.add(arg4);
                return;
            }

            arg6.add(arg4);
            arg6.add(arg5);
        }
        else if(arg5.indexOf(v1) != v2) {
            if(this.withinDomain(arg5.substring(arg4.indexOf(v1) + 1), arg4)) {
                arg6.add(arg4);
            }
            else {
                arg6.add(arg4);
                arg6.add(arg5);
            }
        }
        else if(arg5.startsWith(".")) {
            if(!this.withinDomain(arg4, arg5) && !arg4.equalsIgnoreCase(arg5)) {
                if(this.withinDomain(arg5, arg4)) {
                    arg6.add(arg4);
                }
                else {
                    arg6.add(arg4);
                    arg6.add(arg5);
                }

                return;
            }

            arg6.add(arg5);
        }
        else {
            if(this.withinDomain(arg5, arg4)) {
                arg6.add(arg4);
                return;
            }

            arg6.add(arg4);
            arg6.add(arg5);
        }
    }

    private static boolean withinDNSubtree(ASN1Sequence arg5, ASN1Sequence arg6) {
        boolean v0 = false;
        if(arg6.size() >= 1 && arg6.size() <= arg5.size()) {
            int v2 = arg6.size() - 1;
            while(true) {
                if(v2 < 0) {
                    break;
                }
                else if(arg6.getObjectAt(v2).equals(arg5.getObjectAt(v2))) {
                    --v2;
                    continue;
                }

                return v0;
            }

            v0 = true;
        }

        return v0;
    }

    private boolean withinDomain(String arg10, String arg11) {
        boolean v0;
        char v5 = '.';
        int v1 = -1;
        if(arg11.startsWith(".")) {
            arg11 = arg11.substring(1);
        }

        String[] v4 = Strings.split(arg11, v5);
        String[] v5_1 = Strings.split(arg10, v5);
        if(v5_1.length <= v4.length) {
            v0 = false;
        }
        else {
            int v6 = v5_1.length - v4.length;
            int v0_1;
            for(v0_1 = v1; v0_1 < v4.length; ++v0_1) {
                if(v0_1 == v1) {
                    if(v5_1[v0_1 + v6].equals("")) {
                        return false;
                    }
                }
                else if(!v4[v0_1].equalsIgnoreCase(v5_1[v0_1 + v6])) {
                    return false;
                }
            }

            v0 = true;
        }

        return v0;
    }
}

