// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.ocsp;

import java.security.MessageDigest;
import java.security.Signature;
import java.security.cert.CertStore;
import java.security.cert.CertStoreParameters;
import java.security.cert.CertificateFactory;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.bouncycastle.asn1.DERNull;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.util.Strings;

class OCSPUtil {
    private static Hashtable algorithms;
    private static Set noParams;
    private static Hashtable oids;

    static {
        OCSPUtil.algorithms = new Hashtable();
        OCSPUtil.oids = new Hashtable();
        OCSPUtil.noParams = new HashSet();
        OCSPUtil.algorithms.put("MD2WITHRSAENCRYPTION", PKCSObjectIdentifiers.md2WithRSAEncryption);
        OCSPUtil.algorithms.put("MD2WITHRSA", PKCSObjectIdentifiers.md2WithRSAEncryption);
        OCSPUtil.algorithms.put("MD5WITHRSAENCRYPTION", PKCSObjectIdentifiers.md5WithRSAEncryption);
        OCSPUtil.algorithms.put("MD5WITHRSA", PKCSObjectIdentifiers.md5WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA1WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha1WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA1WITHRSA", PKCSObjectIdentifiers.sha1WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA224WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha224WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA224WITHRSA", PKCSObjectIdentifiers.sha224WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA256WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha256WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA256WITHRSA", PKCSObjectIdentifiers.sha256WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA384WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha384WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA384WITHRSA", PKCSObjectIdentifiers.sha384WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA512WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha512WithRSAEncryption);
        OCSPUtil.algorithms.put("SHA512WITHRSA", PKCSObjectIdentifiers.sha512WithRSAEncryption);
        OCSPUtil.algorithms.put("RIPEMD160WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd160);
        OCSPUtil.algorithms.put("RIPEMD160WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd160);
        OCSPUtil.algorithms.put("RIPEMD128WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd128);
        OCSPUtil.algorithms.put("RIPEMD128WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd128);
        OCSPUtil.algorithms.put("RIPEMD256WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd256);
        OCSPUtil.algorithms.put("RIPEMD256WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd256);
        OCSPUtil.algorithms.put("SHA1WITHDSA", X9ObjectIdentifiers.id_dsa_with_sha1);
        OCSPUtil.algorithms.put("DSAWITHSHA1", X9ObjectIdentifiers.id_dsa_with_sha1);
        OCSPUtil.algorithms.put("SHA224WITHDSA", NISTObjectIdentifiers.dsa_with_sha224);
        OCSPUtil.algorithms.put("SHA256WITHDSA", NISTObjectIdentifiers.dsa_with_sha256);
        OCSPUtil.algorithms.put("SHA1WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA1);
        OCSPUtil.algorithms.put("ECDSAWITHSHA1", X9ObjectIdentifiers.ecdsa_with_SHA1);
        OCSPUtil.algorithms.put("SHA224WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA224);
        OCSPUtil.algorithms.put("SHA256WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA256);
        OCSPUtil.algorithms.put("SHA384WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA384);
        OCSPUtil.algorithms.put("SHA512WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA512);
        OCSPUtil.algorithms.put("GOST3411WITHGOST3410", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94);
        OCSPUtil.algorithms.put("GOST3411WITHGOST3410-94", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94);
        OCSPUtil.oids.put(PKCSObjectIdentifiers.md2WithRSAEncryption, "MD2WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.md5WithRSAEncryption, "MD5WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.sha1WithRSAEncryption, "SHA1WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.sha224WithRSAEncryption, "SHA224WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.sha256WithRSAEncryption, "SHA256WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.sha384WithRSAEncryption, "SHA384WITHRSA");
        OCSPUtil.oids.put(PKCSObjectIdentifiers.sha512WithRSAEncryption, "SHA512WITHRSA");
        OCSPUtil.oids.put(TeleTrusTObjectIdentifiers.rsaSignatureWithripemd160, "RIPEMD160WITHRSA");
        OCSPUtil.oids.put(TeleTrusTObjectIdentifiers.rsaSignatureWithripemd128, "RIPEMD128WITHRSA");
        OCSPUtil.oids.put(TeleTrusTObjectIdentifiers.rsaSignatureWithripemd256, "RIPEMD256WITHRSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.id_dsa_with_sha1, "SHA1WITHDSA");
        OCSPUtil.oids.put(NISTObjectIdentifiers.dsa_with_sha224, "SHA224WITHDSA");
        OCSPUtil.oids.put(NISTObjectIdentifiers.dsa_with_sha256, "SHA256WITHDSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.ecdsa_with_SHA1, "SHA1WITHECDSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.ecdsa_with_SHA224, "SHA224WITHECDSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.ecdsa_with_SHA256, "SHA256WITHECDSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.ecdsa_with_SHA384, "SHA384WITHECDSA");
        OCSPUtil.oids.put(X9ObjectIdentifiers.ecdsa_with_SHA512, "SHA512WITHECDSA");
        OCSPUtil.oids.put(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94, "GOST3411WITHGOST3410");
        OCSPUtil.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA1);
        OCSPUtil.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA224);
        OCSPUtil.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA256);
        OCSPUtil.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA384);
        OCSPUtil.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA512);
        OCSPUtil.noParams.add(X9ObjectIdentifiers.id_dsa_with_sha1);
        OCSPUtil.noParams.add(NISTObjectIdentifiers.dsa_with_sha224);
        OCSPUtil.noParams.add(NISTObjectIdentifiers.dsa_with_sha256);
    }

    OCSPUtil() {
        super();
    }

    static CertStore createCertStoreInstance(String arg1, CertStoreParameters arg2, String arg3) {
        CertStore v0 = arg3 == null ? CertStore.getInstance(arg1, arg2) : CertStore.getInstance(arg1, arg2, arg3);
        return v0;
    }

    static MessageDigest createDigestInstance(String arg1, String arg2) {
        MessageDigest v0 = arg2 == null ? MessageDigest.getInstance(arg1) : MessageDigest.getInstance(arg1, arg2);
        return v0;
    }

    static Signature createSignatureInstance(String arg1, String arg2) {
        Signature v0 = arg2 == null ? Signature.getInstance(arg1) : Signature.getInstance(arg1, arg2);
        return v0;
    }

    static CertificateFactory createX509CertificateFactory(String arg1) {
        CertificateFactory v0 = arg1 == null ? CertificateFactory.getInstance("X.509") : CertificateFactory.getInstance("X.509", arg1);
        return v0;
    }

    static Iterator getAlgNames() {
        Enumeration v0 = OCSPUtil.algorithms.keys();
        ArrayList v1 = new ArrayList();
        while(v0.hasMoreElements()) {
            ((List)v1).add(v0.nextElement());
        }

        return ((List)v1).iterator();
    }

    static String getAlgorithmName(DERObjectIdentifier arg1) {
        String v0_1;
        if(OCSPUtil.oids.containsKey(arg1)) {
            Object v0 = OCSPUtil.oids.get(arg1);
        }
        else {
            v0_1 = arg1.getId();
        }

        return v0_1;
    }

    static DERObjectIdentifier getAlgorithmOID(String arg2) {
        DERObjectIdentifier v0_1;
        String v1 = Strings.toUpperCase(arg2);
        if(OCSPUtil.algorithms.containsKey(v1)) {
            Object v0 = OCSPUtil.algorithms.get(v1);
        }
        else {
            v0_1 = new DERObjectIdentifier(v1);
        }

        return v0_1;
    }

    static AlgorithmIdentifier getSigAlgID(DERObjectIdentifier arg2) {
        AlgorithmIdentifier v0 = OCSPUtil.noParams.contains(arg2) ? new AlgorithmIdentifier(arg2) : new AlgorithmIdentifier(arg2, DERNull.INSTANCE);
        return v0;
    }
}

