// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.pqc.crypto.gmss;

import java.security.SecureRandom;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.pqc.crypto.MessageSigner;
import org.bouncycastle.pqc.crypto.gmss.util.GMSSRandom;
import org.bouncycastle.pqc.crypto.gmss.util.GMSSUtil;
import org.bouncycastle.pqc.crypto.gmss.util.WinternitzOTSVerify;
import org.bouncycastle.pqc.crypto.gmss.util.WinternitzOTSignature;
import org.bouncycastle.util.Arrays;

public class GMSSSigner implements MessageSigner {
    private byte[][][] currentAuthPaths;
    private GMSSDigestProvider digestProvider;
    private GMSSParameters gmssPS;
    private GMSSRandom gmssRandom;
    private GMSSUtil gmssUtil;
    private int[] index;
    GMSSKeyParameters key;
    private int mdLength;
    private Digest messDigestOTS;
    private Digest messDigestTrees;
    private int numLayer;
    private WinternitzOTSignature ots;
    private byte[] pubKeyBytes;
    private SecureRandom random;
    private byte[][] subtreeRootSig;

    public GMSSSigner(GMSSDigestProvider arg3) {
        super();
        this.gmssUtil = new GMSSUtil();
        this.digestProvider = arg3;
        this.messDigestTrees = arg3.get();
        this.messDigestOTS = this.messDigestTrees;
        this.mdLength = this.messDigestTrees.getDigestSize();
        this.gmssRandom = new GMSSRandom(this.messDigestTrees);
    }

    public byte[] generateSignature(byte[] arg11) {
        new byte[this.mdLength];
        byte[] v0 = this.ots.getSignature(arg11);
        byte[] v1 = this.gmssUtil.concatenateArray(this.currentAuthPaths[this.numLayer - 1]);
        byte[] v2 = this.gmssUtil.intToBytesLittleEndian(this.index[this.numLayer - 1]);
        byte[] v3 = new byte[v2.length + v0.length + v1.length];
        System.arraycopy(v2, 0, v3, 0, v2.length);
        System.arraycopy(v0, 0, v3, v2.length, v0.length);
        System.arraycopy(v1, 0, v3, v0.length + v2.length, v1.length);
        v1 = new byte[0];
        int v0_1;
        for(v0_1 = this.numLayer - 2; v0_1 >= 0; --v0_1) {
            v2 = this.gmssUtil.concatenateArray(this.currentAuthPaths[v0_1]);
            byte[] v4 = this.gmssUtil.intToBytesLittleEndian(this.index[v0_1]);
            byte[] v5 = new byte[v1.length];
            System.arraycopy(v1, 0, v5, 0, v1.length);
            v1 = new byte[v5.length + v4.length + this.subtreeRootSig[v0_1].length + v2.length];
            System.arraycopy(v5, 0, v1, 0, v5.length);
            System.arraycopy(v4, 0, v1, v5.length, v4.length);
            System.arraycopy(this.subtreeRootSig[v0_1], 0, v1, v5.length + v4.length, this.subtreeRootSig[v0_1].length);
            System.arraycopy(v2, 0, v1, v4.length + v5.length + this.subtreeRootSig[v0_1].length, v2.length);
        }

        v0 = new byte[v3.length + v1.length];
        System.arraycopy(v3, 0, v0, 0, v3.length);
        System.arraycopy(v1, 0, v0, v3.length, v1.length);
        return v0;
    }

    public void init(boolean arg2, CipherParameters arg3) {
        if(!arg2) {
            this.key = ((GMSSKeyParameters)arg3);
            this.initVerify();
        }
        else if((arg3 instanceof ParametersWithRandom)) {
            this.random = ((ParametersWithRandom)arg3).getRandom();
            this.key = ((ParametersWithRandom)arg3).getParameters();
            this.initSign();
        }
        else {
            this.random = new SecureRandom();
            this.key = ((GMSSKeyParameters)arg3);
            this.initSign();
        }
    }

    private void initSign() {
        this.messDigestTrees.reset();
        GMSSKeyParameters v0 = this.key;
        if(((GMSSPrivateKeyParameters)v0).isUsed()) {
            throw new IllegalStateException("Private key already used");
        }

        if(((GMSSPrivateKeyParameters)v0).getIndex(0) >= ((GMSSPrivateKeyParameters)v0).getNumLeafs(0)) {
            throw new IllegalStateException("No more signatures can be generated");
        }

        this.gmssPS = ((GMSSPrivateKeyParameters)v0).getParameters();
        this.numLayer = this.gmssPS.getNumOfLayers();
        byte[] v1 = ((GMSSPrivateKeyParameters)v0).getCurrentSeeds()[this.numLayer - 1];
        new byte[this.mdLength];
        byte[] v2 = new byte[this.mdLength];
        System.arraycopy(v1, 0, v2, 0, this.mdLength);
        this.ots = new WinternitzOTSignature(this.gmssRandom.nextSeed(v2), this.digestProvider.get(), this.gmssPS.getWinternitzParameter()[this.numLayer - 1]);
        byte[][][] v4 = ((GMSSPrivateKeyParameters)v0).getCurrentAuthPaths();
        this.currentAuthPaths = new byte[this.numLayer][][];
        int v2_1;
        for(v2_1 = 0; v2_1 < this.numLayer; ++v2_1) {
            this.currentAuthPaths[v2_1] = new byte[v4[v2_1].length][this.mdLength];
            int v1_1;
            for(v1_1 = 0; v1_1 < v4[v2_1].length; ++v1_1) {
                System.arraycopy(v4[v2_1][v1_1], 0, this.currentAuthPaths[v2_1][v1_1], 0, this.mdLength);
            }
        }

        this.index = new int[this.numLayer];
        System.arraycopy(((GMSSPrivateKeyParameters)v0).getIndex(), 0, this.index, 0, this.numLayer);
        this.subtreeRootSig = new byte[this.numLayer - 1][];
        for(v1_1 = 0; v1_1 < this.numLayer - 1; ++v1_1) {
            v2 = ((GMSSPrivateKeyParameters)v0).getSubtreeRootSig(v1_1);
            this.subtreeRootSig[v1_1] = new byte[v2.length];
            System.arraycopy(v2, 0, this.subtreeRootSig[v1_1], 0, v2.length);
        }

        ((GMSSPrivateKeyParameters)v0).markUsed();
    }

    private void initVerify() {
        this.messDigestTrees.reset();
        GMSSKeyParameters v0 = this.key;
        this.pubKeyBytes = ((GMSSPublicKeyParameters)v0).getPublicKey();
        this.gmssPS = ((GMSSPublicKeyParameters)v0).getParameters();
        this.numLayer = this.gmssPS.getNumOfLayers();
    }

    public boolean verifySignature(byte[] arg12, byte[] arg13) {
        boolean v2 = false;
        this.messDigestOTS.reset();
        int v7 = this.numLayer - 1;
        int v3 = 0;
        while(true) {
            if(v7 >= 0) {
                WinternitzOTSVerify v0 = new WinternitzOTSVerify(this.digestProvider.get(), this.gmssPS.getWinternitzParameter()[v7]);
                int v1 = v0.getSignatureLength();
                int v5 = this.gmssUtil.bytesToIntLittleEndian(arg13, v3);
                v3 += 4;
                byte[] v4 = new byte[v1];
                System.arraycopy(arg13, v3, v4, 0, v1);
                v1 += v3;
                v4 = v0.Verify(arg12, v4);
                if(v4 == null) {
                    System.err.println("OTS Public Key is null in GMSSSignature.verify");
                }
                else {
                    Object v0_1 = new byte[this.gmssPS.getHeightOfTrees()[v7]][this.mdLength];
                    v3 = v1;
                    for(v1 = 0; v1 < v0_1.length; ++v1) {
                        System.arraycopy(arg13, v3, v0_1[v1], 0, this.mdLength);
                        v3 += this.mdLength;
                    }

                    new byte[this.mdLength];
                    v1 = (1 << v0_1.length) + v5;
                    byte[] v5_1 = v4;
                    int v4_1 = v1;
                    for(v1 = 0; v1 < v0_1.length; ++v1) {
                        byte[] v8 = new byte[this.mdLength << 1];
                        if(v4_1 % 2 == 0) {
                            System.arraycopy(v5_1, 0, v8, 0, this.mdLength);
                            System.arraycopy(v0_1[v1], 0, v8, this.mdLength, this.mdLength);
                            v4_1 /= 2;
                        }
                        else {
                            System.arraycopy(v0_1[v1], 0, v8, 0, this.mdLength);
                            System.arraycopy(v5_1, 0, v8, this.mdLength, v5_1.length);
                            v4_1 = (v4_1 - 1) / 2;
                        }

                        this.messDigestTrees.update(v8, 0, v8.length);
                        v5_1 = new byte[this.messDigestTrees.getDigestSize()];
                        this.messDigestTrees.doFinal(v5_1, 0);
                    }

                    --v7;
                    arg12 = v5_1;
                    continue;
                }
            }
            else {
                break;
            }

            return v2;
        }

        if(Arrays.areEqual(this.pubKeyBytes, arg12)) {
            v2 = true;
        }

        return v2;
    }
}

