// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.pqc.crypto.mceliece;

import java.security.SecureRandom;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.params.ParametersWithRandom;
import org.bouncycastle.crypto.prng.DigestRandomGenerator;
import org.bouncycastle.pqc.crypto.MessageEncryptor;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.bouncycastle.pqc.math.linearalgebra.GF2Vector;
import org.bouncycastle.pqc.math.linearalgebra.IntegerFunctions;

public class McElieceKobaraImaiCipher implements MessageEncryptor {
    private static final String DEFAULT_PRNG_NAME = "SHA1PRNG";
    public static final String OID = "1.3.6.1.4.1.8301.3.1.3.4.2.3";
    public static final byte[] PUBLIC_CONSTANT;
    private int k;
    McElieceCCA2KeyParameters key;
    private Digest messDigest;
    private int n;
    private SecureRandom sr;
    private int t;

    static {
        McElieceKobaraImaiCipher.PUBLIC_CONSTANT = "a predetermined public constant".getBytes();
    }

    public McElieceKobaraImaiCipher() {
        super();
    }

    public int getKeySize(McElieceCCA2KeyParameters arg3) {
        int v0;
        if((arg3 instanceof McElieceCCA2PublicKeyParameters)) {
            v0 = ((McElieceCCA2PublicKeyParameters)arg3).getN();
        }
        else if((arg3 instanceof McElieceCCA2PrivateKeyParameters)) {
            v0 = ((McElieceCCA2PrivateKeyParameters)arg3).getN();
        }
        else {
            goto label_8;
        }

        return v0;
    label_8:
        throw new IllegalArgumentException("unsupported type");
    }

    public void init(boolean arg2, CipherParameters arg3) {
        if(!arg2) {
            this.key = ((McElieceCCA2KeyParameters)arg3);
            this.initCipherDecrypt(this.key);
        }
        else if((arg3 instanceof ParametersWithRandom)) {
            this.sr = ((ParametersWithRandom)arg3).getRandom();
            this.key = ((ParametersWithRandom)arg3).getParameters();
            this.initCipherEncrypt(this.key);
        }
        else {
            this.sr = new SecureRandom();
            this.key = ((McElieceCCA2KeyParameters)arg3);
            this.initCipherEncrypt(this.key);
        }
    }

    public void initCipherDecrypt(McElieceCCA2PrivateKeyParameters arg2) {
        this.messDigest = arg2.getParameters().getDigest();
        this.n = arg2.getN();
        this.k = arg2.getK();
        this.t = arg2.getT();
    }

    private void initCipherEncrypt(McElieceCCA2PublicKeyParameters arg2) {
        this.messDigest = arg2.getParameters().getDigest();
        this.n = arg2.getN();
        this.k = arg2.getK();
        this.t = arg2.getT();
    }

    public byte[] messageDecrypt(byte[] arg10) {
        byte[] v1_1;
        byte[] v0_1;
        int v0 = this.n >> 3;
        if(arg10.length < v0) {
            throw new Exception("Bad Padding: Ciphertext too short.");
        }

        int v2 = this.messDigest.getDigestSize();
        int v3 = this.k >> 3;
        v0 = arg10.length - v0;
        if(v0 > 0) {
            byte[][] v1 = ByteUtils.split(arg10, v0);
            v0_1 = v1[0];
            arg10 = v1[1];
            v1_1 = v0_1;
        }
        else {
            v1_1 = new byte[0];
        }

        GF2Vector[] v4 = McElieceCCA2Primitives.decryptionPrimitive(this.key, GF2Vector.OS2VP(this.n, arg10));
        v0_1 = v4[0].getEncoded();
        GF2Vector v4_1 = v4[1];
        if(v0_1.length > v3) {
            v0_1 = ByteUtils.subArray(v0_1, 0, v3);
        }

        v0_1 = ByteUtils.concatenate(ByteUtils.concatenate(v1_1, Conversions.decode(this.n, this.t, v4_1)), v0_1);
        int v1_2 = v0_1.length - v2;
        byte[][] v0_2 = ByteUtils.split(v0_1, v2);
        byte[] v3_1 = v0_2[0];
        byte[] v4_2 = v0_2[1];
        byte[] v5 = new byte[this.messDigest.getDigestSize()];
        this.messDigest.update(v4_2, 0, v4_2.length);
        this.messDigest.doFinal(v5, 0);
        for(v0 = v2 - 1; v0 >= 0; --v0) {
            v5[v0] = ((byte)(v5[v0] ^ v3_1[v0]));
        }

        DigestRandomGenerator v0_3 = new DigestRandomGenerator(new SHA1Digest());
        v0_3.addSeedMaterial(v5);
        byte[] v2_1 = new byte[v1_2];
        v0_3.nextBytes(v2_1);
        for(v0 = v1_2 - 1; v0 >= 0; --v0) {
            v2_1[v0] = ((byte)(v2_1[v0] ^ v4_2[v0]));
        }

        if(v2_1.length < v1_2) {
            throw new Exception("Bad Padding: invalid ciphertext");
        }

        v0_2 = ByteUtils.split(v2_1, v1_2 - McElieceKobaraImaiCipher.PUBLIC_CONSTANT.length);
        v1_1 = v0_2[0];
        if(!ByteUtils.equals(v0_2[1], McElieceKobaraImaiCipher.PUBLIC_CONSTANT)) {
            throw new Exception("Bad Padding: invalid ciphertext");
        }

        return v1_1;
    }

    public byte[] messageEncrypt(byte[] arg12) {
        int v1 = this.messDigest.getDigestSize();
        int v2 = this.k >> 3;
        int v3 = IntegerFunctions.binomial(this.n, this.t).bitLength() - 1 >> 3;
        int v0 = v2 + v3 - v1 - McElieceKobaraImaiCipher.PUBLIC_CONSTANT.length;
        if(arg12.length > v0) {
            v0 = arg12.length;
        }

        int v4 = McElieceKobaraImaiCipher.PUBLIC_CONSTANT.length + v0;
        int v5 = v4 + v1 - v2 - v3;
        byte[] v6 = new byte[v4];
        System.arraycopy(arg12, 0, v6, 0, arg12.length);
        System.arraycopy(McElieceKobaraImaiCipher.PUBLIC_CONSTANT, 0, v6, v0, McElieceKobaraImaiCipher.PUBLIC_CONSTANT.length);
        byte[] v7 = new byte[v1];
        this.sr.nextBytes(v7);
        DigestRandomGenerator v0_1 = new DigestRandomGenerator(new SHA1Digest());
        v0_1.addSeedMaterial(v7);
        byte[] v8 = new byte[v4];
        v0_1.nextBytes(v8);
        for(v0 = v4 - 1; v0 >= 0; --v0) {
            v8[v0] = ((byte)(v8[v0] ^ v6[v0]));
        }

        byte[] v4_1 = new byte[this.messDigest.getDigestSize()];
        this.messDigest.update(v8, 0, v8.length);
        this.messDigest.doFinal(v4_1, 0);
        for(v0 = v1 - 1; v0 >= 0; --v0) {
            v4_1[v0] = ((byte)(v4_1[v0] ^ v7[v0]));
        }

        v4_1 = ByteUtils.concatenate(v4_1, v8);
        byte[] v0_2 = new byte[0];
        if(v5 > 0) {
            v0_2 = new byte[v5];
            System.arraycopy(v4_1, 0, v0_2, 0, v5);
        }

        byte[] v1_1 = v0_2;
        v0_2 = new byte[v3];
        System.arraycopy(v4_1, v5, v0_2, 0, v3);
        v6 = new byte[v2];
        System.arraycopy(v4_1, v3 + v5, v6, 0, v2);
        v0_2 = McElieceCCA2Primitives.encryptionPrimitive(this.key, GF2Vector.OS2VP(this.k, v6), Conversions.encode(this.n, this.t, v0_2)).getEncoded();
        if(v5 > 0) {
            v0_2 = ByteUtils.concatenate(v1_1, v0_2);
        }

        return v0_2;
    }
}

