// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.IOException;
import java.math.BigInteger;
import java.security.cert.CRL;
import java.security.cert.X509CRL;
import java.security.cert.X509CRLSelector;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.DERInteger;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.Selector;
import org.bouncycastle.x509.extension.X509ExtensionUtil;

public class X509CRLStoreSelector extends X509CRLSelector implements Selector {
    private X509AttributeCertificate attrCertChecking;
    private boolean completeCRLEnabled;
    private boolean deltaCRLIndicator;
    private byte[] issuingDistributionPoint;
    private boolean issuingDistributionPointEnabled;
    private BigInteger maxBaseCRLNumber;

    public X509CRLStoreSelector() {
        super();
        this.deltaCRLIndicator = false;
        this.completeCRLEnabled = false;
        this.maxBaseCRLNumber = null;
        this.issuingDistributionPoint = null;
        this.issuingDistributionPointEnabled = false;
    }

    public Object clone() {
        X509CRLStoreSelector v0 = X509CRLStoreSelector.getInstance(((X509CRLSelector)this));
        v0.deltaCRLIndicator = this.deltaCRLIndicator;
        v0.completeCRLEnabled = this.completeCRLEnabled;
        v0.maxBaseCRLNumber = this.maxBaseCRLNumber;
        v0.attrCertChecking = this.attrCertChecking;
        v0.issuingDistributionPointEnabled = this.issuingDistributionPointEnabled;
        v0.issuingDistributionPoint = Arrays.clone(this.issuingDistributionPoint);
        return v0;
    }

    public X509AttributeCertificate getAttrCertificateChecking() {
        return this.attrCertChecking;
    }

    public static X509CRLStoreSelector getInstance(X509CRLSelector arg2) {
        if(arg2 == null) {
            throw new IllegalArgumentException("cannot create from null selector");
        }

        X509CRLStoreSelector v0 = new X509CRLStoreSelector();
        v0.setCertificateChecking(arg2.getCertificateChecking());
        v0.setDateAndTime(arg2.getDateAndTime());
        try {
            v0.setIssuerNames(arg2.getIssuerNames());
        }
        catch(IOException v0_1) {
            throw new IllegalArgumentException(v0_1.getMessage());
        }

        v0.setIssuers(arg2.getIssuers());
        v0.setMaxCRLNumber(arg2.getMaxCRL());
        v0.setMinCRLNumber(arg2.getMinCRL());
        return v0;
    }

    public byte[] getIssuingDistributionPoint() {
        return Arrays.clone(this.issuingDistributionPoint);
    }

    public BigInteger getMaxBaseCRLNumber() {
        return this.maxBaseCRLNumber;
    }

    public boolean isCompleteCRLEnabled() {
        return this.completeCRLEnabled;
    }

    public boolean isDeltaCRLIndicatorEnabled() {
        return this.deltaCRLIndicator;
    }

    public boolean isIssuingDistributionPointEnabled() {
        return this.issuingDistributionPointEnabled;
    }

    public boolean match(Object arg5) {
        if(!(arg5 instanceof X509CRL)) {
            boolean v0 = false;
            return v0;
        }

        Object v0_1 = arg5;
        ASN1Integer v1 = null;
        try {
            byte[] v3 = ((X509CRL)v0_1).getExtensionValue(X509Extensions.DeltaCRLIndicator.getId());
            if(v3 != null) {
                v1 = DERInteger.getInstance(X509ExtensionUtil.fromExtensionValue(v3));
            }
        }
        catch(Exception v0_2) {
            return false;
        }

        if((this.isDeltaCRLIndicatorEnabled()) && v1 == null) {
            return false;
        }

        if((this.isCompleteCRLEnabled()) && v1 != null) {
            return false;
        }

        if(v1 != null && this.maxBaseCRLNumber != null && ((DERInteger)v1).getPositiveValue().compareTo(this.maxBaseCRLNumber) == 1) {
            return false;
        }

        if(this.issuingDistributionPointEnabled) {
            byte[] v0_3 = ((X509CRL)v0_1).getExtensionValue(X509Extensions.IssuingDistributionPoint.getId());
            if(this.issuingDistributionPoint == null) {
                if(v0_3 != null) {
                    return false;
                }
            }
            else if(!Arrays.areEqual(v0_3, this.issuingDistributionPoint)) {
                return false;
            }
        }

        return super.match(((CRL)arg5));
    }

    public boolean match(CRL arg2) {
        return this.match(arg2);
    }

    public void setAttrCertificateChecking(X509AttributeCertificate arg1) {
        this.attrCertChecking = arg1;
    }

    public void setCompleteCRLEnabled(boolean arg1) {
        this.completeCRLEnabled = arg1;
    }

    public void setDeltaCRLIndicatorEnabled(boolean arg1) {
        this.deltaCRLIndicator = arg1;
    }

    public void setIssuingDistributionPoint(byte[] arg2) {
        this.issuingDistributionPoint = Arrays.clone(arg2);
    }

    public void setIssuingDistributionPointEnabled(boolean arg1) {
        this.issuingDistributionPointEnabled = arg1;
    }

    public void setMaxBaseCRLNumber(BigInteger arg1) {
        this.maxBaseCRLNumber = arg1;
    }
}

