// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.SecureRandom;
import java.security.Security;
import java.security.Signature;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.security.auth.x500.X500Principal;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.DERNull;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.RSASSAPSSparams;
import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.util.Strings;

class X509Util {
    private static Hashtable algorithms;
    private static Set noParams;
    private static Hashtable params;

    static {
        X509Util.algorithms = new Hashtable();
        X509Util.params = new Hashtable();
        X509Util.noParams = new HashSet();
        X509Util.algorithms.put("MD2WITHRSAENCRYPTION", PKCSObjectIdentifiers.md2WithRSAEncryption);
        X509Util.algorithms.put("MD2WITHRSA", PKCSObjectIdentifiers.md2WithRSAEncryption);
        X509Util.algorithms.put("MD5WITHRSAENCRYPTION", PKCSObjectIdentifiers.md5WithRSAEncryption);
        X509Util.algorithms.put("MD5WITHRSA", PKCSObjectIdentifiers.md5WithRSAEncryption);
        X509Util.algorithms.put("SHA1WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha1WithRSAEncryption);
        X509Util.algorithms.put("SHA1WITHRSA", PKCSObjectIdentifiers.sha1WithRSAEncryption);
        X509Util.algorithms.put("SHA224WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha224WithRSAEncryption);
        X509Util.algorithms.put("SHA224WITHRSA", PKCSObjectIdentifiers.sha224WithRSAEncryption);
        X509Util.algorithms.put("SHA256WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha256WithRSAEncryption);
        X509Util.algorithms.put("SHA256WITHRSA", PKCSObjectIdentifiers.sha256WithRSAEncryption);
        X509Util.algorithms.put("SHA384WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha384WithRSAEncryption);
        X509Util.algorithms.put("SHA384WITHRSA", PKCSObjectIdentifiers.sha384WithRSAEncryption);
        X509Util.algorithms.put("SHA512WITHRSAENCRYPTION", PKCSObjectIdentifiers.sha512WithRSAEncryption);
        X509Util.algorithms.put("SHA512WITHRSA", PKCSObjectIdentifiers.sha512WithRSAEncryption);
        X509Util.algorithms.put("SHA1WITHRSAANDMGF1", PKCSObjectIdentifiers.id_RSASSA_PSS);
        X509Util.algorithms.put("SHA224WITHRSAANDMGF1", PKCSObjectIdentifiers.id_RSASSA_PSS);
        X509Util.algorithms.put("SHA256WITHRSAANDMGF1", PKCSObjectIdentifiers.id_RSASSA_PSS);
        X509Util.algorithms.put("SHA384WITHRSAANDMGF1", PKCSObjectIdentifiers.id_RSASSA_PSS);
        X509Util.algorithms.put("SHA512WITHRSAANDMGF1", PKCSObjectIdentifiers.id_RSASSA_PSS);
        X509Util.algorithms.put("RIPEMD160WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd160);
        X509Util.algorithms.put("RIPEMD160WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd160);
        X509Util.algorithms.put("RIPEMD128WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd128);
        X509Util.algorithms.put("RIPEMD128WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd128);
        X509Util.algorithms.put("RIPEMD256WITHRSAENCRYPTION", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd256);
        X509Util.algorithms.put("RIPEMD256WITHRSA", TeleTrusTObjectIdentifiers.rsaSignatureWithripemd256);
        X509Util.algorithms.put("SHA1WITHDSA", X9ObjectIdentifiers.id_dsa_with_sha1);
        X509Util.algorithms.put("DSAWITHSHA1", X9ObjectIdentifiers.id_dsa_with_sha1);
        X509Util.algorithms.put("SHA224WITHDSA", NISTObjectIdentifiers.dsa_with_sha224);
        X509Util.algorithms.put("SHA256WITHDSA", NISTObjectIdentifiers.dsa_with_sha256);
        X509Util.algorithms.put("SHA384WITHDSA", NISTObjectIdentifiers.dsa_with_sha384);
        X509Util.algorithms.put("SHA512WITHDSA", NISTObjectIdentifiers.dsa_with_sha512);
        X509Util.algorithms.put("SHA1WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA1);
        X509Util.algorithms.put("ECDSAWITHSHA1", X9ObjectIdentifiers.ecdsa_with_SHA1);
        X509Util.algorithms.put("SHA224WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA224);
        X509Util.algorithms.put("SHA256WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA256);
        X509Util.algorithms.put("SHA384WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA384);
        X509Util.algorithms.put("SHA512WITHECDSA", X9ObjectIdentifiers.ecdsa_with_SHA512);
        X509Util.algorithms.put("GOST3411WITHGOST3410", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94);
        X509Util.algorithms.put("GOST3411WITHGOST3410-94", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94);
        X509Util.algorithms.put("GOST3411WITHECGOST3410", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001);
        X509Util.algorithms.put("GOST3411WITHECGOST3410-2001", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001);
        X509Util.algorithms.put("GOST3411WITHGOST3410-2001", CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001);
        X509Util.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA1);
        X509Util.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA224);
        X509Util.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA256);
        X509Util.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA384);
        X509Util.noParams.add(X9ObjectIdentifiers.ecdsa_with_SHA512);
        X509Util.noParams.add(X9ObjectIdentifiers.id_dsa_with_sha1);
        X509Util.noParams.add(NISTObjectIdentifiers.dsa_with_sha224);
        X509Util.noParams.add(NISTObjectIdentifiers.dsa_with_sha256);
        X509Util.noParams.add(NISTObjectIdentifiers.dsa_with_sha384);
        X509Util.noParams.add(NISTObjectIdentifiers.dsa_with_sha512);
        X509Util.noParams.add(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94);
        X509Util.noParams.add(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001);
        X509Util.params.put("SHA1WITHRSAANDMGF1", X509Util.creatPSSParams(new AlgorithmIdentifier(OIWObjectIdentifiers.idSHA1, DERNull.INSTANCE), 20));
        X509Util.params.put("SHA224WITHRSAANDMGF1", X509Util.creatPSSParams(new AlgorithmIdentifier(NISTObjectIdentifiers.id_sha224, DERNull.INSTANCE), 28));
        X509Util.params.put("SHA256WITHRSAANDMGF1", X509Util.creatPSSParams(new AlgorithmIdentifier(NISTObjectIdentifiers.id_sha256, DERNull.INSTANCE), 32));
        X509Util.params.put("SHA384WITHRSAANDMGF1", X509Util.creatPSSParams(new AlgorithmIdentifier(NISTObjectIdentifiers.id_sha384, DERNull.INSTANCE), 48));
        X509Util.params.put("SHA512WITHRSAANDMGF1", X509Util.creatPSSParams(new AlgorithmIdentifier(NISTObjectIdentifiers.id_sha512, DERNull.INSTANCE), 64));
    }

    X509Util() {
        super();
    }

    static byte[] calculateSignature(DERObjectIdentifier arg3, String arg4, String arg5, PrivateKey arg6, SecureRandom arg7, ASN1Encodable arg8) {
        if(arg3 == null) {
            throw new IllegalStateException("no signature algorithm specified");
        }

        Signature v0 = X509Util.getSignatureInstance(arg4, arg5);
        if(arg7 != null) {
            v0.initSign(arg6, arg7);
        }
        else {
            v0.initSign(arg6);
        }

        v0.update(arg8.toASN1Primitive().getEncoded("DER"));
        return v0.sign();
    }

    static byte[] calculateSignature(DERObjectIdentifier arg3, String arg4, PrivateKey arg5, SecureRandom arg6, ASN1Encodable arg7) {
        if(arg3 == null) {
            throw new IllegalStateException("no signature algorithm specified");
        }

        Signature v0 = X509Util.getSignatureInstance(arg4);
        if(arg6 != null) {
            v0.initSign(arg5, arg6);
        }
        else {
            v0.initSign(arg5);
        }

        v0.update(arg7.toASN1Primitive().getEncoded("DER"));
        return v0.sign();
    }

    static X509Principal convertPrincipal(X500Principal arg2) {
        try {
            return new X509Principal(arg2.getEncoded());
        }
        catch(IOException v0) {
            throw new IllegalArgumentException("cannot convert principal");
        }
    }

    private static RSASSAPSSparams creatPSSParams(AlgorithmIdentifier arg6, int arg7) {
        return new RSASSAPSSparams(arg6, new AlgorithmIdentifier(PKCSObjectIdentifiers.id_mgf1, ((ASN1Encodable)arg6)), new ASN1Integer(((long)arg7)), new ASN1Integer(1));
    }

    static Iterator getAlgNames() {
        Enumeration v0 = X509Util.algorithms.keys();
        ArrayList v1 = new ArrayList();
        while(v0.hasMoreElements()) {
            ((List)v1).add(v0.nextElement());
        }

        return ((List)v1).iterator();
    }

    static DERObjectIdentifier getAlgorithmOID(String arg2) {
        Object v0;
        String v1 = Strings.toUpperCase(arg2);
        if(X509Util.algorithms.containsKey(v1)) {
            v0 = X509Util.algorithms.get(v1);
        }
        else {
            DERObjectIdentifier v0_1 = new DERObjectIdentifier(v1);
        }

        return ((DERObjectIdentifier)v0);
    }

    static X509Util$Implementation getImplementation(String arg4, String arg5) {
        Provider[] v1 = Security.getProviders();
        int v0;
        for(v0 = 0; v0 != v1.length; ++v0) {
            X509Util$Implementation v2 = X509Util.getImplementation(arg4, Strings.toUpperCase(arg5), v1[v0]);
            if(v2 != null) {
                return v2;
            }

            try {
                X509Util.getImplementation(arg4, arg5, v1[v0]);
            }
            catch(NoSuchAlgorithmException v2_1) {
            }
        }

        throw new NoSuchAlgorithmException("cannot find implementation " + arg5);
    }

    static X509Util$Implementation getImplementation(String arg5, String arg6, Provider arg7) {
        String v0;
        for(v0 = Strings.toUpperCase(arg6); true; v0 = v1) {
            String v1 = arg7.getProperty("Alg.Alias." + arg5 + "." + v0);
            if(v1 == null) {
                break;
            }
        }

        String v2 = arg7.getProperty(arg5 + "." + v0);
        if(v2 != null) {
            try {
                ClassLoader v1_3 = arg7.getClass().getClassLoader();
                Class v1_4 = v1_3 != null ? v1_3.loadClass(v2) : Class.forName(v2);
                return new X509Util$Implementation(v1_4.newInstance(), arg7);
            }
            catch(Exception v1_1) {
                throw new IllegalStateException("algorithm " + v0 + " in provider " + arg7.getName() + " but class \"" + v2 + "\" inaccessible!");
            }
            catch(ClassNotFoundException v1_2) {
                throw new IllegalStateException("algorithm " + v0 + " in provider " + arg7.getName() + " but no class \"" + v2 + "\" found!");
            }
        }

        throw new NoSuchAlgorithmException("cannot find implementation " + v0 + " for provider " + arg7.getName());
    }

    static Provider getProvider(String arg3) {
        Provider v0 = Security.getProvider(arg3);
        if(v0 == null) {
            throw new NoSuchProviderException("Provider " + arg3 + " not found");
        }

        return v0;
    }

    static AlgorithmIdentifier getSigAlgID(DERObjectIdentifier arg3, String arg4) {
        AlgorithmIdentifier v0;
        if(X509Util.noParams.contains(arg3)) {
            v0 = new AlgorithmIdentifier(arg3);
        }
        else {
            String v0_1 = Strings.toUpperCase(arg4);
            v0 = X509Util.params.containsKey(v0_1) ? new AlgorithmIdentifier(arg3, X509Util.params.get(v0_1)) : new AlgorithmIdentifier(arg3, DERNull.INSTANCE);
        }

        return v0;
    }

    static Signature getSignatureInstance(String arg1, String arg2) {
        Signature v0 = arg2 != null ? Signature.getInstance(arg1, arg2) : Signature.getInstance(arg1);
        return v0;
    }

    static Signature getSignatureInstance(String arg1) {
        return Signature.getInstance(arg1);
    }
}

