// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.x509.AttributeCertificate;
import org.bouncycastle.asn1.x509.Extension;
import org.bouncycastle.asn1.x509.Extensions;
import org.bouncycastle.util.Arrays;

public class X509V2AttributeCertificate implements X509AttributeCertificate {
    private AttributeCertificate cert;
    private Date notAfter;
    private Date notBefore;

    public X509V2AttributeCertificate(byte[] arg2) {
        this(new ByteArrayInputStream(arg2));
    }

    public X509V2AttributeCertificate(InputStream arg2) {
        this(X509V2AttributeCertificate.getObject(arg2));
    }

    X509V2AttributeCertificate(AttributeCertificate arg3) {
        super();
        this.cert = arg3;
        try {
            this.notAfter = arg3.getAcinfo().getAttrCertValidityPeriod().getNotAfterTime().getDate();
            this.notBefore = arg3.getAcinfo().getAttrCertValidityPeriod().getNotBeforeTime().getDate();
            return;
        }
        catch(ParseException v0) {
            throw new IOException("invalid data structure in certificate!");
        }
    }

    public void checkValidity() {
        this.checkValidity(new Date());
    }

    public void checkValidity(Date arg4) {
        if(arg4.after(this.getNotAfter())) {
            throw new CertificateExpiredException("certificate expired on " + this.getNotAfter());
        }

        if(arg4.before(this.getNotBefore())) {
            throw new CertificateNotYetValidException("certificate not valid till " + this.getNotBefore());
        }
    }

    public boolean equals(Object arg4) {
        boolean v0 = false;
        if((((X509V2AttributeCertificate)arg4)) == this) {
            return true;
        }

        if((arg4 instanceof X509AttributeCertificate)) {
            try {
                v0 = Arrays.areEqual(this.getEncoded(), ((X509AttributeCertificate)arg4).getEncoded());
            }
            catch(IOException v1) {
            }
        }

        return v0;
    }

    public X509Attribute[] getAttributes() {
        ASN1Sequence v1 = this.cert.getAcinfo().getAttributes();
        X509Attribute[] v2 = new X509Attribute[v1.size()];
        int v0;
        for(v0 = 0; v0 != v1.size(); ++v0) {
            v2[v0] = new X509Attribute(v1.getObjectAt(v0));
        }

        return v2;
    }

    public X509Attribute[] getAttributes(String arg6) {
        Object[] v0_2;
        ASN1Sequence v1 = this.cert.getAcinfo().getAttributes();
        ArrayList v2 = new ArrayList();
        int v0;
        for(v0 = 0; v0 != v1.size(); ++v0) {
            X509Attribute v3 = new X509Attribute(v1.getObjectAt(v0));
            if(v3.getOID().equals(arg6)) {
                ((List)v2).add(v3);
            }
        }

        if(((List)v2).size() == 0) {
            X509Attribute[] v0_1 = null;
        }
        else {
            v0_2 = ((List)v2).toArray(new X509Attribute[((List)v2).size()]);
        }

        return ((X509Attribute[])v0_2);
    }

    public Set getCriticalExtensionOIDs() {
        return this.getExtensionOIDs(true);
    }

    public byte[] getEncoded() {
        return this.cert.getEncoded();
    }

    private Set getExtensionOIDs(boolean arg6) {
        Set v0_2;
        Extensions v2 = this.cert.getAcinfo().getExtensions();
        if(v2 != null) {
            HashSet v1 = new HashSet();
            Enumeration v3 = v2.oids();
            while(v3.hasMoreElements()) {
                Object v0 = v3.nextElement();
                if(v2.getExtension(((ASN1ObjectIdentifier)v0)).isCritical() != arg6) {
                    continue;
                }

                ((Set)v1).add(((ASN1ObjectIdentifier)v0).getId());
            }

            HashSet v0_1 = v1;
        }
        else {
            v0_2 = null;
        }

        return v0_2;
    }

    public byte[] getExtensionValue(String arg5) {
        byte[] v0_3;
        Extensions v0 = this.cert.getAcinfo().getExtensions();
        if(v0 != null) {
            Extension v0_1 = v0.getExtension(new ASN1ObjectIdentifier(arg5));
            if(v0_1 != null) {
                try {
                    v0_3 = v0_1.getExtnValue().getEncoded("DER");
                }
                catch(Exception v0_2) {
                    throw new RuntimeException("error encoding " + v0_2.toString());
                }
            }
            else {
                goto label_23;
            }
        }
        else {
        label_23:
            v0_3 = null;
        }

        return v0_3;
    }

    public AttributeCertificateHolder getHolder() {
        return new AttributeCertificateHolder(this.cert.getAcinfo().getHolder().toASN1Object());
    }

    public AttributeCertificateIssuer getIssuer() {
        return new AttributeCertificateIssuer(this.cert.getAcinfo().getIssuer());
    }

    public boolean[] getIssuerUniqueID() {
        boolean[] v0_2;
        DERBitString v0 = this.cert.getAcinfo().getIssuerUniqueID();
        if(v0 != null) {
            byte[] v4 = v0.getBytes();
            boolean[] v3 = new boolean[v4.length * 8 - v0.getPadBits()];
            int v0_1;
            for(v0_1 = 0; v0_1 != v3.length; ++v0_1) {
                boolean v2 = (v4[v0_1 / 8] & 128 >>> v0_1 % 8) != 0 ? true : false;
                v3[v0_1] = v2;
            }

            v0_2 = v3;
        }
        else {
            v0_2 = null;
        }

        return v0_2;
    }

    public Set getNonCriticalExtensionOIDs() {
        return this.getExtensionOIDs(false);
    }

    public Date getNotAfter() {
        return this.notAfter;
    }

    public Date getNotBefore() {
        return this.notBefore;
    }

    private static AttributeCertificate getObject(InputStream arg4) {
        try {
            return AttributeCertificate.getInstance(new ASN1InputStream(arg4).readObject());
        }
        catch(Exception v0) {
            throw new IOException("exception decoding certificate structure: " + v0.toString());
        }
        catch(IOException v0_1) {
            throw v0_1;
        }
    }

    public BigInteger getSerialNumber() {
        return this.cert.getAcinfo().getSerialNumber().getValue();
    }

    public byte[] getSignature() {
        return this.cert.getSignatureValue().getBytes();
    }

    public int getVersion() {
        return this.cert.getAcinfo().getVersion().getValue().intValue() + 1;
    }

    public boolean hasUnsupportedCriticalExtension() {
        Set v0 = this.getCriticalExtensionOIDs();
        boolean v0_1 = v0 == null || (v0.isEmpty()) ? false : true;
        return v0_1;
    }

    public int hashCode() {
        int v0_1;
        try {
            v0_1 = Arrays.hashCode(this.getEncoded());
        }
        catch(IOException v0) {
            v0_1 = 0;
        }

        return v0_1;
    }

    public final void verify(PublicKey arg3, String arg4) {
        if(!this.cert.getSignatureAlgorithm().equals(this.cert.getAcinfo().getSignature())) {
            throw new CertificateException("Signature algorithm in certificate info not same as outer certificate");
        }

        Signature v0 = Signature.getInstance(this.cert.getSignatureAlgorithm().getObjectId().getId(), arg4);
        v0.initVerify(arg3);
        try {
            v0.update(this.cert.getAcinfo().getEncoded());
        }
        catch(IOException v0_1) {
            throw new SignatureException("Exception encoding certificate info object");
        }

        if(!v0.verify(this.getSignature())) {
            throw new InvalidKeyException("Public key presented not for certificate signature");
        }
    }
}

