// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.IOException;
import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.util.Date;
import java.util.Iterator;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1GeneralizedTime;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.AttCertIssuer;
import org.bouncycastle.asn1.x509.Attribute;
import org.bouncycastle.asn1.x509.AttributeCertificate;
import org.bouncycastle.asn1.x509.AttributeCertificateInfo;
import org.bouncycastle.asn1.x509.V2AttributeCertificateInfoGenerator;
import org.bouncycastle.asn1.x509.X509ExtensionsGenerator;

public class X509V2AttributeCertificateGenerator {
    private V2AttributeCertificateInfoGenerator acInfoGen;
    private X509ExtensionsGenerator extGenerator;
    private AlgorithmIdentifier sigAlgId;
    private DERObjectIdentifier sigOID;
    private String signatureAlgorithm;

    public X509V2AttributeCertificateGenerator() {
        super();
        this.acInfoGen = new V2AttributeCertificateInfoGenerator();
        this.extGenerator = new X509ExtensionsGenerator();
    }

    public void addAttribute(X509Attribute arg3) {
        this.acInfoGen.addAttribute(Attribute.getInstance(arg3.toASN1Object()));
    }

    public void addExtension(String arg3, boolean arg4, ASN1Encodable arg5) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg3), arg4, arg5);
    }

    public void addExtension(String arg3, boolean arg4, byte[] arg5) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg3), arg4, arg5);
    }

    public X509AttributeCertificate generate(PrivateKey arg2, String arg3) {
        return this.generate(arg2, arg3, null);
    }

    public X509AttributeCertificate generate(PrivateKey arg9, String arg10, SecureRandom arg11) {
        if(!this.extGenerator.isEmpty()) {
            this.acInfoGen.setExtensions(this.extGenerator.generate());
        }

        AttributeCertificateInfo v5 = this.acInfoGen.generateAttributeCertificateInfo();
        ASN1EncodableVector v6 = new ASN1EncodableVector();
        v6.add(((ASN1Encodable)v5));
        v6.add(this.sigAlgId);
        try {
            v6.add(new DERBitString(X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, arg10, arg9, arg11, ((ASN1Encodable)v5))));
            return new X509V2AttributeCertificate(new AttributeCertificate(new DERSequence(v6)));
        }
        catch(IOException v0) {
            throw new ExtCertificateEncodingException("constructed invalid certificate", ((Throwable)v0));
        }
    }

    public X509AttributeCertificate generateCertificate(PrivateKey arg2, String arg3) {
        return this.generateCertificate(arg2, arg3, null);
    }

    public X509AttributeCertificate generateCertificate(PrivateKey arg5, String arg6, SecureRandom arg7) {
        try {
            return this.generate(arg5, arg6, arg7);
        }
        catch(GeneralSecurityException v0) {
            throw new SecurityException("exception creating certificate: " + v0);
        }
        catch(InvalidKeyException v0_1) {
            throw v0_1;
        }
        catch(SignatureException v0_2) {
            throw v0_2;
        }
        catch(NoSuchProviderException v0_3) {
            throw v0_3;
        }
    }

    public Iterator getSignatureAlgNames() {
        return X509Util.getAlgNames();
    }

    public void reset() {
        this.acInfoGen = new V2AttributeCertificateInfoGenerator();
        this.extGenerator.reset();
    }

    public void setHolder(AttributeCertificateHolder arg3) {
        this.acInfoGen.setHolder(arg3.holder);
    }

    public void setIssuer(AttributeCertificateIssuer arg3) {
        this.acInfoGen.setIssuer(AttCertIssuer.getInstance(arg3.form));
    }

    public void setIssuerUniqueId(boolean[] arg3) {
        throw new RuntimeException("not implemented (yet)");
    }

    public void setNotAfter(Date arg3) {
        this.acInfoGen.setEndDate(new ASN1GeneralizedTime(arg3));
    }

    public void setNotBefore(Date arg3) {
        this.acInfoGen.setStartDate(new ASN1GeneralizedTime(arg3));
    }

    public void setSerialNumber(BigInteger arg3) {
        this.acInfoGen.setSerialNumber(new ASN1Integer(arg3));
    }

    public void setSignatureAlgorithm(String arg3) {
        this.signatureAlgorithm = arg3;
        try {
            this.sigOID = X509Util.getAlgorithmOID(arg3);
        }
        catch(Exception v0) {
            throw new IllegalArgumentException("Unknown signature type requested");
        }

        this.sigAlgId = X509Util.getSigAlgID(this.sigOID, arg3);
        this.acInfoGen.setSignature(this.sigAlgId);
    }
}

