// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.IOException;
import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.cert.CRLException;
import java.security.cert.X509CRL;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import javax.security.auth.x500.X500Principal;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1GeneralizedTime;
import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.CertificateList;
import org.bouncycastle.asn1.x509.Extensions;
import org.bouncycastle.asn1.x509.TBSCertList;
import org.bouncycastle.asn1.x509.Time;
import org.bouncycastle.asn1.x509.V2TBSCertListGenerator;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.asn1.x509.X509ExtensionsGenerator;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.jce.provider.X509CRLObject;

public class X509V2CRLGenerator {
    private X509ExtensionsGenerator extGenerator;
    private AlgorithmIdentifier sigAlgId;
    private DERObjectIdentifier sigOID;
    private String signatureAlgorithm;
    private V2TBSCertListGenerator tbsGen;

    public X509V2CRLGenerator() {
        super();
        this.tbsGen = new V2TBSCertListGenerator();
        this.extGenerator = new X509ExtensionsGenerator();
    }

    public void addCRL(X509CRL arg5) {
        Set v0 = arg5.getRevokedCertificates();
        if(v0 != null) {
            Iterator v1 = v0.iterator();
            while(v1.hasNext()) {
                ASN1InputStream v2 = new ASN1InputStream(v1.next().getEncoded());
                try {
                    this.tbsGen.addCRLEntry(ASN1Sequence.getInstance(v2.readObject()));
                    continue;
                }
                catch(IOException v0_1) {
                    throw new CRLException("exception processing encoding of CRL: " + v0_1.toString());
                }

                return;
            }
        }
    }

    public void addCRLEntry(BigInteger arg4, Date arg5, int arg6) {
        this.tbsGen.addCRLEntry(new ASN1Integer(arg4), new Time(arg5), arg6);
    }

    public void addCRLEntry(BigInteger arg5, Date arg6, int arg7, Date arg8) {
        this.tbsGen.addCRLEntry(new ASN1Integer(arg5), new Time(arg6), arg7, new ASN1GeneralizedTime(arg8));
    }

    public void addCRLEntry(BigInteger arg5, Date arg6, X509Extensions arg7) {
        this.tbsGen.addCRLEntry(new ASN1Integer(arg5), new Time(arg6), Extensions.getInstance(arg7));
    }

    public void addExtension(String arg2, boolean arg3, ASN1Encodable arg4) {
        this.addExtension(new DERObjectIdentifier(arg2), arg3, arg4);
    }

    public void addExtension(DERObjectIdentifier arg4, boolean arg5, ASN1Encodable arg6) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg4.getId()), arg5, arg6);
    }

    public void addExtension(String arg2, boolean arg3, byte[] arg4) {
        this.addExtension(new DERObjectIdentifier(arg2), arg3, arg4);
    }

    public void addExtension(DERObjectIdentifier arg4, boolean arg5, byte[] arg6) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg4.getId()), arg5, arg6);
    }

    public X509CRL generate(PrivateKey arg2) {
        return this.generate(arg2, null);
    }

    public X509CRL generate(PrivateKey arg4, SecureRandom arg5) {
        byte[] v1;
        TBSCertList v0 = this.generateCertList();
        try {
            v1 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, arg4, arg5, ((ASN1Encodable)v0));
        }
        catch(IOException v0_1) {
            throw new X509V2CRLGenerator$ExtCRLException("cannot generate CRL encoding", ((Throwable)v0_1));
        }

        return this.generateJcaObject(v0, v1);
    }

    public X509CRL generate(PrivateKey arg2, String arg3) {
        return this.generate(arg2, arg3, null);
    }

    public X509CRL generate(PrivateKey arg7, String arg8, SecureRandom arg9) {
        byte[] v0_1;
        TBSCertList v5 = this.generateCertList();
        try {
            v0_1 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, arg8, arg7, arg9, ((ASN1Encodable)v5));
        }
        catch(IOException v0) {
            throw new X509V2CRLGenerator$ExtCRLException("cannot generate CRL encoding", ((Throwable)v0));
        }

        return this.generateJcaObject(v5, v0_1);
    }

    private TBSCertList generateCertList() {
        if(!this.extGenerator.isEmpty()) {
            this.tbsGen.setExtensions(this.extGenerator.generate());
        }

        return this.tbsGen.generateTBSCertList();
    }

    private X509CRL generateJcaObject(TBSCertList arg5, byte[] arg6) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        v0.add(((ASN1Encodable)arg5));
        v0.add(this.sigAlgId);
        v0.add(new DERBitString(arg6));
        return new X509CRLObject(new CertificateList(new DERSequence(v0)));
    }

    public X509CRL generateX509CRL(PrivateKey arg3) {
        try {
            return this.generateX509CRL(arg3, "BC", null);
        }
        catch(NoSuchProviderException v0) {
            throw new SecurityException("BC provider not installed!");
        }
    }

    public X509CRL generateX509CRL(PrivateKey arg5, String arg6, SecureRandom arg7) {
        try {
            return this.generate(arg5, arg6, arg7);
        }
        catch(GeneralSecurityException v0) {
            throw new SecurityException("exception: " + v0);
        }
        catch(InvalidKeyException v0_1) {
            throw v0_1;
        }
        catch(SignatureException v0_2) {
            throw v0_2;
        }
        catch(NoSuchProviderException v0_3) {
            throw v0_3;
        }
    }

    public X509CRL generateX509CRL(PrivateKey arg2, String arg3) {
        return this.generateX509CRL(arg2, arg3, null);
    }

    public X509CRL generateX509CRL(PrivateKey arg3, SecureRandom arg4) {
        try {
            return this.generateX509CRL(arg3, "BC", arg4);
        }
        catch(NoSuchProviderException v0) {
            throw new SecurityException("BC provider not installed!");
        }
    }

    public Iterator getSignatureAlgNames() {
        return X509Util.getAlgNames();
    }

    public void reset() {
        this.tbsGen = new V2TBSCertListGenerator();
        this.extGenerator.reset();
    }

    public void setIssuerDN(X500Principal arg5) {
        try {
            this.tbsGen.setIssuer(new X509Principal(arg5.getEncoded()));
            return;
        }
        catch(IOException v0) {
            throw new IllegalArgumentException("can\'t process principal: " + v0);
        }
    }

    public void setIssuerDN(X509Name arg2) {
        this.tbsGen.setIssuer(arg2);
    }

    public void setNextUpdate(Date arg3) {
        this.tbsGen.setNextUpdate(new Time(arg3));
    }

    public void setSignatureAlgorithm(String arg3) {
        this.signatureAlgorithm = arg3;
        try {
            this.sigOID = X509Util.getAlgorithmOID(arg3);
        }
        catch(Exception v0) {
            throw new IllegalArgumentException("Unknown signature type requested");
        }

        this.sigAlgId = X509Util.getSigAlgID(this.sigOID, arg3);
        this.tbsGen.setSignature(this.sigAlgId);
    }

    public void setThisUpdate(Date arg3) {
        this.tbsGen.setThisUpdate(new Time(arg3));
    }
}

