// Decompiled by JEB v1.5.201411100 - http://www.android-decompiler.com

package org.bouncycastle.x509;

import java.io.IOException;
import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Iterator;
import javax.security.auth.x500.X500Principal;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.ASN1Integer;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.Certificate;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x509.TBSCertificate;
import org.bouncycastle.asn1.x509.Time;
import org.bouncycastle.asn1.x509.V3TBSCertificateGenerator;
import org.bouncycastle.asn1.x509.X509ExtensionsGenerator;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.jce.provider.X509CertificateObject;
import org.bouncycastle.x509.extension.X509ExtensionUtil;

public class X509V3CertificateGenerator {
    private X509ExtensionsGenerator extGenerator;
    private AlgorithmIdentifier sigAlgId;
    private DERObjectIdentifier sigOID;
    private String signatureAlgorithm;
    private V3TBSCertificateGenerator tbsGen;

    public X509V3CertificateGenerator() {
        super();
        this.tbsGen = new V3TBSCertificateGenerator();
        this.extGenerator = new X509ExtensionsGenerator();
    }

    public void addExtension(DERObjectIdentifier arg4, boolean arg5, ASN1Encodable arg6) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg4.getId()), arg5, arg6);
    }

    public void addExtension(String arg2, boolean arg3, ASN1Encodable arg4) {
        this.addExtension(new DERObjectIdentifier(arg2), arg3, arg4);
    }

    public void addExtension(String arg2, boolean arg3, byte[] arg4) {
        this.addExtension(new DERObjectIdentifier(arg2), arg3, arg4);
    }

    public void addExtension(DERObjectIdentifier arg4, boolean arg5, byte[] arg6) {
        this.extGenerator.addExtension(new ASN1ObjectIdentifier(arg4.getId()), arg5, arg6);
    }

    private DERBitString booleanToBitString(boolean[] arg8) {
        byte[] v3 = new byte[(arg8.length + 7) / 8];
        int v0;
        for(v0 = 0; v0 != arg8.length; ++v0) {
            int v4 = v0 / 8;
            int v5 = v3[v4];
            int v2 = arg8[v0] ? 1 << 7 - v0 % 8 : 0;
            v3[v4] = ((byte)(v2 | v5));
        }

        int v1 = arg8.length % 8;
        DERBitString v0_1 = v1 == 0 ? new DERBitString(v3) : new DERBitString(v3, 8 - v1);
        return v0_1;
    }

    public void copyAndAddExtension(String arg4, boolean arg5, X509Certificate arg6) {
        byte[] v0 = arg6.getExtensionValue(arg4);
        if(v0 == null) {
            throw new CertificateParsingException("extension " + arg4 + " not present");
        }

        try {
            this.addExtension(arg4, arg5, X509ExtensionUtil.fromExtensionValue(v0));
            return;
        }
        catch(IOException v0_1) {
            throw new CertificateParsingException(v0_1.toString());
        }
    }

    public void copyAndAddExtension(DERObjectIdentifier arg2, boolean arg3, X509Certificate arg4) {
        this.copyAndAddExtension(arg2.getId(), arg3, arg4);
    }

    public X509Certificate generate(PrivateKey arg2) {
        return this.generate(arg2, null);
    }

    public X509Certificate generate(PrivateKey arg4, SecureRandom arg5) {
        byte[] v1;
        TBSCertificate v0 = this.generateTbsCert();
        try {
            v1 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, arg4, arg5, ((ASN1Encodable)v0));
        }
        catch(IOException v0_1) {
            throw new ExtCertificateEncodingException("exception encoding TBS cert", ((Throwable)v0_1));
        }

        try {
            return this.generateJcaObject(v0, v1);
        }
        catch(CertificateParsingException v0_2) {
            throw new ExtCertificateEncodingException("exception producing certificate object", ((Throwable)v0_2));
        }
    }

    public X509Certificate generate(PrivateKey arg2, String arg3) {
        return this.generate(arg2, arg3, null);
    }

    public X509Certificate generate(PrivateKey arg7, String arg8, SecureRandom arg9) {
        byte[] v0_1;
        TBSCertificate v5 = this.generateTbsCert();
        try {
            v0_1 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, arg8, arg7, arg9, ((ASN1Encodable)v5));
        }
        catch(IOException v0) {
            throw new ExtCertificateEncodingException("exception encoding TBS cert", ((Throwable)v0));
        }

        try {
            return this.generateJcaObject(v5, v0_1);
        }
        catch(CertificateParsingException v0_2) {
            throw new ExtCertificateEncodingException("exception producing certificate object", ((Throwable)v0_2));
        }
    }

    private X509Certificate generateJcaObject(TBSCertificate arg4, byte[] arg5) {
        ASN1EncodableVector v0 = new ASN1EncodableVector();
        v0.add(((ASN1Encodable)arg4));
        v0.add(this.sigAlgId);
        v0.add(new DERBitString(arg5));
        return new X509CertificateObject(Certificate.getInstance(new DERSequence(v0)));
    }

    private TBSCertificate generateTbsCert() {
        if(!this.extGenerator.isEmpty()) {
            this.tbsGen.setExtensions(this.extGenerator.generate());
        }

        return this.tbsGen.generateTBSCertificate();
    }

    public X509Certificate generateX509Certificate(PrivateKey arg3) {
        try {
            return this.generateX509Certificate(arg3, "BC", null);
        }
        catch(NoSuchProviderException v0) {
            throw new SecurityException("BC provider not installed!");
        }
    }

    public X509Certificate generateX509Certificate(PrivateKey arg5, String arg6, SecureRandom arg7) {
        try {
            return this.generate(arg5, arg6, arg7);
        }
        catch(GeneralSecurityException v0) {
            throw new SecurityException("exception: " + v0);
        }
        catch(InvalidKeyException v0_1) {
            throw v0_1;
        }
        catch(SignatureException v0_2) {
            throw v0_2;
        }
        catch(NoSuchProviderException v0_3) {
            throw v0_3;
        }
    }

    public X509Certificate generateX509Certificate(PrivateKey arg2, String arg3) {
        return this.generateX509Certificate(arg2, arg3, null);
    }

    public X509Certificate generateX509Certificate(PrivateKey arg3, SecureRandom arg4) {
        try {
            return this.generateX509Certificate(arg3, "BC", arg4);
        }
        catch(NoSuchProviderException v0) {
            throw new SecurityException("BC provider not installed!");
        }
    }

    public Iterator getSignatureAlgNames() {
        return X509Util.getAlgNames();
    }

    public void reset() {
        this.tbsGen = new V3TBSCertificateGenerator();
        this.extGenerator.reset();
    }

    public void setIssuerDN(X509Name arg2) {
        this.tbsGen.setIssuer(arg2);
    }

    public void setIssuerDN(X500Principal arg5) {
        try {
            this.tbsGen.setIssuer(new X509Principal(arg5.getEncoded()));
            return;
        }
        catch(IOException v0) {
            throw new IllegalArgumentException("can\'t process principal: " + v0);
        }
    }

    public void setIssuerUniqueID(boolean[] arg3) {
        this.tbsGen.setIssuerUniqueID(this.booleanToBitString(arg3));
    }

    public void setNotAfter(Date arg3) {
        this.tbsGen.setEndDate(new Time(arg3));
    }

    public void setNotBefore(Date arg3) {
        this.tbsGen.setStartDate(new Time(arg3));
    }

    public void setPublicKey(PublicKey arg5) {
        try {
            this.tbsGen.setSubjectPublicKeyInfo(SubjectPublicKeyInfo.getInstance(new ASN1InputStream(arg5.getEncoded()).readObject()));
            return;
        }
        catch(Exception v0) {
            throw new IllegalArgumentException("unable to process key - " + v0.toString());
        }
    }

    public void setSerialNumber(BigInteger arg3) {
        if(arg3.compareTo(BigInteger.ZERO) <= 0) {
            throw new IllegalArgumentException("serial number must be a positive integer");
        }

        this.tbsGen.setSerialNumber(new ASN1Integer(arg3));
    }

    public void setSignatureAlgorithm(String arg4) {
        this.signatureAlgorithm = arg4;
        try {
            this.sigOID = X509Util.getAlgorithmOID(arg4);
        }
        catch(Exception v0) {
            throw new IllegalArgumentException("Unknown signature type requested: " + arg4);
        }

        this.sigAlgId = X509Util.getSigAlgID(this.sigOID, arg4);
        this.tbsGen.setSignature(this.sigAlgId);
    }

    public void setSubjectDN(X509Name arg2) {
        this.tbsGen.setSubject(arg2);
    }

    public void setSubjectDN(X500Principal arg5) {
        try {
            this.tbsGen.setSubject(new X509Principal(arg5.getEncoded()));
            return;
        }
        catch(IOException v0) {
            throw new IllegalArgumentException("can\'t process principal: " + v0);
        }
    }

    public void setSubjectUniqueID(boolean[] arg3) {
        this.tbsGen.setSubjectUniqueID(this.booleanToBitString(arg3));
    }
}

